@@ -1,12 +1,13 @@
 
-/* $Id: ai.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: ai.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2009 - 2010 by Yurii Chernyi <terraninfo@terraninfo.net>
+   Copyright (C) 2009 - 2011 by Yurii Chernyi <terraninfo@terraninfo.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,12 +16,17 @@
 
 /**
  * Composite AI with turn sequence which is a vector of stages
- * @file ai/composite/ai.cpp
+ * @file
  */
 
 #include "ai.hpp"
+#include "aspect.hpp"
+#include "engine.hpp"
+#include "goal.hpp"
+#include "property_handler.hpp"
 #include "stage.hpp"
 #include "../manager.hpp"
+#include "../../actions.hpp"
 #include "../../foreach.hpp"
 #include "../../log.hpp"
 
@@ -74,11 +80,10 @@ void ai_composite::on_create()
 	boost::function2<void, std::vector<stage_ptr>&, const config&> factory_stages =
 		boost::bind(&ai::ai_composite::create_stage,*this,_1,_2);
 
-	register_vector_property("engine",get_engines(), factory_engines);
-	register_vector_property("goal",get_goals(), factory_goals);
-	register_vector_property("stage",stages_, factory_stages);
-
-	register_aspect_property("aspect",get_aspects());
+	register_vector_property(property_handlers(),"engine",get_engines(), factory_engines);
+	register_vector_property(property_handlers(),"goal",get_goals(), factory_goals);
+	register_vector_property(property_handlers(),"stage",stages_, factory_stages);
+	register_aspect_property(property_handlers(),"aspect",get_aspects());
 
 }
 
@@ -138,20 +143,20 @@ void ai_composite::play_turn(){
 }
 
 
-const std::string& ai_composite::get_id() const
+std::string ai_composite::get_id() const
 {
 	return cfg_["id"];
 }
 
 
 
-const std::string& ai_composite::get_name() const
+std::string ai_composite::get_name() const
 {
 	return cfg_["name"];
 }
 
 
-const std::string& ai_composite::get_engine() const
+std::string ai_composite::get_engine() const
 {
 	return cfg_["engine"];
 }
@@ -160,7 +165,7 @@ const std::string& ai_composite::get_engine() const
 std::string ai_composite::evaluate(const std::string& str)
 {
 	config cfg;
-	cfg["engine"] = "fai";//@todo 1.9 : consider allowing other engines to evaluate
+	cfg["engine"] = "fai";///@todo 1.9 : consider allowing other engines to evaluate
 	engine_ptr e_ptr = get_engine_by_cfg(cfg);
 	if (!e_ptr) {
 		return interface::evaluate(str);
@@ -171,7 +176,7 @@ std::string ai_composite::evaluate(const std::string& str)
 
 void ai_composite::new_turn()
 {
-	//@todo 1.9 replace with event system
+	///@todo 1.9 replace with event system
 	recalculate_move_maps();
 	invalidate_defensive_position_cache();
 	invalidate_keeps_cache();
