@@ -1,11 +1,12 @@
-/* $Id: aspect.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: aspect.hpp 50501 2011-07-30 12:05:19Z mordante $ */
 /*
-   Copyright (C) 2009 - 2010 by Yurii Chernyi <terraninfo@terraninfo.net>
+   Copyright (C) 2009 - 2011 by Yurii Chernyi <terraninfo@terraninfo.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,33 +14,20 @@
 */
 
 /**
- * @file ai/composite/aspect.hpp
+ * @file
  */
 
 #ifndef AI_COMPOSITE_ASPECT_HPP_INCLUDED
 #define AI_COMPOSITE_ASPECT_HPP_INCLUDED
 
-#include "../../global.hpp"
-
-#include "component.hpp"
-#include "engine.hpp"
-#include "stage.hpp"
+#include "property_handler.hpp"
 #include "value_translator.hpp"
+#include "../lua/lua_object.hpp"
+#include "../lua/core.hpp"
+#include "../../scripting/lua.hpp"
 
-#include "../contexts.hpp"
-#include "../default/contexts.hpp"
-#include "../game_info.hpp"
-#include "../manager.hpp"
-#include "../../foreach.hpp"
 #include "../../log.hpp"
-#include "../../terrain_filter.hpp"
 
-#include <map>
-#include <stack>
-#include <vector>
-#include <deque>
-#include <iterator>
-#include <algorithm>
 #include <boost/bind.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/pointer_cast.hpp>
@@ -62,6 +50,7 @@ public:
 	{
 		valid_ = false;
 		valid_variant_ = false;
+		valid_lua_ = false;
 	}
 
 
@@ -97,21 +86,21 @@ public:
 		return true;
 	}
 
+	virtual std::string get_name() const
+	{ return name_; }
 
-	const std::string& get_name() const;
-
-
-	const std::string& get_id() const;
-
-
-	const std::string& get_engine() const;
+	virtual std::string get_id() const
+	{ return id_; }
 
+	virtual std::string get_engine() const
+	{ return engine_; }
 
 	static lg::log_domain& log();
 
 protected:
 	mutable bool valid_;
 	mutable bool valid_variant_;
+	mutable bool valid_lua_;
 
 	config cfg_;
 	bool invalidate_on_turn_start_;
@@ -131,6 +120,7 @@ public:
 		: aspect(context,cfg,id)
 		, value_()
 		, value_variant_()
+		, value_lua_()
 	{
 	}
 
@@ -160,6 +150,10 @@ public:
 			if (!valid_variant_ && valid_ ) {
 				value_variant_ = boost::shared_ptr<variant>(new variant(variant_value_translator<T>::value_to_variant(this->get())));
 				valid_variant_ = true;
+			} else if (!valid_variant_ && valid_lua_) {
+				value_ = value_lua_->get();
+				value_variant_ = boost::shared_ptr<variant>(new variant(variant_value_translator<T>::value_to_variant(this->get())));
+				valid_variant_ = true; // @note: temporary workaround
 			} else {
 				assert(valid_variant_);
 			}
@@ -173,15 +167,20 @@ public:
 	virtual boost::shared_ptr<T> get_ptr() const
 	{
 		if (!valid_) {
-			if (!valid_variant_) {
+			if (!(valid_variant_ || valid_lua_)) {
 				recalculate();
 			}
 
-			if (!valid_ && valid_variant_) {
-				value_ = boost::shared_ptr<T>(new T(variant_value_translator<T>::variant_to_value(get_variant())));
-				valid_ = true;
-			} else {
-				assert(valid_);
+			if (!valid_ ) {
+				if (valid_variant_) {
+					value_ = boost::shared_ptr<T>(new T(variant_value_translator<T>::variant_to_value(get_variant())));
+					valid_ = true;
+				} else if (valid_lua_){
+					value_ = value_lua_->get();
+					valid_ = true;
+				} else {
+					assert(valid_);
+				}
 			}
 		}
 		return value_;
@@ -190,6 +189,7 @@ public:
 protected:
 	mutable boost::shared_ptr<T> value_;
 	mutable boost::shared_ptr<variant> value_variant_;
+	mutable boost::shared_ptr< lua_object<T> > value_lua_;
 };
 
 
@@ -282,7 +282,7 @@ public:
 		boost::function2<void, typename aspect_type<T>::typesafe_ptr_vector&, const config&> factory_facets =
                         boost::bind(&ai::composite_aspect<T>::create_facet,*this,_1,_2);
 
-                register_vector_property("facet",facets_, factory_facets);
+                register_vector_property(this->property_handlers(),"facet",facets_, factory_facets);
 
 	}
 
@@ -300,7 +300,7 @@ public:
 
 	virtual void recalculate() const
 	{
-		//@todo 1.9 optimize in case of an aspect which returns variant
+		///@todo 1.9 optimize in case of an aspect which returns variant
 		foreach (const typename aspect_type<T>::typesafe_ptr &f, make_pair(facets_.rbegin(),facets_.rend())) {
 			if (f->active()) {
 				this->value_ = boost::shared_ptr<T>(f->get_ptr());
@@ -396,6 +396,58 @@ protected:
 
 };
 
+template<typename T>
+class lua_aspect : public typesafe_aspect<T>
+{
+public:
+	lua_aspect(readonly_context &context, const config &cfg, const std::string &id, boost::shared_ptr<lua_ai_context>& l_ctx)
+		: typesafe_aspect<T>(context, cfg, id)
+		, handler_(), code_()
+	{
+		std::string value;
+		if (cfg.has_attribute("value"))
+		{
+			value = cfg["value"].str();
+			if (value == "yes") /** @todo for Nephro or Crab: get rid of this workaround */
+			{
+				value = "true";
+			}
+			value = "return " + value;
+		}
+		else if (cfg.has_attribute("code"))
+		{
+			value = cfg["code"].str();
+		}
+		else
+		{
+			// error
+			return;
+		}
+		code_ = value;
+		handler_ = boost::shared_ptr<lua_ai_action_handler>(resources::lua_kernel->create_lua_ai_action_handler(value.c_str(), *l_ctx));
+	}
+
+	void recalculate() const
+	{
+		this->valid_lua_ = true;
+		boost::shared_ptr< lua_object<T> > l_obj = boost::shared_ptr< lua_object<T> >(new lua_object<T>());
+		config c = config();
+		handler_->handle(c, true, l_obj);
+		this->value_lua_ = l_obj;
+	}
+
+	config to_config() const
+	{
+		config cfg = aspect::to_config();
+		cfg["code"] = code_;
+		return cfg;
+	}
+
+private:
+	boost::shared_ptr<lua_ai_action_handler> handler_;
+	std::string code_;
+};
+
 
 class aspect_factory{
 public:
@@ -431,7 +483,8 @@ public:
 	{
 	}
 
-	virtual aspect_ptr get_new_instance( readonly_context &context, const config &cfg, const std::string &id){
+	aspect_ptr get_new_instance( readonly_context &context, const config &cfg, const std::string &id)
+	{
 		boost::shared_ptr<ASPECT> _a(new ASPECT(context,cfg,id));
 		aspect_ptr a = _a;
 		a->on_create();
@@ -439,6 +492,48 @@ public:
 	}
 };
 
+class lua_aspect_factory{
+public:
+	typedef boost::shared_ptr< lua_aspect_factory > factory_ptr;
+	typedef std::map<std::string, factory_ptr> factory_map;
+	typedef std::pair<const std::string, factory_ptr> factory_map_pair;
+
+	static factory_map& get_list() {
+		static factory_map *aspect_factories;
+		if (aspect_factories==NULL) {
+			aspect_factories = new factory_map;
+		}
+		return *aspect_factories;
+	}
+
+	virtual aspect_ptr get_new_instance( readonly_context &context, const config &cfg, const std::string &id, boost::shared_ptr<lua_ai_context>& l_ctx) = 0;
+
+	lua_aspect_factory( const std::string &name )
+	{
+		factory_ptr ptr_to_this(this);
+		get_list().insert(make_pair(name,ptr_to_this));
+	}
+
+	virtual ~lua_aspect_factory() {}
+};
+
+template<class ASPECT>
+class register_lua_aspect_factory : public lua_aspect_factory {
+public:
+	register_lua_aspect_factory( const std::string &name )
+		: lua_aspect_factory( name )
+	{
+	}
+
+	aspect_ptr get_new_instance( readonly_context &context, const config &cfg, const std::string &id, boost::shared_ptr<lua_ai_context>& l_ctx)
+	{
+		boost::shared_ptr<ASPECT> _a(new ASPECT(context,cfg,id,l_ctx));
+		aspect_ptr a = _a;
+		a->on_create();
+		return a;
+	}
+};
+
 
 } //end of namespace ai
 
