@@ -1,11 +1,12 @@
-/* $Id: ai.cpp 41353 2010-02-22 20:08:38Z boucman $ */
+/* $Id: ai.cpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
-   Copyright (C) 2008 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2008 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- * @file ai/formula/ai.cpp
+ * @file
  * Defines formula ai candidate actions - headers
  */
 
@@ -30,12 +31,16 @@
 #include "../manager.hpp"
 
 #include "../../callable_objects.hpp"
+#include "../../game_display.hpp"
 #include "../../foreach.hpp"
 #include "../../formula_debugger.hpp"
 #include "../../log.hpp"
 #include "../../menu_events.hpp"
+#include "../../pathfind/teleport.hpp"
+#include "../../resources.hpp"
 #include "../../terrain_filter.hpp"
 #include "../../tod_manager.hpp"
+#include "../../pathfind/pathfind.hpp"
 
 static lg::log_domain log_formula_ai("ai/engine/fai");
 #define DBG_AI LOG_STREAM(debug, log_formula_ai)
@@ -65,7 +70,6 @@ formula_ai::formula_ai(readonly_context &context, const config &cfg)
 	ai_ptr_(NULL),
 	cfg_(cfg),
 	recursion_counter_(context.get_recursion_count()),
-	outcome_positions_(),
 	keeps_cache_(),
 	infinite_loop_guardian_(),
 	vars_(),
@@ -74,6 +78,7 @@ formula_ai::formula_ai(readonly_context &context, const config &cfg)
 {
 	add_ref();
 	init_readonly_context_proxy(context);
+	LOG_AI << "creating new formula ai"<< std::endl;
 }
 
 void formula_ai::handle_exception(game_logic::formula_error& e) const
@@ -97,7 +102,7 @@ void formula_ai::handle_exception(game_logic::formula_error& e, const std::strin
 
 void formula_ai::display_message(const std::string& msg) const
 {
-	get_info().disp.add_chat_message(time(NULL), "fai", get_side(), msg,
+	resources::screen->add_chat_message(time(NULL), "fai", get_side(), msg,
 				events::chat_handler::MESSAGE_PUBLIC, false);
 
 }
@@ -131,8 +136,6 @@ std::string formula_ai::evaluate(const std::string& formula_str)
 		//formula_debugger fdb;
 		const variant v = f.evaluate(callable,NULL);
 
-                outcome_positions_.clear();
-
 		if (ai_ptr_) {
 			variant var = execute_variant(v, *ai_ptr_, true );
 
@@ -150,39 +153,32 @@ std::string formula_ai::evaluate(const std::string& formula_str)
 	}
 }
 
-void formula_ai::store_outcome_position(const variant& var)
-{
-    outcome_positions_.push_back(var);
-}
-
-
 variant formula_ai::make_action(game_logic::const_formula_ptr formula_, const game_logic::formula_callable& variables)
 {
 	if (!formula_) {
 		throw formula_error("null formula passed to make_action","","formula",0);
 	}
 	LOG_AI << "do move...\n";
-	const variant var = formula_->evaluate(variables);//@todo 1.9 add formula_debugger
+	const variant var = formula_->evaluate(variables);///@todo 1.9 add formula_debugger
 	variant res;
 
 	if (ai_ptr_) {
 		res = execute_variant(var, *ai_ptr_, false);
+	} else {
+		ERR_AI << "skipped execution of action because ai context is not set correctly" << std::endl;
 	}
 
-        //remove outcome_positions
-        outcome_positions_.clear();
-
 	return res;
 }
 
 pathfind::plain_route formula_ai::shortest_path_calculator(const map_location &src,
 	const map_location &dst, unit_map::iterator &unit_it,
-	std::set<map_location> & allowed_teleports) const
+	pathfind::teleport_map& allowed_teleports) const
 {
     map_location destination = dst;
 
-    unit_map &units_ = get_info().units;
-    pathfind::shortest_path_calculator calc(unit_it->second, current_team(), units_, get_info().teams, get_info().map);
+    unit_map &units_ = *resources::units;
+    pathfind::shortest_path_calculator calc(*unit_it, current_team(), units_, *resources::teams, *resources::game_map);
 
     unit_map::const_iterator dst_un = units_.find(destination);
 
@@ -197,7 +193,7 @@ pathfind::plain_route formula_ai::shortest_path_calculator(const map_location &s
         get_adjacent_tiles(destination,adj);
 
         for(size_t n = 0; n != 6; ++n) {
-                if(get_info().map.on_board(adj[n]) == false) {
+                if(resources::game_map->on_board(adj[n]) == false) {
                         continue;
                 }
 
@@ -224,54 +220,14 @@ pathfind::plain_route formula_ai::shortest_path_calculator(const map_location &s
     }
 
     pathfind::plain_route route = pathfind::a_star_search(src, destination, 1000.0, &calc,
-            get_info().map.w(), get_info().map.h(), &allowed_teleports);
+            resources::game_map->w(), resources::game_map->h(), &allowed_teleports);
 
     return route;
 }
 
-std::set<map_location> formula_ai::get_allowed_teleports(unit_map::iterator& unit_it) const
+pathfind::teleport_map formula_ai::get_allowed_teleports(unit_map::iterator& unit_it) const
 {
-  return pathfind::get_teleport_locations(unit_it->second, get_info().units, current_team(), true);
-}
-
-map_location formula_ai::path_calculator(const map_location& src, const map_location& dst, unit_map::iterator& unit_it) const{
-	std::map<map_location,pathfind::paths>::const_iterator path = get_possible_moves().find(src);
-
-	map_location destination = dst;
-
-	//check if destination is within unit's reach, if not, calculate where to move
-	if (!path->second.destinations.contains(dst))
-	{
-		std::set<map_location> allowed_teleports = get_allowed_teleports(unit_it);
-		//destination is too far, check where unit can go
-		pathfind::plain_route route = shortest_path_calculator( src, dst, unit_it, allowed_teleports );
-
-		if( route.steps.size() == 0 ) {
-			pathfind::emergency_path_calculator em_calc(unit_it->second, get_info().map);
-			route = pathfind::a_star_search(src, dst, 1000.0, &em_calc, get_info().map.w(), get_info().map.h(), &allowed_teleports);
-			if( route.steps.size() < 2 ) {
-				return map_location();
-			}
-		}
-		destination = map_location();
-		for (std::vector<map_location>::const_iterator loc_iter = route.steps.begin() + 1 ; loc_iter !=route.steps.end(); ++loc_iter) {
-			typedef move_map::const_iterator Itor;
-			std::pair<Itor,Itor> range = get_srcdst().equal_range(src);
-			bool found = false;
-			for(Itor i = range.first; i != range.second; ++i) {
-				if (i->second == *loc_iter ) {
-					found = true;
-					break;
-				}
-			}
-			if ( !found ) {
-				continue;
-			}
-			destination = *loc_iter;
-		}
-		return destination;
-	}
-	return destination;
+  return pathfind::get_teleport_locations(*unit_it, current_team(), true);
 }
 
 //commandline=true when we evaluate formula from commandline, false otherwise (default)
@@ -290,6 +246,8 @@ variant formula_ai::execute_variant(const variant& var, ai_context &ai_, bool co
 
 	variant error;
 
+	unit_map& units = *resources::units;
+
 	while( !vars.empty() ) {
 
 		if(vars.top().is_null()) {
@@ -391,8 +349,8 @@ variant formula_ai::execute_variant(const variant& var, ai_context &ai_, bool co
 			assert(_attack_analysis->movements.empty() == false);
 
 			//make sure that unit which has to attack is at given position and is able to attack
-			unit_map::const_iterator unit = get_info().units.find(_attack_analysis->movements.front().first);
-			if ( ( unit == get_info().units.end() ) || (unit->second.attacks_left() == 0) )
+			unit_map::const_iterator unit = units.find(_attack_analysis->movements.front().first);
+			if (!unit.valid() || unit->attacks_left() == 0)
 				continue;
 
 			const map_location& move_from = _attack_analysis->movements.front().first;
@@ -400,22 +358,22 @@ variant formula_ai::execute_variant(const variant& var, ai_context &ai_, bool co
 			const map_location& att_dst = _attack_analysis->target;
 
 			//check if target is still valid
-			unit = get_info().units.find(att_dst);
-			if ( unit == get_info().units.end() )
+			unit = units.find(att_dst);
+			if ( unit == units.end() )
 				continue;
 
                         //check if we need to move
                         if( move_from != att_src ) {
                             //now check if location to which we want to move is still unoccupied
-				unit = get_info().units.find(att_src);
-				if ( unit != get_info().units.end() ) {
+				unit = units.find(att_src);
+				if ( unit != units.end() ) {
 					continue;
 				}
 
 				ai_.execute_move_action(move_from, att_src);
                         }
 
-			if(get_info().units.count(att_src)) {
+			if(units.count(att_src)) {
 				ai_.execute_attack_action(_attack_analysis->movements.front().second,_attack_analysis->target,-1);
 			}
 			made_moves.push_back(action);
@@ -488,15 +446,15 @@ variant formula_ai::execute_variant(const variant& var, ai_context &ai_, bool co
 
 			if( !infinite_loop_guardian_.set_unit_var_check() ) {
 			    status = 5001; //exceeded nmber of calls in a row - possible infinite loop
-			} else if( (unit = get_info().units.find(set_unit_var_command->loc())) == get_info().units.end() ) {
+			} else if( (unit = units.find(set_unit_var_command->loc())) == units.end() ) {
 			    status = 5002; //unit not found
-			} else if( unit->second.side() != get_side() ) {
+			} else if (unit->side() != get_side()) {
 			    status = 5003;//unit does not belong to our side
 			}
 
 			if( status == 0 ){
 				LOG_AI << "Setting unit variable: " << set_unit_var_command->key() << " -> " << set_unit_var_command->value().to_debug_string() << "\n";
-				unit->second.add_formula_var(set_unit_var_command->key(), set_unit_var_command->value());
+				unit->add_formula_var(set_unit_var_command->key(), set_unit_var_command->value());
 				made_moves.push_back(action);
 			} else {
 				ERR_AI << "ERROR #" << status << " while executing 'set_unit_var' formula function\n";
@@ -548,7 +506,7 @@ variant formula_ai::execute_variant(const variant& var, ai_context &ai_, bool co
 		}
 
 		if( safe_call && (error != variant() || made_moves.empty() || made_moves.back() != action) ){
-		    /*if we have safe_call formula and either error occured, or current action
+		    /*if we have safe_call formula and either error occurred, or current action
 		     *was not reckognized, then evaluate backup formula from safe_call and execute it
 		     *during the next loop
 		     */
@@ -603,6 +561,8 @@ variant villages_from_set(const Container& villages,
 
 variant formula_ai::get_value(const std::string& key) const
 {
+	const unit_map& units = *resources::units;
+
 	if(key == "aggression")
 	{
 		return variant(get_aggression()*1000,variant::DECIMAL_VARIANT);
@@ -684,15 +644,15 @@ variant formula_ai::get_value(const std::string& key) const
 
 	} else if(key == "turn")
 	{
-		return variant(get_info().tod_manager_.turn());
+		return variant(resources::tod_manager->turn());
 
 	} else if(key == "time_of_day")
 	{
-		return variant(get_info().tod_manager_.get_time_of_day().id);
+		return variant(resources::tod_manager->get_time_of_day().id);
 
 	} else if(key == "my_side")
 	{
-		return variant(new team_callable((get_info().teams)[get_side()-1]));
+		return variant(new team_callable((*resources::teams)[get_side()-1]));
 
 	} else if(key == "my_side_number")
 	{
@@ -701,7 +661,7 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "teams")
 	{
 		std::vector<variant> vars;
-		for(std::vector<team>::const_iterator i = get_info().teams.begin(); i != get_info().teams.end(); ++i) {
+		for(std::vector<team>::const_iterator i = resources::teams->begin(); i != resources::teams->end(); ++i) {
 			vars.push_back(variant(new team_callable(*i)));
 		}
 		return variant(&vars);
@@ -709,7 +669,7 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "allies")
 	{
 		std::vector<variant> vars;
-		for( size_t i = 0; i < get_info().teams.size(); ++i) {
+		for( size_t i = 0; i < resources::teams->size(); ++i) {
 			if ( !current_team().is_enemy( i+1 ) )
 				vars.push_back(variant( i ));
 		}
@@ -718,7 +678,7 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "enemies")
 	{
 		std::vector<variant> vars;
-		for( size_t i = 0; i < get_info().teams.size(); ++i) {
+		for( size_t i = 0; i < resources::teams->size(); ++i) {
 			if ( current_team().is_enemy( i+1 ) )
 				vars.push_back(variant( i ));
 		}
@@ -731,8 +691,9 @@ variant formula_ai::get_value(const std::string& key) const
 		unit_types.build_all(unit_type::FULL);
 
 		const std::set<std::string>& recruits = current_team().recruits();
-		if(recruits.size()==0)
+		if(recruits.empty()) {
 			return variant( &vars );
+		}
 		for(std::set<std::string>::const_iterator i = recruits.begin(); i != recruits.end(); ++i)
 		{
 			const unit_type *ut = unit_types.find(*i);
@@ -750,14 +711,15 @@ variant formula_ai::get_value(const std::string& key) const
 
 		unit_types.build_all(unit_type::FULL);
 
-		for( size_t i = 0; i<get_info().teams.size(); ++i)
+		for( size_t i = 0; i<resources::teams->size(); ++i)
 		{
 			std::vector<variant> v;
 			tmp.push_back( v );
 
-			const std::set<std::string>& recruits = get_info().teams[i].recruits();
-			if(recruits.size()==0)
+			const std::set<std::string>& recruits = (*resources::teams)[i].recruits();
+			if(recruits.empty()) {
 				continue;
+			}
 			for(std::set<std::string>::const_iterator str_it = recruits.begin(); str_it != recruits.end(); ++str_it)
 			{
 				const unit_type *ut = unit_types.find(*str_it);
@@ -775,7 +737,7 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "units")
 	{
 		std::vector<variant> vars;
-		for(unit_map::const_iterator i = get_info().units.begin(); i != get_info().units.end(); ++i) {
+		for(unit_map::const_iterator i = units.begin(); i != units.end(); ++i) {
 			vars.push_back(variant(new unit_callable(*i)));
 		}
 		return variant(&vars);
@@ -784,13 +746,13 @@ variant formula_ai::get_value(const std::string& key) const
 	{
 		std::vector<variant> vars;
 		std::vector< std::vector< variant> > tmp;
-		for( size_t i = 0; i<get_info().teams.size(); ++i)
+		for( size_t i = 0; i<resources::teams->size(); ++i)
 		{
 			std::vector<variant> v;
 			tmp.push_back( v );
 		}
-		for(unit_map::const_iterator i = get_info().units.begin(); i != get_info().units.end(); ++i) {
-			tmp[ i->second.side()-1 ].push_back( variant(new unit_callable(*i)) );
+		foreach (const unit &u, units) {
+			tmp[u.side() - 1].push_back(variant(new unit_callable(u)));
 		}
 		for( size_t i = 0; i<tmp.size(); ++i)
 			vars.push_back( variant( &tmp[i] ));
@@ -799,8 +761,8 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "my_units")
 	{
 		std::vector<variant> vars;
-		for(unit_map::const_iterator i = get_info().units.begin(); i != get_info().units.end(); ++i) {
-			if(i->second.side() == get_side()) {
+		for(unit_map::const_iterator i = units.begin(); i != units.end(); ++i) {
+			if (i->side() == get_side()) {
 				vars.push_back(variant(new unit_callable(*i)));
 			}
 		}
@@ -809,9 +771,9 @@ variant formula_ai::get_value(const std::string& key) const
 	} else if(key == "enemy_units")
 	{
 		std::vector<variant> vars;
-		for(unit_map::const_iterator i = get_info().units.begin(); i != get_info().units.end(); ++i) {
-			if(current_team().is_enemy(i->second.side())) {
-				if (!i->second.incapacitated()) {
+		for(unit_map::const_iterator i = units.begin(); i != units.end(); ++i) {
+			if (current_team().is_enemy(i->side())) {
+				if (!i->incapacitated()) {
 					vars.push_back(variant(new unit_callable(*i)));
 				}
 			}
@@ -820,19 +782,19 @@ variant formula_ai::get_value(const std::string& key) const
 
 	} else if(key == "my_moves")
 	{
-		return variant(new move_map_callable(get_srcdst(), get_dstsrc(), get_info().units));
+		return variant(new move_map_callable(get_srcdst(), get_dstsrc(), units));
 
 	} else if(key == "my_attacks")
 	{
-		return variant(new attack_map_callable(*this, get_srcdst(), get_info().units));
+		return variant(new attack_map_callable(*this, get_srcdst(), units));
 	} else if(key == "enemy_moves")
 	{
-		return variant(new move_map_callable(get_enemy_srcdst(), get_enemy_dstsrc(), get_info().units));
+		return variant(new move_map_callable(get_enemy_srcdst(), get_enemy_dstsrc(), units));
 
 	} else if(key == "my_leader")
 	{
-		unit_map::const_iterator i = get_info().units.find_leader(get_side());
-		if(i == get_info().units.end()) {
+		unit_map::const_iterator i = units.find_leader(get_side());
+		if(i == units.end()) {
 			return variant();
 		}
 		return variant(new unit_callable(*i));
@@ -855,20 +817,20 @@ variant formula_ai::get_value(const std::string& key) const
 		return get_keeps();
 	} else if(key == "map")
 	{
-		return variant(new gamemap_callable(get_info().map));
+		return variant(new gamemap_callable(*resources::game_map));
 	} else if(key == "villages")
 	{
-		return villages_from_set(get_info().map.villages());
+		return villages_from_set(resources::game_map->villages());
 	} else if(key == "villages_of_side")
 	{
 		std::vector<variant> vars;
-		for(size_t i = 0; i<get_info().teams.size(); ++i)
+		for(size_t i = 0; i<resources::teams->size(); ++i)
 		{
 			vars.push_back( variant() );
 		}
 		for(size_t i = 0; i<vars.size(); ++i)
 		{
-			vars[i] = villages_from_set(get_info().teams[i].villages());
+			vars[i] = villages_from_set((*resources::teams)[i].villages());
 		}
 		return variant(&vars);
 
@@ -878,7 +840,7 @@ variant formula_ai::get_value(const std::string& key) const
 
 	} else if(key == "enemy_and_unowned_villages")
 	{
-		return villages_from_set(get_info().map.villages(), &current_team().villages());
+		return villages_from_set(resources::game_map->villages(), &current_team().villages());
 	}
 
 	return variant();
@@ -920,14 +882,14 @@ variant formula_ai::get_keeps() const
 {
 	if(keeps_cache_.is_null()) {
 		std::vector<variant> vars;
-		for(size_t x = 0; x != size_t(get_info().map.w()); ++x) {
-			for(size_t y = 0; y != size_t(get_info().map.h()); ++y) {
+		for(size_t x = 0; x != size_t(resources::game_map->w()); ++x) {
+			for(size_t y = 0; y != size_t(resources::game_map->h()); ++y) {
 				const map_location loc(x,y);
-				if(get_info().map.is_keep(loc)) {
+				if(resources::game_map->is_keep(loc)) {
 					map_location adj[6];
 					get_adjacent_tiles(loc,adj);
 					for(size_t n = 0; n != 6; ++n) {
-						if(get_info().map.is_castle(adj[n])) {
+						if(resources::game_map->is_castle(adj[n])) {
 							vars.push_back(variant(new location_callable(loc)));
 							break;
 						}
@@ -997,7 +959,7 @@ void formula_ai::on_create(){
 
 void formula_ai::evaluate_candidate_action(game_logic::candidate_action_ptr fai_ca)
 {
-	fai_ca->evaluate(this,get_info().units);
+	fai_ca->evaluate(this,*resources::units);
 
 }
 
@@ -1070,7 +1032,7 @@ config formula_ai::to_config() const
 		{
 			try {
 				i->second.serialize_to_string(str);
-			} catch (type_error te) {
+			} catch (type_error&) {
 				WRN_AI << "variable ["<< i->first <<"] is not serializable - it will not be persisted across savegames"<<std::endl;
 				continue;
 			}
