@@ -1,11 +1,12 @@
-/* $Id: builder.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: builder.hpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
-   Copyright (C) 2004 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2011 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- * @file builder.hpp
+ * @file
  * Definitions for the terrain builder.
  */
 
@@ -21,14 +22,12 @@
 #define BUILDER_H_INCLUDED
 
 #include "animated.hpp"
-#include "image.hpp"
 #include "map_location.hpp"
-
-#include <map>
-#include <set>
+#include "terrain_translation.hpp"
 
 class config;
-
+class gamemap;
+namespace image{ class locator; }
 /**
  * The class terrain_builder is constructed from a config object, and a
  * gamemap object. On construction, it parses the configuration and extracts
@@ -45,17 +44,32 @@ class terrain_builder
 {
 public:
 	/** Used as a parameter for the get_terrain_at function. */
-	enum ADJACENT_TERRAIN_TYPE {
-			ADJACENT_BACKGROUND,	/**<
-									 * Represents terrains which are to be
-									 * drawn behind unit sprites
-									 */
-			ADJACENT_FOREGROUND	    /**<
-									 * Represents terrains which are to be
-									 * drawn in front of them.
-									 */
+	enum TERRAIN_TYPE {
+			BACKGROUND,	/**<
+						* Represents terrains which are to be
+						* drawn behind unit sprites
+						*/
+			FOREGROUND	/**<
+						* Represents terrains which are to be
+						* drawn in front of them.
+						 */
 	};
 
+	/** The tile width used when using basex and basey. This is not,
+	 * necessarily, the tile width in pixels, this is totally
+	 * arbitrary. However, it will be set to 72 for convenience.
+	 */
+	static const int TILEWIDTH = 72;
+
+	/** The position of unit graphics in a tile. Graphics whose y
+	 * position is below this value are considered background for
+	 * this tile; graphics whose y position is above this value are
+	 * considered foreground.
+	 */
+	static const int UNITPOS = 36 + 18;
+
+	static const unsigned int DUMMY_HASH = 0;
+
 	/** A shorthand typedef for a list of animated image locators,
 	 * the base data type returned by the get_terrain_at method.
 	 */
@@ -63,8 +77,6 @@ public:
 
 	/** Constructor for the terrain_builder class.
 	 *
-	 * @param cfg			The main grame configuration object, where the
-	 *						[terrain_graphics] rule reside.
 	 * @param level		A level (scenario)-specific configuration file,
 	 *		    containing scenario-specific [terrain_graphics] rules.
 	 * @param map			A properly-initialized gamemap object representing
@@ -74,8 +86,16 @@ public:
 	 *						This image automatically gets the 'terrain/' prefix
 	 *						and '.png' suffix
 	 */
-	terrain_builder(const config& cfg, const config &level,
-		const gamemap* map, const std::string& offmap_image);
+	terrain_builder(const config &level, const gamemap* map,
+			const std::string& offmap_image);
+
+	/**  Set the config where we will parse the global terrain rules.
+	 *   This also flushes the terrain rules cache.
+	 *
+	 * @param cfg			The main grame configuration object, where the
+	 *						[terrain_graphics] rule reside.
+	 */
+	static void set_terrain_rules_cfg(const config& cfg);
 
 	const gamemap& map() const { return *map_; }
 
@@ -95,15 +115,15 @@ public:
 	 * @param tod   The string representing the current time-of day.
 	 *				Will be used if some images specify several
 	 *				time-of-day- related variants.
-	 * @param terrain_type ADJACENT_BACKGROUND or ADJACENT_FOREGROUND,
-	 *              depending on wheter we ask for the terrain which is
+	 * @param terrain_type BACKGROUND or FOREGROUND,
+	 *              depending on whether we ask for the terrain which is
 	 *              before, or after the unit sprite.
 	 *
 	 * @return      Returns a pointer list of animated images corresponding
 	 *              to the parameters, or NULL if there is none.
 	 */
 	const imagelist *get_terrain_at(const map_location &loc,
-			const std::string &tod, ADJACENT_TERRAIN_TYPE const terrain_type);
+			const std::string &tod, TERRAIN_TYPE const terrain_type);
 
 	/** Updates the animation at a given tile.
 	 * Returns true if something has changed, and must be redrawn.
@@ -135,13 +155,18 @@ public:
 	 * the [variant] tag may be omitted.
 	 */
 	struct rule_image_variant {
-		/** Shorthand constructor for this structure */
-		rule_image_variant(const std::string &image_string, const std::string &tod) :
+		/** Constructor for the normal defaut case */
+		rule_image_variant(const std::string &image_string, const std::string& variations, bool random_start = true) :
 			image_string(image_string),
-			image(),
-			tod(tod)
+			variations(variations),
+			images(),
+			tods(),
+			random_start(random_start)
 			{};
 
+		/** Constructor for true [variant] cases */
+		rule_image_variant(const std::string &image_string, const std::string& variations, const std::string& tod, bool random_start = true);
+
 		/** A string representing either the filename for an image, or
 		 *  a list of images, with an optional timing for each image.
 		 *  Corresponds to the "name" parameter of the [variant] (or of
@@ -161,23 +186,23 @@ public:
 		 */
 		std::string image_string;
 
+		/** A semi-solon separated list of string used to replace
+		 * @verbatim <code>@V</code> @endverbatim in image_string (if present)
+		 */
+		std::string variations;
+
 		/** An animated image locator built according to the image string.
 		 * This will be the image locator which will actually
 		 * be returned to the user.
 		 */
-		animated<image::locator> image;
-		/**
-		 * The time-of-day to which this variant applies.
-		 * Set to the empty string, this variant applies to all TODs.
-		 */
-		std::string tod;
-	};
+		std::vector< animated<image::locator> > images;
 
-	/**
-	 * A map associating a rule_image_variant to a string representing
-	 * the time of day.
-	 */
-	typedef std::map<std::string, rule_image_variant> rule_image_variantlist;
+		/** The Time of Day associated to this variant (if any)*/
+		std::set<std::string> tods;
+
+		/** Indicate if the animation uses a random shift */
+		bool random_start;
+	};
 
 	/**
 	 * Each terrain_graphics rule is associated a set of images, which are
@@ -189,6 +214,10 @@ public:
 	struct rule_image {
 		rule_image(int layer, int x, int y, bool global_image=false, int center_x=-1, int center_y=-1);
 
+		bool is_background() const {
+		return layer < 0 || (layer == 0 && basey < UNITPOS);
+		}
+
 		/** The layer of the image for horizontal layering */
 		int layer;
 		/** The position of the image base (that is, the point where
@@ -196,9 +225,8 @@ public:
 		 */
 		int basex, basey;
 
-		/** A list of Time-Of-Day-related variants for this image
-		 */
-		rule_image_variantlist variants;
+		/** A list of variants for this image */
+		std::vector<rule_image_variant> variants;
 
 		/** Set to true if the image was defined as a child of the
 		 * [terrain_graphics] tag, set to false if it was defined as a
@@ -255,30 +283,19 @@ public:
 	 */
 	struct tile
 	{
-		/** An ordered rule_image list */
-		typedef std::multimap<int, const rule_image*> ordered_ri_list;
-
 		/** Contructor for the tile() structure */
 		tile();
 
-		/** Adds an image, extracted from an ordered rule_image list,
-		 * to the background or foreground image cache.
-		 *
-		 * @param tod    The current time-of-day, to select between
-		 *               images presenting several variants.
-		 * @param itor   An iterator pointing to the rule_image where
-		 *               to extract the image we wish to add to the
-		 *               cache.
-		 */
-		void add_image_to_cache(const std::string &tod, ordered_ri_list::const_iterator itor);
-
+		struct rule_image_rand;
+		typedef std::pair<const rule_image_rand*, const rule_image_variant*> log_details;
+		typedef std::vector<log_details> logs;
 		/** Rebuilds the whole image cache, for a given time-of-day.
 		 * Must be called when the time-of-day has changed,
 		 * to select the correct images.
 		 *
 		 * @param tod    The current time-of-day
 		 */
-		void rebuild_cache(const std::string &tod);
+		void rebuild_cache(const std::string &tod, logs* log = NULL);
 
 		/** Clears all data in this tile, and resets the cache */
 		void clear();
@@ -286,10 +303,23 @@ public:
 		/** The list of flags present in this tile */
 		std::set<std::string> flags;
 
-		/** The list of images associated to this tile, ordered by
-		 * their layer first and base-y position second.
+		/** Represent a rule_image applied with a random seed.*/
+		struct rule_image_rand{
+			rule_image_rand(const rule_image* r_i, unsigned int rnd) : ri(r_i), rand(rnd) {}
+
+			const rule_image* operator->() const {return ri;}
+			/** sort by layer first then by basey */
+			bool operator<(const rule_image_rand& o) const {
+				return ri->layer < o.ri->layer ||
+						(ri->layer == o.ri->layer && ri->basey < o.ri->basey);}
+
+			const rule_image* ri;
+			unsigned int rand;
+		};
+
+		/** The list of rule_images and random seeds associated to this tile.
 		 */
-		ordered_ri_list images;
+		std::vector<rule_image_rand> images;
 
 		/** The list of images which are in front of the unit sprites,
 		 * attached to this tile. This member is considered a cache:
@@ -306,13 +336,17 @@ public:
 		 */
 		std::string last_tod;
 
+		/** Indicates if 'images' is sorted */
+		bool sorted_images;
 	};
 
+	tile* get_tile(const map_location &loc);
+
 private:
 	/**
 	 * The list of constraints attached to a terrain_graphics WML rule.
 	 */
-	typedef std::map<map_location, terrain_constraint> constraint_set;
+	typedef std::vector<terrain_constraint> constraint_set;
 
 	/**
 	 * The in-memory representation of a [terrain_graphics] WML rule.
@@ -322,8 +356,10 @@ private:
 		building_rule() :
 			constraints(),
 			location_constraints(),
-			probability(0),
-			precedence(0)
+			probability(100),
+			precedence(0),
+			local(false),
+			hash_(DUMMY_HASH)
 		{}
 
 		/**
@@ -346,12 +382,21 @@ private:
 		int probability;
 
 		/**
-		 * The precedence of this rule. Used to order rules differently
-		 * that the order in which they appear.
-		 * Defined if the "precedence" parameter of the
-		 * [terrain_graphics] element is set.
+		 * Ordering relation between the rules.
 		 */
 		int precedence;
+
+		/**
+		 * Indicate if the rule is only for this scenario
+		 */
+		bool local;
+
+		bool operator<(building_rule const &that) const
+		{ return precedence < that.precedence; }
+
+		unsigned int get_hash() const;
+	private:
+		mutable unsigned int hash_;
 	};
 
 	/**
@@ -367,7 +412,7 @@ private:
 				tiles_((x + 4) * (y + 4)),
 				x_(x),
 				y_(y)
-			{}
+			{reset();}
 
 		/**
 		 * Returns a reference to the tile which is at the position
@@ -415,18 +460,18 @@ private:
 	 * A set of building rules. In-memory representation
 	 * of the whole set of [terrain_graphics] rules.
 	 */
-	typedef std::multimap<int, building_rule> building_ruleset;
+	typedef std::multiset<building_rule> building_ruleset;
 
 	/**
-	 * Tests for validity of a rule. A rule is considered valid if all its
-	 * images are present. This method is used, when building the ruleset,
-	 * to only add rules which are valid to the ruleset.
+	 * Load images and tests for validity of a rule. A rule is considered
+	 * valid if all its images are present. This method is used, when building
+	 * the ruleset, to only add rules which are valid to the ruleset.
 	 *
 	 * @param rule  The rule to test for validity
 	 *
 	 * @return		true if the rule is valid, false if it is not.
 	 */
-	bool rule_valid(const building_rule &rule) const;
+	bool load_images(building_rule &rule);
 
 	/**
 	 * Starts the animation on a rule.
@@ -439,8 +484,8 @@ private:
 
 	/**
 	 *  "Rotates" a constraint from a rule.
-	 *  Takes a template constraint from a template rule, and creates
-	 *  a constraint from this template, rotated to the given angle.
+	 *  Takes a template constraint from a template rule, and rotates
+	 *  to the given angle.
 	 *
 	 *  On a constraint, the relative position of each rule, and the "base"
 	 *  of each vertical images, are rotated according to the given angle.
@@ -457,70 +502,70 @@ private:
 	 *  @param constraint  A template constraint to rotate
 	 *  @param angle       An int, from 0 to 5, representing the rotation angle.
 	 */
-	terrain_constraint rotate(const terrain_constraint &constraint, int angle);
+	void rotate(terrain_constraint &constraint, int angle);
 
 	/**
-	 * Replaces, in a given string, a token with its value.
+	 * Replaces, in a given string, rotation tokens with their values.
 	 *
-	 * @param s            The string in which to do the replacement
-	 * @param token        The token to substitute
-	 * @param replacement  The replacement string
+	 * @param s            the string in which to do the replacement
+	 * @param angle        the angle for substituting the correct replacement.
+	 * @param replacement  the replacement strings.
 	 */
-	void replace_token(std::string &s, const std::string &token,
-			const std::string& replacement);
+	void replace_rotate_tokens(std::string &s, int angle,
+		const std::vector<std::string> &replacement);
 
 	/**
-	 * Replaces, in a given rule_image, a token with its value.
+	 * Replaces, in a given rule_image, rotation tokens with their values.
 	 * The actual substitution is done in all variants of the given image.
 	 *
-	 * @param image        The rule_image in which to do the replacement
-	 * @param token        The token to substitute
-	 * @param replacement  The replacement string
+	 * @param image        the rule_image in which to do the replacement.
+	 * @param angle        the angle for substituting the correct replacement.
+	 * @param replacement  the replacement strings.
 	 */
-	void replace_token(rule_image &image, const std::string &token,
-			const std::string& replacement);
+	void replace_rotate_tokens(rule_image &image, int angle,
+		const std::vector<std::string> &replacement);
 
 	/**
-	 * Replaces, in a given rule_variant_image, a token with its value.
+	 * Replaces, in a given rule_variant_image, rotation tokens with their values.
 	 * The actual substitution is done in the "image_string" parameter
 	 * of this rule_variant_image.
 	 *
-	 * @param variant      The rule_variant_image in which to do the replacement
-	 * @param token        The token to substitute
-	 * @param replacement  The replacement string
+	 * @param variant      the rule_variant_image in which to do the replacement.
+	 * @param angle        the angle for substituting the correct replacement.
+	 * @param replacement  the replacement strings.
 	 */
-	void replace_token(rule_image_variant &variant, const std::string &token,
-			const std::string& replacement)
-		{ replace_token(variant.image_string, token, replacement); }
+	void replace_rotate_tokens(rule_image_variant &variant, int angle,
+		const std::vector<std::string> &replacement)
+	{ replace_rotate_tokens(variant.image_string, angle, replacement); }
 
 	/**
-	 * Replaces, in a given rule_imagelist, a token with its value.
+	 * Replaces, in a given rule_imagelist, rotation tokens with their values.
 	 * The actual substitution is done in all rule_images contained
 	 * in the rule_imagelist.
 	 *
-	 * @param &					The rule_imagelist in which to do the replacement
-	 * @param token		The token to substitute
-	 * @param replacement		The replacement string
+	 * @param list         the rule_imagelist in which to do the replacement.
+	 * @param angle        the angle for substituting the correct replacement.
+	 * @param replacement  the replacement strings.
 	 */
-	void replace_token(rule_imagelist &, const std::string &token,
-			const std::string& replacement);
+	void replace_rotate_tokens(rule_imagelist &list, int angle,
+		const std::vector<std::string> &replacement);
 
 	/**
-	 * Replaces, in a given building_rule, a token with its value.
+	 * Replaces, in a given building_rule, rotation tokens with their values.
 	 * The actual substitution is done in the rule_imagelists contained
 	 * in all constraints of the building_rule, and in the flags
 	 * (has_flag, set_flag and no_flag) contained in all constraints
 	 * of the building_rule.
 	 *
-	 * @param s            The building_rule  in which to do the replacement
-	 * @param token        The token to substitute
-	 * @param replacement  The replacement string
+	 * @param rule         the building_rule in which to do the replacement.
+	 * @param angle        the angle for substituting the correct replacement.
+	 * @param replacement  the replacement strings.
 	 */
-	void replace_token(building_rule &s, const std::string &token,
-			const std::string& replacement);
+	void replace_rotate_tokens(building_rule &rule, int angle,
+		const std::vector<std::string> &replacement);
 
 	/**
-	 *  Rotates a template rule to a given angle, and returns the rotated rule.
+	 *  Rotates a template rule to a given angle.
 	 *
 	 *  Template rules are defined like normal rules, except that:
 	 *  * Flags and image filenames may contain template strings of the form
@@ -555,8 +600,8 @@ private:
 	 *    r2, r3, r4, r5, r0, r1 and so on.
 	 *
 	 */
-	building_rule rotate_rule(const building_rule &rule, int angle,
-							  const std::vector<std::string>& angle_name);
+	void rotate_rule(building_rule &rule, int angle,
+		const std::vector<std::string> &angle_name);
 
 	/**
 	 * Parses a "config" object, which should contains [image] children,
@@ -588,7 +633,7 @@ private:
 	 * @param global_images A configuration object containing [image] tags
 	 *                      describing rule-global images.
 	 */
-	void add_constraints(constraint_set& constraints,
+	terrain_constraint &add_constraints(constraint_set& constraints,
 			const map_location &loc, const t_translation::t_match& type,
 			const config& global_images);
 
@@ -626,8 +671,8 @@ private:
 	/**
 	 * Adds a rule to a ruleset. Checks for validity before adding the rule.
 	 *
-	 * @param rules   The ruleset into which to add the rules.
-	 * @param rule    The rule to add.
+	 * @param rules      The ruleset into which to add the rules.
+	 * @param rule       The rule to add.
 	 */
 	void add_rule(building_ruleset& rules, building_rule &rule);
 
@@ -638,19 +683,23 @@ private:
 	 * @param rules      The ruleset into which to add the rules.
 	 * @param tpl        The template rule
 	 * @param rotations  A comma-separated string containing the
-	 *					 6 values for replacing rotation
-	 *					 template strings @verbatim (@Rn) @endverbatim
+	 *                   6 values for replacing rotation template
+	 *                   template strings @verbatim (@Rn) @endverbatim
 	 */
-	void add_rotated_rules(building_ruleset& rules, building_rule& tpl, const std::string &rotations);
+	void add_rotated_rules(building_ruleset& rules, building_rule& tpl,
+		const std::string &rotations);
 
 	/**
 	 * Parses a configuration object containing [terrain_graphics] rules,
 	 * and fills the building_rules_ member of the current class according
 	 * to those.
 	 *
-	 * @param cfg		The configuration object to parse.
+	 * @param cfg       The configuration object to parse.
+	 * @param local     Mark the rules as local only.
 	 */
-	void parse_config(const config &cfg);
+	void parse_config(const config &cfg, bool local = true);
+
+	void parse_global_config(const config &cfg) { parse_config(cfg, false); }
 
 	/**
 	 * Adds a builder rule for the _off^_usr tile, this tile only has 1 image.
@@ -659,6 +708,8 @@ private:
 	 */
 	void add_off_map_rule(const std::string& image);
 
+	void flush_local_rules();
+
 	/**
 	 * Checks whether a terrain code matches a given list of terrain codes.
 	 *
@@ -693,15 +744,10 @@ private:
 	 * @param rule      The rule to check.
 	 * @param loc       The location in the map where we want to check
 	 *                  whether the rule matches.
-	 * @param rule_index The index of the rule, relative to the start of
-	 *                  the rule list. Rule indices are used for seeding
-	 *                  the pseudo-random-number generator used for
-	 *                  probability calculations.
 	 * @param type_checked The constraint which we already know that its
 	 *                  terrain types matches.
 	 */
-	bool rule_matches(const building_rule &rule, const map_location &loc,
-			const int rule_index, const constraint_set::const_iterator type_checked) const;
+	bool rule_matches(const building_rule &rule, const map_location &loc, const terrain_constraint *type_checked) const;
 
 	/**
 	 * Applies a rule at a given location: applies the result of a
@@ -741,8 +787,11 @@ private:
 	 */
 	terrain_by_type_map terrain_by_type_;
 
-	building_ruleset building_rules_;
+	/** Parsed terrain rules. Cached between instances */
+	static building_ruleset building_rules_;
 
+	/** Config used to parse global terrain rules */
+	static const config* rules_cfg_;
 };
 
 #endif
