@@ -1,12 +1,13 @@
-/* $Id: construct_dialog.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: construct_dialog.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2006 - 2010 by Patrick Parker <patrick_x99@hotmail.com>
+   Copyright (C) 2006 - 2011 by Patrick Parker <patrick_x99@hotmail.com>
    wesnoth widget Copyright (C) 2003-5 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -20,8 +21,6 @@
 
 #include "widgets/label.hpp"
 #include "widgets/textbox.hpp"
-#include "widgets/button.hpp"
-#include "widgets/menu.hpp"
 #include "key.hpp"
 
 namespace gui {
@@ -321,21 +320,6 @@ private:
 };
 
 typedef Uint32 msecs;
-const msecs three_blinks = 300;	// 3 times the 0.1sec human reflex-arc time
-
-class message_dialog : public gui::dialog
-{
-public:
-	message_dialog(display &disp, const std::string& title="", const std::string& message="", const gui::DIALOG_TYPE type=gui::MESSAGE)
-		: dialog(disp, title, message, type, message_style), prevent_misclick_until_(0)
-	{}
-	~message_dialog();
-	int show(msecs minimum_lifetime = three_blinks);
-protected:
-	void action(gui::dialog_process_info &dp_info);
-private:
-	msecs prevent_misclick_until_;
-};
 
 } //end namespace gui
 #endif
