@@ -1,12 +1,13 @@
-/* $Id: controller_base.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: controller_base.hpp 49958 2011-06-20 23:33:10Z fendrin $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2011 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -18,13 +19,15 @@
 
 #include "global.hpp"
 
-#include "generic_event.hpp"
 #include "hotkeys.hpp"
 #include "key.hpp"
-#include "preferences_display.hpp"
-#include "sdl_utils.hpp"
 
-#include <vector>
+#include "joystick.hpp"
+
+#include "map.hpp"
+
+class CVideo;
+
 namespace events {
 class mouse_handler_base;
 }
@@ -35,7 +38,7 @@ public:
 	controller_base(int ticks, const config& game_config, CVideo& video);
 	virtual ~controller_base();
 
-	void play_slice();
+	void play_slice(bool is_delay_enabled = true);
 
 	int get_ticks();
 
@@ -51,11 +54,6 @@ protected:
 	 */
 	virtual void slice_end();
 
-	/**
-	 * Inherited from command_executor, still declared pure virtual but might
-	 * provide some defaults
-	 */
-	virtual bool can_execute_command(hotkey::HOTKEY_COMMAND command, int index=-1) const = 0;
 	/**
 	 * Get a reference to a mouse handler member a derived class uses
 	 */
@@ -74,35 +72,41 @@ protected:
 	 */
 	virtual bool have_keyboard_focus();
 
+
 	/**
-	 * Handle scrolling by keyboard and moving mouse near map edges
+	 * Handle scrolling by keyboard, joystick and moving mouse near map edges
 	 * @see is_keyboard_scroll_active
 	 * @return true when there was any scrolling, false otherwise
 	 */
-	bool handle_scroll(CKey& key, int mousex, int mousey, int mouse_flags);
+	bool handle_scroll(CKey& key, int mousex, int mousey, int mouse_flags, double joystickx, double joysticky);
 
 	/**
 	 * Process mouse- and keypress-events from SDL.
 	 * Not virtual but calls various virtual function to allow specialized
-	 * behaviour of derived classess.
+	 * behaviour of derived classes.
 	 */
 	void handle_event(const SDL_Event& event);
 
 	/**
 	 * Process keydown (only when the general map display does not have focus).
-	 * Overriden in derived classes
+	 */
+	virtual void process_focus_keydown_event(const SDL_Event& event);
+
+	/**
+	 * Process keydown (always).
+	 * Overridden in derived classes
 	 */
 	virtual void process_keydown_event(const SDL_Event& event);
 
 	/**
 	 * Process keyup (always).
-	 * Overriden in derived classes
+	 * Overridden in derived classes
 	 */
 	virtual void process_keyup_event(const SDL_Event& event);
 
 	/**
 	 * Called after processing a mouse button up or down event
-	 * Overriden in derived classes
+	 * Overridden in derived classes
 	 */
 	virtual void post_mouse_press(const SDL_Event& event);
 
@@ -110,11 +114,14 @@ protected:
 
 	virtual bool in_context_menu(hotkey::HOTKEY_COMMAND command) const;
 
+	const config &get_theme(const config& game_config, std::string theme_name);
 	const config& game_config_;
 	const int ticks_;
 	CKey key_;
 	bool browse_;
 	bool scrolling_;
+	joystick_manager joystick_manager_;
+
 };
 
 
