@@ -1,11 +1,12 @@
-/* $Id: display.hpp 41179 2010-02-13 10:18:45Z silene $ */
+/* $Id: display.hpp 51824 2011-11-04 04:27:51Z shadowmaster $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- * @file display.hpp
+ * @file
  *
  * map_display and display: classes which take care of
  * displaying the map and game-data on the screen.
@@ -35,25 +36,18 @@
 
 class config;
 class terrain_builder;
+struct time_of_day;
 class map_labels;
+class arrow;
 
-#include "generic_event.hpp"
-#include "image.hpp"
 #include "font.hpp"
 #include "key.hpp"
-#include "map_location.hpp"
-#include "reports.hpp"
 #include "team.hpp"
-#include "time_of_day.hpp"
 #include "theme.hpp"
 #include "video.hpp"
 #include "widgets/button.hpp"
 
-#include "SDL.h"
-
-#include <map>
-#include <set>
-#include <string>
+#include <list>
 
 #include <boost/function.hpp>
 #include <boost/scoped_ptr.hpp>
@@ -86,6 +80,20 @@ public:
 	static Uint32 max_rgb(Uint32 first,Uint32 second)
 		{ return rgb(std::max(red(first),red(second)),std::max(green(first),green(second)),std::max(blue(first),blue(second))) ; }
 
+	/**
+	 * Add r,g,b from tod_manager to the map
+	 *
+	 */
+	void update_tod();
+
+	/**
+	 * Add r,g,b to the colors for all images displayed on the map.
+	 *
+	 * Used for special effects like flashes.
+	 */
+	void adjust_color_overlay(int r, int g, int b);
+
+
 	/** Gets the underlying screen object. */
 	CVideo& video() { return screen_; }
 
@@ -108,7 +116,7 @@ public:
 		{ return theme_.unit_image_location(screen_area()); }
 
 	SDL_Rect screen_area() const
-		{ const SDL_Rect res = {0,0,w(),h()}; return res; }
+		{ return create_rect(0, 0, w(), h()); }
 
 	/**
 	 * Returns the maximum area used for the map
@@ -199,7 +207,7 @@ public:
 				: loc_(loc), rect_(rect){};
 
 			/** increment y first, then when reaching bottom, increment x */
-			void operator++();
+			iterator& operator++();
 			bool operator==(const iterator &that) const { return that.loc_ == loc_; }
 			bool operator!=(const iterator &that) const { return that.loc_ != loc_; }
 			const map_location& operator*() const {return loc_;};
@@ -264,12 +272,23 @@ public:
 	void clear_redraw_observers();
 
 	theme& get_theme() { return theme_; }
+
+	/**
+	 * Retrieves a pointer to a theme UI button.
+	 *
+	 * @note The returned pointer may either be NULL, meaning the button
+	 *       isn't defined by the current theme, or point to a valid
+	 *       gui::button object. However, the objects retrieved will be
+	 *       destroyed and recreated by draw() method calls. Do *NOT* store
+	 *       these pointers for longer than strictly necessary to
+	 *       accomplish a specific task before the next screen refresh.
+	 */
 	gui::button* find_button(const std::string& id);
 	gui::button::TYPE string_to_button_type(std::string type);
 	void create_buttons();
 	void invalidate_theme() { panelsDrawn_ = false; }
 
-	void refresh_report(reports::TYPE report_num, reports::report report);
+	void refresh_report(std::string const &report_name, const config &);
 
 	// Will be overridden in the display subclass
 	virtual void draw_minimap_units() {};
@@ -299,36 +318,16 @@ public:
 
 	/**
 	 * Per-location invalidation called by invalidate_animations()
-	 * defaults to no action, overriden by derived classes
+	 * defaults to no action, overridden by derived classes
 	 */
 	virtual void invalidate_animations_location(const map_location& /*loc*/) {}
 
-	/**
-	 * What hex are currently invalidated (read only)
-	 * used for some fine grained invalidation algorithm which need recurstion
-	 */
-	const std::set<map_location> & get_invalidated() { return invalidated_; }
-
-
-
 	const gamemap& get_map() const { return *map_; }
 
 	/**
-	 * The last action in drawing a tile is adding the overlays.
-	 * These overlays are drawn in the following order:
-	 * hex_overlay_			if the drawn location is in the map
-	 * selected_hex_overlay_	if the drawn location is selected
-	 * mouseover_hex_overlay_	if the drawn location is underneath the mouse
-	 *
-	 * These functions require a prerendered surface.
-	 * Since they are drawn at the top, they are not influenced by TOD, shroud etc.
+	 * mouseover_hex_overlay_ require a prerendered surface
+	 * and is drawn underneath the mouse's location
 	 */
-	void set_selected_hex_overlay(const surface& image)
-		{ selected_hex_overlay_ = image; }
-
-	void clear_selected_hex_overlay()
-		{ selected_hex_overlay_ = NULL; }
-
 	void set_mouseover_hex_overlay(const surface& image)
 		{ mouseover_hex_overlay_ = image; }
 
@@ -342,10 +341,19 @@ public:
 	 * delay is the number of frames between each darkening
 	 * (0 to toggle).
 	 */
-	void sunset(const size_t delay = 0);
+	static void sunset(const size_t delay = 0);
 
-	/** Toogle to continuously redraw the screen. */
-	void toggle_benchmark();
+	/** Toggle to continuously redraw the screen. */
+	static void toggle_benchmark();
+
+	/**
+	 * Toggle to debug foreground terrain.
+	 * Separate background and foreground layer
+	 * to better spot any error there.
+	 */
+	static void toggle_debug_foreground();
+
+	terrain_builder& get_builder() {return *builder_;};
 
 	void flip();
 
@@ -406,13 +414,13 @@ public:
 	/** Sets the zoom amount to the default. */
 	void set_default_zoom();
 
-	enum SCROLL_TYPE { SCROLL, WARP, ONSCREEN };
+	enum SCROLL_TYPE { SCROLL, WARP, ONSCREEN, ONSCREEN_WARP };
 
 	/**
 	 * Scroll such that location loc is on-screen.
 	 * WARP jumps to loc; SCROLL uses scroll speed;
 	 * ONSCREEN only scrolls if x,y is offscreen
-	 * force : scroll even if prefferences tell us not to
+	 * force : scroll even if preferences tell us not to
 	 */
 	void scroll_to_tile(const map_location& loc, SCROLL_TYPE scroll_type=ONSCREEN, bool check_fogged=true,bool force = true);
 
@@ -435,6 +443,8 @@ public:
 	/** Expose the event, so observers can be notified about map scrolling. */
 	events::generic_event &scroll_event() const { return scroll_event_; }
 
+	events::generic_event& complete_redraw_event() { return complete_redraw_event_; }
+
 	/** Check if a tile is fully visible on screen. */
 	bool tile_fully_on_screen(const map_location& loc);
 
@@ -455,7 +465,7 @@ public:
 
 	/** Announce a message prominently. */
 	void announce(const std::string& msg,
-		       const SDL_Color& colour = font::GOOD_COLOUR);
+		       const SDL_Color& color = font::GOOD_COLOR);
 
 	/**
 	 * Schedule the minimap for recalculation.
@@ -469,15 +479,13 @@ public:
 	 */
 	void redraw_minimap() { redrawMinimap_ = true; }
 
-	/**
-	 * Set what will be shown for the report with type which_report.
-	 * Note that this only works for some reports,
-	 * i.e. reports that can not be deducted
-	 * from the supplied arguments to generate_report,
-	 * currently: SELECTED_TERRAIN, EDIT_LEFT_BUTTON_FUNCTION
-	 */
-	void set_report_content(const reports::TYPE which_report, const std::string &content);
-	std::map<reports::TYPE, std::string> get_report_contents() {return report_;};
+	virtual const time_of_day& get_time_of_day(const map_location& loc = map_location::null_location) const;
+
+	virtual bool has_time_area() const {return false;};
+
+	void write(config& cfg) const;
+private:
+	void read(const config& cfg);
 
 protected:
 	/** Clear the screen contents */
@@ -521,11 +529,6 @@ protected:
 	 */
 	virtual image::TYPE get_image_type(const map_location& loc);
 
-	/**
-	 * Update time of day member to the tod to be used in the current drawing
-	 */
-	virtual void update_time_of_day();
-
 	/**
 	 * Called near the end of a draw operation, derived classes can use this
 	 * to render a specific sidebar. Very similar to post_commit.
@@ -547,19 +550,23 @@ protected:
 
 	void draw_minimap();
 
-	enum ADJACENT_TERRAIN_TYPE { ADJACENT_BACKGROUND, ADJACENT_FOREGROUND, ADJACENT_FOGSHROUD };
+	enum TERRAIN_TYPE { BACKGROUND, FOREGROUND};
 
 	std::vector<surface> get_terrain_images(const map_location &loc,
 					const std::string& timeid,
 					image::TYPE type,
-					ADJACENT_TERRAIN_TYPE terrain_type);
+					TERRAIN_TYPE terrain_type);
 
-	std::vector<std::string> get_fog_shroud_graphics(const map_location& loc);
+	std::vector<surface> get_fog_shroud_images(const map_location& loc, image::TYPE image_type);
 
 	void draw_image_for_report(surface& img, SDL_Rect& rect);
 
 	void scroll_to_xy(int screenxpos, int screenypos, SCROLL_TYPE scroll_type,bool force = true);
 
+	void fill_images_list(const std::string& prefix, std::vector<std::string>& images);
+
+	const std::string& get_variant(const std::vector<std::string>& variants, const map_location &loc) const;
+
 	CVideo& screen_;
 	const gamemap* map_;
 	const team *viewpoint_;
@@ -580,13 +587,17 @@ protected:
 	bool turbo_;
 	bool invalidateGameStatus_;
 	boost::scoped_ptr<map_labels> map_labels_;
-	std::string shroud_image_;
-	std::string fog_image_;
-	time_of_day tod_;
+
 
 	/** Event raised when the map is being scrolled */
 	mutable events::generic_event scroll_event_;
 
+	/**
+	 * notify observers that the screen has been redrawn completely
+	 * atm this is used for replay_controller to add replay controls to the standard theme
+	 */
+	events::generic_event complete_redraw_event_;
+
 	/**
 	 * Holds the tick count for when the next drawing event is scheduled.
 	 * Drawing shouldn't occur before this time.
@@ -594,55 +605,30 @@ protected:
 	int nextDraw_;
 
 	// Not set by the initializer:
-	SDL_Rect reportRects_[reports::NUM_REPORTS];
-	surface reportSurfaces_[reports::NUM_REPORTS];
-	reports::report reports_[reports::NUM_REPORTS];
-	std::map<reports::TYPE, std::string> report_;
+	std::map<std::string, SDL_Rect> reportRects_;
+	std::map<std::string, surface> reportSurfaces_;
+	std::map<std::string, config> reports_;
 	std::vector<gui::button> buttons_;
 	std::set<map_location> invalidated_;
 	std::set<map_location> previous_invalidated_;
-	surface selected_hex_overlay_;
 	surface mouseover_hex_overlay_;
+	// If we're transitioning from one time of day to the next,
+	// then we will use these two masks on top of all hexes when we blit.
+	surface tod_hex_mask1, tod_hex_mask2;
+	std::vector<std::string> fog_images_;
+	std::vector<std::string> shroud_images_;
+
 	map_location selectedHex_;
 	map_location mouseoverHex_;
 	CKey keys_;
 
+	/** Local cache for preferences::animate_map, since it is constantly queried. */
+	bool animate_map_;
+
+	/** Local cache for preferences "local_tod_light" */
+	bool local_tod_light_;
+
 public:
-	/** Helper structure for rendering the terrains. */
-	struct tblit{
-		tblit(const int x, const int y) :
-			x(x),
-			y(y),
-			surf(),
-			clip()
-			{}
-
-		tblit(const int x, const int y, const surface& surf,
-				const SDL_Rect& clip = SDL_Rect()) :
-			x(x),
-			y(y),
-			surf(1, surf),
-			clip(clip)
-			{}
-
-		tblit(const int x, const int y, const std::vector<surface>& surf,
-				const SDL_Rect& clip = SDL_Rect()) :
-			x(x),
-			y(y),
-			surf(surf),
-			clip(clip)
-			{}
-
-
-		int x;                      /**< x screen coordinate to render at. */
-		int y;                      /**< y screen coordinate to render at. */
-		std::vector<surface> surf;  /**< surface(s) to render. */
-		SDL_Rect clip;              /**<
-		                             * The clipping area of the source if
-		                             * ommitted the entire source is used.
-		                             */
-	};
-
 	/**
 	 * The layers to render something on. This value should never be stored
 	 * it's the internal drawing order and adding removing and reordering
@@ -654,17 +640,21 @@ public:
 		                            * Layer for the terrain drawn behind the
 		                            * unit.
 		                            */
-		LAYER_TERRAIN_TMP_BG,      /**<
-		                            * Layer which holds stuff that needs to be
-		                            * sorted out further, but under units.
-		                            */
-		LAYER_UNIT_FIRST,          /**< Reserve layeres to be selected for WML. */
+		LAYER_GRID_TOP,            /**< Top half part of grid image */
+		LAYER_MOUSEOVER_OVERLAY,   /**< Mouseover overlay used by editor*/
+		LAYER_FOOTSTEPS,           /**< Footsteps showing path from unit to mouse */
+		LAYER_MOUSEOVER_TOP,       /**< Top half of image following the mouse */
+		LAYER_UNIT_FIRST,          /**< Reserve layers to be selected for WML. */
 		LAYER_UNIT_BG = LAYER_UNIT_FIRST+10,             /**< Used for the ellipse behind the unit. */
 		LAYER_UNIT_DEFAULT=LAYER_UNIT_FIRST+40,/**<default layer for drawing units */
 		LAYER_TERRAIN_FG = LAYER_UNIT_FIRST+50, /**<
 		                            * Layer for the terrain drawn in front of
 		                            * the unit.
 		                            */
+		LAYER_GRID_BOTTOM,         /**<
+		                            * Used for the bottom half part of grid image.
+		                            * Should be under moving units, to avoid masking south move.
+		                            */
 		LAYER_UNIT_MOVE_DEFAULT=LAYER_UNIT_FIRST+60/**<default layer for drawing moving units */,
 		LAYER_UNIT_FG =  LAYER_UNIT_FIRST+80, /**<
 		                            * Used for the ellipse in front of the
@@ -673,12 +663,16 @@ public:
 		LAYER_UNIT_MISSILE_DEFAULT = LAYER_UNIT_FIRST+90, /**< default layer for missile frames*/
 		LAYER_UNIT_LAST=LAYER_UNIT_FIRST+100,
 		LAYER_REACHMAP,            /**< "black stripes" on unreachable hexes. */
+		LAYER_MOUSEOVER_BOTTOM,    /**< Bottom half of image following the mouse */
 		LAYER_FOG_SHROUD,          /**< Fog and shroud. */
+		LAYER_ARROWS,              /**< Arrows from the arrows framework. Used for planned moves display. */
+		LAYER_ACTIONS_NUMBERING,   /**< Move numbering for the whiteboard. */
+		LAYER_SELECTED_HEX,        /**< Image on the selected unit */
+		LAYER_ATTACK_INDICATOR,    /**< Layer which holds the attack indicator. */
 		LAYER_UNIT_BAR,            /**<
 		                            * Unit bars and overlays are drawn on this
 		                            * layer (for testing here).
 		                            */
-		LAYER_ATTACK_INDICATOR,    /**< Layer which holds the attack indicator. */
 		LAYER_MOVE_INFO,           /**< Movement info (defense%, ect...). */
 		LAYER_LINGER_OVERLAY,      /**< The overlay used for the linger mode. */
 		LAYER_BORDER,              /**< The border of the map. */
@@ -690,17 +684,17 @@ public:
 		};
 
 	/**
-	 * Draw the image of a unit at a certain location.
-	 * x,y: pixel location on screen to draw the unit
-	 * image: the image of the unit
-	 * reverse: if the unit should be flipped across the x axis
-	 * greyscale: used when the unit is petrified
+	 * Draw an image at a certain location.
+	 * x,y: pixel location on screen to draw the image
+	 * image: the image to draw
+	 * reverse: if the image should be flipped across the x axis
+	 * greyscale: used for instance to give the petrified appearance to a unit image
 	 * alpha: the merging to use with the background
-	 * blendto: blend to this colour using blend_ratio
+	 * blendto: blend to this color using blend_ratio
 	 * submerged: the amount of the unit out of 1.0 that is submerged
 	 *            (presumably under water) and thus shouldn't be drawn
 	 */
-	void render_unit_image(int x, int y, const display::tdrawing_layer drawing_layer,
+	void render_image(int x, int y, const display::tdrawing_layer drawing_layer,
 			const map_location& loc, surface image,
 			bool hreverse=false, bool greyscale=false,
 			fixed_t alpha=ftofxp(1.0), Uint32 blendto=0,
@@ -716,37 +710,26 @@ public:
 		SDL_Color color, double x_in_hex=0.5, double y_in_hex=0.5);
 
 protected:
-
-	// Initially tdrawing_buffer was a vector but profiling showed that a map
-	// was more efficient. Tested with the LAYER_UNIT_LAST for various values
-	// and different types the results were. (Tested with oprofile.)
-
- 	// container    unit layers    counts
-	// vector       100            3748
-	// vector       10000          147338
-	// map          10000          3362
-
-	// Since a map with 10000 items was more efficient I didn't test the map
-	// with 100 items. I want to retest once more is converted, since with
-	// a different usage it numbers might differ so the old code is disabled
-	// with TDRAWING_BUFFER_USES_VECTOR
-	//     20080308 -- Mordante
-
 	/**
-	 * Compare functor for the blitting.
+	 * In order to render a hex properly it needs to be rendered per row. On
+	 * this row several layers need to be drawn at the same time. Mainly the
+	 * unit and the background terrain. This is needed since both can spill
+	 * in the next hex. The foreground terrain needs to be drawn before to
+	 * avoid decapitation a unit.
+	 *
+	 * In other words:
+	 * for every layer
+	 *   for every row (starting from the top)
+	 *     for every hex in the row
+	 *       ...
+	 *
+	 * this is modified to:
+	 * for every layer group
+	 *   for every row (starting from the top)
+	 *     for every layer in the group
+	 *       for every hex in the row
+	 *         ...
 	 *
-	 * The blitting order must be sorted on y value first instead of x so added
-	 * another functor since map_location::operator< sorts on x value.
-	 */
-	struct draw_order
-	{
-		bool operator()(const map_location& lhs, const map_location& rhs) const
-		{
-			return lhs.y < rhs.y || (lhs.y == rhs.y && lhs.x < rhs.x);
-		}
-	};
-
-	/**
 	 * * Surfaces are rendered per level in a map.
 	 * * Per level the items are rendered per location these locations are
 	 *   stored in the drawing order required for units.
@@ -754,18 +737,63 @@ protected:
 	 *   coordinate to render at.
 	 * * every vector element has a vector with surfaces to render.
 	 */
-	typedef std::map<map_location, std::vector<tblit>, draw_order> tblit_map;
-#if TDRAWING_BUFFER_USES_VECTOR
-	typedef std::vector<tblit_map> tdrawing_buffer;
-#else
-	typedef std::map<tdrawing_layer, tblit_map> tdrawing_buffer;
-#endif
+	class drawing_buffer_key
+	{
+	private:
+		unsigned int key_;
+
+		static const tdrawing_layer layer_groups[];
+		static const unsigned int max_layer_group;
+
+	public:
+		drawing_buffer_key(const map_location &loc, tdrawing_layer layer);
+
+		bool operator<(const drawing_buffer_key &rhs) const { return key_ < rhs.key_; }
+	};
+
+	/** Helper structure for rendering the terrains. */
+	class tblit
+	{
+	public:
+		tblit(const tdrawing_layer layer, const map_location& loc,
+				const int x, const int y, const surface& surf,
+				const SDL_Rect& clip)
+			: x_(x), y_(y), surf_(1, surf), clip_(clip),
+			key_(loc, layer)
+		{}
+
+		tblit(const tdrawing_layer layer, const map_location& loc,
+				const int x, const int y, const std::vector<surface>& surf,
+				const SDL_Rect& clip)
+			: x_(x), y_(y), surf_(surf), clip_(clip),
+			key_(loc, layer)
+		{}
+
+		int x() const { return x_; }
+		int y() const { return y_; }
+		const std::vector<surface> &surf() const { return surf_; }
+		const SDL_Rect &clip() const { return clip_; }
+
+		bool operator<(const tblit &rhs) const { return key_ < rhs.key_; }
+
+	private:
+		int x_;                      /**< x screen coordinate to render at. */
+		int y_;                      /**< y screen coordinate to render at. */
+		std::vector<surface> surf_;  /**< surface(s) to render. */
+		SDL_Rect clip_;              /**<
+		                              * The clipping area of the source if
+		                              * ommitted the entire source is used.
+		                              */
+		drawing_buffer_key key_;
+	};
+
+	typedef std::list<tblit> tdrawing_buffer;
 	tdrawing_buffer drawing_buffer_;
 
 public:
 
 	/**
-	 * Add an item to the drawing buffer.
+	 * Add an item to the drawing buffer. You need to update screen on affected area
 	 *
 	 * @param layer              The layer to draw on.
 	 * @param loc                The hex the image belongs to, needed for the
@@ -773,10 +801,13 @@ public:
 	 * @param blit               The structure to blit.
 	 */
 	void drawing_buffer_add(const tdrawing_layer layer,
-			const map_location& loc, const tblit& blit)
-	{
-		drawing_buffer_[layer][loc].push_back(blit);
-	}
+			const map_location& loc, int x, int y, const surface& surf,
+			const SDL_Rect &clip = SDL_Rect());
+
+	void drawing_buffer_add(const tdrawing_layer layer,
+			const map_location& loc, int x, int y,
+			const std::vector<surface> &surf,
+			const SDL_Rect &clip = SDL_Rect());
 
 protected:
 
@@ -789,29 +820,27 @@ protected:
 	/** redraw all panels associated with the map display */
 	void draw_all_panels();
 
-	/**
-	 * Strict weak ordering to sort a STL-set of hexes
-	 * for drawing using the z-order.
-	 * (1000 are just to weight the y compare to x)
-	 */
-	struct ordered_draw : public std::binary_function<map_location, map_location, bool> {
-		bool operator()(map_location a, map_location b) {
-			return (a.y*2 + a.x%2) * 1024 + a.x < (b.y*2 + b.x%2) * 1024 + b.x;
-		}
-	};
-
 	/**
 	 * Initiate a redraw.
 	 *
 	 * Invalidate controls and panels when changed after they have been drawn
 	 * initially. Useful for dynamic theme modification.
 	 */
-	bool draw_init();
-	void draw_wrap(bool update,bool force,bool changed);
+	void draw_init();
+	void draw_wrap(bool update,bool force);
 
-	/** Used to indicate to drawing funtions that we are doing a map screenshot */
+	/** Used to indicate to drawing functions that we are doing a map screenshot */
 	bool map_screenshot_;
 
+public: //operations for the arrow framework
+
+	void add_arrow(arrow&);
+
+	void remove_arrow(arrow&);
+
+	/** Called by arrow objects when they change. You should not need to call this directly. */
+	void update_arrow(arrow & a);
+
 private:
 	/** Handle for the label which displays frames per second. */
 	int fps_handle_;
@@ -830,6 +859,20 @@ private:
 	bool draw_coordinates_;
 	/** Debug flag - overlay terrain codes on tiles */
 	bool draw_terrain_codes_;
+
+	typedef std::list<arrow*> arrows_list_t;
+	typedef std::map<map_location, arrows_list_t > arrows_map_t;
+	/** Maps the list of arrows for each location */
+	arrows_map_t arrows_map_;
+
+	int color_adjust_red_;
+	int color_adjust_green_;
+	int color_adjust_blue_;
+
+#if defined(__GLIBC__)
+	/** Flag for bug #17573 - this is set in the constructor **/
+	bool do_reverse_memcpy_workaround_;
+#endif
 };
 
 #endif
