@@ -1,11 +1,12 @@
-/* $Id: editor_main.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: editor_main.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by Tomasz Sniatowski <kailoran@gmail.com>
+   Copyright (C) 2008 - 2011 by Tomasz Sniatowski <kailoran@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -34,7 +35,7 @@ EXIT_STATUS start(const config& game_conf, CVideo& video, const std::string& fil
 		hotkey::deactivate_all_scopes();
 		hotkey::set_scope_active(hotkey::SCOPE_GENERAL);
 		hotkey::set_scope_active(hotkey::SCOPE_EDITOR);
-		editor_controller editor(game_conf, video, NULL);
+		editor_controller editor(game_conf, video);
 		if (!filename.empty()) {
 			if (is_directory(filename)) {
 				editor.set_default_dir(filename);
