@@ -1,11 +1,12 @@
-/* $Id: filechooser.cpp 40541 2010-01-03 11:45:18Z mordante $ */
+/* $Id: filechooser.cpp 50076 2011-06-29 01:46:25Z shadowmaster $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -18,6 +19,7 @@
 
 #include "display.hpp"
 #include "gettext.hpp"
+#include "gui/dialogs/folder_create.hpp"
 #include "gui/dialogs/transient_message.hpp"
 #include "filechooser.hpp"
 #include "widgets/file_menu.hpp"
@@ -202,12 +204,10 @@ void file_dialog::action(gui::dialog_process_info &dp_info) {
 	//handle "create item" requests
 	else if(result() == gui::CREATE_ITEM)
 	{
-		gui::dialog d(get_display(), _("New Folder"), "", gui::OK_CANCEL);
-		d.set_textbox(_("Name: "));
-		d.show();
-		if(d.result() != gui::CLOSE_DIALOG && !d.textbox_text().empty())
+		std::string new_dir_name = "";
+		if(gui2::tfolder_create::execute(new_dir_name, get_display().video()))
 		{
-			if( !files_list_->make_directory(d.textbox_text()) ) {
+			if( !files_list_->make_directory(new_dir_name) ) {
 				gui2::show_transient_error_message(get_display().video()
 						, _("Creation of the directory failed."));
 			} else {
