@@ -1,12 +1,13 @@
-/* $Id: font.cpp 35228 2009-04-26 08:38:25Z silene $ */
+/* $Id: font.cpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /* vim:set encoding=utf-8: */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -20,14 +21,16 @@
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "font.hpp"
+#include "foreach.hpp"
 #include "game_config.hpp"
 #include "log.hpp"
+#include "marked-up_text.hpp"
+#include "text.hpp"
 #include "tooltips.hpp"
 #include "video.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include "marked-up_text.hpp"
-#include "foreach.hpp"
+#include "serialization/string_utils.hpp"
 
 #include <list>
 #include <set>
@@ -35,28 +38,25 @@
 
 #include <cairo-features.h>
 
+#ifdef CAIRO_HAS_WIN32_FONT
+#include <windows.h>
+#undef CAIRO_HAS_FT_FONT
+#endif
+
 #ifdef CAIRO_HAS_FT_FONT
 #include <fontconfig/fontconfig.h>
 #endif
 
-#ifdef CAIRO_HAS_WIN32_FONT
-#include <windows.h>
-#endif
-
-#define DBG_FT LOG_STREAM(debug, display)
-#define LOG_FT LOG_STREAM(info, display)
-#define WRN_FT LOG_STREAM(warn, display)
-#define ERR_FT LOG_STREAM(err, display)
+static lg::log_domain log_font("font");
+#define DBG_FT LOG_STREAM(debug, log_font)
+#define LOG_FT LOG_STREAM(info, log_font)
+#define WRN_FT LOG_STREAM(warn, log_font)
+#define ERR_FT LOG_STREAM(err, log_font)
 
 #ifdef	HAVE_FRIBIDI
-#include <fribidi/fribidi.h>
-
-#else
-
+#include <fribidi.h>
 #endif
 
-namespace {
-
 // Signed int. Negative values mean "no subset".
 typedef int subset_id;
 
@@ -76,57 +76,96 @@ struct font_id
 	int size;
 };
 
-std::map<font_id, TTF_Font*> font_table;
-std::vector<std::string> font_names;
+static std::map<font_id, TTF_Font*> font_table;
+static std::vector<std::string> font_names;
 
 struct text_chunk
 {
 	text_chunk(subset_id subset) :
 		subset(subset),
-		text(),
-		ucs2_text()
+		text()
 	{
 	}
 
-	text_chunk(subset_id subset, std::string const & text) :
-		subset(subset),
-		text(text),
-		ucs2_text()
-	{
-	}
-
-	text_chunk(subset_id subset, ucs2_string const & ucs2_text) :
-		subset(subset),
-		text(),
-		ucs2_text(ucs2_text)
-	{
-	}
-
-	text_chunk(subset_id subset, std::string const & text, ucs2_string const & ucs2_text) :
-		subset(subset),
-		text(text),
-		ucs2_text(ucs2_text)
-	{
-	}
-
-	bool operator==(text_chunk const & t) const { return subset == t.subset && ucs2_text == t.ucs2_text; }
+	bool operator==(text_chunk const & t) const { return subset == t.subset && text == t.text; }
 	bool operator!=(text_chunk const & t) const { return !operator==(t); }
 
 	subset_id subset;
-	//FIXME if we don't need the utf8 here remove it
 	std::string text;
-	ucs2_string ucs2_text;
 };
 
-std::vector<subset_id> font_map;
+struct char_block_map
+{
+	char_block_map()
+		: cbmap()
+	{
+	}
+
+	typedef std::pair<int, subset_id> block_t;
+	typedef std::map<int, block_t> cbmap_t;
+	cbmap_t cbmap;
+	/** Associates not-associated parts of a range with a new font. */
+	void insert(int first, int last, subset_id id)
+	{
+		if (first > last) return;
+		cbmap_t::iterator i = cbmap.lower_bound(first);
+		// At this point, either first <= i->first or i is past the end.
+		if (i != cbmap.begin()) {
+			cbmap_t::iterator j = i;
+			--j;
+			if (first <= j->second.first /* prev.last */) {
+				insert(j->second.first + 1, last, id);
+				return;
+			}
+		}
+		if (i != cbmap.end()) {
+			if (/* next.first */ i->first <= last) {
+				insert(first, i->first - 1, id);
+				return;
+			}
+		}
+		cbmap.insert(std::make_pair(first, block_t(last, id)));
+	}
+	/**
+	 * Compresses map by merging consecutive ranges with the same font, even
+	 * if there is some unassociated ranges inbetween.
+	 */
+	void compress()
+	{
+		LOG_FT << "Font map size before compression: " << cbmap.size() << " ranges\n";
+		cbmap_t::iterator i = cbmap.begin(), e = cbmap.end();
+		while (i != e) {
+			cbmap_t::iterator j = i;
+			++j;
+			if (j == e || i->second.second != j->second.second) {
+				i = j;
+				continue;
+			}
+			i->second.first = j->second.first;
+			cbmap.erase(j);
+		}
+		LOG_FT << "Font map size after compression: " << cbmap.size() << " ranges\n";
+	}
+	subset_id get_id(int ch)
+	{
+		cbmap_t::iterator i = cbmap.upper_bound(ch);
+		// At this point, either ch < i->first or i is past the end.
+		if (i != cbmap.begin()) {
+			--i;
+			if (ch <= i->second.first /* prev.last */)
+				return i->second.second;
+		}
+		return -1;
+	}
+};
+
+static char_block_map char_blocks;
 
 //cache sizes of small text
 typedef std::map<std::string,SDL_Rect> line_size_cache_map;
 
 //map of styles -> sizes -> cache
-std::map<int,std::map<int,line_size_cache_map> > line_size_cache;
-
-}
+static std::map<int,std::map<int,line_size_cache_map> > line_size_cache;
 
 //Splits the UTF-8 text into text_chunks using the same font.
 static std::vector<text_chunk> split_text(std::string const & utf8_text) {
@@ -138,28 +177,23 @@ static std::vector<text_chunk> split_text(std::string const & utf8_text) {
 
 	try {
 		utils::utf8_iterator ch(utf8_text);
-		if(size_t(*ch) < font_map.size() && font_map[size_t(*ch)] >= 0) {
-			current_chunk.subset = font_map[size_t(*ch)];
-		}
-		for(utils::utf8_iterator end = utils::utf8_iterator::end(utf8_text); ch != end; ++ch) {
-			if(size_t(*ch) < font_map.size() &&
-					font_map[size_t(*ch)] >= 0 &&
-					font_map[size_t(*ch)] != current_chunk.subset) {
-				//null-terminate ucs2_text so we can pass it to SDL_ttf later
-				current_chunk.ucs2_text.push_back(0);
+		int sub = char_blocks.get_id(*ch);
+		if (sub >= 0) current_chunk.subset = sub;
+		for(utils::utf8_iterator end = utils::utf8_iterator::end(utf8_text); ch != end; ++ch)
+		{
+			sub = char_blocks.get_id(*ch);
+			if (sub >= 0 && sub != current_chunk.subset) {
 				chunks.push_back(current_chunk);
-				current_chunk.text = "";
-				current_chunk.ucs2_text.clear();
-				current_chunk.subset = font_map[size_t(*ch)];
+				current_chunk.text.clear();
+				current_chunk.subset = sub;
 			}
-			current_chunk.ucs2_text.push_back(static_cast<Uint16>(*ch));
 			current_chunk.text.append(ch.substr().first, ch.substr().second);
 		}
 		if (!current_chunk.text.empty()) {
 			chunks.push_back(current_chunk);
 		}
 	}
-	catch(utils::invalid_utf8_exception e) {
+	catch(utils::invalid_utf8_exception&) {
 		WRN_FT << "Invalid UTF-8 string: \"" << utf8_text << "\"\n";
 	}
 	return chunks;
@@ -230,7 +264,7 @@ static void clear_fonts()
 
 	font_table.clear();
 	font_names.clear();
-	font_map.clear();
+	char_blocks.cbmap.clear();
 	line_size_cache.clear();
 }
 
@@ -312,11 +346,10 @@ void manager::update_font_path() const
 void manager::init() const
 {
 #ifdef CAIRO_HAS_FT_FONT
-	if(!FcConfigAppFontAddDir( FcConfigGetCurrent(),
-			reinterpret_cast<const FcChar8*>
-			((game_config::path + "/fonts/").c_str()))) {
-
-		ERR_FT << "Could not load the  true type fonts\n";
+	if (!FcConfigAppFontAddDir(FcConfigGetCurrent(),
+		reinterpret_cast<const FcChar8 *>((game_config::path + "/fonts").c_str())))
+	{
+		ERR_FT << "Could not load the true type fonts\n";
 		throw error();
 	}
 #endif
@@ -326,7 +359,7 @@ void manager::init() const
 		std::vector<std::string> files;
 		get_files_in_dir(path, &files, NULL, ENTIRE_FILE_PATH);
 		foreach(const std::string& file, files)
-			if(file.substr(file.length() - 4) == ".ttf")
+			if(file.substr(file.length() - 4) == ".ttf" || file.substr(file.length() - 4) == ".ttc")
 				AddFontResource(file.c_str());
 	}
 #endif
@@ -343,7 +376,7 @@ void manager::deinit() const
 		std::vector<std::string> files;
 		get_files_in_dir(path, &files, NULL, ENTIRE_FILE_PATH);
 		foreach(const std::string& file, files)
-			if(file.substr(file.length() - 4) == ".ttf")
+			if(file.substr(file.length() - 4) == ".ttf" || file.substr(file.length() - 4) == ".ttc")
 				RemoveFontResource(file.c_str());
 	}
 #endif
@@ -360,14 +393,14 @@ struct subset_descriptor
 	}
 
 	std::string name;
-	std::vector<std::pair<size_t, size_t> > present_codepoints;
+	typedef std::pair<int, int> range;
+	std::vector<range> present_codepoints;
 };
 
 //sets the font list to be used.
 static void set_font_list(const std::vector<subset_descriptor>& fontlist)
 {
 	clear_fonts();
-	font_map.reserve(0x10000);
 
 	std::vector<subset_descriptor>::const_iterator itor;
 	for(itor = fontlist.begin(); itor != fontlist.end(); ++itor) {
@@ -392,35 +425,26 @@ static void set_font_list(const std::vector<subset_descriptor>& fontlist)
 		const subset_id subset = font_names.size();
 		font_names.push_back(itor->name);
 
-		std::vector<std::pair<size_t,size_t> >::const_iterator cp_range;
-		for(cp_range = itor->present_codepoints.begin();
-				cp_range != itor->present_codepoints.end(); ++cp_range) {
-
-			size_t cp_max = std::max<size_t>(cp_range->first, cp_range->second);
-			if(cp_max >= font_map.size()) {
-				font_map.resize(cp_max+1, -1);
-			}
-			for(size_t cp = cp_range->first; cp <= cp_range->second; ++cp) {
-				if(font_map[cp] < 0)
-					font_map[cp] = subset;
-			}
+		foreach (const subset_descriptor::range &cp_range, itor->present_codepoints) {
+			char_blocks.insert(cp_range.first, cp_range.second, subset);
 		}
 	}
+	char_blocks.compress();
 }
 
-const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
-                GRAY_COLOUR   = {0x77,0x77,0x77,0},
-                LOBBY_COLOUR  = {0xBB,0xBB,0xBB,0},
-                GOOD_COLOUR   = {0x00,0xFF,0x00,0},
-                BAD_COLOUR    = {0xFF,0x00,0x00,0},
-                BLACK_COLOUR  = {0x00,0x00,0x00,0},
-                YELLOW_COLOUR = {0xFF,0xFF,0x00,0},
-                BUTTON_COLOUR = {0xBC,0xB0,0x88,0},
-                STONED_COLOUR = {0xA0,0xA0,0xA0,0},
-                TITLE_COLOUR  = {0xBC,0xB0,0x88,0},
-				LABEL_COLOUR  = {0x6B,0x8C,0xFF,0},
-				BIGMAP_COLOUR = {0xFF,0xFF,0xFF,0};
-const SDL_Color DISABLED_COLOUR = inverse(STONED_COLOUR);
+const SDL_Color NORMAL_COLOR = {0xDD,0xDD,0xDD,0},
+                GRAY_COLOR   = {0x77,0x77,0x77,0},
+                LOBBY_COLOR  = {0xBB,0xBB,0xBB,0},
+                GOOD_COLOR   = {0x00,0xFF,0x00,0},
+                BAD_COLOR    = {0xFF,0x00,0x00,0},
+                BLACK_COLOR  = {0x00,0x00,0x00,0},
+                YELLOW_COLOR = {0xFF,0xFF,0x00,0},
+                BUTTON_COLOR = {0xBC,0xB0,0x88,0},
+                PETRIFIED_COLOR = {0xA0,0xA0,0xA0,0},
+                TITLE_COLOR  = {0xBC,0xB0,0x88,0},
+				LABEL_COLOR  = {0x6B,0x8C,0xFF,0},
+				BIGMAP_COLOR = {0xFF,0xFF,0xFF,0};
+const SDL_Color DISABLED_COLOR = inverse(PETRIFIED_COLOR);
 
 namespace {
 
@@ -467,16 +491,25 @@ private:
 void text_surface::bidi_cvt()
 {
 	char		*c_str = const_cast<char *>(str_.c_str());	// fribidi forgot const...
-	int		len = str_.length();
+	FriBidiStrIndex	len = str_.length();
 	FriBidiChar	*bidi_logical = new FriBidiChar[len + 2];
 	FriBidiChar	*bidi_visual = new FriBidiChar[len + 2];
 	char		*utf8str = new char[4*len + 1];	//assume worst case here (all 4 Byte characters)
 	FriBidiCharType	base_dir = FRIBIDI_TYPE_ON;
-	int n;
+	FriBidiStrIndex n;
 
+
+#ifdef	OLD_FRIBIDI
 	n = fribidi_utf8_to_unicode (c_str, len, bidi_logical);
+#else
+	n = fribidi_charset_to_unicode(FRIBIDI_CHAR_SET_UTF8, c_str, len, bidi_logical);
+#endif
 	fribidi_log2vis(bidi_logical, n, &base_dir, bidi_visual, NULL, NULL, NULL);
+#ifdef	OLD_FRIBIDI
 	fribidi_unicode_to_utf8 (bidi_visual, n, utf8str);
+#else
+	fribidi_unicode_to_charset(FRIBIDI_CHAR_SET_UTF8, bidi_visual, n, utf8str);
+#endif
 	is_rtl_ = base_dir == FRIBIDI_TYPE_RTL;
 	str_ = std::string(utf8str);
 	delete[] bidi_logical;
@@ -549,23 +582,15 @@ void text_surface::measure() const
 	w_ = 0;
 	h_ = 0;
 
-	for(std::vector<text_chunk>::iterator itor = chunks_.begin();
-			itor != chunks_.end(); ++itor) {
-
-		TTF_Font* ttfont = get_font(font_id(itor->subset, font_size_));
+	foreach (text_chunk const &chunk, chunks_)
+	{
+		TTF_Font* ttfont = get_font(font_id(chunk.subset, font_size_));
 		if(ttfont == NULL)
 			continue;
 		font_style_setter const style_setter(ttfont, style_);
 
-		int w;
-		int h;
-
-		if(itor->ucs2_text.back() != 0) {
-			itor->ucs2_text.push_back(0);
-		}
-
-		TTF_SizeUNICODE(ttfont,
-			static_cast<Uint16 const *>(&(itor->ucs2_text.front())), &w, &h);
+		int w, h;
+		TTF_SizeUTF8(ttfont, chunk.text.c_str(), &w, &h);
 		w_ += w;
 		h_ = std::max<int>(h_, h);
 	}
@@ -603,15 +628,14 @@ std::vector<surface> const &text_surface::get_surfaces() const
 	if(width() > max_text_line_width)
 		return surfs_;
 
-	for(std::vector<text_chunk>::const_iterator itor = chunks_.begin();
-			itor != chunks_.end(); ++itor) {
-		TTF_Font* ttfont = get_font(font_id(itor->subset, font_size_));
+	foreach (text_chunk const &chunk, chunks_)
+	{
+		TTF_Font* ttfont = get_font(font_id(chunk.subset, font_size_));
 		if (ttfont == NULL)
 			continue;
 		font_style_setter const style_setter(ttfont, style_);
 
-		surface s = surface(TTF_RenderUNICODE_Blended(ttfont,
-			static_cast<Uint16 const *>(&(itor->ucs2_text.front())), color_));
+		surface s = surface(TTF_RenderUTF8_Blended(ttfont, chunk.text.c_str(), color_));
 		if(!s.null())
 			surfs_.push_back(s);
 	}
@@ -667,7 +691,7 @@ text_surface &text_cache::find(text_surface const &t)
 
 }
 
-static surface render_text(const std::string& text, int fontsize, const SDL_Color& colour, int style, bool use_markup)
+static surface render_text(const std::string& text, int fontsize, const SDL_Color& color, int style, bool use_markup)
 {
 	// we keep blank lines and spaces (may be wanted for indentation)
 	const std::vector<std::string> lines = utils::split(text, '\n', 0);
@@ -677,17 +701,16 @@ static surface render_text(const std::string& text, int fontsize, const SDL_Colo
 
 	for(std::vector< std::string >::const_iterator ln = lines.begin(), ln_end = lines.end(); ln != ln_end; ++ln) {
 
-		SDL_Color col = colour;
 		int sz = fontsize;
 		int text_style = style;
 
 		std::string::const_iterator after_markup = use_markup ?
-			parse_markup(ln->begin(),ln->end(),&sz,&col,&text_style) : ln->begin();
-		text_surface txt_surf(sz, col, text_style);
+			parse_markup(ln->begin(), ln->end(), &sz, NULL, &text_style) : ln->begin();
+		text_surface txt_surf(sz, color, text_style);
 
-		if (after_markup == ln->end() && ln+1 != ln_end) {
+		if (after_markup == ln->end() && (ln+1 != ln_end || lines.begin()+1 == ln_end)) {
 			// we replace empty line by a space (to have a line height)
-			// except for the blank following the last '\n'
+			// except for the last line if we have several
 			txt_surf.set_text(" ");
 		} else if (after_markup == ln->begin()) {
 		 	// simple case, no markup to skip
@@ -728,8 +751,8 @@ static surface render_text(const std::string& text, int fontsize, const SDL_Colo
 			for(std::vector<surface>::const_iterator j = i->begin(),
 					j_end = i->end(); j != j_end; ++j) {
 				SDL_SetAlpha(*j, 0, 0); // direct blit without alpha blending
-				SDL_Rect dstrect = {xpos, ypos, 0, 0};
-				SDL_BlitSurface(*j, NULL, res, &dstrect);
+				SDL_Rect dstrect = create_rect(xpos, ypos, 0, 0);
+				sdl_blit(*j, NULL, res, &dstrect);
 				xpos += (*j)->w;
 				height = std::max<size_t>((*j)->h, height);
 			}
@@ -741,34 +764,31 @@ static surface render_text(const std::string& text, int fontsize, const SDL_Colo
 }
 
 
-surface get_rendered_text(const std::string& str, int size, const SDL_Color& colour, int style)
+surface get_rendered_text(const std::string& str, int size, const SDL_Color& color, int style)
 {
 	// TODO maybe later also to parse markup here, but a lot to check
-	return render_text(str, size, colour, style, false);
+	return render_text(str, size, color, style, false);
 }
 
 SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
-		   const SDL_Color& colour, const std::string& text,
+		   const SDL_Color& color, const std::string& text,
 		   int x, int y, bool use_tooltips, int style)
 {
 	if (gui_surface.null()) {
-		text_surface const &u = text_cache::find(text_surface(text, size, colour, style));
-		SDL_Rect res = {0, 0, u.width(), u.height()};
-		return res;
+		text_surface const &u = text_cache::find(text_surface(text, size, color, style));
+		return create_rect(0, 0, u.width(), u.height());
 	}
 
 	if(area.w == 0) {  // no place to draw
-		SDL_Rect res = {0,0,0,0};
-		return res;
+		return create_rect(0, 0, 0, 0);
 	}
 
 	const std::string etext = make_text_ellipsis(text, size, area.w);
 
 	// for the main current use, we already parsed markup
-	surface surface(render_text(etext,size,colour,style,false));
+	surface surface(render_text(etext,size,color,style,false));
 	if(surface == NULL) {
-		SDL_Rect res = {0,0,0,0};
-		return res;
+		return create_rect(0, 0, 0, 0);
 	}
 
 	SDL_Rect dest;
@@ -777,7 +797,7 @@ SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 #ifdef	HAVE_FRIBIDI
 		// Oron -- Conditional, until all draw_text_line calls have fixed area parameter
 		if(getenv("NO_RTL") == NULL) {
-			bool is_rtl = text_cache::find(text_surface(text, size, colour, style)).is_rtl();
+			bool is_rtl = text_cache::find(text_surface(text, size, color, style)).is_rtl();
 			if(is_rtl)
 				dest.x = area.x + area.w - surface->w - (x - area.x);
 		}
@@ -807,7 +827,7 @@ SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 		SDL_Rect src = dest;
 		src.x = 0;
 		src.y = 0;
-		SDL_BlitSurface(surface,&src,gui_surface,&dest);
+		sdl_blit(surface,&src,gui_surface,&dest);
 	}
 
 	if(use_tooltips) {
@@ -854,16 +874,11 @@ SDL_Rect line_size(const std::string& line, int font_size, int style)
 }
 
 std::string make_text_ellipsis(const std::string &text, int font_size,
-		int max_width, bool with_tags, bool parse_for_style)
+	int max_width, int style)
 {
 	static const std::string ellipsis = "...";
 
-	SDL_Color unused_color;
-	int unused_int;
-	int style = TTF_STYLE_NORMAL;
-	if(parse_for_style) parse_markup(text.begin(), text.end(), &unused_int, &unused_color, &style);
-
-	if(line_width(with_tags ? text : del_tags(text), font_size, style) <= max_width)
+	if (line_width(text, font_size, style) <= max_width)
 		return text;
 	if(line_width(ellipsis, font_size, style) > max_width)
 		return "";
@@ -875,9 +890,8 @@ std::string make_text_ellipsis(const std::string &text, int font_size,
 	for(; itor != utils::utf8_iterator::end(text); ++itor) {
 		std::string tmp = current_substring;
 		tmp.append(itor.substr().first, itor.substr().second);
-		tmp += ellipsis;
 
-		if (line_width(with_tags ? tmp : del_tags(tmp), font_size, style) > max_width) {
+		if (line_width(tmp + ellipsis, font_size, style) > max_width) {
 			return current_substring + ellipsis;
 		}
 
@@ -889,59 +903,29 @@ std::string make_text_ellipsis(const std::string &text, int font_size,
 
 }
 
-//floating labels
 namespace {
 
-class floating_label
-{
-public:
-	floating_label(const std::string& text, int font_size, const SDL_Color& colour, const SDL_Color& bgcolour,
-			double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect,
-			font::ALIGN align, int border_size, bool scroll_with_map)
-		: surf_(NULL), buf_(NULL), text_(text), font_size_(font_size), colour_(colour),
-		bgcolour_(bgcolour), bgalpha_(bgcolour.unused), xpos_(xpos), ypos_(ypos),
-		xmove_(xmove), ymove_(ymove), lifetime_(lifetime), clip_rect_(clip_rect),
-		alpha_change_(-255/lifetime), visible_(true), align_(align), border_(border_size), scroll_(scroll_with_map)
-	{}
-
-	void move(double xmove, double ymove);
-
-	void draw(surface screen);
-	void undraw(surface screen);
-
-	surface create_surface();
-
-	bool expired() const { return lifetime_ == 0; }
-
-
-	void show(const bool value) { visible_ = value; }
-
-	bool scroll() const { return scroll_; }
-
-private:
-
-	int xpos(size_t width) const;
-
-	surface surf_, buf_;
-	std::string text_;
-	int font_size_;
-	SDL_Color colour_, bgcolour_;
-	int bgalpha_;
-	double xpos_, ypos_, xmove_, ymove_;
-	int lifetime_;
-	SDL_Rect clip_rect_;
-	int alpha_change_;
-	bool visible_;
-	font::ALIGN align_;
-	int border_;
-	bool scroll_;
-};
-
-typedef std::map<int,floating_label> label_map;
+typedef std::map<int, font::floating_label> label_map;
 label_map labels;
 int label_id = 1;
 
 std::stack<std::set<int> > label_contexts;
+}
+
+
+namespace font {
+
+floating_label::floating_label(const std::string& text)
+		: surf_(NULL), buf_(NULL), text_(text),
+		font_size_(SIZE_NORMAL),
+		color_(NORMAL_COLOR),	bgcolor_(), bgalpha_(0),
+		xpos_(0), ypos_(0),
+		xmove_(0), ymove_(0), lifetime_(-1),
+		width_(-1), height_(-1),
+		clip_rect_(screen_area()),
+		alpha_change_(0), visible_(true), align_(CENTER_ALIGN),
+		border_(0), scroll_(ANCHOR_LABEL_SCREEN), use_markup_(true)
+{}
 
 void floating_label::move(double xmove, double ymove)
 {
@@ -964,7 +948,20 @@ int floating_label::xpos(size_t width) const
 surface floating_label::create_surface()
 {
 	if (surf_.null()) {
-		surface foreground = font::render_text(text_, font_size_, colour_, 0, true);
+		font::ttext text;
+		text.set_foreground_color((color_.r << 24) | (color_.g << 16) | (color_.b << 8) | 255);
+		text.set_font_size(font_size_);
+		text.set_maximum_width(width_ < 0 ? clip_rect_.w : width_);
+		text.set_maximum_height(height_ < 0 ? clip_rect_.h : height_);
+
+		//ignore last '\n'
+		if(!text_.empty() && *(text_.rbegin()) == '\n'){
+			text.set_text(std::string(text_.begin(), text_.end()-1), use_markup_);
+		} else {
+			text.set_text(text_, use_markup_);
+		}
+
+		surface foreground = text.render();
 
 		if(foreground == NULL) {
 			ERR_FT << "could not create floating label's text" << std::endl;
@@ -982,8 +979,8 @@ surface floating_label::create_surface()
 				return surf_;
 			}
 
-			Uint32 color = SDL_MapRGBA(foreground->format, bgcolour_.r,bgcolour_.g, bgcolour_.b, bgalpha_);
-			SDL_FillRect(background,NULL, color);
+			Uint32 color = SDL_MapRGBA(foreground->format, bgcolor_.r,bgcolor_.g, bgcolor_.b, bgalpha_);
+			sdl_fill_rect(background,NULL, color);
 
 			// we make the text less transparent, because the blitting on the
 			// dark background will darken the anti-aliased part.
@@ -991,7 +988,7 @@ surface floating_label::create_surface()
 			// (where the text was blitted directly on screen)
 			foreground = adjust_surface_alpha(foreground, ftofxp(1.13), false);
 
-			SDL_Rect r = { border_, border_, 0, 0 };
+			SDL_Rect r = create_rect( border_, border_, 0, 0);
 			SDL_SetAlpha(foreground,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
 			blit_surface(foreground, NULL, background, &r);
 
@@ -1002,8 +999,12 @@ surface floating_label::create_surface()
 		}
 		else {
 			// background is blurred shadow of the text
-			//TODO add a little extra space for the blur of letters with a lower part
-			surface background = shadow_image(foreground, false);
+			surface background = create_neutral_surface
+				(foreground->w + 4, foreground->h + 4);
+			sdl_fill_rect(background, NULL, 0);
+			SDL_Rect r = { 2, 2, 0, 0 };
+			blit_surface(foreground, NULL, background, &r);
+			background = shadow_image(background, false);
 
 			if (background == NULL) {
 				ERR_FT << "could not create floating label's shadow" << std::endl;
@@ -1011,7 +1012,7 @@ surface floating_label::create_surface()
 				return surf_;
 			}
 			SDL_SetAlpha(foreground,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
-			blit_surface(foreground, NULL, background, NULL);
+			blit_surface(foreground, NULL, background, &r);
 			surf_ = create_optimized_surface(background);
 		}
 	}
@@ -1032,7 +1033,7 @@ void floating_label::draw(surface screen)
 	}
 
 	if(buf_ == NULL) {
-		buf_.assign(create_compatible_surface(surf_));
+		buf_.assign(create_compatible_surface(screen, surf_->w, surf_->h));
 		if(buf_ == NULL) {
 			return;
 		}
@@ -1042,10 +1043,10 @@ void floating_label::draw(surface screen)
 		return;
 	}
 
-	SDL_Rect rect = {xpos(surf_->w),int(ypos_),surf_->w,surf_->h};
-	const clip_rect_setter clip_setter(screen,clip_rect_);
-	SDL_BlitSurface(screen,&rect,buf_,NULL);
-	SDL_BlitSurface(surf_,NULL,screen,&rect);
+	SDL_Rect rect = create_rect(xpos(surf_->w), ypos_, surf_->w, surf_->h);
+	const clip_rect_setter clip_setter(screen, &clip_rect_);
+	sdl_blit(screen,&rect,buf_,NULL);
+	sdl_blit(surf_,NULL,screen,&rect);
 
 	update_rect(rect);
 }
@@ -1056,9 +1057,9 @@ void floating_label::undraw(surface screen)
 		return;
 	}
 
-	SDL_Rect rect = {xpos(surf_->w),int(ypos_),surf_->w,surf_->h};
-	const clip_rect_setter clip_setter(screen,clip_rect_);
-	SDL_BlitSurface(buf_,NULL,screen,&rect);
+	SDL_Rect rect = create_rect(xpos(surf_->w), ypos_, surf_->w, surf_->h);
+	const clip_rect_setter clip_setter(screen, &clip_rect_);
+	sdl_blit(buf_,NULL,screen,&rect);
 
 	update_rect(rect);
 
@@ -1073,28 +1074,14 @@ void floating_label::undraw(surface screen)
 	}
 }
 
-}
-
-namespace font {
-int add_floating_label(const std::string& text, int font_size, const SDL_Color& colour,
-		double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect, ALIGN align,
-		const SDL_Color* bg_colour, int border_size, LABEL_SCROLL_MODE scroll_mode)
+int add_floating_label(const floating_label& flabel)
 {
 	if(label_contexts.empty()) {
 		return 0;
 	}
 
-	if(lifetime <= 0) {
-		lifetime = -1;
-	}
-
-	SDL_Color bg = {0,0,0,0};
-	if(bg_colour != NULL) {
-		bg = *bg_colour;
-	}
-
 	++label_id;
-	labels.insert(std::pair<int,floating_label>(label_id,floating_label(text,font_size,colour,bg,xpos,ypos,xmove,ymove,lifetime,clip_rect,align,border_size,scroll_mode == ANCHOR_LABEL_MAP)));
+	labels.insert(std::pair<int, floating_label>(label_id, flabel));
 	label_contexts.top().insert(label_id);
 	return label_id;
 }
@@ -1110,7 +1097,7 @@ void move_floating_label(int handle, double xmove, double ymove)
 void scroll_floating_labels(double xmove, double ymove)
 {
 	for(label_map::iterator i = labels.begin(); i != labels.end(); ++i) {
-		if(i->second.scroll()) {
+		if(i->second.scroll() == ANCHOR_LABEL_MAP) {
 			i->second.move(xmove,ymove);
 		}
 	}
@@ -1142,8 +1129,7 @@ SDL_Rect get_floating_label_rect(int handle)
 	if(i != labels.end()) {
 		const surface surf = i->second.create_surface();
 		if(surf != NULL) {
-			SDL_Rect rect = {0,0,surf->w,surf->h};
-			return rect;
+			return create_rect(0, 0, surf->w, surf->h);
 		}
 	}
 
@@ -1214,22 +1200,23 @@ void undraw_floating_labels(surface screen)
 			context.erase(j->first);
 			labels.erase(j++);
 		} else {
-			j++;
+			++j;
 		}
 	}
 }
 
 }
 
-static bool add_font_to_fontlist(config* fonts_config, std::vector<font::subset_descriptor>& fontlist, const std::string& name)
-	{
-		config* font = fonts_config->find_child("font", "name", name);
-		if(font == NULL)
-			return false;
+static bool add_font_to_fontlist(config &fonts_config,
+	std::vector<font::subset_descriptor>& fontlist, const std::string& name)
+{
+	config &font = fonts_config.find_child("font", "name", name);
+	if (!font)
+		return false;
 
 		fontlist.push_back(font::subset_descriptor());
 		fontlist.back().name = name;
-		std::vector<std::string> ranges = utils::split((*font)["codepoints"]);
+		std::vector<std::string> ranges = utils::split(font["codepoints"]);
 
 		for(std::vector<std::string>::const_iterator itor = ranges.begin();
 				itor != ranges.end(); ++itor) {
@@ -1269,18 +1256,17 @@ bool load_font_config()
 		return false;
 	}
 
-	config* fonts_config = cfg.child("fonts");
-	if(fonts_config == NULL)
+	config &fonts_config = cfg.child("fonts");
+	if (!fonts_config)
 		return false;
 
 	std::set<std::string> known_fonts;
-	const config::child_list fonts = fonts_config->get_children("font");
-	for (config::child_list::const_iterator child = fonts.begin(); child != fonts.end(); ++child) {
-		known_fonts.insert((**child)["name"]);
+	foreach (const config &font, fonts_config.child_range("font")) {
+		known_fonts.insert(font["name"]);
 	}
 
-	family_order = (*fonts_config)["family_order"];
-	const std::vector<std::string> font_order = utils::split((*fonts_config)["order"]);
+	family_order = fonts_config["family_order"];
+	const std::vector<std::string> font_order = utils::split(fonts_config["order"]);
 	std::vector<font::subset_descriptor> fontlist;
 	std::vector<std::string>::const_iterator font;
 	for(font = font_order.begin(); font != font_order.end(); ++font) {
