@@ -1,11 +1,12 @@
-/* $Id: formula_callable.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: formula_callable.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2008 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,9 +16,6 @@
 #ifndef FORMULA_CALLABLE_HPP_INCLUDED
 #define FORMULA_CALLABLE_HPP_INCLUDED
 
-#include <map>
-#include <string>
-
 #include "reference_counted_object.hpp"
 #include "variant.hpp"
 
