@@ -1,11 +1,12 @@
-/* $Id: game_preferences.cpp 33728 2009-03-16 18:09:00Z mordante $ */
+/* $Id: game_preferences.cpp 51516 2011-10-16 20:59:09Z shadowmaster $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -17,13 +18,17 @@
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "foreach.hpp"
+#include "game_display.hpp"
 #include "game_preferences.hpp"
 #include "gamestatus.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
 #include "map.hpp"
 #include "network.hpp" // ping_timeout
+#include "serialization/string_utils.hpp"
 #include "settings.hpp"
+#include "unit.hpp"
+#include "unit_map.hpp"
 #include "wml_exception.hpp"
 
 
@@ -41,6 +46,7 @@ const unsigned max_history_saved = 50;
 
 std::set<std::string> friends;
 std::set<std::string> ignores;
+
 bool friends_initialized = false;
 bool ignores_initialized = false;
 
@@ -55,8 +61,8 @@ manager::manager() :
 	set_music_volume(music_volume());
 	set_sound_volume(sound_volume());
 
-	set_show_haloes(utils::string_bool(preferences::get("show_haloes"), true));
-	if(!utils::string_bool(preferences::get("remember_timer_settings"), false)) {
+	set_show_haloes(preferences::get("show_haloes", true));
+	if (!preferences::get("remember_timer_settings", false)) {
 		preferences::erase("mp_countdown_init_time");
 		preferences::erase("mp_countdown_reservoir_time");
 		preferences::erase("mp_countdown_turn_bonus");
@@ -72,8 +78,8 @@ manager::manager() :
 	std::copy(terrain.begin(), terrain.end(),
 			std::inserter(encountered_terrains_set, encountered_terrains_set.begin()));
 
-	config* history = preferences::get_child("history");
-	if (history) {
+	if (const config &history = preferences::get_child("history"))
+	{
 /* Structure of the history
 	[history]
 		[history_id]
@@ -81,21 +87,11 @@ manager::manager() :
 				message = foobar
 			[/line]
 */
-		const config::child_map& history_id_list = history->all_children();
-		typedef std::pair<std::string, config::child_list> hack;
-		foreach(const hack& history_id, history_id_list) {
-
-			std::vector<std::string> current_history;
-			foreach(const config* history_id_child, history_id.second) {
-
-				const config::child_list& line = history_id_child->get_children("line");
-				foreach(const config* line_data, line) {
-
-					current_history.push_back((*line_data)["message"]);
-				}
+		foreach (const config::any_child &h, history.all_children_range())
+		{
+			foreach (const config &l, h.cfg.child_range("line")) {
+				history_map[h.key].push_back(l["message"]);
 			}
-
-			history_map[history_id.first] = current_history;
 		}
 	}
 
@@ -138,14 +134,13 @@ manager::~manager()
 	history_map.clear();
 	encountered_units_set.clear();
 	encountered_terrains_set.clear();
-	set_ping_timeout(network::ping_timeout);
 }
 
 bool is_authenticated() {
 	return authenticated;
 }
 
-void admin_authentication(const std::string& sender, const std::string& message) {
+void parse_admin_authentication(const std::string& sender, const std::string& message) {
 	if(sender != "server") return;
 	if(message.find("You are now recognized as an administrator.") == 0) {
 		authenticated = true;
@@ -213,14 +208,6 @@ void remove_ignore(const std::string& nick) {
 	}
 }
 
-void clear_friends() {
-	preferences::set("friends", "");
-}
-
-void clear_ignores() {
-	preferences::set("ignores", "");
-}
-
 bool is_friend(const std::string& nick) {
 	initialize_friends();
 	return friends.find(nick) != friends.end();
@@ -231,25 +218,42 @@ bool is_ignored(const std::string& nick) {
 	return ignores.find(nick) != ignores.end();
 }
 
-bool show_lobby_join(const std::string& sender, const std::string& message) {
+void add_completed_campaign(const std::string& campaign_id) {
+	std::vector<std::string> completed = utils::split(preferences::get("completed_campaigns"));
+
+	if(std::find(completed.begin(), completed.end(), campaign_id) != completed.end())
+		return;
+
+	completed.push_back(campaign_id);
+	preferences::set("completed_campaigns", utils::join(completed));
+}
+
+bool is_campaign_completed(const std::string& campaign_id) {
+	std::vector<std::string> completed = utils::split(preferences::get("completed_campaigns"));
+
+	return std::find(completed.begin(), completed.end(), campaign_id) != completed.end();
+}
+
+bool parse_should_show_lobby_join(const std::string &sender, const std::string &message)
+{
 	// If it's actually not a lobby join message return true (show it).
-	if (sender != "server" || message.find("has logged into the lobby") == std::string::npos) return true;
-	if (lobby_joins() == SHOW_NONE) return false;
-	if (lobby_joins() == SHOW_ALL) return true;
-	const std::string::const_iterator i =
-			std::find(message.begin(), message.end(), ' ');
-	const std::string joiner(message.begin(), i);
-	if (lobby_joins() == SHOW_FRIENDS && is_friend(joiner)) return true;
-	return false;
+	if (sender != "server") return true;
+	std::string::size_type pos = message.find(" has logged into the lobby");
+	if (pos == std::string::npos) return true;
+	int lj = lobby_joins();
+	if (lj == SHOW_NONE) return false;
+	if (lj == SHOW_ALL) return true;
+	return is_friend(message.substr(0, pos));
 }
 
 int lobby_joins()
 {
-    if(preferences::get("lobby_joins") == "friends") {
+	std::string pref = preferences::get("lobby_joins");
+	if (pref == "friends") {
 		return SHOW_FRIENDS;
-	} else if(preferences::get("lobby_joins") == "all") {
+	} else if (pref == "all") {
 		return SHOW_ALL;
-	} else if(preferences::get("lobby_joins") == "none") {
+	} else if (pref == "none") {
 		return SHOW_NONE;
 	} else {
 		return SHOW_FRIENDS;
@@ -268,25 +272,14 @@ void _set_lobby_joins(int show)
 	}
 }
 
-bool sort_list()
+bool new_lobby()
 {
-	return utils::string_bool(preferences::get("sort_list"), true);
+	return get("new_lobby", false);
 }
 
-
-void _set_sort_list(bool sort)
+void set_new_lobby(bool value)
 {
-	preferences::set("sort_list", sort ? "yes" : "no");
-}
-
-bool iconize_list()
-{
-	return utils::string_bool(preferences::get("iconize_list"), true);
-}
-
-void _set_iconize_list(bool sort)
-{
-	preferences::set("iconize_list", sort ? "yes" : "no");
+	preferences::set("new_lobby", value);
 }
 
 const std::vector<game_config::server_info>& server_list()
@@ -294,14 +287,12 @@ const std::vector<game_config::server_info>& server_list()
 	static std::vector<game_config::server_info> pref_servers;
 	if(pref_servers.empty()) {
 		std::vector<game_config::server_info> &game_servers = game_config::server_list;
-		VALIDATE(game_servers.size() > 0, _("No server has been defined."));
+		VALIDATE(!game_servers.empty(), _("No server has been defined."));
 		pref_servers.insert(pref_servers.begin(), game_servers.begin(), game_servers.end());
-		const std::vector<config *> &user_servers = get_prefs()->get_children("server");
-		std::vector<config *>::const_iterator server;
-		for(server = user_servers.begin(); server != user_servers.end(); ++server) {
+		foreach (const config &server, get_prefs()->child_range("server")) {
 			game_config::server_info sinf;
-			sinf.name = (**server)["name"];
-			sinf.address = (**server)["address"];
+			sinf.name = server["name"].str();
+			sinf.address = server["address"].str();
 			pref_servers.push_back(sinf);
 		}
 	}
@@ -328,12 +319,6 @@ unsigned int get_ping_timeout()
 	return lexical_cast_default<unsigned>(preferences::get("ping_timeout"), 0);
 }
 
-void set_ping_timeout(unsigned int timeout)
-{
-	network::ping_timeout = timeout;
-	preferences::set("ping_timeout", lexical_cast<std::string>(timeout));
-}
-
 std::string campaign_server()
 {
 	if(!preferences::get("campaign_server").empty()) {
@@ -393,48 +378,78 @@ void set_password(const std::string& password)
 
 bool remember_password()
 {
-	return utils::string_bool(preferences::get("remember_password"), false);
+	return preferences::get("remember_password", false);
 }
 
 void set_remember_password(bool remember)
 {
-	preferences::set("remember_password", remember ? "yes" : "no");
+	preferences::set("remember_password", remember);
 	preferences::set("password", remember ? prv::password : "");
 }
 
 bool turn_dialog()
 {
-	return utils::string_bool(preferences::get("turn_dialog"), false);
+	return preferences::get("turn_dialog", false);
 }
 
 void set_turn_dialog(bool ison)
 {
-	preferences::set("turn_dialog", ison ? "yes" : "no");
+	preferences::set("turn_dialog", ison);
+}
+
+bool enable_whiteboard_mode_on_start()
+{
+	return preferences::get("enable_planning_mode_on_start", false);
+}
+
+void set_enable_whiteboard_mode_on_start(bool value)
+{
+	preferences::set("enable_planning_mode_on_start", value);
+}
+
+bool hide_whiteboard()
+{
+	return preferences::get("hide_whiteboard", false);
+}
+
+void set_hide_whiteboard(bool value)
+{
+	preferences::set("hide_whiteboard", value);
 }
 
 bool show_combat()
 {
-	return utils::string_bool(preferences::get("show_combat"), true);
+	return preferences::get("show_combat", true);
 }
 
 bool allow_observers()
 {
-	return utils::string_bool(preferences::get("allow_observers"), true);
+	return preferences::get("allow_observers", true);
 }
 
 void set_allow_observers(bool value)
 {
-	preferences::set("allow_observers", value ? "yes" : "no");
+	preferences::set("allow_observers", value);
+}
+
+bool shuffle_sides()
+{
+	return preferences::get("shuffle_sides", false);
+}
+
+void set_shuffle_sides(bool value)
+{
+	preferences::set("shuffle_sides", value);
 }
 
 bool use_map_settings()
 {
-	return utils::string_bool(preferences::get("mp_use_map_settings"), true);
+	return preferences::get("mp_use_map_settings", true);
 }
 
 void set_use_map_settings(bool value)
 {
-	preferences::set("mp_use_map_settings", value ? "yes" : "no");
+	preferences::set("mp_use_map_settings", value);
 }
 
 int mp_server_warning_disabled()
@@ -444,7 +459,7 @@ int mp_server_warning_disabled()
 
 void set_mp_server_warning_disabled(int value)
 {
-	preferences::set("mp_server_warning_disabled", lexical_cast<std::string>(value));
+	preferences::set("mp_server_warning_disabled", value);
 }
 
 void set_mp_server_program_name(const std::string& path)
@@ -466,32 +481,32 @@ std::string get_mp_server_program_name()
 
 bool random_start_time()
 {
-	return settings::use_random_start_time(preferences::get("mp_random_start_time"));
+	return preferences::get("mp_random_start_time", true);
 }
 
 void set_random_start_time(bool value)
 {
-	preferences::set("mp_random_start_time", value ? "yes" : "no");
+	preferences::set("mp_random_start_time", value);
 }
 
 bool fog()
 {
-	return settings::use_fog(preferences::get("mp_fog"));
+	return preferences::get("mp_fog", true);
 }
 
 void set_fog(bool value)
 {
-	preferences::set("mp_fog", value ? "yes" : "no");
+	preferences::set("mp_fog", value);
 }
 
 bool shroud()
 {
-	return settings::use_shroud(preferences::get("mp_shroud"));
+	return preferences::get("mp_shroud", false);
 }
 
 void set_shroud(bool value)
 {
-	preferences::set("mp_shroud", value ? "yes" : "no");
+	preferences::set("mp_shroud", value);
 }
 
 int turns()
@@ -501,27 +516,27 @@ int turns()
 
 void set_turns(int value)
 {
-	preferences::set("mp_turns", lexical_cast<std::string>(value));
+	preferences::set("mp_turns", value);
 }
 
 bool skip_mp_replay()
 {
-	return utils::string_bool(preferences::get("skip_mp_replay"), false);
+	return preferences::get("skip_mp_replay", false);
 }
 
 void set_skip_mp_replay(bool value)
 {
-	preferences::set("skip_mp_replay", value ? "yes" : "no");
+	preferences::set("skip_mp_replay", value);
 }
 
 bool countdown()
 {
-	return utils::string_bool(preferences::get("mp_countdown"), false);
+	return preferences::get("mp_countdown", false);
 }
 
 void set_countdown(bool value)
 {
-	preferences::set("mp_countdown", value ? "yes" : "no");
+	preferences::set("mp_countdown", value);
 }
 
 int countdown_init_time()
@@ -532,7 +547,7 @@ int countdown_init_time()
 
 void set_countdown_init_time(int value)
 {
-	preferences::set("mp_countdown_init_time", lexical_cast<std::string>(value));
+	preferences::set("mp_countdown_init_time", value);
 }
 
 int countdown_reservoir_time()
@@ -543,7 +558,7 @@ int countdown_reservoir_time()
 
 void set_countdown_reservoir_time(int value)
 {
-	preferences::set("mp_countdown_reservoir_time", lexical_cast<std::string>(value));
+	preferences::set("mp_countdown_reservoir_time", value);
 }
 
 int countdown_turn_bonus()
@@ -554,7 +569,7 @@ int countdown_turn_bonus()
 
 void set_countdown_turn_bonus(int value)
 {
-	preferences::set("mp_countdown_turn_bonus", lexical_cast<std::string>(value));
+	preferences::set("mp_countdown_turn_bonus", value);
 }
 
 int countdown_action_bonus()
@@ -565,7 +580,7 @@ int countdown_action_bonus()
 
 void set_countdown_action_bonus(int value)
 {
-	preferences::set("mp_countdown_action_bonus", lexical_cast<std::string>(value));
+	preferences::set("mp_countdown_action_bonus", value);
 }
 
 int village_gold()
@@ -575,7 +590,7 @@ int village_gold()
 
 void set_village_gold(int value)
 {
-	preferences::set("mp_village_gold", lexical_cast<std::string>(value));
+	preferences::set("mp_village_gold", value);
 }
 
 int xp_modifier()
@@ -585,7 +600,7 @@ int xp_modifier()
 
 void set_xp_modifier(int value)
 {
-	preferences::set("mp_xp_modifier", lexical_cast<std::string>(value));
+	preferences::set("mp_xp_modifier", value);
 }
 
 int era()
@@ -595,7 +610,7 @@ int era()
 
 void set_era(int value)
 {
-	preferences::set("mp_era", lexical_cast<std::string>(value));
+	preferences::set("mp_era", value);
 }
 
 int map()
@@ -605,57 +620,67 @@ int map()
 
 void set_map(int value)
 {
-	preferences::set("mp_map", lexical_cast<std::string>(value));
+	preferences::set("mp_map", value);
 }
 
 bool show_ai_moves()
 {
-	return utils::string_bool(preferences::get("show_ai_moves"), true);
+	return preferences::get("show_ai_moves", true);
 }
 
 void set_show_ai_moves(bool value)
 {
-	preferences::set("show_ai_moves", value ? "yes" : "no");
+	preferences::set("show_ai_moves", value);
 }
 
-void set_show_side_colours(bool value)
+void set_show_side_colors(bool value)
 {
-	preferences::set("show_side_colours", value ? "yes" : "no");
+	preferences::set("show_side_colors", value);
 }
 
-bool show_side_colours()
+bool show_side_colors()
 {
-	return utils::string_bool(preferences::get("show_side_colours"), true);
+	return preferences::get("show_side_colors", true);
 }
 
 void set_save_replays(bool value)
 {
-	preferences::set("save_replays", value ? "yes" : "no");
+	preferences::set("save_replays", value);
 }
 
 bool save_replays()
 {
-	return utils::string_bool(preferences::get("save_replays"), true);
+	return preferences::get("save_replays", true);
 }
 
 void set_delete_saves(bool value)
 {
-	preferences::set("delete_saves", value ? "yes" : "no");
+	preferences::set("delete_saves", value);
 }
 
 bool delete_saves()
 {
-	return utils::string_bool(preferences::get("delete_saves"), false);
+	return preferences::get("delete_saves", false);
 }
 
 void set_ask_delete_saves(bool value)
 {
-	preferences::set("ask_delete", value ? "yes" : "no");
+	preferences::set("ask_delete", value);
 }
 
 bool ask_delete_saves()
 {
-	return utils::string_bool(preferences::get("ask_delete"), true);
+	return preferences::get("ask_delete", true);
+}
+
+void set_interrupt_when_ally_sighted(bool value)
+{
+	preferences::set("ally_sighted_interrupts", value);
+}
+
+bool interrupt_when_ally_sighted()
+{
+	return preferences::get("ally_sighted_interrupts", true);
 }
 
 int autosavemax()
@@ -665,7 +690,7 @@ int autosavemax()
 
 void set_autosavemax(int value)
 {
-	preferences::set("auto_save_max", lexical_cast<std::string>(value));
+	preferences::set("auto_save_max", value);
 }
 
 std::string client_type()
@@ -673,15 +698,6 @@ std::string client_type()
 	return preferences::get("client_type") == "ai" ? "ai" : "human";
 }
 
-std::string clock_format()
-{
-	if(preferences::get("clock_format").size())
-		return preferences::get("clock_format");
-	else
-		preferences::set("clock_format", "%H:%M");
-	return "%H:%M";
-}
-
 std::string theme()
 {
 	if(non_interactive()) {
@@ -706,12 +722,12 @@ void set_theme(const std::string& theme)
 
 bool show_floating_labels()
 {
-	return utils::string_bool(preferences::get("floating_labels"), true);
+	return preferences::get("floating_labels", true);
 }
 
 void set_show_floating_labels(bool value)
 {
-	preferences::set("floating_labels", value ? "yes" : "no");
+	preferences::set("floating_labels", value);
 }
 
 bool message_private()
@@ -724,11 +740,6 @@ void set_message_private(bool value)
 	message_private_on = value;
 }
 
-bool show_tip_of_day()
-{
-	return utils::string_bool(preferences::get("tip_of_day"), true);
-}
-
 bool show_haloes()
 {
 	return haloes;
@@ -737,70 +748,47 @@ bool show_haloes()
 void set_show_haloes(bool value)
 {
 	haloes = value;
-	preferences::set("show_haloes", value ? "yes" : "no");
+	preferences::set("show_haloes", value);
 }
 
 bool flip_time()
 {
-	return utils::string_bool(preferences::get("flip_time"), false);
+	return preferences::get("flip_time", false);
 }
 
 void set_flip_time(bool value)
 {
-	preferences::set("flip_time", value ? "yes" : "no");
-}
-
-bool has_upload_log()
-{
-	return preferences::get("upload_log_new").empty() == false;
-}
-
-bool upload_log()
-{
-	return utils::string_bool(preferences::get("upload_log_new"), true);
-}
-
-void set_upload_log(bool value)
-{
-	preferences::set("upload_log_new", value ? "yes" : "no");
-}
-
-std::string upload_id()
-{
-	// We create a unique id for each person, *when asked for* to increase
-	// randomness.
-	if (preferences::get("upload_id") == "") {
-		srand(time(NULL));
-		preferences::set("upload_id",
-			lexical_cast<std::string>(rand())
-				 + lexical_cast<std::string>(SDL_GetTicks()));
-	}
-	return preferences::get("upload_id");
+	preferences::set("flip_time", value);
 }
 
 bool compress_saves()
 {
-	return utils::string_bool(preferences::get("compress_saves"), true);
+	return preferences::get("compress_saves", true);
 }
 
 bool startup_effect()
 {
-	return utils::string_bool(preferences::get("startup_effect"), true);
+	return preferences::get("startup_effect", false);
 }
 
 std::string get_chat_timestamp(const time_t& t) {
 	if (chat_timestamping()) {
-		return lg::get_timestamp(t, clock_format()) + " ";
+		if(preferences::use_twelve_hour_clock_format() == false) {
+			return lg::get_timestamp(t, _("%H:%M")) + " ";
+		}
+		else {
+			return lg::get_timestamp(t, _("%I:%M %p")) + " ";
+		}
 	}
 	return "";
 }
 
 bool chat_timestamping() {
-	return utils::string_bool(preferences::get("chat_timestamp"), false);
+	return preferences::get("chat_timestamp", false);
 }
 
 void set_chat_timestamping(bool value) {
-	preferences::set("chat_timestamp", value ? "yes" : "no");
+	preferences::set("chat_timestamp", value);
 }
 
 int chat_lines()
@@ -810,7 +798,25 @@ int chat_lines()
 
 void set_chat_lines(int lines)
 {
-	preferences::set("chat_lines", lexical_cast<std::string>(lines));
+	preferences::set("chat_lines", lines);
+}
+
+void set_chat_message_aging(const int aging)
+{
+	preferences::set("chat_message_aging", aging);
+}
+
+int chat_message_aging()
+{
+	return lexical_cast_default<int>(preferences::get("chat_message_aging"), 20);
+}
+
+bool show_all_units_in_help() {
+	return preferences::get("show_all_units_in_help", false);
+}
+
+void set_show_all_units_in_help(bool value) {
+	preferences::set("show_all_units_in_help", value);
 }
 
 std::set<std::string> &encountered_units() {
@@ -826,7 +832,7 @@ std::string custom_command() {
 }
 
 void set_custom_command(const std::string& command) {
-	return preferences::set("custom_command", command);
+	preferences::set("custom_command", command);
 }
 
 /**
@@ -863,7 +869,7 @@ bool confirm_no_moves()
 
 void encounter_recruitable_units(std::vector<team>& teams){
 	for (std::vector<team>::iterator help_team_it = teams.begin();
-		help_team_it != teams.end(); help_team_it++) {
+		help_team_it != teams.end(); ++help_team_it) {
 		help_team_it->log_recruitable();
 		std::copy(help_team_it->recruits().begin(), help_team_it->recruits().end(),
 				  std::inserter(encountered_units_set, encountered_units_set.begin()));
@@ -872,27 +878,27 @@ void encounter_recruitable_units(std::vector<team>& teams){
 
 void encounter_start_units(unit_map& units){
 	for (unit_map::const_iterator help_unit_it = units.begin();
-		 help_unit_it != units.end(); help_unit_it++) {
-		const std::string name = help_unit_it->second.type_id();
+		 help_unit_it != units.end(); ++help_unit_it) {
+		const std::string name = help_unit_it->type_id();
 		encountered_units_set.insert(name);
 	}
 }
 
-void encounter_recallable_units(game_state& gamestate){
-	for(std::map<std::string, player_info>::iterator pi = gamestate.players.begin(); pi!=gamestate.players.end(); ++pi) {
-		for(std::vector<unit>::iterator help_recall_it = pi->second.available_units.begin(); help_recall_it != pi->second.available_units.end(); help_recall_it++) {
-			encountered_units_set.insert(help_recall_it->type_id());
+void encounter_recallable_units(std::vector<team>& teams){
+	foreach(const team& t, teams) {
+		foreach(const unit& u, t.recall_list()) {
+			encountered_units_set.insert(u.type_id());
 		}
 	}
 }
 
 void encounter_map_terrain(gamemap& map){
-	for (int map_x = 0; map_x < map.w(); map_x++) {
-		for (int map_y = 0; map_y < map.h(); map_y++) {
+	for (int map_x = 0; map_x < map.w(); ++map_x) {
+		for (int map_y = 0; map_y < map.h(); ++map_y) {
 			const t_translation::t_terrain t = map.get_terrain(map_location(map_x, map_y));
 			preferences::encountered_terrains().insert(t);
 			const t_translation::t_list& underlaying_list = map.underlying_union_terrain(map_location(map_x, map_y));
-			for (std::vector<t_translation::t_terrain>::const_iterator ut = underlaying_list.begin(); ut != underlaying_list.end(); ut++) {
+			for (std::vector<t_translation::t_terrain>::const_iterator ut = underlaying_list.begin(); ut != underlaying_list.end(); ++ut) {
 				preferences::encountered_terrains().insert(*ut);
 			};
 		}
