@@ -1,18 +1,19 @@
-/* $Id: gamestatus.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: gamestatus.hpp 51437 2011-10-10 17:12:26Z anonymissimus $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
-/** @file gamestatus.hpp */
+/** @file */
 
 #ifndef GAME_STATUS_HPP_INCLUDED
 #define GAME_STATUS_HPP_INCLUDED
@@ -20,16 +21,21 @@
 #include "mp_game_settings.hpp"
 #include "random.hpp"
 #include "simple_rng.hpp"
-#include "team.hpp"
+#include "map_location.hpp"
 #include "variable.hpp"
-#include "savegame_config.hpp"
 #include "serialization/binary_or_text.hpp"
-
-#include <time.h>
-#include <string>
-#include <vector>
+#include "serialization/string_utils.hpp"
+#include "boost/shared_ptr.hpp"
 
 class scoped_wml_variable;
+class team;
+class gamemap;
+namespace t_translation {
+	struct t_match;
+}
+
+class team_builder;
+typedef boost::shared_ptr<team_builder> team_builder_ptr;
 
 //meta information of the game
 class game_classification : public savegame::savegame_config
@@ -84,7 +90,7 @@ public:
 	std::vector<scoped_wml_variable*> scoped_variables;
 	std::map<std::string, wml_menu_item*> wml_menu_items;
 
-	const config& get_variables() const { return variables; }
+	const config& get_variables() const { return variables_; }
 	void set_variables(const config& vars);
 
 	void set_menu_items(const config::const_child_itors &menu_items);
@@ -96,10 +102,9 @@ public:
 
 	// Variable access
 
-	t_string& get_variable(const std::string& varname);
-	virtual const t_string& get_variable_const(const std::string& varname) const;
+	config::attribute_value &get_variable(const std::string &varname);
+	virtual config::attribute_value get_variable_const(const std::string& varname) const;
 	config& get_variable_cfg(const std::string& varname);
-	variable_info::array_range get_variable_cfgs(const std::string& varname);
 
 	void set_variable(const std::string& varname, const t_string& value);
 	config& add_variable_cfg(const std::string& varname, const config& value=config());
@@ -110,11 +115,27 @@ public:
 	const rand_rng::simple_rng& rng() const { return rng_; }
 	rand_rng::simple_rng& rng() { return rng_; }
 
-	//create and populate a team from a config
-	void build_team(const config& side_cfg, std::string save_id
+	enum PHASE {
+		INITIAL,
+		PRELOAD,
+		PRESTART,
+		START,
+		PLAY
+	};
+	PHASE phase() const { return phase_; }
+	void set_phase(PHASE phase) { phase_ = phase; }
+
+	//create an object responsible for creating and populating a team from a config
+	team_builder_ptr create_team_builder(const config& side_cfg, std::string save_id
 			, std::vector<team>& teams, const config& level, gamemap& map
 			, unit_map& units, bool snapshot);
 
+	//do first stage of team initialization (everything except unit placement)
+	void build_team_stage_one(team_builder_ptr tb_ptr);
+
+	//do second stage of team initialization (unit placement)
+	void build_team_stage_two(team_builder_ptr tb_ptr);
+
 	game_classification& classification() { return classification_; }
 	const game_classification& classification() const { return classification_; } //FIXME: const getter to allow use from const gamestatus::sog() (see ai.cpp:344) - remove after merge?
 
@@ -122,6 +143,9 @@ public:
 	mp_game_settings& mp_settings() { return mp_settings_; }
 	const mp_game_settings& mp_settings() const { return mp_settings_; }
 
+	bool allow_end_turn() const { return can_end_turn_; }
+	void set_allow_end_turn(bool value) { can_end_turn_ = value; }
+
 	/**
 	 * If the game is saved mid-level, we have a series of replay steps
 	 * to take the game up to the position it was saved at.
@@ -148,18 +172,18 @@ public:
 	map_location last_selected;
 
 private:
-	rand_rng::simple_rng rng_ ;
-	config variables;
-	mutable config temporaries; // lengths of arrays, etc.
-	const rand_rng::set_random_generator generator_setter; /**< Make sure that rng is initialized first */
+	rand_rng::simple_rng rng_;
+	config variables_;
+	mutable config temporaries_; // lengths of arrays, etc.
+	const rand_rng::set_random_generator generator_setter_; /**< Make sure that rng is initialized first */
 	friend struct variable_info;
 	game_classification classification_;
 	mp_game_settings mp_settings_;
+	PHASE phase_;
+	bool can_end_turn_;
 };
 
 
-std::string generate_game_uuid();
-
 void write_players(game_state& gamestate, config& cfg, const bool use_snapshot=true, const bool merge_side = false);
 
 void extract_summary_from_config(config& cfg_save, config& cfg_summary);
