@@ -1,11 +1,12 @@
-/* $Id: image.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: image.cpp 50749 2011-08-13 22:20:07Z sytyi $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -36,8 +37,8 @@ twidget* tbuilder_image::build() const
 	init_control(widget);
 
 	DBG_GUI_G << "Window builder: placed image '"
-		<< id << "' with defintion '"
-		<< definition << "'.\n";
+			<< id << "' with definition '"
+			<< definition << "'.\n";
 
 	return widget;
 }
@@ -47,10 +48,9 @@ twidget* tbuilder_image::build() const
 } // namespace gui2
 
 /*WIKI_MACRO
- * @start_macro = image_description
- *
- *        An image shows a static image.
- * @end_macro
+ * @begin{macro}{image_description}
+ * An image shows a static image.
+ * @end{macro}
  */
 
 /*WIKI
@@ -62,5 +62,9 @@ twidget* tbuilder_image::build() const
  * @macro = image_description
  *
  * An image has no extra fields.
+ * @begin{parent}{name="gui/window/resolution/grid/row/column/"}
+ * @begin{tag}{name="image"}{min=0}{max=-1}{super="generic/widget_instance"}
+ * @end{tag}{name="image"}
+ * @end{parent}{name="gui/window/resolution/grid/row/column/"}
  */
 
