@@ -1,11 +1,12 @@
-/* $Id: minimap.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: minimap.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -24,7 +25,7 @@ namespace implementation {
 struct tbuilder_minimap
 	: public tbuilder_control
 {
-	tbuilder_minimap(const config& cfg);
+	explicit tbuilder_minimap(const config& cfg);
 
 	twidget* build () const;
 };
