@@ -1,11 +1,12 @@
-/* $Id: multi_page.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: multi_page.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,13 +16,12 @@
 #ifndef GUI_AUXILIARY_WINDOW_BUILDER_MULTI_PAGE_HPP_INCLUDED
 #define GUI_AUXILIARY_WINDOW_BUILDER_MULTI_PAGE_HPP_INCLUDED
 
+#include "gui/widgets/widget.hpp"
 #include "gui/auxiliary/window_builder/control.hpp"
 
 #include <map>
 #include <vector>
 
-typedef std::map<std::string,t_string> string_map;
-
 namespace gui2 {
 
 namespace implementation {
@@ -29,7 +29,7 @@ namespace implementation {
 struct tbuilder_multi_page
 	: public tbuilder_control
 {
-	tbuilder_multi_page(const config& cfg);
+	explicit tbuilder_multi_page(const config& cfg);
 
 	twidget* build () const;
 
@@ -50,4 +50,3 @@ struct tbuilder_multi_page
 
 #endif
 
-
