@@ -1,11 +1,12 @@
-/* $Id: vertical_scrollbar.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: vertical_scrollbar.cpp 50749 2011-08-13 22:20:07Z sytyi $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -37,7 +38,7 @@ twidget* tbuilder_vertical_scrollbar::build() const
 
 	DBG_GUI_G << "Window builder:"
 			<< " placed vertical scrollbar '" << id
-			<< "' with defintion '" << definition
+			<< "' with definition '" << definition
 			<< "'.\n";
 
 	return widget;
@@ -53,7 +54,10 @@ twidget* tbuilder_vertical_scrollbar::build() const
  *
  * == Vertical scrollbar ==
  *
- * A vertical scrollbar has no special fields.
  *
+ * @begin{parent}{name="gui/window/resolution/grid/row/column/"}
+ * @begin{tag}{name="vertical_scrollbar"}{min=0}{max=1}{super="generic/widget_instance"}
+ * @end{tag}{name="vertical_scrollbar"}
+ * @end{parent}{name="gui/window/resolution/grid/row/column/"}
  */
 
