@@ -1,11 +1,12 @@
-/* $Id: campaign_selection.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: campaign_selection.hpp 49604 2011-05-22 17:56:21Z mordante $ */
 /*
-   Copyright (C) 2009 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2009 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -25,22 +26,24 @@ class tcampaign_selection
 	: public tdialog
 {
 public:
-	tcampaign_selection(const std::vector<config> &c)
-		: campaigns_(c)
+	explicit tcampaign_selection(const std::vector<config>& campaigns)
+		: campaigns_(campaigns)
 		, choice_(-1)
 
 	{
 	}
 
+	/***** ***** ***** setters / getters for members ***** ****** *****/
+
 	int get_choice() const { return choice_; }
 
+private:
+
 	/** Called when another campaign is selected. */
 	void campaign_selected(twindow& window);
 
-private:
-
-	/** Inherited from tdialog. */
-	twindow* build_window(CVideo& video);
+	/** Inherited from tdialog, implemented by REGISTER_DIALOG. */
+	virtual const std::string& window_id() const;
 
 	/** Inherited from tdialog. */
 	void pre_show(CVideo& video, twindow& window);
@@ -48,12 +51,10 @@ private:
 	/** Inherited from tdialog. */
 	void post_show(twindow& window);
 
-	/** Config which contains the list with the campaigns. */
-
-	/** Campaign list. */
+	/** Contains the config objects for all campaigns. */
 	const std::vector<config> &campaigns_;
 
-	/** The choosen campaign. */
+	/** The chosen campaign. */
 	int choice_;
 };
 
