@@ -1,11 +1,12 @@
-/* $Id: editor_settings.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: editor_settings.hpp 51731 2011-10-30 04:39:34Z shadowmaster $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -19,20 +20,32 @@
 #include "gui/dialogs/dialog.hpp"
 
 #include <vector>
-#include <boost/function.hpp>
+
+namespace editor {
+
+class editor_display;
+
+} // namespace editor
 
 namespace gui2 {
 
 class tlabel;
 class ttoggle_button;
-class tslider;
 
 class teditor_settings : public tdialog
 {
 public:
-	teditor_settings();
+	teditor_settings(editor::editor_display* display
+			, const std::vector<time_of_day>& tods);
 
-	void set_redraw_callback(boost::function<void (int, int, int)> callback) { redraw_callback_ = callback; }
+	static bool execute(editor::editor_display* display
+			, const std::vector<time_of_day>& tods
+			, CVideo& video)
+	{
+		return teditor_settings(display, tods).show(video);
+	}
+
+private:
 
 	/** Callback for the next tod button */
 	void do_next_tod(twindow& window);
@@ -41,31 +54,20 @@ public:
 
 	void slider_update_callback(twindow& window);
 
-	void set_tods(const std::vector<time_of_day>& tods) { tods_ = tods; }
-	const std::vector<time_of_day>& get_tods() const { return tods_; }
-
-	void set_current_adjustment(int r, int g, int b);
-
 	void set_selected_tod(time_of_day tod);
 	const time_of_day& get_selected_tod() const;
 
-	int get_red() const;
-	int get_green() const;
-	int get_blue() const;
-
 	void update_selected_tod_info(twindow& window);
 
-	bool get_use_mdi() const;
-	void set_use_mdi(bool value);
-
 private:
-	/** Inherited from tdialog. */
-	twindow* build_window(CVideo& video);
+	/** Inherited from tdialog, implemented by REGISTER_DIALOG. */
+	virtual const std::string& window_id() const;
 
 	/** Inherited from tdialog. */
 	void pre_show(CVideo& video, twindow& window);
 
-	boost::function<void (int, int, int)> redraw_callback_;
+	/** Inherited from tdialog. */
+	void post_show(twindow& window);
 
 	/** Available time_of_days */
 	std::vector<time_of_day> tods_;
@@ -84,15 +86,19 @@ private:
 
 	tfield_bool* custom_tod_toggle_field_;
 
-	tslider* custom_tod_red_;
-	tslider* custom_tod_green_;
-	tslider* custom_tod_blue_;
-
 	tfield_integer* custom_tod_red_field_;
 	tfield_integer* custom_tod_green_field_;
 	tfield_integer* custom_tod_blue_field_;
 
-	tfield_bool* use_mdi_field_;
+	/**
+	 * The display to update when the ToD changes.
+	 *
+	 * The pointer may be NULL, in the unit tests, but normally it should be a
+	 * pointer to a valid object.
+	 */
+	editor::editor_display* display_;
+
+	bool can_update_display_;
 };
 
 } // namespace gui2
