@@ -1,11 +1,12 @@
-/* $Id: gamestate_inspector.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: gamestate_inspector.hpp 49606 2011-05-22 17:56:26Z mordante $ */
 /*
-   Copyright (C) 2009 - 2010 by Yurii Chernyi <terraninfo@terraninfo.net>
+   Copyright (C) 2009 - 2011 by Yurii Chernyi <terraninfo@terraninfo.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -27,17 +28,17 @@ public:
 	class model;
 	class view;
 	class controller;
-	tgamestate_inspector(const vconfig &cfg);
-
-	/** Inherited from tdialog. */
-	twindow* build_window(CVideo& video);
-
-	/** Inherited from tdialog. */
-	void pre_show(CVideo& video, twindow& window);
+	explicit tgamestate_inspector(const vconfig &cfg);
 
 	boost::shared_ptr<view> get_view();
 
 private:
+	/** Inherited from tdialog. */
+	void pre_show(CVideo& video, twindow& window);
+
+	/** Inherited from tdialog, implemented by REGISTER_DIALOG. */
+	virtual const std::string& window_id() const;
+
 	boost::shared_ptr<view> view_;
 
 };
