@@ -1,11 +1,12 @@
-/* $Id$ */
+/* $Id: lobby_player_info.hpp 48440 2011-02-07 20:57:31Z mordante $ */
 /*
-   Copyright (C) 2009 - 2010 by Tomasz Sniatowski <kailoran@gmail.com>
+   Copyright (C) 2009 - 2011 by Tomasz Sniatowski <kailoran@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -36,8 +37,9 @@ public:
 	bool result_open_whisper() const { return result_open_whisper_; }
 
 private:
-	/** Inherited from tdialog. */
-	twindow* build_window(CVideo& video);
+
+	/** Inherited from tdialog, implemented by REGISTER_DIALOG. */
+	virtual const std::string& window_id() const;
 
 	/** Inherited from tdialog. */
 	void pre_show(CVideo& video, twindow& window);
