@@ -1,12 +1,13 @@
-/* $Id: password_box.hpp 32658 2009-02-07 18:04:43Z baufo $ */
+/* $Id: password_box.hpp 49599 2011-05-22 17:55:59Z mordante $ */
 /*
-   Copyright (C) 2009 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
-   Copyright (C) 2009 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2009 - 2011 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
+   Copyright (C) 2009 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -51,7 +52,8 @@ class tpassword_box : public ttext_box {
 public:
 	tpassword_box() : ttext_box(), real_value_() {}
 
-	void set_value(const std::string& text);
+	/** Inherited from ttext_. */
+	virtual void set_value(const std::string& text);
 	std::string get_real_value() const { return real_value_; }
 
 
@@ -74,6 +76,8 @@ private:
 
 	std::string real_value_;
 
+	/** Inherited from ttext_box. */
+	const std::string& get_control_type() const;
 };
 
 } //namespace gui2
