@@ -1,11 +1,12 @@
-/* $Id: scrollbar.cpp 41498 2010-03-07 18:43:34Z mordante $ */
+/* $Id: scrollbar.cpp 51570 2011-10-21 20:07:12Z espreon $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -38,7 +39,6 @@ tscrollbar_::tscrollbar_()
 	, mouse_(0, 0)
 	, positioner_offset_(0)
 	, positioner_length_(0)
-	, callback_positioner_move_(0)
 {
 	connect_signal<event::MOUSE_ENTER>(boost::bind(
 				&tscrollbar_::signal_handler_mouse_enter, this, _2, _3, _4));
@@ -182,7 +182,7 @@ void tscrollbar_::recalculate()
 		assert(window);
 		window->invalidate_layout();
 		ERR_GUI_G << LOG_HEADER
-				<< " Can't recalculate size, force a Window layout phase.\n";
+				<< " Can't recalculate size, force a window layout phase.\n";
 		return;
 	}
 
@@ -261,9 +261,9 @@ void tscrollbar_::move_positioner(const int distance)
 
 		child_callback_positioner_moved();
 
-		if(callback_positioner_move_) {
-			callback_positioner_move_(this);
-		}
+		fire(event::NOTIFY_MODIFIED, *this, NULL);
+
+//		positioner_moved_notifier_.notify();
 	}
 #if 0
 	std::cerr << "Scrollbar move overview:\n"
@@ -379,14 +379,12 @@ void tscrollbar_::signal_handler_left_button_down(
 
 	if(bar == -1) {
 		scroll(HALF_JUMP_BACKWARDS);
-		if(callback_positioner_move_) {
-			callback_positioner_move_(this);
-		}
+		fire(event::NOTIFY_MODIFIED, *this, NULL);
+//		positioner_moved_notifier_.notify();
 	} else if(bar == 1) {
 		scroll(HALF_JUMP_FORWARD);
-		if(callback_positioner_move_) {
-			callback_positioner_move_(this);
-		}
+		fire(event::NOTIFY_MODIFIED, *this, NULL);
+//		positioner_moved_notifier_.notify();
 	} else {
 		assert(bar == 0);
 	}
