@@ -1,11 +1,12 @@
-/* $Id: slider.cpp 40674 2010-01-11 22:11:12Z mordante $ */
+/* $Id: slider.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -20,14 +21,20 @@
 #include "formatter.hpp"
 #include "gui/auxiliary/log.hpp"
 #include "gui/auxiliary/widget_definition/slider.hpp"
+#include "gui/auxiliary/window_builder/slider.hpp"
+#include "gui/widgets/window.hpp"
 #include "gui/widgets/settings.hpp"
 #include "sound.hpp"
 
+#include <boost/bind.hpp>
+
 #define LOG_SCOPE_HEADER get_control_type() + " [" + id() + "] " + __func__
 #define LOG_HEADER LOG_SCOPE_HEADER + ':'
 
 namespace gui2 {
 
+REGISTER_WIDGET(slider)
+
 static int distance(const int a, const int b)
 {
 	/**
@@ -39,6 +46,21 @@ static int distance(const int a, const int b)
 	return result;
 }
 
+tslider::tslider():
+		tscrollbar_(),
+		best_slider_length_(0),
+		minimum_value_(0),
+		minimum_value_label_(),
+		maximum_value_label_(),
+		value_labels_()
+{
+	connect_signal<event::SDL_KEY_DOWN>(boost::bind(
+		&tslider::signal_handler_sdl_key_down, this, _2, _3, _5));
+	connect_signal<event::LEFT_BUTTON_UP>(boost::bind(
+		&tslider::signal_handler_left_button_up, this, _2, _3));
+
+}
+
 tpoint tslider::calculate_best_size() const
 {
 	log_scope2(log_gui_layout, LOG_SCOPE_HEADER);
@@ -221,5 +243,49 @@ const std::string& tslider::get_control_type() const
 	return type;
 }
 
+void tslider::handle_key_decrease(bool& handled)
+{
+	DBG_GUI_E << LOG_HEADER << '\n';
+
+	handled = true;
+
+	scroll(tscrollbar_::ITEM_BACKWARDS);
+}
+
+void tslider::handle_key_increase(bool& handled)
+{
+	DBG_GUI_E << LOG_HEADER << '\n';
+
+	handled = true;
+
+	scroll(tscrollbar_::ITEM_FORWARD);
+}
+
+void tslider::signal_handler_sdl_key_down(const event::tevent event
+		, bool& handled
+		, const SDLKey key)
+{
+
+	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
+
+	if (key == SDLK_DOWN || key == SDLK_LEFT) {
+		handle_key_decrease(handled);
+	} else if (key == SDLK_UP || key == SDLK_RIGHT) {
+		handle_key_increase(handled);
+	} else {
+		// Do nothing. Ignore other keys.
+	}
+}
+
+void tslider::signal_handler_left_button_up(
+		const event::tevent event, bool& handled)
+{
+	DBG_GUI_E << LOG_HEADER << ' ' << event << ".\n";
+
+	get_window()->keyboard_capture(this);
+
+	handled = true;
+}
+
 } // namespace gui2
 
