@@ -1,11 +1,12 @@
-/* $Id: slider.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: slider.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -25,15 +26,7 @@ class tslider : public tscrollbar_, public tinteger_selector_
 {
 public:
 
-	tslider() :
-		tscrollbar_(),
-		best_slider_length_(0),
-		minimum_value_(0),
-		minimum_value_label_(),
-		maximum_value_label_(),
-		value_labels_()
-	{
-	}
+	tslider();
 
 	/***** ***** ***** ***** layout functions ***** ***** ***** *****/
 
@@ -87,6 +80,7 @@ public:
 	 *                            the result of get_value().
 	 */
 	t_string get_value_label() const;
+
 protected:
 
 	/** Inherited from tscrollbar. */
@@ -155,6 +149,24 @@ private:
 
 	/** Inherited from tcontrol. */
 	const std::string& get_control_type() const;
+
+	/**
+	 * Handlers for keyboard input
+	 */
+	void handle_key_decrease(bool& handled);
+	void handle_key_increase(bool& handled);
+
+	/**
+	 * Signal handlers:
+	 */
+	void signal_handler_sdl_key_down(const event::tevent event
+			, bool& handled
+			, const SDLKey key);
+
+	// In this subclass, only used to grab keyboard focus -
+	// see tscrollbar class for more handling of this event.
+	void signal_handler_left_button_up(
+			const event::tevent event, bool& handled);
 };
 
 } // namespace gui2
