@@ -1,15 +1,16 @@
-/* $Id: text_box.hpp 33687 2009-03-15 16:50:42Z mordante $ */
+/* $Id: text_box.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
-   or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_TEXT_BOX_HPP_INCLUDED
@@ -113,16 +114,7 @@ private:
 class ttext_box : public ttext_
 {
 public:
-	ttext_box() :
-		ttext_(),
-		history_(),
-		text_x_offset_(0),
-		text_y_offset_(0),
-		text_height_(0),
-		dragging_(false)
-	{
-		set_wants_mouse_left_double_click();
-	}
+	ttext_box();
 
 	/** Saves the text in the widget to the history. */
 	void save_to_history() { history_.push(get_value()); }
@@ -137,25 +129,13 @@ protected:
 	/***** ***** ***** ***** layout functions ***** ***** ***** *****/
 
 	/** Inherited from tcontrol. */
-	void set_size(const tpoint& origin, const tpoint& size);
+	void place(const tpoint& origin, const tpoint& size);
 
 	/***** ***** ***** ***** Inherited ***** ***** ***** *****/
 
 	/** Inherited from tcontrol. */
 	void update_canvas();
 
-	/** Inherited from ttext_. */
-	void mouse_move(tevent_handler& event);
-
-	/** Inherited from ttext_. */
-	void mouse_left_button_down(tevent_handler& event);
-
-	/** Inherited from ttext_. */
-	void mouse_left_button_up(tevent_handler&);
-
-	/** Inherited from ttext_. */
-	void mouse_left_button_double_click(tevent_handler&);
-
 	/** Inherited from ttext_. */
 	void goto_end_of_line(const bool select = false)
 		{ goto_end_of_data(select); }
@@ -170,10 +150,7 @@ protected:
 	/** Inherited from ttext_. */
 	void delete_selection();
 
-	/** Handles the selection in a mouse down or mouse move event. */
-	void handle_mouse_selection(
-		tevent_handler& event, const bool start_selection);
-
+	void handle_mouse_selection(tpoint mouse, const bool start_selection);
 private:
 
 	/** The history text for this widget. */
@@ -249,12 +226,24 @@ private:
 	void handle_key_clear_line(SDLMod modifier, bool& handled);
 
 	/** Inherited from tcontrol. */
-	const std::string& get_control_type() const
-		{ static const std::string type = "text_box"; return type; }
+	const std::string& get_control_type() const;
 
 	/** Inherited from tcontrol. */
 	void load_config_extra();
 
+	/***** ***** ***** signal handlers ***** ****** *****/
+
+	void signal_handler_mouse_motion(
+			const event::tevent event, bool& handled, const tpoint& coordinate);
+
+	void signal_handler_left_button_down(
+			const event::tevent event, bool& handled);
+
+	void signal_handler_left_button_up(
+			const event::tevent event, bool& handled);
+
+	void signal_handler_left_button_double_click(
+			const event::tevent event, bool& handled);
 };
 
 } //namespace gui2
