@@ -1,11 +1,12 @@
-/* $Id: vertical_scrollbar.cpp 40574 2010-01-04 21:31:06Z mordante $ */
+/* $Id: vertical_scrollbar.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -17,9 +18,15 @@
 #include "gui/widgets/vertical_scrollbar.hpp"
 
 #include "gui/auxiliary/widget_definition/vertical_scrollbar.hpp"
+#include "gui/auxiliary/window_builder/vertical_scrollbar.hpp"
+#include "gui/widgets/settings.hpp"
+
+#include <boost/bind.hpp>
 
 namespace gui2 {
 
+REGISTER_WIDGET(vertical_scrollbar)
+
 unsigned tvertical_scrollbar::minimum_positioner_length() const
 {
 	boost::intrusive_ptr<const tvertical_scrollbar_definition::tresolution> conf =
