@@ -1,11 +1,12 @@
-/* $Id: widget.hpp 41319 2010-02-21 10:43:59Z mordante $ */
+/* $Id: widget.hpp 49926 2011-06-19 12:05:27Z mordante $ */
 /*
-   Copyright (C) 2007 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2007 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -18,7 +19,6 @@
 #include "gui/auxiliary/event/dispatcher.hpp"
 #include "gui/widgets/event_executor.hpp"
 #include "gui/widgets/helper.hpp"
-#include "sdl_utils.hpp"
 #include "wml_exception.hpp"
 
 #include <boost/noncopyable.hpp>
@@ -31,6 +31,12 @@ namespace gui2 {
 class tdialog;
 class twindow;
 
+namespace iterator {
+	class twalker_;
+} // namespace iterator
+
+typedef std::map< std::string, t_string > string_map;
+
 /**
  * Base class for all widgets.
  *
@@ -157,7 +163,7 @@ public:
 	/**
 	 * Tries to reduce the width of a widget.
 	 *
-	 * This function does it more agressive and should only be used when
+	 * This function does it more aggressive and should only be used when
 	 * using scrollbars and wrapping failed.
 	 *
 	 * @todo Make pure virtual.
@@ -184,7 +190,7 @@ public:
 	/**
 	 * Tries to reduce the height of a widget.
 	 *
-	 * This function does it more agressive and should only be used when
+	 * This function does it more aggressive and should only be used when
 	 * using scrollbars failed.
 	 *
 	 * @todo Make pure virtual.
@@ -329,6 +335,10 @@ public:
 	 * A window is most of the time created by a dialog, this function returns
 	 * that dialog.
 	 *
+	 * @deprecated The function was used to install callbacks to member
+	 * functions of the dialog. Once all widgets are converted to signals this
+	 * function will be removed.
+	 *
 	 * @returns                   The toplevel dialog.
 	 * @retval 0                  No toplevel window or the toplevel window is
 	 *                            not owned by a dialog.
@@ -340,6 +350,9 @@ public:
 	/** Does the widget disable easy close? */
 	virtual bool disable_click_dismiss() const = 0;
 
+	/** Creates a new walker object on the heap. */
+	virtual iterator::twalker_* create_walker() = 0;
+
 	/***** ***** ***** setters / getters for members ***** ****** *****/
 
 	twidget* parent() { return parent_; }
@@ -436,9 +449,9 @@ public:
 		debug_border_mode_ = debug_border_mode;
 	}
 
-	void set_debug_border_colour(const unsigned debug_border_colour)
+	void set_debug_border_color(const unsigned debug_border_color)
 	{
-		debug_border_colour_ = debug_border_colour;
+		debug_border_color_ = debug_border_color;
 	}
 #endif
 
@@ -534,24 +547,27 @@ public:
 private:
 
 	/**
-	 * The id is the unique name of the widget in a certain context. This is
-	 * needed for certain widgets so the engine knows which widget is which.
-	 * Eg it knows which button is pressed and thus which engine action is
-	 * connected to the button. This doesn't mean that the id is unique in a
+	 * The id is the unique name of the widget in a certain context.
+	 *
+	 * This is needed for certain widgets so the engine knows which widget is
+	 * which. Eg it knows which button is pressed and thus which engine action
+	 * is connected to the button. This doesn't mean that the id is unique in a
 	 * window, eg a listbox can have the same id for every row.
 	 */
 	std::string id_;
 
 	/**
-	 * The definition is the id of that widget class. Eg for a button it
-	 * [button_definition]id. A button can have multiple definitions which all
-	 * look different but for the engine still is a button.
+	 * The definition is the id of that widget class.
+	 *
+	 * Eg for a button it [button_definition]id. A button can have multiple
+	 * definitions which all look different but for the engine still is a
+	 * button.
 	 */
 	std::string definition_;
 
 	/**
 	 * The parent widget, if the widget has a parent it contains a pointer to
-	 * the parent, else it's set to 0.
+	 * the parent, else it's set to NULL.
 	 */
 	twidget* parent_;
 
@@ -622,7 +638,7 @@ private:
 	unsigned debug_border_mode_;
 
 	/** The color for the debug border. */
-	unsigned debug_border_colour_;
+	unsigned debug_border_color_;
 
 	void draw_debug_border(surface& frame_buffer);
 #else
@@ -748,5 +764,55 @@ T& find_widget(typename tconst_duplicator<T, twidget>::type* widget
 
 } // namespace gui2
 
+/**
+ * Registers a widget.
+ *
+ * Call this function to register a widget. Use this macro in the
+ * implementation, inside the gui2 namespace.
+ *
+ * @see @ref gui2::load_widget_definitions for more information.
+ *
+ * @note When the type is tfoo_definition, the id "foo" and no special key best
+ * use RESISTER_WIDGET(foo) instead.
+ *
+ * @param type                    Class type of the window to register.
+ * @param id                      Id of the widget
+ * @param key                     The id to load if differs from id.
+ */
+#define REGISTER_WIDGET3(                                                  \
+		  type                                                             \
+		, id                                                               \
+		, key)                                                             \
+namespace {                                                                \
+                                                                           \
+	namespace ns_##type {                                                  \
+                                                                           \
+		struct tregister_helper {                                          \
+			tregister_helper()                                             \
+			{                                                              \
+				register_widget(#id, boost::bind(                          \
+						  load_widget_definitions<type>                    \
+						, _1                                               \
+						, _2                                               \
+						, _3                                               \
+						, key));                                           \
+                                                                           \
+				register_builder_widget(#id, boost::bind(                  \
+							  build_widget<implementation::tbuilder_##id>  \
+							, _1));                                        \
+			}                                                              \
+		};                                                                 \
+                                                                           \
+		static tregister_helper register_helper;                           \
+	}                                                                      \
+}
+
+/**
+ * Wrapper for REGISTER_WIDGET3.
+ *
+ * "Calls" REGISTER_WIDGET3(tid_definition, id, _4)
+ */
+#define REGISTER_WIDGET(id) REGISTER_WIDGET3(t##id##_definition, id, _4)
+
 #endif
 
