@@ -1,11 +1,12 @@
-/* $Id: hash.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: hash.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
+   Copyright (C) 2008 - 2011 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -25,7 +26,6 @@ std::string get_salt(const std::string& hash);
 bool is_valid_hash(const std::string& hash);
 std::string encode_hash(unsigned char* input);
 std::string create_hash(const std::string& password, const std::string& salt, int iteration_count =10);
-bool password_matches_hash(const std::string& password, const std::string& hash);
 
 } // namespace util
 
