@@ -1,11 +1,12 @@
-/* $Id: intro.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: intro.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- * @file intro.cpp
+ * @file
  * Introduction sequence at start of a scenario, End-screen after end of
  * campaign.
  */
@@ -23,16 +24,10 @@
 #include "intro.hpp"
 
 #include "display.hpp"
-#include "foreach.hpp"
-#include "game_events.hpp"
 #include "gettext.hpp"
-#include "language.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
-#include "sound.hpp"
 #include "storyscreen/interface.hpp"
-#include "unit.hpp"
-#include "variable.hpp"
 
 static lg::log_domain log_engine("engine");
 #define ERR_NG LOG_STREAM(err, log_engine)
@@ -52,7 +47,7 @@ static void the_end_old(display &disp, std::string text, unsigned int duration)
 
 	SDL_Rect area = screen_area();
 	CVideo &video = disp.video();
-	SDL_FillRect(video.getSurface(),&area,0);
+	sdl_fill_rect(video.getSurface(),&area,0);
 
 	update_whole_screen();
 	disp.flip();
@@ -65,9 +60,9 @@ static void the_end_old(display &disp, std::string text, unsigned int duration)
 
 	for(size_t n = 0; n < 255; n += 5) {
 		if(n)
-			SDL_FillRect(video.getSurface(),&area,0);
+			sdl_fill_rect(video.getSurface(),&area,0);
 
-		const SDL_Color col = {n,n,n,n};
+		const SDL_Color col = create_color(n, n, n, n);
 		font::draw_text(&video,area,font_size,col,text,area.x,area.y);
 		update_rect(area);
 
@@ -99,11 +94,6 @@ void set_new_storyscreen(bool enabled)
 	LOG_NG << "enabled experimental story/endscreen code\n";
 }
 
-bool get_new_storyscreen_status()
-{
-	return use_shadowm_storyscreen;
-}
-
 void the_end(display &disp, std::string text, unsigned int duration)
 {
 	if(use_shadowm_storyscreen) {
