@@ -1,31 +1,27 @@
-/* $Id: loadscreen.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: loadscreen.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2005 - 2010 by Joeri Melis <joeri_melis@hotmail.com>
+   Copyright (C) 2005 - 2011 by Joeri Melis <joeri_melis@hotmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
-/** @file loadscreen.hpp */
+/** @file */
 
 #ifndef JM_LOADSCREEN_HPP
 #define JM_LOADSCREEN_HPP
 
 class CVideo;
 
-#include "SDL.h"
-
-#include "global.hpp"
 #include "sdl_utils.hpp"
 
-#include <string>
-
 class loadscreen {
 	public:
 		// Preferred constructor
@@ -38,17 +34,18 @@ class loadscreen {
 			dump_counters();
 		}
 
-		/** Function to display a load progress bar. */
-		void set_progress(const int percentage=0, const std::string &text="", const bool commit=true);
-		/** Function to increment the progress bar. */
-		void increment_progress(const int percentage=1, const std::string &text="", const bool commit=true);
-		/** Function to draw a blank screen. */
-		void clear_screen(const bool commit=true);
+	/**
+	 * Starts the stage with identifier @a id.
+	 */
+	static void start_stage(char const *id);
 
-		// Counters
-		int filesystem_counter;
-		int setconfig_counter;
-		int parser_counter;
+	/**
+	 * Increments the current stage for the progress bar.
+	 */
+	static void increment_progress();
+
+		/** Function to draw a blank screen. */
+		void clear_screen();
 
 		/**
 		 * A global loadscreen instance that can be used to avoid
@@ -66,7 +63,12 @@ private:
 			static global_loadscreen_manager* manager;
 			bool owns;
 		};
-	private:
+private:
+	/**
+	 * Displays a load progress bar.
+	 */
+	void draw_screen(const std::string &text);
+
 		// Prohibit default constructor
 		loadscreen();
 
@@ -81,9 +83,4 @@ private:
 		void dump_counters() const;
 };
 
-// Global accessible functions that centralize the loadscreen related work.
-void increment_filesystem_progress();
-void increment_set_config_progress();
-void increment_parser_progress();
-
 #endif
