@@ -1,12 +1,13 @@
-/* $Id: log.cpp 32825 2009-02-15 16:51:31Z mordante $ */
+/* $Id: log.cpp 48725 2011-03-02 21:40:59Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-                 2004 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+                 2004 - 2011 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,9 +15,9 @@
 */
 
 /**
- * @file log.cpp
+ * @file
  * Standard logging facilities (implementation).
- * See also the command line switches --logdomains and --log-<level>="domain".
+ * See also the command line switches --logdomains and --log-@<level@>="domain".
  */
 
 #include "global.hpp"
@@ -24,7 +25,9 @@
 #include "SDL.h"
 
 #include "log.hpp"
+#include "foreach.hpp"
 
+#include <map>
 #include <sstream>
 #include <ctime>
 
@@ -43,56 +46,73 @@ static std::ostream null_ostream(new null_streambuf);
 static int indent = 0;
 static bool timestamp = true;
 
+static std::ostream *output_stream = NULL;
+
+static std::ostream& output()
+{
+	if(output_stream) {
+		return *output_stream;
+	}
+	return std::cerr;
+}
+
 namespace lg {
 
-std::vector< lg::logd > log_domains;
+tredirect_output_setter::tredirect_output_setter(std::ostream& stream)
+	: old_stream_(output_stream)
+{
+	output_stream = &stream;
+}
+
+tredirect_output_setter::~tredirect_output_setter()
+{
+	output_stream = old_stream_;
+}
+
+typedef std::map<std::string, int> domain_map;
+static domain_map *domains;
 void timestamps(bool t) { timestamp = t; }
 
 logger err("error", 0), warn("warning", 1), info("info", 2), debug("debug", 3);
-log_domain general("general"), ai("ai"), formula_ai("formula_ai"), cache("cache"), config("config"), display("display"),
-	engine("engine"), network("network"), mp_server("server"),
-	filesystem("filesystem"), audio("audio"),
-	replay("replay"), help("help"), gui("gui"), gui_parse("gui_parse"),
-	gui_draw("gui_draw"), gui_layout("gui_layout"), gui_event("gui_event"), editor("editor"), wml("wml"),
-	mp_user_handler("user_handler");
+log_domain general("general");
 
-log_domain::log_domain(char const *name) : domain_(log_domains.size())
+log_domain::log_domain(char const *name)
+	: domain_(NULL)
 {
-	logd d = { name, 0 };
-	log_domains.push_back(d);
+	// Indirection to prevent initialization depending on link order.
+	if (!domains) domains = new domain_map;
+	domain_ = &*domains->insert(logd(name, 1)).first;
 }
 
 bool set_log_domain_severity(std::string const &name, int severity)
 {
-	std::vector< logd >::iterator
-		it = log_domains.begin(),
-		it_end = log_domains.end();
+	std::string::size_type s = name.size();
 	if (name == "all") {
-		for(; it != it_end; ++it)
-			it->severity_ = severity;
+		foreach (logd &l, *domains) {
+			l.second = severity;
+		}
+	} else if (s > 2 && name.compare(s - 2, 2, "/*") == 0) {
+		foreach (logd &l, *domains) {
+			if (l.first.compare(0, s - 1, name, 0, s - 1) == 0)
+				l.second = severity;
+		}
 	} else {
-		for(; it != it_end; ++it)
-			if (name == it->name_) break;
-		if (it == it_end)
+		domain_map::iterator it = domains->find(name);
+		if (it == domains->end())
 			return false;
-		it->severity_ = severity;
+		it->second = severity;
 	}
 	return true;
 }
 
-std::string list_logdomains()
+std::string list_logdomains(const std::string& filter)
 {
-	std::vector< logd >::iterator
-		it_begin = log_domains.begin(),
-		it_end = log_domains.end(),
-		it;
-	std::string domainlist = "";
-	for(it = it_begin; it != it_end; ++it) {
-		if (it != it_begin)
-			domainlist += ", ";
-		domainlist += it->name_;
+	std::ostringstream res;
+	foreach (logd &l, *domains) {
+		if(l.first.find(filter) != std::string::npos)
+			res << l.first << "\n";
 	}
-	return domainlist;
+	return res.str();
 }
 
 std::string get_timestamp(const time_t& t, const std::string& format) {
@@ -106,21 +126,21 @@ std::string get_timestamp(const time_t& t, const std::string& format) {
 
 std::ostream &logger::operator()(log_domain const &domain, bool show_names, bool do_indent) const
 {
-	logd const &d = log_domains[domain.domain_];
-	if (severity_ > d.severity_)
+	if (severity_ > domain.domain_->second)
 		return null_ostream;
 	else {
+		std::ostream& stream = output();
 		if(do_indent) {
 			for(int i = 0; i != indent; ++i)
-				std::cerr << "  ";
+				stream << "  ";
 			}
 		if (timestamp) {
-			std::cerr << get_timestamp(time(NULL));
+			stream << get_timestamp(time(NULL));
 		}
 		if (show_names) {
-			std::cerr << name_ << ' ' << d.name_ << ": ";
+			stream << name_ << ' ' << domain.domain_->first << ": ";
 		}
-		return std::cerr;
+		return stream;
 	}
 }
 
