@@ -1,12 +1,13 @@
-/* $Id: log.hpp 32075 2009-01-12 00:14:25Z shadowmaster $ */
+/* $Id: log.hpp 48725 2011-03-02 21:40:59Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-                 2004 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+                 2004 - 2011 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,39 +15,68 @@
 */
 
 /**
- * @file log.hpp
+ * @file
  * Standard logging facilities (interface).
  */
 
 #ifndef LOG_HPP_INCLUDED
 #define LOG_HPP_INCLUDED
 
-#include <iosfwd>
-#include <iostream>
+#ifndef __func__
+ #ifdef __FUNCTION__
+  #define __func__ __FUNCTION__
+ #endif
+#endif
+
+#include <iostream> // needed else all files including log.hpp need to do it.
+#include <sstream> // as above. iostream (actually, iosfwd) declares stringstream as an incomplete type, but does not define it
 #include <string>
-#include <vector>
+#include <utility>
 
 namespace lg {
 
-class logger;
+/**
+ * Helper class to redirect the output of the logger in a certain scope.
+ *
+ * The main usage of the redirection is for the unit tests to validate the
+ * ourput on the logger with the expected output.
+ */
+class tredirect_output_setter
+{
+public:
 
-struct logd {
-	char const *name_;
-	int severity_;
+	/**
+	 * Constructor.
+	 *
+	 * @param stream              The stream to direct the output to.
+	 */
+	explicit tredirect_output_setter(std::ostream& stream);
+
+	~tredirect_output_setter();
+
+private:
+
+	/**
+	 * The previously set redirection.
+	 *
+	 * This value is stored here to be restored in this destructor.
+	 */
+	std::ostream* old_stream_;
 };
 
+class logger;
+
+typedef std::pair<const std::string, int> logd;
+
 class log_domain {
-	int domain_;
+	logd *domain_;
 public:
 	log_domain(char const *name);
 	friend class logger;
 };
 
-//exposed to make inlining possible
-extern std::vector<logd> log_domains;
-
 bool set_log_domain_severity(std::string const &name, int severity);
-std::string list_logdomains();
+std::string list_logdomains(const std::string& filter);
 
 class logger {
 	char const *name_;
@@ -58,8 +88,7 @@ public:
 
 	bool dont_log(log_domain const &domain) const
 	{
-		logd const &d = log_domains[domain.domain_];
-		return severity_ > d.severity_;
+		return severity_ > domain.domain_->second;
 	}
 };
 
@@ -67,9 +96,7 @@ void timestamps(bool);
 std::string get_timestamp(const time_t& t, const std::string& format="%Y%m%d %H:%M:%S ");
 
 extern logger err, warn, info, debug;
-extern log_domain general, ai, formula_ai, cache, config, display, engine, network, mp_server,
-	filesystem, audio, replay, help, gui, gui_parse, gui_layout, gui_draw,
-	gui_event, editor, wml, mp_user_handler;
+extern log_domain general;
 
 class scope_logger
 {
@@ -115,38 +142,11 @@ extern std::stringstream wml_error;
 } // namespace lg
 
 #define log_scope(a) lg::scope_logger scope_logging_object__(lg::general, a);
-#define log_scope2(a,b) lg::scope_logger scope_logging_object__(lg::a, b);
+#define log_scope2(a,b) lg::scope_logger scope_logging_object__(a, b);
 
-#define LOG_STREAM(a, b) if (lg::a.dont_log(lg::b)) ; else lg::a(lg::b)
+#define LOG_STREAM(a, b) if (lg::a.dont_log(b)) ; else lg::a(b)
 
 // When using log_scope/log_scope2 it is nice to have all output indented.
-#define LOG_STREAM_INDENT(a,b) if (lg::a.dont_log(lg::b)) ; else lg::a(lg::b, true, true)
-
-
-// Define a list of standard loggers.
-#define DBG_GUI LOG_STREAM_INDENT(debug, gui)
-#define LOG_GUI LOG_STREAM_INDENT(info, gui)
-#define WRN_GUI LOG_STREAM_INDENT(warn, gui)
-#define ERR_GUI LOG_STREAM_INDENT(err, gui)
-
-#define DBG_G_P LOG_STREAM_INDENT(debug, gui_parse)
-#define LOG_G_P LOG_STREAM_INDENT(info, gui_parse)
-#define WRN_G_P LOG_STREAM_INDENT(warn, gui_parse)
-#define ERR_G_P LOG_STREAM_INDENT(err, gui_parse)
-
-#define DBG_G_L LOG_STREAM_INDENT(debug, gui_layout)
-#define LOG_G_L LOG_STREAM_INDENT(info, gui_layout)
-#define WRN_G_L LOG_STREAM_INDENT(warn, gui_layout)
-#define ERR_G_L LOG_STREAM_INDENT(err, gui_layout)
-
-#define DBG_G_D LOG_STREAM_INDENT(debug, gui_draw)
-#define LOG_G_D LOG_STREAM_INDENT(info, gui_draw)
-#define WRN_G_D LOG_STREAM_INDENT(warn, gui_draw)
-#define ERR_G_D LOG_STREAM_INDENT(err, gui_draw)
-
-#define DBG_G_E LOG_STREAM_INDENT(debug, gui_event)
-#define LOG_G_E LOG_STREAM_INDENT(info, gui_event)
-#define WRN_G_E LOG_STREAM_INDENT(warn, gui_event)
-#define ERR_G_E LOG_STREAM_INDENT(err, gui_event)
+#define LOG_STREAM_INDENT(a,b) if (lg::a.dont_log(b)) ; else lg::a(b, true, true)
 
 #endif
