@@ -1,11 +1,12 @@
-/* $Id: map.cpp 40885 2010-01-26 20:33:20Z mordante $ */
+/* $Id: map.cpp 51304 2011-09-26 20:27:09Z mordante $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- * @file map.cpp
+ * @file
  * Routines related to game-maps, terrain, locations, directions. etc.
  */
 
@@ -81,23 +82,53 @@ const t_translation::t_list& gamemap::underlying_union_terrain(t_translation::t_
 	}
 }
 
+
+
 std::string gamemap::get_terrain_string(const t_translation::t_terrain& terrain) const
 {
-	std::stringstream ss;
-	const std::string& name = get_terrain_info(terrain).description();
+	std::string str =
+		get_terrain_info(terrain).description();
+
+	str += get_underlying_terrain_string(terrain);
+
+	return str;
+}
+
+std::string gamemap::get_terrain_editor_string(const t_translation::t_terrain& terrain) const
+{
+	std::string str =
+		get_terrain_info(terrain).editor_name();
+	const std::string desc =
+		get_terrain_info(terrain).description();
+
+	if(str != desc) {
+		str += "/";
+		str += desc;
+	}
+
+	str += get_underlying_terrain_string(terrain);
+
+	return str;
+}
+
+std::string gamemap::get_underlying_terrain_string(const t_translation::t_terrain& terrain) const
+{
+	std::string str;
+
 	const t_translation::t_list& underlying = underlying_union_terrain(terrain);
 	assert(!underlying.empty());
-	ss << name;
+
 	if(underlying.size() > 1 || underlying[0] != terrain) {
-		ss << " (";
+		str += " (";
         t_translation::t_list::const_iterator i = underlying.begin();
-        ss << get_terrain_info(*i).name();
+        str += get_terrain_info(*i).name();
         while (++i != underlying.end()) {
-            ss << "," << get_terrain_info(*i).name();
+			str += ", " + get_terrain_info(*i).name();
         }
-		ss << ")";
+		str += ")";
 	}
-	return ss.str();
+
+	return str;
 }
 
 void gamemap::write_terrain(const map_location &loc, config& cfg) const
@@ -166,7 +197,7 @@ void gamemap::read(const std::string& data)
 	config header;
 	::read(header, header_str);
 
-	border_size_ = lexical_cast_default<int>(header["border_size"], 0);
+	border_size_ = header["border_size"];
 	const std::string usage = header["usage"];
 
 	utils::string_map symbols;
@@ -185,14 +216,13 @@ void gamemap::read(const std::string& data)
 		symbols["border_size_val"] = "0";
 		VALIDATE(border_size_ == 0, vgettext(msg.c_str(), symbols));
 	} else if(usage == "") {
-		throw incorrect_map_format_exception("Map has a header but no usage");
+		throw incorrect_map_format_error("Map has a header but no usage");
 	} else {
 		std::string msg = "Map has a header but an unknown usage:" + usage;
-		throw incorrect_map_format_exception(msg.c_str());
+		throw incorrect_map_format_error(msg.c_str());
 	}
 
-	/* The third parameter is required for MSVC++ 6.0 */
-	const std::string& map = std::string(data, header_offset + 2, std::string::npos);
+	const std::string& map = std::string(data, header_offset + 2);
 
 	try {
 		tiles_ = t_translation::read_game_map(map, starting_positions);
@@ -200,7 +230,7 @@ void gamemap::read(const std::string& data)
 	} catch(t_translation::error& e) {
 		// We re-throw the error but as map error.
 		// Since all codepaths test for this, it's the least work.
-		throw incorrect_map_format_exception(e.message);
+		throw incorrect_map_format_error(e.message);
 	}
 
 	// Convert the starting positions to the array
@@ -217,7 +247,7 @@ void gamemap::read(const std::string& data)
 			ss << "Starting position " << itor->first << " out of range\n";
 			ERR_CF << ss.str();
 			ss << "The map cannot be loaded.";
-			throw incorrect_map_format_exception(ss.str().c_str());
+			throw incorrect_map_format_error(ss.str().c_str());
 		}
 
 		// Add to the starting position array
@@ -237,11 +267,11 @@ void gamemap::read(const std::string& data)
 			if(tcodeToTerrain_.count(tiles_[x][y]) == 0) {
 				if(!try_merge_terrains(tiles_[x][y])) {
 					std::stringstream ss;
-					ss << "Illegal character in map: (" << t_translation::write_terrain_code(tiles_[x][y])
+					ss << "Illegal tile in map: (" << t_translation::write_terrain_code(tiles_[x][y])
 						   << ") '" << tiles_[x][y] << "'\n";
 					ERR_CF << ss.str();
 					ss << "The map cannot be loaded.";
-					throw incorrect_map_format_exception(ss.str().c_str());
+					throw incorrect_map_format_error(ss.str().c_str());
 				}
 			}
 
@@ -257,23 +287,23 @@ void gamemap::read(const std::string& data)
 
 std::string gamemap::write() const
 {
-	std::map<int, t_translation::coordinate> starting_positions = std::map<int, t_translation::coordinate>();
-
 	// Convert the starting positions to a map
-	for(int i = 0; i < MAX_PLAYERS+1; ++i) {
-	if(on_board(startingPositions_[i])) {
-			const struct t_translation::coordinate position =
-				{startingPositions_[i].x + border_size_, startingPositions_[i].y + border_size_};
-
-			 starting_positions.insert(std::pair<int, t_translation::coordinate>(i, position));
-		}
+	std::map<int, t_translation::coordinate> starting_positions;
+	for (int i = 0; i < MAX_PLAYERS + 1; ++i)
+	{
+		if (!on_board(startingPositions_[i])) continue;
+		t_translation::coordinate position(
+				  startingPositions_[i].x + border_size_
+				, startingPositions_[i].y + border_size_);
+		starting_positions[i] = position;
 	}
 
 	// Let the low level convertor do the conversion
-	const std::string& data = t_translation::write_game_map(tiles_, starting_positions);
-	const std::string& header = "border_size=" + lexical_cast<std::string>(border_size_)
-		+ "\nusage=" + (usage_ == IS_MAP ? "map" : "mask");
-	return header + "\n\n" + data;
+	std::ostringstream s;
+	s << "border_size=" << border_size_ << "\nusage="
+		<< (usage_ == IS_MAP ? "map" : "mask") << "\n\n"
+		<< t_translation::write_game_map(tiles_, starting_positions);
+	return s.str();
 }
 
 void gamemap::overlay(const gamemap& m, const config& rules_cfg, int xpos, int ypos, bool border)
@@ -351,8 +381,8 @@ void gamemap::overlay(const gamemap& m, const config& rules_cfg, int xpos, int y
 					new_terrain = terrain[0];
 				}
 
-				if(!utils::string_bool(cfg["use_old"])) {
-					set_terrain(map_location(x2,y2), new_terrain, mode, utils::string_bool(cfg["replace_if_failed"]));
+				if (!cfg["use_old"].to_bool()) {
+					set_terrain(map_location(x2, y2), new_terrain, mode, cfg["replace_if_failed"].to_bool());
 				}
 
 			} else {
@@ -384,14 +414,14 @@ t_translation::t_terrain gamemap::get_terrain(const map_location& loc) const
 
 	// If not on the board, decide based on what surrounding terrain is
 	t_translation::t_terrain items[6];
-	int nitems = 0;
+	int number_of_items = 0;
 
 	map_location adj[6];
 	get_adjacent_tiles(loc,adj);
 	for(int n = 0; n != 6; ++n) {
 		if(on_board(adj[n])) {
-			items[nitems] = tiles_[adj[n].x][adj[n].y];
-			++nitems;
+			items[number_of_items] = tiles_[adj[n].x][adj[n].y];
+			++number_of_items;
 		} else {
 			// If the terrain is off map but already in the border cache,
 			// this will be used to determine the terrain.
@@ -409,8 +439,8 @@ t_translation::t_terrain gamemap::get_terrain(const map_location& loc) const
 			if(itor != borderCache_.end() &&
 					itor->second != t_translation::NONE_TERRAIN)  {
 
-				items[nitems] = itor->second;
-				++nitems;
+				items[number_of_items] = itor->second;
+				++number_of_items;
 			}
 		}
 
@@ -420,9 +450,9 @@ t_translation::t_terrain gamemap::get_terrain(const map_location& loc) const
 	// and see which one is the most common, and use it.
 	t_translation::t_terrain used_terrain;
 	int terrain_count = 0;
-	for(int i = 0; i != nitems; ++i) {
+	for(int i = 0; i != number_of_items; ++i) {
 		if(items[i] != used_terrain && !is_village(items[i]) && !is_keep(items[i])) {
-			const int c = std::count(items+i+1,items+nitems,items[i]) + 1;
+			const int c = std::count(items+i+1,items+number_of_items,items[i]) + 1;
 			if(c > terrain_count) {
 				used_terrain = items[i];
 				terrain_count = c;
