@@ -1,39 +1,28 @@
-/* $Id: map.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: map.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
-/** @file map.hpp  */
+/** @file */
 
 #ifndef MAP_H_INCLUDED
 #define MAP_H_INCLUDED
 
 class config;
-class gamestatus;
-class unit;
-class unit_map;
-class vconfig;
 
 #include "terrain.hpp"
 #include "map_location.hpp"
 
-#include "serialization/string_utils.hpp"
-
-#include <iostream>
-#include <map>
-#include <string>
-#include <vector>
-#include <set>
-
 #define MAX_MAP_AREA	65536
 
 /**
@@ -58,12 +47,8 @@ public:
 	 * Get a formatted terrain name -- terrain (underlying, terrains)
 	 */
 	std::string get_terrain_string(const t_translation::t_terrain& terrain) const;
-
-	/** Throws exception if the map file is not in the correct format. */
-	struct incorrect_format_exception {
-		incorrect_format_exception(const char* msg) : msg_(msg) {}
-		const char* const msg_;
-	};
+	std::string get_terrain_editor_string(const t_translation::t_terrain& terrain) const;
+	std::string get_underlying_terrain_string(const t_translation::t_terrain& terrain) const;
 
 	const t_translation::t_list& underlying_mvt_terrain(const map_location& loc) const
 		{ return underlying_mvt_terrain(get_terrain(loc)); }
@@ -73,6 +58,8 @@ public:
 		{ return underlying_union_terrain(get_terrain(loc)); }
 	std::string get_terrain_string(const map_location& loc) const
 		{ return get_terrain_string(get_terrain(loc)); }
+	std::string get_terrain_editor_string(const map_location& loc) const
+		{ return get_terrain_editor_string(get_terrain(loc)); }
 	bool is_village(t_translation::t_terrain terrain) const
 		{ return get_terrain_info(terrain).is_village(); }
 	int gives_healing(t_translation::t_terrain terrain) const
@@ -104,7 +91,7 @@ public:
 	enum tmerge_mode {
 		BASE,
 		OVERLAY,
-        BOTH
+		BOTH
 		};
 
 	/**
@@ -114,10 +101,10 @@ public:
 	 * hex on the map.  Starting locations are represented by numbers, and will
 	 * be of type keep.
 	 *
-	 * @param cfg                 The game config.
-	 * @param data			      The mapdata to load.
+	 * @param cfg the game config.
+	 * @param data the map data to load.
 	 */
-	gamemap(const config& terrain_cfg, const std::string& data); //throw(incorrect_format_exception)
+	gamemap(const config &cfg, const std::string &data); //throw(incorrect_map_format_error)
 
 	virtual ~gamemap();
 
@@ -174,7 +161,7 @@ public:
 	 * Tell if a location is on the map.
 	 *
 	 * Should be called before indexing using [].
-	 * @todo inline for performace? -- Ilor
+	 * @todo inline for performance? -- Ilor
 	 */
 	bool on_board(const map_location& loc) const;
 	bool on_board_with_border(const map_location& loc) const;
@@ -278,7 +265,7 @@ private:
 	/**
 	 * Tries to find out if "terrain" can be created by combining two existing
 	 * terrains Will add the resulting terrain to the terrain list if
-	 * successfull
+	 * successful
 	 */
 	bool try_merge_terrains(const t_translation::t_terrain terrain);
 
