@@ -1,11 +1,12 @@
-/* $Id: map_location.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: map_location.cpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- * @file map_location.cpp
+ * @file
  * Routines related to game-maps, terrain, locations, directions. etc.
  */
 
@@ -129,11 +130,8 @@ map_location::map_location(const config& cfg, const variable_set *variables) :
 
 void map_location::write(config& cfg) const
 {
-	char buf[50];
-	snprintf(buf,sizeof(buf),"%d",x+1);
-	cfg["x"] = buf;
-	snprintf(buf,sizeof(buf),"%d",y+1);
-	cfg["y"] = buf;
+	cfg["x"] = x + 1;
+	cfg["y"] = y + 1;
 }
 
 map_location map_location::legacy_negation() const
@@ -182,11 +180,6 @@ map_location& map_location::vector_sum_assign(const map_location &a)
 	return *this;
 }
 
-map_location map_location::vector_difference(const map_location &a) const
-{
-	return vector_sum(a.vector_negation());
-}
-
 map_location& map_location::vector_difference_assign(const map_location &a)
 {
 	return vector_sum_assign(a.vector_negation());
@@ -272,7 +265,7 @@ bool map_location::matches_range(const std::string& xloc, const std::string &ylo
 	if(!xloc.empty()) {
 		const std::string::const_iterator dash =
 		             std::find(xloc.begin(),xloc.end(),'-');
-		if(dash != xloc.end()) {
+		if(dash != xloc.begin() && dash != xloc.end()) {
 			const std::string beg(xloc.begin(),dash);
 			const std::string end(dash+1,xloc.end());
 
@@ -291,7 +284,7 @@ bool map_location::matches_range(const std::string& xloc, const std::string &ylo
 		const std::string::const_iterator dash =
 		             std::find(yloc.begin(),yloc.end(),'-');
 
-		if(dash != yloc.end()) {
+		if(dash != yloc.begin() && dash != yloc.end()) {
 			const std::string beg(yloc.begin(),dash);
 			const std::string end(dash+1,yloc.end());
 
