@@ -1,11 +1,12 @@
-/* $Id: mapgen.cpp 33770 2009-03-17 19:41:19Z jhinrichs $ */
+/* $Id: mapgen.cpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,26 +14,30 @@
 */
 
 /**
- * @file mapgen.cpp
+ * @file
  * Map-generator, with standalone testprogram.
  */
 
 #include "global.hpp"
 
+#include "foreach.hpp"
 #include "gettext.hpp"
 #include "language.hpp"
 #include "log.hpp"
 #include "map.hpp"
 #include "mapgen.hpp"
-#include "pathfind.hpp"
+#include "pathfind/pathfind.hpp"
+#include "pathutils.hpp"
 #include "race.hpp"
+#include "util.hpp"
 #include "wml_exception.hpp"
 #include "formula_string_utils.hpp"
+#include "SDL.h"
 
 
-#define ERR_CF LOG_STREAM(err, config)
-#define ERR_NG LOG_STREAM(err, engine)
-#define LOG_NG LOG_STREAM(info, engine)
+static lg::log_domain log_engine("engine");
+#define ERR_NG LOG_STREAM(err, log_engine)
+#define LOG_NG LOG_STREAM(info, log_engine)
 
 config map_generator::create_scenario(const std::vector<std::string>& args)
 {
@@ -49,7 +54,7 @@ typedef map_location location;
 /**
  * Generate a height-map.
  *
- * Basically we generate alot of hills, each hill being centered at a certain
+ * Basically we generate a lot of hills, each hill being centered at a certain
  * point, with a certain radius - being a half sphere.  Hills are combined
  * additively to form a bumpy surface.  The size of each hill varies randomly
  * from 1-hill_size.  We generate 'iterations' hills in total.  The range of
@@ -357,20 +362,20 @@ namespace {
  * Calculates the cost of building a road over terrain.  For use in the
  * a_star_search algorithm.
  */
-struct road_path_calculator : cost_calculator
+struct road_path_calculator : pathfind::cost_calculator
 {
 	road_path_calculator(const terrain_map& terrain, const config& cfg) :
     	calls(0),
 		map_(terrain),
 		cfg_(cfg),
 		// Find out how windy roads should be.
-		windiness_(std::max<int>(1,atoi(cfg["road_windiness"].c_str()))),
+		windiness_(std::max<int>(1, cfg["road_windiness"].to_int())),
 		seed_(rand()),
 		cache_()
 	{
 	}
 
-	virtual double cost(const location& src, const location& loc, const double so_far) const;
+	virtual double cost(const location& loc, const double so_far) const;
 
 	mutable int calls;
 private:
@@ -381,14 +386,14 @@ private:
 	mutable std::map<t_translation::t_terrain, double> cache_;
 };
 
-double road_path_calculator::cost(const location& /*src*/, const location& loc,
+double road_path_calculator::cost(const location& loc,
 	const double /*so_far*/) const
 {
 	++calls;
 	if (loc.x < 0 || loc.y < 0 || loc.x >= static_cast<long>(map_.size()) ||
 			loc.y >= static_cast<long>(map_.front().size())) {
 
-		return (getNoPathValue());
+		return (pathfind::cost_calculator::getNoPathValue());
 	}
 
 	// We multiply the cost by a random amount,
@@ -421,10 +426,9 @@ double road_path_calculator::cost(const location& /*src*/, const location& loc,
 
 	static std::string terrain;
 	terrain = t_translation::write_terrain_code(c);
-	const config* const child = cfg_.find_child("road_cost","terrain",terrain);
 	double res = getNoPathValue();
-	if(child != NULL) {
-		res = double(atof((*child)["cost"].c_str()));
+	if (const config &child = cfg_.find_child("road_cost", "terrain", terrain)) {
+		res = child["cost"].to_double();
 	}
 
 	cache_.insert(std::pair<t_translation::t_terrain, double>(c,res));
@@ -534,7 +538,7 @@ static map_location place_village(const t_translation::t_map& map,
 	std::set<map_location> locs;
 	get_tiles_radius(loc,radius,locs);
 	map_location best_loc;
-	size_t best_rating = 0;
+	int best_rating = 0;
 	for(std::set<map_location>::const_iterator i = locs.begin();
 			i != locs.end(); ++i) {
 
@@ -546,18 +550,17 @@ static map_location place_village(const t_translation::t_map& map,
 
 		const t_translation::t_terrain t = map[i->x][i->y];
 		const std::string str = t_translation::write_terrain_code(t);
-		const config* const child = cfg.find_child("village","terrain",str);
-		if(child != NULL) {
+		if (const config &child = cfg.find_child("village", "terrain", str)) {
 			tcode_list_cache::iterator l = adj_liked_cache.find(t);
 			t_translation::t_list *adjacent_liked;
 			if (l != adj_liked_cache.end()) {
 				adjacent_liked = &(l->second);
 			} else {
-				adj_liked_cache[t] = t_translation::read_list((*child)["adjacent_liked"]);
+				adj_liked_cache[t] = t_translation::read_list(child["adjacent_liked"]);
 				adjacent_liked = &(adj_liked_cache[t]);
 			}
 
-			size_t rating = atoi((*child)["rating"].c_str());
+			int rating = child["rating"];
 			map_location adj[6];
 			get_adjacent_tiles(map_location(i->x,i->y),adj);
 			for(size_t n = 0; n != 6; ++n) {
@@ -612,6 +615,36 @@ static std::string generate_name(const unit_race& name_generator, const std::str
 	return "";
 }
 
+// "flood fill" a tile name to adjacent tiles of certain terrain
+static void flood_name(const map_location& start, const std::string& name, std::map<map_location,std::string>& tile_names,
+	const t_translation::t_match& tile_types, const terrain_map& terrain,
+	unsigned width, unsigned height,
+	size_t label_count, std::map<map_location,std::string>* labels, const std::string& full_name) {
+	map_location adj[6];
+	get_adjacent_tiles(start,adj);
+	size_t n;
+	//if adjacent tiles are tiles and unnamed, name them
+	for (n = 0; n < 6; n++) {
+		//we do not care for tiles outside the middle part
+		//cast to unsigned to skip x < 0 || y < 0 as well.
+		if (unsigned(adj[n].x) >= width / 3 || unsigned(adj[n].y) >= height / 3) {
+			continue;
+		}
+
+		const t_translation::t_terrain terr = terrain[adj[n].x + (width / 3)][adj[n].y + (height / 3)];
+		const location loc(adj[n].x, adj[n].y);
+		if((t_translation::terrain_matches(terr, tile_types)) && (tile_names.find(loc) == tile_names.end())) {
+			tile_names.insert(std::pair<location, std::string>(loc, name));
+			//labeling decision: this is result of trial and error on what looks best in game
+			if (label_count % 6 == 0) { //ensure that labels do not occur more often than every 6 recursions
+				labels->insert(std::pair<map_location, std::string>(loc, full_name));
+				label_count++; //ensure that no adjacent tiles get labeled
+			}
+			flood_name(adj[n], name, tile_names, tile_types, terrain, width, height, label_count++, labels, full_name);
+		}
+	}
+}
+
 namespace {
 
 // the configuration file should contain a number of [height] tags:
@@ -636,7 +669,7 @@ private:
 };
 
 terrain_height_mapper::terrain_height_mapper(const config& cfg) :
-	terrain_height(lexical_cast_default<int>(cfg["height"],0)),
+	terrain_height(cfg["height"]),
 	to(t_translation::GRASS_LAND)
 {
 	const std::string& terrain = cfg["terrain"];
@@ -674,10 +707,10 @@ terrain_converter::terrain_converter(const config& cfg) : min_temp(-1),
 	  from(t_translation::read_list(cfg["from"])),
 	  to(t_translation::NONE_TERRAIN)
 {
-	min_temp = lexical_cast_default<int>(cfg["min_temperature"],-100000);
-	max_temp = lexical_cast_default<int>(cfg["max_temperature"],100000);
-	min_height = lexical_cast_default<int>(cfg["min_height"],-100000);
-	max_height = lexical_cast_default<int>(cfg["max_height"],100000);
+	min_temp = cfg["min_temperature"].to_int(-100000);
+	max_temp = cfg["max_temperature"].to_int(100000);
+	min_height = cfg["min_height"].to_int(-100000);
+	max_height = cfg["max_height"].to_int(100000);
 
 	const std::string& to_str = cfg["to"];
 	if(to_str != "") {
@@ -734,20 +767,18 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	LOG_NG << "done generating height map...\n";
 	LOG_NG << (SDL_GetTicks() - ticks) << "\n"; ticks = SDL_GetTicks();
 
-	const config* const names_info = cfg.child("naming");
-	config naming;
-	if(names_info != NULL) {
-		naming = *names_info;
-	}
+	config naming = cfg.child_or_empty("naming");
+	// HACK: dummy names to satisfy unit_race requirements
+	naming["id"] = "village_naming";
+	naming["plural_name"] = "villages";
 
 	// Make a dummy race for generating names
-	unit_race name_generator(naming);
+	const unit_race name_generator(naming);
 
 	std::vector<terrain_height_mapper> height_conversion;
 
-	const config::child_list& height_cfg = cfg.get_children("height");
-	for(config::child_list::const_iterator h = height_cfg.begin(); h != height_cfg.end(); ++h) {
-		height_conversion.push_back(terrain_height_mapper(**h));
+	foreach (const config &h, cfg.child_range("height")) {
+		height_conversion.push_back(terrain_height_mapper(h));
 	}
 
 	terrain_map terrain(width, t_translation::t_list(height, grassland));
@@ -784,15 +815,16 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	// We will also attempt to source a river from each lake.
 	std::set<location> lake_locs;
 
-	std::map<location,std::string> river_names, lake_names;
+	std::map<location, std::string> river_names, lake_names, road_names, bridge_names, mountain_names, forest_names, swamp_names;
 
 	const size_t nlakes = max_lakes > 0 ? (rand()%max_lakes) : 0;
 	for(size_t lake = 0; lake != nlakes; ++lake) {
 		for(int tries = 0; tries != 100; ++tries) {
 			const int x = rand()%width;
 			const int y = rand()%height;
-			if(heights[x][y] > atoi(cfg["min_lake_height"].c_str())) {
-				const std::vector<location> river = generate_river(heights,terrain,x,y,atoi(cfg["river_frequency"].c_str()));
+			if (heights[x][y] > cfg["min_lake_height"].to_int()) {
+				std::vector<location> river = generate_river(heights,
+					terrain, x, y, cfg["river_frequency"]);
 
 				if(river.empty() == false && labels != NULL) {
 					std::string base_name;
@@ -816,7 +848,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 				LOG_NG << "generating lake...\n";
 				std::set<location> locs;
-				const bool res = generate_lake(terrain,x,y,atoi(cfg["lake_size"].c_str()),locs);
+				bool res = generate_lake(terrain, x, y, cfg["lake_size"], locs);
 				if(res && labels != NULL) {
 					bool touches_other_lake = false;
 
@@ -850,7 +882,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 					for(i = locs.begin(); i != locs.end(); ++i) {
 						const location loc(i->x-width/3,i->y-height/3);
-						lake_names.insert(std::pair<location,std::string>(*i,base_name));
+						lake_names.insert(std::pair<location, std::string>(loc, base_name));
 					}
 				}
 
@@ -873,16 +905,15 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	 * more interesting types than the default.
 	 */
 	const height_map temperature_map = generate_height_map(width,height,
-	                                                       (atoi(cfg["temperature_iterations"].c_str())*width*height)/default_dimensions,
-														   atoi(cfg["temperature_size"].c_str()),0,0);
+		cfg["temperature_iterations"].to_int() * width * height / default_dimensions,
+		cfg["temperature_size"], 0, 0);
 
 	LOG_NG << "generated temperature map...\n";
 	LOG_NG << (SDL_GetTicks() - ticks) << "\n"; ticks = SDL_GetTicks();
 
 	std::vector<terrain_converter> converters;
-	const config::child_list& converter_items = cfg.get_children("convert");
-	for(config::child_list::const_iterator cv = converter_items.begin(); cv != converter_items.end(); ++cv) {
-		converters.push_back(terrain_converter(**cv));
+	foreach (const config &cv, cfg.child_range("convert")) {
+		converters.push_back(terrain_converter(cv));
 	}
 
 	LOG_NG << "created terrain converters\n";
@@ -915,10 +946,10 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	LOG_NG << "placing castles...\n";
 
 	/** Try to find configuration for castles. */
-	const config* const castle_config = cfg.child("castle");
-	if(castle_config == NULL) {
+	const config &castle_config = cfg.child("castle");
+	if (!castle_config) {
 		LOG_NG << "Could not find castle configuration\n";
-		return "";
+		return std::string();
 	}
 
 	/*
@@ -926,7 +957,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	 * list of terrains that the castle may appear on.
 	 */
 	const t_translation::t_list list =
-		t_translation::read_list((*castle_config)["valid_terrain"]);
+		t_translation::read_list(castle_config["valid_terrain"]);
 
 	const is_valid_terrain terrain_tester(terrain, list);
 
@@ -949,7 +980,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 		const int min_y = height/3 + 3;
 		const int max_x = (width/3)*2 - 4;
 		const int max_y = (height/3)*2 - 4;
-		const size_t min_distance = atoi((*castle_config)["min_distance"].c_str());
+		int min_distance = castle_config["min_distance"];
 
 		location best_loc;
 		int best_ranking = 0;
@@ -972,9 +1003,8 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 			}
 		}
 		if(best_ranking == 0) {
-			//FIXME: Make this error message translatable (not possible atm due to string freeze)
 			ERR_NG << "No castle location found, aborting.\n";
-			std::string error = "No valid castle location found. Too many or too few mountain hexes? (please check the 'max hill size' parameter)";
+			std::string error = _("No valid castle location found. Too many or too few mountain hexes? (please check the 'max hill size' parameter)");
 			throw mapgen_exception(error);
 		}
 		assert(std::find(castles.begin(), castles.end(), best_loc) == castles.end());
@@ -989,7 +1019,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	// Place roads.
 	// We select two tiles at random locations on the borders
 	// of the map, and try to build roads between them.
-	size_t nroads = atoi(cfg["roads"].c_str());
+	int nroads = cfg["roads"];
 	if(roads_between_castles) {
 		nroads += castles.size()*castles.size();
 	}
@@ -997,7 +1027,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	std::set<location> bridges;
 
 	road_path_calculator calc(terrain,cfg);
-	for(size_t road = 0; road != nroads; ++road) {
+	for (int road = 0; road != nroads; ++road) {
 		log_scope("creating road");
 
 		/*
@@ -1013,7 +1043,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 		dst.x += width/3 - 1;
 		dst.y += height/3 - 1;
 
-		if(roads_between_castles && road < castles.size()*castles.size()) {
+		if (roads_between_castles && road < int(castles.size() * castles.size())) {
 			const size_t src_castle = road/castles.size();
 			const size_t dst_castle = road%castles.size();
 			if(src_castle >= dst_castle) {
@@ -1029,14 +1059,15 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 			continue;
 		}
 
-		if (calc.cost(src,src, 0.0) >= 1000.0 || calc.cost(src,dst, 0.0) >= 1000.0) {
+		if (calc.cost(src, 0.0) >= 1000.0 || calc.cost(dst, 0.0) >= 1000.0) {
 			continue;
 		}
 
 		// Search a path out for the road
-		const paths::route rt = a_star_search(src, dst, 10000.0, &calc, width, height);
+		pathfind::plain_route rt = pathfind::a_star_search(src, dst, 10000.0, &calc, width, height);
 
-		const std::string& name = generate_name(name_generator,"road_name");
+		std::string road_base_name;
+		const std::string& name = generate_name(name_generator, "road_name", &road_base_name);
 		const int name_frequency = 20;
 		int name_count = 0;
 
@@ -1058,9 +1089,9 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 			// Find the configuration which tells us
 			// what to convert this tile to, to make it into a road.
-			const config* const child = cfg.find_child("road_cost", "terrain",
-					t_translation::write_terrain_code(terrain[x][y]));
-			if(child != NULL) {
+			if (const config &child = cfg.find_child("road_cost", "terrain",
+					t_translation::write_terrain_code(terrain[x][y])))
+			{
 				// Convert to bridge means that we want to convert
 				// depending upon the direction the road is going.
 				// Typically it will be in a format like,
@@ -1070,7 +1101,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 				// '\' will be used if the road is going south east-north west
 				// The terrain will be left unchanged otherwise
 				// (if there is no clear direction).
-				const std::string& convert_to_bridge = (*child)["convert_to_bridge"];
+				const std::string &convert_to_bridge = child["convert_to_bridge"];
 				if(convert_to_bridge.empty() == false) {
 					if(step == rt.steps.begin() || step+1 == rt.steps.end())
 						continue;
@@ -1100,9 +1131,11 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 					if(labels != NULL && on_bridge == false) {
 						on_bridge = true;
-						const std::string& name = generate_name(name_generator,"bridge_name");
-						const location loc(x-width/3,y-height/3);
+						std::string bridge_base_name;
+						const std::string& name = generate_name(name_generator, "bridge_name", &bridge_base_name);
+						const location loc(x - width / 3, y-height/3);
 						labels->insert(std::pair<map_location,std::string>(loc,name));
+						bridge_names.insert(std::pair<location,std::string>(loc, bridge_base_name)); //add to use for village naming
 						bridges.insert(loc);
 					}
 
@@ -1119,7 +1152,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 				}
 
 				// Just a plain terrain substitution for a road
-				const std::string& convert_to = (*child)["convert_to"];
+				const std::string &convert_to = child["convert_to"];
 				if(convert_to.empty() == false) {
 					const t_translation::t_terrain letter =
 						t_translation::read_terrain_code(convert_to);
@@ -1129,6 +1162,8 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 					}
 
 					terrain[x][y] = letter;
+					const location loc(x - width / 3, y - height / 3); //add to use for village naming
+					road_names.insert(std::pair<location,std::string>(loc, road_base_name));
 				}
 			}
 		}
@@ -1146,7 +1181,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 		const int x = c->x;
 		const int y = c->y;
 		const int player = c - castles.begin() + 1;
-		const struct t_translation::coordinate coord = {x, y};
+		const struct t_translation::coordinate coord(x, y);
 		starting_positions.insert(std::pair<int, t_translation::coordinate>(player, coord));
 		terrain[x][y] = t_translation::HUMAN_KEEP;
 
@@ -1174,12 +1209,61 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 	LOG_NG << "placed castles\n";
 	LOG_NG << (SDL_GetTicks() - ticks) << "\n"; ticks = SDL_GetTicks();
 
-	if(nvillages > 0) {
-		const config* const naming = cfg.child("village_naming");
-		config naming_cfg;
-		if(naming != NULL) {
-			naming_cfg = *naming;
+	/*Random naming for landforms: mountains, forests, swamps, hills
+	 *we name these now that everything else is placed (as e.g., placing
+	 * roads could split a forest)
+	 */
+	for (x = width / 3; x < (width / 3)*2; x++) {
+		for (y = height / 3; y < (height / 3) * 2;y++) {
+		//check the terrain of the tile
+		const location loc(x - width / 3, y - height / 3);
+		const t_translation::t_terrain terr = terrain[x][y];
+		std::string name, base_name;
+		std::set<std::string> used_names;
+		if (t_translation::terrain_matches(terr, t_translation::ALL_MOUNTAINS)) {
+			//name every 15th mountain
+			if ((rand()%15) == 0) {
+				for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+					name = generate_name(name_generator, "mountain_name", &base_name);
+				}
+				labels->insert(std::pair<map_location, std::string>(loc, name));
+				mountain_names.insert(std::pair<location, std::string>(loc, base_name));
+			}
 		}
+		else if (t_translation::terrain_matches(terr, t_translation::ALL_FORESTS)) {
+			//if the forest tile is not named yet, name it
+			const std::map<location, std::string>::const_iterator forest_name = forest_names.find(loc);
+			if(forest_name == forest_names.end()) {
+				for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+					name = generate_name(name_generator, "forest_name", &base_name);
+				}
+				forest_names.insert(std::pair<location, std::string>(loc, base_name));
+				// name all connected forest tiles accordingly
+				flood_name(loc, base_name, forest_names, t_translation::ALL_FORESTS, terrain, width, height, 0, labels, name);
+			}
+		}
+		else if (t_translation::terrain_matches(terr, t_translation::ALL_SWAMPS)) {
+			//if the swamp tile is not named yet, name it
+			const std::map<location, std::string>::const_iterator swamp_name = swamp_names.find(loc);
+			if(swamp_name == swamp_names.end()) {
+				for(size_t ntry = 0; ntry != 30 && (ntry == 0 || used_names.count(name) > 0); ++ntry) {
+					name = generate_name(name_generator, "swamp_name", &base_name);
+				}
+				swamp_names.insert(std::pair<location, std::string>(loc, base_name));
+				// name all connected swamp tiles accordingly
+				flood_name(loc, base_name, swamp_names, t_translation::ALL_SWAMPS, terrain, width, height, 0, labels, name);
+			}
+		}
+
+		}
+	}
+
+	if (nvillages > 0)
+	{
+		config naming_cfg = cfg.child_or_empty("village_naming");
+		// HACK: dummy names to satisfy unit_race requirements
+		naming_cfg["id"] = "village_naming";
+		naming_cfg["plural_name"] = "villages";
 
 		const unit_race village_names_generator(naming_cfg);
 
@@ -1219,10 +1303,9 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 					const std::string str =
 						t_translation::write_terrain_code(terrain[res.x][res.y]);
-					const config* const child =
-						cfg.find_child("village","terrain",str);
-					if(child != NULL) {
-						const std::string& convert_to = (*child)["convert_to"];
+					if (const config &child = cfg.find_child("village", "terrain", str))
+					{
+						const std::string &convert_to = child["convert_to"];
 						if(convert_to != "") {
 							terrain[res.x][res.y] =
 								t_translation::read_terrain_code(convert_to);
@@ -1248,18 +1331,35 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 
 								size_t n;
 								for(n = 0; n != 6; ++n) {
+									const std::map<location,std::string>::const_iterator road_name = road_names.find(adj[n]);
+									if(road_name != road_names.end()) {
+										symbols["road"] = road_name->second;
+										name_type = "village_name_road";
+										break;
+									}
+
 									const std::map<location,std::string>::const_iterator river_name = river_names.find(adj[n]);
 									if(river_name != river_names.end()) {
 										symbols["river"] = river_name->second;
 										name_type = "village_name_river";
 
-										if(bridges.count(loc)) {
-											name_type = "village_name_river_bridge";
+										const std::map<location,std::string>::const_iterator bridge_name = bridge_names.find(adj[n]);
+										if(bridge_name != bridge_names.end()) {
+										//we should always end up here, since if there is an adjacent bridge, there has to be an adjacent river too
+										symbols["bridge"] = bridge_name->second;
+										name_type = "village_name_river_bridge";
 										}
 
 										break;
 									}
 
+									const std::map<location,std::string>::const_iterator forest_name = forest_names.find(adj[n]);
+									if(forest_name != forest_names.end()) {
+										symbols["forest"] = forest_name->second;
+										name_type = "village_name_forest";
+										break;
+									}
+
 									const std::map<location,std::string>::const_iterator lake_name = lake_names.find(adj[n]);
 									if(lake_name != lake_names.end()) {
 										symbols["lake"] = lake_name->second;
@@ -1267,6 +1367,20 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 										break;
 									}
 
+									const std::map<location,std::string>::const_iterator mountain_name = mountain_names.find(adj[n]);
+									if(mountain_name != mountain_names.end()) {
+										symbols["mountain"] = mountain_name->second;
+										name_type = "village_name_mountain";
+										break;
+									}
+
+									const std::map<location,std::string>::const_iterator swamp_name = swamp_names.find(adj[n]);
+									if(swamp_name != swamp_names.end()) {
+										symbols["swamp"] = swamp_name->second;
+										name_type = "village_name_swamp";
+										break;
+									}
+
 									const t_translation::t_terrain terr =
 										terrain[adj[n].x+width/3][adj[n].y+height/3];
 
@@ -1287,7 +1401,7 @@ std::string default_generate_map(size_t width, size_t height, size_t island_size
 									} else if(forest_count >= 2) {
 										name_type = "village_name_forest";
 									} else if(mountain_count >= 1) {
-										name_type = "village_name_mountain";
+										name_type = "village_name_mountain_anonymous";
 									} else if(hill_count >= 2) {
 										name_type = "village_name_hill";
 									}
