@@ -1,11 +1,12 @@
-/* $Id: multiplayer.cpp 42333 2010-04-30 23:22:12Z loonycyborg $ */
+/* $Id: multiplayer.cpp 51986 2011-11-15 23:46:46Z espreon $ */
 /*
-   Copyright (C) 2007 - 2010
+   Copyright (C) 2007 - 2011
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -16,11 +17,13 @@
 
 #include "dialogs.hpp"
 #include "gettext.hpp"
+#include "game_preferences.hpp"
 #include "log.hpp"
 #include "gui/dialogs/lobby_main.hpp"
 #include "gui/dialogs/message.hpp"
 #include "gui/dialogs/mp_connect.hpp"
 #include "gui/dialogs/mp_create_game.hpp"
+#include "gui/dialogs/mp_login.hpp"
 #include "gui/widgets/settings.hpp"
 #include "gui/widgets/window.hpp"
 #include "hash.hpp"
@@ -34,12 +37,16 @@
 #include "playcampaign.hpp"
 #include "formula_string_utils.hpp"
 #include "sound.hpp"
+#include "unit_id.hpp"
 
 #include <boost/bind.hpp>
 
 static lg::log_domain log_network("network");
 #define LOG_NW LOG_STREAM(info, log_network)
 
+static lg::log_domain log_mp("mp/main");
+#define DBG_MP LOG_STREAM(debug, log_mp)
+
 namespace {
 
 class network_game_manager
@@ -54,7 +61,7 @@ public:
 			LOG_NW << "sending leave_game\n";
 			config cfg;
 			cfg.add_child("leave_game");
-			network::send_data(cfg, 0, true);
+			network::send_data(cfg, 0);
 			LOG_NW << "sent leave_game\n";
 		}
 	};
@@ -64,12 +71,16 @@ public:
 
 static void run_lobby_loop(display& disp, mp::ui& ui)
 {
+	DBG_MP << "running lobby loop" << std::endl;
 	disp.video().modeChanged();
 	bool first = true;
 	font::cache_mode(font::CACHE_LOBBY);
 	while (ui.get_result() == mp::ui::CONTINUE) {
 		if (disp.video().modeChanged() || first) {
-			SDL_Rect lobby_pos = { 0, 0, disp.video().getx(), disp.video().gety() };
+			SDL_Rect lobby_pos = create_rect(0
+					, 0
+					, disp.video().getx()
+					, disp.video().gety());
 			ui.set_location(lobby_pos);
 			first = false;
 		}
@@ -100,6 +111,7 @@ enum server_type {
 
 static server_type open_connection(game_display& disp, const std::string& original_host)
 {
+	DBG_MP << "opening connection" << std::endl;
 	std::string h = original_host;
 
 	if(h.empty()) {
@@ -161,8 +173,8 @@ static server_type open_connection(game_display& disp, const std::string& origin
 		// Check for "redirect" messages
 		if (const config &redirect = data.child("redirect"))
 		{
-			host = redirect["host"];
-			port = lexical_cast_default<unsigned int>(redirect["port"], 15000);
+			host = redirect["host"].str();
+			port =redirect["port"].to_int(15000);
 
 			if(shown_hosts.find(hostpair(host,port)) != shown_hosts.end()) {
 				throw network::error(_("Server-side redirect loop"));
@@ -180,7 +192,7 @@ static server_type open_connection(game_display& disp, const std::string& origin
 			config res;
 			cfg["version"] = game_config::version;
 			res.add_child("version", cfg);
-			network::send_data(res, 0, true);
+			network::send_data(res, 0);
 		}
 
 		//if we got a direction to login
@@ -199,15 +211,15 @@ static server_type open_connection(game_display& disp, const std::string& origin
 				// If ping_timeout has a non-zero value, do not enable
 				// selective pings as this will cause clients to falsely
 				// believe the server has died and disconnect.
-				if( preferences::get_ping_timeout() ) {
-				  // Pings required so disable selective pings
-				  sp["selective_ping"] = "0" ;
+				if (preferences::get_ping_timeout()) {
+					// Pings required so disable selective pings
+					sp["selective_ping"] = false;
 				} else {
-				  // Client is bandwidth friendly so allow
-				  // server to optimize ping frequency as needed.
-				  sp["selective_ping"] = "1" ;
+					// Client is bandwidth friendly so allow
+					// server to optimize ping frequency as needed.
+					sp["selective_ping"] = true;
 				}
-				network::send_data(response, 0, true);
+				network::send_data(response, 0);
 
 				// Get response for our login request...
 				network::connection data_res = network::receive_data(data, 0, 3000);
@@ -215,6 +227,31 @@ static server_type open_connection(game_display& disp, const std::string& origin
 					throw network::error(_("Connection timed out"));
 				}
 
+				config *warning = &data.child("warning");
+
+				if(*warning) {
+					std::string warning_msg;
+
+					utils::string_map i18n_symbols;
+					i18n_symbols["nick"] = login;
+
+					if((*warning)["warning_code"] == MP_NAME_INACTIVE_WARNING) {
+						warning_msg = vgettext("The nickname ‘$nick’ is inactive. "
+							"You cannot claim ownership of this nickname until you "
+							"activate your account via email or ask an "
+							"administrator to do it for you.", i18n_symbols);
+					} else {
+						warning_msg = (*warning)["message"].str();
+					}
+
+					warning_msg += "\n\n";
+					warning_msg += _("Do you want to continue?");
+
+					if(gui2::show_message(disp.video(), _("Warning"), warning_msg, gui2::tmessage::yes_no_buttons) != gui2::twindow::OK) {
+						return ABORT_SERVER;
+					}
+				}
+
 				config *error = &data.child("error");
 
 				// ... and get us out of here if the server did not complain
@@ -223,8 +260,8 @@ static server_type open_connection(game_display& disp, const std::string& origin
 				do {
 					std::string password = preferences::password();
 
-					bool fall_through = (*error)["force_confirmation"] == "yes" ?
-						(gui::dialog(disp,_("Confirm"),(*error)["message"],gui::OK_CANCEL).show() != 0) :
+					bool fall_through = (*error)["force_confirmation"].to_bool() ?
+						(gui2::show_message(disp.video(), _("Confirm"), (*error)["message"], gui2::tmessage::ok_cancel_buttons) == gui2::twindow::CANCEL) :
 						false;
 
 					const bool is_pw_request = !((*error)["password_request"].empty()) && !(password.empty());
@@ -235,7 +272,8 @@ static server_type open_connection(game_display& disp, const std::string& origin
 					// above go directly to the username/password dialog
 					if((is_pw_request || !password_reminder.empty()) && !fall_through) {
 						if(is_pw_request) {
-							if((*error)["phpbb_encryption"] == "yes") {
+							if ((*error)["phpbb_encryption"].to_bool())
+							{
 
 								// Apparently HTML key-characters are passed to the hashing functions of phpbb in this escaped form.
 								// I will do closer investigations on this, for now let's just hope these are all of them.
@@ -267,7 +305,7 @@ static server_type open_connection(game_display& disp, const std::string& origin
 						sp["password_reminder"] = password_reminder;
 
 						// Once again send our request...
-						network::send_data(response, 0, true);
+						network::send_data(response, 0);
 
 						network::connection data_res = network::receive_data(data, 0, 3000);
 						if(!data_res) {
@@ -296,32 +334,34 @@ static server_type open_connection(game_display& disp, const std::string& origin
 					if((*error)["error_code"] == MP_MUST_LOGIN) {
 						error_message = _("You must login first.");
 					} else if((*error)["error_code"] == MP_NAME_TAKEN_ERROR) {
-						error_message = vgettext("The nick '$nick' is already taken.", i18n_symbols);
+						error_message = vgettext("The nickname ‘$nick’ is already taken.", i18n_symbols);
 					} else if((*error)["error_code"] == MP_INVALID_CHARS_IN_NAME_ERROR) {
-						error_message = vgettext("The nick '$nick' contains invalid "
+						error_message = vgettext("The nickname ‘$nick’ contains invalid "
 								"characters. Only alpha-numeric characters, underscores and "
 								"hyphens are allowed.", i18n_symbols);
 					} else if((*error)["error_code"] == MP_NAME_TOO_LONG_ERROR) {
-						error_message = vgettext("The nick '$nick' is too long. Nicks must "
-								"be 18 characters or less.", i18n_symbols);
+						error_message = vgettext("The nickname ‘$nick’ is too long. Nicks must "
+								"be 20 characters or less.", i18n_symbols);
 					} else if((*error)["error_code"] == MP_NAME_RESERVED_ERROR) {
-						error_message = vgettext("The nick '$nick' is reserved and cannot be used by players.", i18n_symbols);
+						error_message = vgettext("The nickname ‘$nick’ is reserved and cannot be used by players.", i18n_symbols);
 					} else if((*error)["error_code"] == MP_NAME_UNREGISTERED_ERROR) {
-						error_message = vgettext("The nick '$nick' is not registered on this server.", i18n_symbols)
-								+ _(" This server disallows unregistered nicks.");
+						error_message = vgettext("The nickname ‘$nick’ is not registered on this server.", i18n_symbols)
+								+ _(" This server disallows unregistered nicknames.");
 					} else if((*error)["error_code"] == MP_PASSWORD_REQUEST) {
-						error_message = vgettext("The nick '$nick' is registered on this server.", i18n_symbols);
+						error_message = vgettext("The nickname ‘$nick’ is registered on this server.", i18n_symbols);
 					} else if((*error)["error_code"] == MP_PASSWORD_REQUEST_FOR_LOGGED_IN_NAME) {
-						error_message = vgettext("The nick '$nick' is registered on this server.", i18n_symbols)
-								+ "\n\n" + _("WARNING: There is already a client using this nick, "
+						error_message = vgettext("The nickname ‘$nick’ is registered on this server.", i18n_symbols)
+								+ "\n\n" + _("WARNING: There is already a client using this nickname, "
 								"logging in will cause that client to be kicked!");
 					} else if((*error)["error_code"] == MP_NO_SEED_ERROR) {
 						error_message = _("Error in the login procedure (the server had no "
 								"seed for your connection).");
 					} else if((*error)["error_code"] == MP_INCORRECT_PASSWORD_ERROR) {
 						error_message = _("The password you provided was incorrect.");
+					} else if((*error)["error_code"] == MP_TOO_MANY_ATTEMPTS_ERROR) {
+						error_message = _("You have made too many login attempts.");
 					} else {
-						error_message = (*error)["message"];
+						error_message = (*error)["message"].str();
 					}
 
 					gui2::tmp_login dlg(error_message, !((*error)["password_request"].empty()));
@@ -376,12 +416,15 @@ static server_type open_connection(game_display& disp, const std::string& origin
 
 static void enter_wait_mode(game_display& disp, const config& game_config, mp::chat& chat, config& gamelist, bool observe)
 {
+	DBG_MP << "entering wait mode" << std::endl;
+
 	mp::ui::result res;
 	game_state state;
 	network_game_manager m;
 
 	gamelist.clear();
 	statistics::fresh_stats();
+	n_unit::id_manager::instance().clear(); //< reset the unit underlying_id counter back to zero
 
 	{
 		mp::wait ui(disp, game_config, chat, gamelist);
@@ -424,6 +467,8 @@ static void enter_connect_mode(game_display& disp, const config& game_config,
 		mp::chat& chat, config& gamelist, const mp_game_settings& params,
 		const int num_turns, mp::controller default_controller, bool local_players_only = false)
 {
+	DBG_MP << "entering connect mode" << std::endl;
+
 	mp::ui::result res;
 	game_state state;
 	const network::manager net_manager(1,1);
@@ -457,20 +502,21 @@ static void enter_connect_mode(game_display& disp, const config& game_config,
 		break;
 	case mp::ui::QUIT:
 	default:
-		network::send_data(config("refresh_lobby"), 0, true);
+		network::send_data(config("refresh_lobby"), 0);
 		break;
 	}
 }
 
 static void enter_create_mode(game_display& disp, const config& game_config, mp::chat& chat, config& gamelist, mp::controller default_controller, bool local_players_only)
 {
-	if (0 && gui2::new_widgets) {
+	DBG_MP << "entering create mode" << std::endl;
+	if (gui2::new_widgets) {
 
 		gui2::tmp_create_game dlg(game_config);
 
 		dlg.show(disp.video());
 
-		network::send_data(config("refresh_lobby"), 0, true);
+		network::send_data(config("refresh_lobby"), 0);
 	} else {
 
 		mp::ui::result res;
@@ -492,7 +538,7 @@ static void enter_create_mode(game_display& disp, const config& game_config, mp:
 		case mp::ui::QUIT:
 		default:
 			//update lobby content
-			network::send_data(config("refresh_lobby"), 0, true);
+			network::send_data(config("refresh_lobby"), 0);
 			break;
 		}
 	}
@@ -500,6 +546,7 @@ static void enter_create_mode(game_display& disp, const config& game_config, mp:
 
 static void do_preferences_dialog(game_display& disp, const config& game_config)
 {
+	DBG_MP << "displaying preferences dialog" << std::endl;
 	const preferences::display_manager disp_manager(&disp);
 	preferences::show_preferences_dialog(disp,game_config);
 
@@ -519,7 +566,7 @@ static void do_preferences_dialog(game_display& disp, const config& game_config)
 
 static void enter_lobby_mode(game_display& disp, const config& game_config, mp::chat& chat, config& gamelist)
 {
-
+	DBG_MP << "entering lobby mode" << std::endl;
 
 	mp::ui::result res;
 
@@ -537,13 +584,13 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 		lobby_info li(game_config);
 
 		// Force a black background
-		const Uint32 colour = SDL_MapRGBA(disp.video().getSurface()->format
+		const Uint32 color = SDL_MapRGBA(disp.video().getSurface()->format
 				, 0
 				, 0
 				, 0
 				, 255);
 
-		SDL_FillRect(disp.video().getSurface(), NULL, colour);
+		sdl_fill_rect(disp.video().getSurface(), NULL, color);
 
 		if(preferences::new_lobby()) {
 			gui2::tlobby_main dlg(game_config, li, disp);
@@ -580,7 +627,7 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 					gui2::show_error_message(disp.video(), error.message);
 				}
 				//update lobby content
-				network::send_data(config("refresh_lobby"), 0, true);
+				network::send_data(config("refresh_lobby"), 0);
 			}
 			break;
 		case mp::ui::OBSERVE:
@@ -593,7 +640,7 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 			}
 			// update lobby content unconditionally because we might have left only after the
 			// game ended in which case we ignored the gamelist and need to request it again
-			network::send_data(config("refresh_lobby"), 0, true);
+			network::send_data(config("refresh_lobby"), 0);
 			break;
 		case mp::ui::CREATE:
 			try {
@@ -602,7 +649,7 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 				if (!error.message.empty())
 					gui2::show_error_message(disp.video(), error.message);
 				//update lobby content
-				network::send_data(config("refresh_lobby"), 0, true);
+				network::send_data(config("refresh_lobby"), 0);
 			}
 			break;
 		case mp::ui::QUIT:
@@ -611,7 +658,7 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 			{
 				do_preferences_dialog(disp, game_config);
 				//update lobby content
-				network::send_data(config("refresh_lobby"), 0, true);
+				network::send_data(config("refresh_lobby"), 0);
 			}
 			break;
 		default:
@@ -625,6 +672,7 @@ namespace mp {
 void start_local_game(game_display& disp, const config& game_config,
 		mp::controller default_controller)
 {
+	DBG_MP << "starting local game" << std::endl;
 	const rand_rng::set_random_generator generator_setter(&recorder);
 	mp::chat chat;
 	config gamelist;
@@ -636,6 +684,7 @@ void start_local_game(game_display& disp, const config& game_config,
 void start_client(game_display& disp, const config& game_config,
 		const std::string& host)
 {
+	DBG_MP << "starting client" << std::endl;
 	const rand_rng::set_random_generator generator_setter(&recorder);
 	const network::manager net_manager(1,1);
 
