@@ -1,25 +1,26 @@
-/* $Id: multiplayer_lobby.hpp 33092 2009-02-25 01:45:48Z soliton $ */
+/* $Id: multiplayer_lobby.hpp 49494 2011-05-12 17:22:20Z zaroth $ */
 /*
    Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
-/** @file multiplayer_lobby.hpp. */
+/** @file multiplayer_lobby.hpp */
 
 #ifndef MULTIPLAYER_LOBBY_HPP_INCLUDED
 #define MULTIPLAYER_LOBBY_HPP_INCLUDED
 
 #include "multiplayer_ui.hpp"
-
-#include <map>
+#include "game_preferences.hpp"
+#include "image.hpp"
 
 class config;
 class video;
@@ -55,6 +56,7 @@ public:
 			fog(false),
 			shroud(false),
 			observers(false),
+			shuffle_sides(false),
 			use_map_settings(false),
 			verified(false),
 			password_required(false),
@@ -80,12 +82,13 @@ public:
 		bool fog;
 		bool shroud;
 		bool observers;
+		bool shuffle_sides;
 		bool use_map_settings;
 		bool verified;
 		bool password_required;
 		bool have_era;
 	};
-	gamebrowser(CVideo& video,const config* map_hashes);
+	gamebrowser(CVideo& video, const config &map_hashes);
 	void scroll(unsigned int pos);
 	void handle_event(const SDL_Event& event);
 	void set_inner_location(const SDL_Rect& rect);
@@ -104,7 +107,8 @@ public:
 	void reset_selection() { double_clicked_ = false; }
 	int selection() const { return selected_; }
 	game_item selected_game() { return games_[selected_]; }
-	void select_game(const std::string id);
+	void select_game(const std::string& id);
+	bool game_matches_filter(const game_item& i, const config& cfg);
 protected:
 	unsigned int row_height() const { return item_height_ + (2 * style_->get_thickness()); }
 private:
@@ -114,8 +118,9 @@ private:
 	image::locator time_limit_icon_locator_;
 	image::locator observer_icon_locator_;
 	image::locator no_observer_icon_locator_;
+	image::locator shuffle_sides_icon_locator_;
 
-	const config* map_hashes_;
+	const config &map_hashes_;
 
 	unsigned int item_height_;
 	int margin_;
@@ -166,17 +171,24 @@ private:
 	gui::button join_game_;
 	gui::button create_game_;
 	gui::button skip_replay_;
-#ifndef USE_TINY_GUI
 	gui::button game_preferences_;
-#endif
 	gui::button quit_game_;
 
+	gui::button apply_filter_;
+	gui::button invert_filter_;
+	gui::button vacant_slots_;
+	gui::button friends_in_game_;
+	gui::label filter_label_;
+	gui::textbox filter_text_;
+
 	int last_selected_game_;
 
 	lobby_sorter sorter_;
 	gamebrowser games_menu_;
 
 	std::map<std::string,std::string> minimaps_;
+
+	std::string search_string_;
 };
 
 } // end namespace mp
