@@ -1,11 +1,12 @@
-/* $Id: multiplayer_ui.cpp 35301 2009-04-28 17:56:25Z mordante $ */
+/* $Id: multiplayer_ui.cpp 48564 2011-02-20 02:59:29Z shadowmaster $ */
 /*
-   Copyright (C) 2005 - 2009
+   Copyright (C) 2005 - 2011
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,27 +16,39 @@
 #include "global.hpp"
 
 #include "construct_dialog.hpp"
+#include "foreach.hpp"
+#include "gamestatus.hpp"
+#include "game_display.hpp"
+#include "game_preferences.hpp"
 #include "gettext.hpp"
 #include "gui/dialogs/mp_cmd_wrapper.hpp"
+#include "lobby_preferences.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
+#include "menu_events.hpp"
 #include "multiplayer.hpp"
 #include "multiplayer_ui.hpp"
 #include "sound.hpp"
 #include "replay.hpp"
 #include "wml_separators.hpp"
+#include "formula_string_utils.hpp"
 
-#define LOG_NG LOG_STREAM(info, engine)
-#define ERR_NG LOG_STREAM(err, engine)
-#define ERR_CF LOG_STREAM(err, config)
-#define DBG_NW LOG_STREAM(debug, network)
-#define LOG_NW LOG_STREAM(info, network)
-#define ERR_NW LOG_STREAM(err, network)
+static lg::log_domain log_engine("engine");
+#define LOG_NG LOG_STREAM(info, log_engine)
+#define ERR_NG LOG_STREAM(err, log_engine)
 
-#define MAX_MESSAGES 256
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
+
+static lg::log_domain log_network("network");
+#define LOG_NW LOG_STREAM(info, log_network)
+#define ERR_NW LOG_STREAM(err, log_network)
 
 namespace {
 
+	/** The maximum number of messages in the chat history. */
+	const size_t max_messages = 256;
+
 	class user_menu_style : public gui::menu::imgsel_style {
 	public:
 		user_menu_style() : gui::menu::imgsel_style("misc/selection", false,
@@ -70,9 +83,8 @@ void check_response(network::connection res, const config& data)
 		throw network::error(_("Connection timed out"));
 	}
 
-	const config* err = data.child("error");
-	if(err != NULL) {
-		throw network::error((*err)["message"]);
+	if (const config &err = data.child("error")) {
+		throw network::error(err["message"]);
 	}
 }
 
@@ -80,12 +92,12 @@ void level_to_gamestate(config& level, game_state& state)
 {
 	//any replay data is only temporary and should be removed from
 	//the level data in case we want to save the game later
-	config * const replay_data = level.child("replay");
+	const config &replay_data = level.child("replay");
 	config replay_data_store;
-	if(replay_data != NULL) {
-		replay_data_store = *replay_data;
+	if (replay_data) {
+		replay_data_store = replay_data;
 		LOG_NW << "setting replay\n";
-		state.replay_data = *replay_data;
+		state.replay_data = replay_data;
 		recorder = replay(replay_data_store);
 		if(!recorder.empty()) {
 			recorder.set_skip(false);
@@ -104,67 +116,65 @@ void level_to_gamestate(config& level, game_state& state)
 	}
 
 	//adds the starting pos to the level
-	if(level.child("replay_start") == NULL){
+	if (!level.child("replay_start")) {
 		level.add_child("replay_start", level);
+		level.child("replay_start").remove_child("multiplayer", 0);
 	}
 	//this is important, if it does not happen, the starting position is missing and
 	//will be drawn from the snapshot instead (which is not what we want since we have
 	//all needed information here already)
-	state.starting_pos = *(level.child("replay_start"));
+	state.starting_pos = level.child("replay_start");
 
 	level["campaign_type"] = "multiplayer";
-	state.campaign_type = "multiplayer";
-	state.completion = level["completion"];
-	state.version = level["version"];
+	state.classification().campaign_type = "multiplayer";
+	state.classification().completion = level["completion"].str();
+	state.classification().version = level["version"].str();
 
-	const config* const vars = level.child("variables");
-	if(vars != NULL) {
-		state.set_variables(*vars);
+	if (const config &vars = level.child("variables")) {
+		state.set_variables(vars);
 	}
-	state.set_menu_items(level.get_children("menu_item"));
+	state.set_menu_items(level.child_range("menu_item"));
+	state.mp_settings().set_from_config(level);
 
 	//Check whether it is a save-game by looking for snapshot data
-	const bool saved_game = (level.child("snapshot") && level.child("snapshot")->child("side"));
+	const config &snapshot = level.child("snapshot");
+	const bool saved_game = snapshot && snapshot.child("side");
 
 	//It might be a MP campaign start-of-scenario save
 	//In this case, it's not entirely a new game, but not a save, either
 	//Check whether it is no savegame and the starting_pos contains [player] information
-	const bool start_of_scenario = (!saved_game && (state.starting_pos.child("player") != NULL));
+	bool start_of_scenario = !saved_game && state.starting_pos.child("player");
 
 	//If we start a fresh game, there won't be any snapshot information. If however this
 	//is a savegame, we got a valid snapshot here.
-	if (saved_game){
-		state.snapshot = *(level.child("snapshot"));
-		if (state.snapshot.child("variables") != NULL){
-			state.set_variables(*state.snapshot.child("variables"));
+	if (saved_game) {
+		state.snapshot = snapshot;
+		if (const config &v = snapshot.child("variables")) {
+			state.set_variables(v);
 		}
-		state.set_menu_items(state.snapshot.get_children("menu_item"));
-	}
-
-	//If it is a start-of-scenario save, we need to load the player information from
-	//the [player] tags
-	if(start_of_scenario){
-		state.load_recall_list(state.starting_pos.get_children("player"));
+		state.set_menu_items(snapshot.child_range("menu_item"));
 	}
 
 	//In any type of reload(normal save or start-of-scenario) the players could have
 	//changed and need to be replaced
 	if(saved_game || start_of_scenario){
-		const config::child_list& saved_sides = (saved_game ?
-			state.snapshot.get_children("side") :
-			state.starting_pos.get_children("side"));
+		config::child_itors saved_sides = saved_game ?
+			state.snapshot.child_range("side") :
+			state.starting_pos.child_range("side");
+		config::const_child_itors level_sides = level.child_range("side");
 
-		const config::child_list& level_sides = level.get_children("side");
-		for(config::child_list::const_iterator side = saved_sides.begin(); side != saved_sides.end(); ++side) {
-			for(config::child_list::const_iterator lside = level_sides.begin(); lside != level_sides.end(); ++lside) {
-				if ( ((**side)["side"] == (**lside)["side"]) && (
-					 ((**side)["current_player"] != (**lside)["current_player"]) ||
-					 ((**side)["controller"] != (**lside)["controller"]) )){
-
-					(**side)["current_player"] = (**lside)["current_player"];
-					(**side)["id"] = (**lside)["id"];
-					(**side)["save_id"] = (**lside)["save_id"];
-					(**side)["controller"] = (**lside)["controller"];
+		foreach (config &side, saved_sides)
+		{
+			foreach (const config &lside, level_sides)
+			{
+				if (side["side"] == lside["side"] &&
+						(side["current_player"] != lside["current_player"] ||
+						 side["controller"] != lside["controller"]))
+				{
+					side["current_player"] = lside["current_player"];
+					side["id"] = lside["id"];
+					side["save_id"] = lside["save_id"];
+					side["controller"] = lside["controller"];
 					break;
 				}
 			}
@@ -178,12 +188,10 @@ void level_to_gamestate(config& level, game_state& state)
 	}
 }
 
-std::string get_colour_string(int id)
+std::string get_color_string(int id)
 {
 	std::string prefix = team::get_side_highlight(id);
-	std::stringstream side_id;
-	side_id << (id + 1);
-	std::map<std::string, t_string>::iterator name = game_config::team_rgb_name.find(side_id.str());
+	std::map<std::string, t_string>::iterator name = game_config::team_rgb_name.find(str_cast(id + 1));
 	if(name != game_config::team_rgb_name.end()){
 		return prefix + name->second;
 	}else{
@@ -202,7 +210,7 @@ void chat::add_message(const time_t& time, const std::string& user,
 {
 	message_history_.push_back(msg(time, user, message));
 
-	while (message_history_.size() > MAX_MESSAGES) {
+	while (message_history_.size() > max_messages) {
 		message_history_.pop_front();
 
 		if (last_update_ > 0)
@@ -233,11 +241,10 @@ void chat::update_textbox(gui::textbox& textbox)
 std::string chat::format_message(const msg& message)
 {
 	std::string msg_text = message.message;
-	if(message.user == "server") {
-		SDL_Color c;
-		int unused;
+	if(message.user == "server"
+	|| message.user.substr(0,29) == "whisper: server message from ") {
 		std::string::const_iterator after_markup =
-			font::parse_markup(message.message.begin(), message.message.end(), &unused, &c, &unused);
+			font::parse_markup(message.message.begin(), message.message.end(), NULL, NULL, NULL);
 
 		msg_text = std::string(after_markup,message.message.end());
 	}
@@ -251,14 +258,14 @@ std::string chat::format_message(const msg& message)
 }
 
 SDL_Color chat::color_message(const msg& message) {
-	SDL_Color c = font::NORMAL_COLOUR;
+	SDL_Color c = font::NORMAL_COLOR;
 	// Normal users are not allowed to color their messages
-	if(message.user == "server") {
-		int unused;
-		font::parse_markup(message.message.begin(), message.message.end(), &unused, &c, &unused);
+	if(message.user == "server"
+	|| message.user.substr(0,29) == "whisper: server message from ") {
+		font::parse_markup(message.message.begin(), message.message.end(), NULL, &c, NULL);
 	// Highlight private messages too
 	} else if(message.user.substr(0,8) == "whisper:") {
-	    c = font::LABEL_COLOUR;
+	    c = font::LABEL_COLOR;
 	}
 	return c;
 }
@@ -276,12 +283,8 @@ ui::ui(game_display& disp, const std::string& title, const config& cfg, chat& c,
 	chat_(c),
 	gamelist_(gamelist),
 
-#ifdef USE_TINY_GUI
-	title_(disp.video(), title, font::SIZE_SMALL, font::TITLE_COLOUR),
-#else
-	title_(disp.video(), title, font::SIZE_LARGE, font::TITLE_COLOUR),
+	title_(disp.video(), title, font::SIZE_LARGE, font::TITLE_COLOR),
 	entry_textbox_(disp.video(), 100),
-#endif
 	chat_textbox_(disp.video(), 100, "", false),
 	users_menu_(disp.video(), std::vector<std::string>(), false, -1, -1, NULL, &umenu_style),
 
@@ -294,7 +297,10 @@ ui::ui(game_display& disp, const std::string& title, const config& cfg, chat& c,
 	gamelist_refresh_(false),
 	lobby_clock_(0)
 {
-	const SDL_Rect area = { 0, 0, disp.video().getx(), disp.video().gety() };
+	const SDL_Rect area = create_rect(0
+			, 0
+			, disp.video().getx()
+			, disp.video().gety());
 	users_menu_.set_numeric_keypress_selection(false);
 	set_location(area);
 }
@@ -376,15 +382,11 @@ void ui::draw_contents()
 {
 	hide_children();
 
-#ifdef USE_TINY_GUI
-	surface background(image::get_image("misc/lobby_tiny.png"));
-#else
 	surface background(image::get_image("misc/lobby.png"));
-#endif
 	background = scale_surface(background, video().getx(), video().gety());
 	if(background == NULL)
 		return;
-	SDL_BlitSurface(background, NULL, video().getSurface(), NULL);
+	sdl_blit(background, NULL, video().getSurface(), NULL);
 	update_whole_screen();
 
 	hide_children(false);
@@ -456,7 +458,7 @@ void ui::handle_event(const SDL_Event& event)
 			//if the dialog has been open for a long time, refresh the lobby
 			config request;
 			request.add_child("refresh_lobby");
-			network::send_data(request, 0, true);
+			network::send_data(request, 0);
 		}
 	}
 	if(users_menu_.selection() > 0 // -1 indicates an invalid selection
@@ -466,7 +468,7 @@ void ui::handle_event(const SDL_Event& event)
 	}
 }
 
-void ui::add_chat_message(const time_t& time, const std::string& speaker, int /*side*/, const std::string& message, game_display::MESSAGE_TYPE /*type*/)
+void ui::add_chat_message(const time_t& time, const std::string& speaker, int /*side*/, const std::string& message, events::chat_handler::MESSAGE_TYPE /*type*/)
 {
 	chat_.add_message(time, speaker, message);
 	chat_.update_textbox(chat_textbox_);
@@ -480,13 +482,11 @@ void ui::send_chat_message(const std::string& message, bool /*allies_only*/)
 	data.add_child("message", msg);
 
 	add_chat_message(time(NULL), preferences::login(),0, message);	//local echo
-	network::send_data(data, 0, true);
+	network::send_data(data, 0);
 }
 
-
 void ui::handle_key_event(const SDL_KeyboardEvent& event)
 {
-#ifndef USE_TINY_GUI
 	//On enter, adds the current chat message to the chat textbox.
 	if((event.keysym.sym == SDLK_RETURN || event.keysym.sym == SDLK_KP_ENTER) && !entry_textbox_.text().empty()) {
 
@@ -506,22 +506,22 @@ void ui::handle_key_event(const SDL_KeyboardEvent& event)
 		if (matches.size() == 1) {
 			text.append(line_start ? ": " : " ");
 		} else {
-			std::string completion_list = utils::join(matches, ' ');
+			std::string completion_list = utils::join(matches, " ");
 			chat_.add_message(time(NULL), "", completion_list);
 			chat_.update_textbox(chat_textbox_);
 		}
 		entry_textbox_.set_text(text);
 	}
-#endif
 }
 
 void ui::process_message(const config& msg, const bool whisper) {
 	const std::string& sender = msg["sender"];
 	const std::string& message = msg["message"];
-	if (!preferences::show_lobby_join(sender, message)) return;
+	std::string room = msg["room"];
+	if (!preferences::parse_should_show_lobby_join(sender, message)) return;
 	if (preferences::is_ignored(sender)) return;
 
-	preferences::admin_authentication(sender, message);
+	preferences::parse_admin_authentication(sender, message);
 
 	if (whisper || utils::word_match(message, preferences::login())) {
 		sound::play_UI_sound(game_config::sounds::receive_message_highlight);
@@ -533,19 +533,32 @@ void ui::process_message(const config& msg, const bool whisper) {
 		// too annoying and probably not any helpful
 		//sound::play_UI_sound(game_config::sounds::receive_message);
 	}
-	chat_.add_message(time(NULL), (whisper ? "whisper: " : "") + msg["sender"],
-			msg["message"]);
+
+	std::string prefix;
+
+	if(whisper) {
+		utils::string_map symbols;
+		symbols["sender"] = msg["sender"].str();
+		prefix = VGETTEXT("whisper: $sender", symbols);
+	}
+	else {
+		prefix = msg["sender"].str();
+	}
+
+	if (!room.empty()) room = room + ": ";
+
+	chat_.add_message(time(NULL), room + prefix, msg["message"]);
 	chat_.update_textbox(chat_textbox_);
 }
 
 void ui::process_network_data(const config& data, const network::connection /*sock*/)
 {
-	if(data.child("error")) {
-		throw network::error((*data.child("error"))["message"]);
-	} else if (data.child("message")) {
-		process_message(*data.child("message"));
-	} else if(data.child("whisper")){
-		process_message(*data.child("whisper"), true);
+	if (const config &c = data.child("error")) {
+		throw network::error(c["message"]);
+	} else if (const config &c = data.child("message")) {
+		process_message(c);
+	} else if (const config &c = data.child("whisper")) {
+		process_message(c, true);
 	} else if(data.child("gamelist")) {
 		const cursor::setter cursor_setter(cursor::WAIT);
 		gamelist_initialized_ = true;
@@ -553,17 +566,54 @@ void ui::process_network_data(const config& data, const network::connection /*so
 		gamelist_updated(false);
 		gamelist_refresh_ = false;
 		lobby_clock_ = SDL_GetTicks();
-	} else if(data.child("gamelist_diff")) {
+	} else if (const config &c = data.child("gamelist_diff")) {
 		if(gamelist_initialized_) {
 			try {
-				gamelist_.apply_diff(*data.child("gamelist_diff"));
+				gamelist_.apply_diff(c);
 			} catch(config::error& e) {
 				ERR_CF << "Error while applying the gamelist diff: '"
 					<< e.message << "' Getting a new gamelist.\n";
-				network::send_data(config("refresh_lobby"), 0, true);
+				network::send_data(config("refresh_lobby"), 0);
 			}
 			gamelist_refresh_ = true;
 		}
+	} else if (const config &c = data.child("room_join")) {
+		if (c["player"] == preferences::login()) {
+			chat_.add_message(time(NULL), "server",
+				"You have joined the room '" + c["room"].str() + "'");
+		} else {
+			chat_.add_message(time(NULL), "server",
+				c["player"].str() + " has joined the room '" + c["room"].str() + "'");
+		}
+		chat_.update_textbox(chat_textbox_);
+	} else if (const config &c = data.child("room_part")) {
+		if (c["player"] == preferences::login()) {
+			chat_.add_message(time(NULL), "server",
+				"You have left the room '" + c["room"].str() + "'");
+		} else {
+			chat_.add_message(time(NULL), "server",
+				c["player"].str() + " has left the room '" + c["room"].str() + "'");
+		}
+		chat_.update_textbox(chat_textbox_);
+	} else if (const config &c = data.child("room_query_response")) {
+		if (const config &ms = c.child("members")) {
+			std::stringstream ss;
+			ss << "Room " << c["room"].str() << " members: ";
+			foreach (const config& m, ms.child_range("member")) {
+				ss << m["name"] << " ";
+			}
+			chat_.add_message(time(NULL), "server", ss.str());
+			chat_.update_textbox(chat_textbox_);
+		}
+		if (const config &rs = c.child("rooms")) {
+			std::stringstream ss;
+			ss << "Rooms: ";
+			foreach (const config& r, rs.child_range("room")) {
+				ss << r["name"].str() << "(" << r["size"].str() << ") ";
+			}
+			chat_.add_message(time(NULL), "server", ss.str());
+			chat_.update_textbox(chat_textbox_);
+		}
 	}
 }
 
@@ -584,9 +634,7 @@ void ui::hide_children(bool hide)
 {
 	title_.hide(hide);
 	chat_textbox_.hide(hide);
-#ifndef USE_TINY_GUI
 	entry_textbox_.hide(hide);
-#endif
 	users_menu_.hide(hide);
 }
 
@@ -599,16 +647,10 @@ void ui::layout_children(const SDL_Rect& /*rect*/)
 	users_menu_.set_location(xscale(856), yscale(42));
 	users_menu_.set_height(yscale(715));
 	users_menu_.set_max_height(yscale(715));
-#ifdef USE_TINY_GUI
-	chat_textbox_.set_location(xscale(11) + 4, yscale(625) + 4);
-	chat_textbox_.set_measurements(xscale(833) - 8, yscale(143) - 8);
-
-#else
 	chat_textbox_.set_location(xscale(11) + 4, yscale(573) + 4);
 	chat_textbox_.set_measurements(xscale(833) - 8, yscale(143) - 8);
 	entry_textbox_.set_location(xscale(11) + 4, yscale(732));
 	entry_textbox_.set_width(xscale(833) - 8);
-#endif
 }
 
 bool ui::user_info::operator> (const user_info& b) const {
@@ -662,25 +704,18 @@ bool ui::user_info::operator> (const user_info& b) const {
 
 void ui::gamelist_updated(bool silent)
 {
-	config::child_list users = gamelist_.get_children("user");
-	config::child_iterator user;
 	std::list<user_info> u_list;
 
-	for (user = users.begin(); user != users.end(); ++user) {
+	foreach (const config &user, gamelist_.child_range("user"))
+	{
 		user_info u_elem;
-		u_elem.name = (**user)["name"];
-		u_elem.game_id = "";
-		u_elem.location = "";
-		u_elem.state = (**user)["available"] == "no" ? GAME : LOBBY;
-		u_elem.registered = utils::string_bool((**user)["registered"]);
-		if(!(**user)["game_id"].empty()) {
-			u_elem.game_id = (**user)["game_id"];
-			if (u_elem.game_id == selected_game_) {
-				u_elem.state = SEL_GAME;
-			}
-		}
-		if(!(**user)["location"].empty()) {
-			u_elem.location = (**user)["location"];
+		u_elem.name = user["name"].str();
+		u_elem.state = user["available"].to_bool(true) ? LOBBY : GAME;
+		u_elem.registered = user["registered"].to_bool();
+		u_elem.game_id = user["game_id"].str();
+		u_elem.location = user["location"].str();
+		if (!u_elem.game_id.empty() && u_elem.game_id == selected_game_) {
+			u_elem.state = SEL_GAME;
 		}
 		if (u_elem.name == preferences::login()) {
 			u_elem.relation = ME;
@@ -735,14 +770,14 @@ void ui::gamelist_updated(bool silent)
 		reg_str = u_itor->registered ? registered_user_tag : "";
 		user_strings.push_back(u_itor->name);
 		menu_strings.push_back(img_str + reg_str + color_str + name_str + HELP_STRING_SEPARATOR + name_str);
-		u_itor++;
+		++u_itor;
 	}
 
 	set_user_list(user_strings, silent);
 	set_user_menu_items(menu_strings);
 }
 
-void ui::set_selected_game(const std::string game_id)
+void ui::set_selected_game(const std::string& game_id)
 {
 	// reposition the player list to show the players in the selected game
 	if (preferences::sort_list() && (selected_game_ != game_id)) {
@@ -776,17 +811,11 @@ void ui::set_user_list(const std::vector<std::string>& list, bool silent)
 	user_list_ = list;
 }
 
-std::string ui::get_selected_user_game() {
-	config::child_list users = gamelist_.get_children("user");
-	config::child_iterator user;
-
-	for (user = users.begin(); user != users.end(); ++user) {
-		if((**user)["name"] == selected_user_) {
-			return (**user)["game_id"];
-		}
-	}
-
-	return "";
+std::string ui::get_selected_user_game()
+{
+	const config &u = gamelist_.find_child("user", "name", selected_user_);
+	if (u) return u["game_id"];
+	return std::string();
 }
 
 void ui::append_to_title(const std::string& text) {
@@ -798,5 +827,46 @@ const gui::label& ui::title() const
 	return title_;
 }
 
+int find_suitable_faction(faction_list const &fl, const config &cfg)
+{
+	std::vector<std::string> find;
+	std::string search_field;
+	if (const config::attribute_value *f = cfg.get("faction")) {
+		// Choose based on faction.
+		find.push_back(f->str());
+		search_field = "id";
+	} else if (cfg["faction_from_recruit"].to_bool()) {
+		// Choose based on recruit.
+		find = utils::split(cfg["recruit"]);
+		search_field = "recruit";
+	} else if (const config::attribute_value *l = cfg.get("leader")) {
+		// Choose based on leader.
+		find.push_back(*l);
+		search_field = "leader";
+	} else {
+		return -1;
+	}
+
+	int res = -1, index = 0, best_score = 0;
+	foreach (const config *faction, fl)
+	{
+		int faction_score = 0;
+		std::vector<std::string> recruit = utils::split((*faction)[search_field]);
+		foreach (const std::string &search, find) {
+			foreach (const std::string &r, recruit) {
+				if (r == search) {
+					++faction_score;
+					break;
+				}
+			}
+		}
+		if (faction_score > best_score) {
+			best_score = faction_score;
+			res = index;
+		}
+		++index;
+	}
+	return res;
+}
 
 }// namespace mp
