@@ -1,11 +1,12 @@
-/* $Id: multiplayer_ui.hpp 32674 2009-02-08 13:02:29Z cib $ */
+/* $Id: multiplayer_ui.hpp 48480 2011-02-12 16:20:24Z ivanovic $ */
 /*
-   Copyright (C) 2007 - 2009
+   Copyright (C) 2007 - 2011
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,18 +15,18 @@
 #ifndef MULTIPLAYER_UI_HPP_INCLUDED
 #define MULTIPLAYER_UI_HPP_INCLUDED
 
+#include "chat_events.hpp"
 #include "hotkeys.hpp"
 #include "network.hpp"
 #include "preferences_display.hpp"
 #include "widgets/label.hpp"
 #include "widgets/menu.hpp"
 #include "widgets/textbox.hpp"
-#include "menu_events.hpp"
 
 #include <deque>
-#include <string>
 
 class display;
+class game_display;
 class config;
 class game_state;
 
@@ -37,7 +38,7 @@ void check_response(network::connection res, const config& data);
 
 void level_to_gamestate(config& level, game_state& state);
 
-std::string get_colour_string(int id);
+std::string get_color_string(int id);
 
 /** this class memorizes a chat session. */
 class chat
@@ -130,7 +131,7 @@ protected:
 	/** Override chat_handler. */
 	void add_chat_message(const time_t& time, const std::string& speaker,
 			int side, const std::string& message,
-			game_display::MESSAGE_TYPE type=game_display::MESSAGE_PRIVATE);
+			events::chat_handler::MESSAGE_TYPE type=events::chat_handler::MESSAGE_PRIVATE);
 	void send_chat_message(const std::string& message, bool allies_only=false);
 
 	/** Process chat messages. */
@@ -177,7 +178,7 @@ protected:
 	 * Sets the name of the selected game which is used to highlight the names
 	 * of the players which have joined this game.
 	 */
-	void set_selected_game(const std::string game_name);
+	void set_selected_game(const std::string& game_name);
 
 	/**
 	 * Called each time the gamelist_ variable is updated. May be
@@ -223,9 +224,7 @@ private:
 	config& gamelist_;
 
 	gui::label title_;
-#ifndef USE_TINY_GUI
 	gui::textbox entry_textbox_;
-#endif
 	gui::textbox chat_textbox_;
 
 	gui::menu users_menu_;
@@ -271,6 +270,10 @@ private:
 	};
 };
 
+typedef std::vector<const config *> faction_list;
+/** Picks the first faction with the greater amount of data matching the criteria. */
+int find_suitable_faction(faction_list const &fl, const config &side);
+
 }
 
 #endif
