@@ -1,11 +1,12 @@
-/* $Id: multiplayer_wait.cpp 34750 2009-04-12 04:27:54Z alink $ */
+/* $Id: multiplayer_wait.cpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
-   Copyright (C) 2007 - 2009
+   Copyright (C) 2007 - 2011
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,19 +16,23 @@
 #include "global.hpp"
 
 #include "dialogs.hpp"
+#include "foreach.hpp"
 #include "gettext.hpp"
+#include "game_preferences.hpp"
+#include "gui/dialogs/transient_message.hpp"
+#include "game_display.hpp"
 #include "leader_list.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
 #include "multiplayer_wait.hpp"
 #include "statistics.hpp"
+#include "wml_exception.hpp"
 #include "wml_separators.hpp"
 #include "formula_string_utils.hpp"
 
-
-#define DBG_NW LOG_STREAM(debug, network)
-#define LOG_NW LOG_STREAM(info, network)
-#define ERR_NW LOG_STREAM(err, network)
+static lg::log_domain log_network("network");
+#define DBG_NW LOG_STREAM(debug, log_network)
+#define LOG_NW LOG_STREAM(info, log_network)
 
 namespace {
 const SDL_Rect leader_pane_position = {-260,-370,260,370};
@@ -37,7 +42,7 @@ const int leader_pane_border = 10;
 namespace mp {
 
 wait::leader_preview_pane::leader_preview_pane(game_display& disp,
-		const config::child_list& side_list, int color) :
+		const std::vector<const config *> &side_list, int color) :
 	gui::preview_pane(disp.video()),
 	side_list_(side_list),
 	color_(color),
@@ -46,7 +51,7 @@ wait::leader_preview_pane::leader_preview_pane(game_display& disp,
 	leaders_(side_list, &leader_combo_, &gender_combo_),
 	selection_(0)
 {
-	leaders_.set_colour(color_);
+	leaders_.set_color(color_);
 	set_location(leader_pane_position);
 }
 
@@ -64,13 +69,14 @@ void wait::leader_preview_pane::draw_contents()
 {
 	bg_restore();
 
-	surface const screen = video().getSurface();
+	surface screen = video().getSurface();
 
 	SDL_Rect const &loc = location();
-	const SDL_Rect area = { loc.x + leader_pane_border, loc.y + leader_pane_border,
-	                        loc.w - leader_pane_border * 2, loc.h - leader_pane_border * 2 };
-	SDL_Rect clip_area = area;
-	const clip_rect_setter clipper(screen,clip_area);
+	const SDL_Rect area = create_rect(loc.x + leader_pane_border
+			, loc.y + leader_pane_border
+			, loc.w - leader_pane_border * 2
+			, loc.h - leader_pane_border * 2);
+	const clip_rect_setter clipper(screen, &area);
 
 	if(selection_ < side_list_.size()) {
 		const config& side = *side_list_[selection_];
@@ -80,7 +86,7 @@ void wait::leader_preview_pane::draw_contents()
 		const std::vector<std::string> recruit_list = utils::split(recruits);
 		std::ostringstream recruit_string;
 
-		if(faction[0] == font::IMAGE) {
+		if(!faction.empty() && faction[0] == font::IMAGE) {
 			std::string::size_type p = faction.find_first_of(COLUMN_SEPARATOR);
 			if(p != std::string::npos && p < faction.size())
 				faction = faction.substr(p+1);
@@ -88,35 +94,24 @@ void wait::leader_preview_pane::draw_contents()
 		std::string leader = leaders_.get_leader();
 		std::string gender = leaders_.get_gender();
 
-		unit_type_data::unit_type_map_wrapper& utypes = unit_type_data::types();
-		std::string leader_name;
 		std::string image;
 
-		const unit_type* ut;
-		const unit_type* utg;
+		const unit_type *ut = unit_types.find(leader);
 
-		if (utypes.unit_type_exists(leader) && leader != "random") {
-			ut = &(utypes.find_unit_type(leader)->second);
-			if (!gender.empty()) {
-				if (gender == "female")
-					utg = &(ut->get_gender_unit_type(unit_race::FEMALE));
-				else
-					utg = &(ut->get_gender_unit_type(unit_race::MALE));
-			} else
-				utg = ut;
+		if (ut) {
+			const unit_type &utg = ut->get_gender_unit_type(gender);
 
-			leader_name = utg->type_name();
-			image = utg->image() + leaders_.get_RC_suffix(utg->flag_rgb());
+			image = utg.image() + leaders_.get_RC_suffix(utg.flag_rgb());
 		}
 
 		for(std::vector<std::string>::const_iterator itor = recruit_list.begin();
 				itor != recruit_list.end(); ++itor) {
+			const unit_type *rt = unit_types.find(*itor);
+			if (!rt) continue;
 
-			if (utypes.unit_type_exists(leader)) {
-				if(itor != recruit_list.begin())
-					recruit_string << ", ";
-				recruit_string << utypes.find_unit_type(*itor)->second.type_name();
-			}
+			if(itor != recruit_list.begin())
+				recruit_string << ", ";
+			recruit_string << rt->type_name();
 		}
 
 		SDL_Rect image_rect = {area.x,area.y,0,0};
@@ -126,15 +121,15 @@ void wait::leader_preview_pane::draw_contents()
 		if(!unit_image.null()) {
 			image_rect.w = unit_image->w;
 			image_rect.h = unit_image->h;
-			SDL_BlitSurface(unit_image,NULL,screen,&image_rect);
+			sdl_blit(unit_image,NULL,screen,&image_rect);
 		}
 
-		font::draw_text(&video(),area,font::SIZE_PLUS,font::NORMAL_COLOUR,faction,area.x + 110, area.y + 60);
-		const SDL_Rect leader_rect = font::draw_text(&video(),area,font::SIZE_SMALL,font::NORMAL_COLOUR,
+		font::draw_text(&video(),area,font::SIZE_PLUS,font::NORMAL_COLOR,faction,area.x + 110, area.y + 60);
+		const SDL_Rect leader_rect = font::draw_text(&video(),area,font::SIZE_SMALL,font::NORMAL_COLOR,
 				_("Leader: "),area.x, area.y + 110);
-		const SDL_Rect gender_rect = font::draw_text(&video(),area,font::SIZE_SMALL,font::NORMAL_COLOUR,
+		const SDL_Rect gender_rect = font::draw_text(&video(),area,font::SIZE_SMALL,font::NORMAL_COLOR,
 				_("Gender: "),area.x, leader_rect.y + 30 + (leader_rect.h - leader_combo_.height()) / 2);
-		font::draw_wrapped_text(&video(),area,font::SIZE_SMALL,font::NORMAL_COLOUR,
+		font::draw_wrapped_text(&video(),area,font::SIZE_SMALL,font::NORMAL_COLOR,
 				_("Recruits: ") + recruit_string.str(),area.x, area.y + 132 + 30 + (leader_rect.h - leader_combo_.height()) / 2,
 				area.w);
 		leader_combo_.set_location(leader_rect.x + leader_rect.w + 16, leader_rect.y + (leader_rect.h - leader_combo_.height()) / 2);
@@ -182,7 +177,7 @@ wait::wait(game_display& disp, const config& cfg,
 		mp::chat& c, config& gamelist) :
 	ui(disp, _("Game Lobby"), cfg, c, gamelist),
 	cancel_button_(disp.video(), _("Cancel")),
-	start_label_(disp.video(), _("Waiting for game to start..."), font::SIZE_SMALL, font::LOBBY_COLOUR),
+	start_label_(disp.video(), _("Waiting for game to start..."), font::SIZE_SMALL, font::LOBBY_COLOR),
 	game_menu_(disp.video(), std::vector<std::string>(), false, -1, -1, NULL, &gui::menu::bluebg_style),
 	level_(),
 	state_(),
@@ -203,7 +198,7 @@ void wait::join_game(bool observe)
 	//if we have got valid side data
 	//the first condition is to make sure that we don't have another
 	//WML message with a side-tag in it
-	while((level_.values.find("version") == level_.values.end()) || (level_.child("side") == NULL)) {
+	while (!level_.has_attribute("version") || !level_.child("side")) {
 		network::connection data_res = dialogs::network_receive_dialog(disp(),
 				_("Getting game data..."), level_);
 		if (!data_res) {
@@ -218,73 +213,88 @@ void wait::join_game(bool observe)
 	}
 
 	// Add the map name to the title.
-	append_to_title(": " + level_["name"]);
+	append_to_title(": " + level_["name"].t_str());
 
 	if (!observe) {
-		const config::child_list& sides_list = level_.get_children("side");
-
 		//search for an appropriate vacant slot. If a description is set
 		//(i.e. we're loading from a saved game), then prefer to get the side
 		//with the same description as our login. Otherwise just choose the first
 		//available side.
-		int side_choice = -1;
-		for(config::child_list::const_iterator s = sides_list.begin(); s != sides_list.end(); ++s) {
-			if((**s)["controller"] == "reserved" && (**s)["current_player"] == preferences::login())
+		const config *side_choice = NULL;
+		int side_num = -1, nb_sides = 0;
+		foreach (const config &sd, level_.child_range("side"))
+		{
+			if (sd["controller"] == "reserved" && sd["current_player"] == preferences::login())
 			{
-				side_choice = s - sides_list.begin();
+				side_choice = &sd;
+				side_num = nb_sides;
 				break;
 			}
-			if((**s)["controller"] == "network" && (**s)["id"].empty()) {
-				if (side_choice < 0)  // found the first empty side
-					side_choice = s - sides_list.begin();
-				if((**s)["current_player"] == preferences::login()) {
-					side_choice = s - sides_list.begin();
-					break;  // found the prefered one
+			if (sd["controller"] == "network" && sd["player_id"].empty())
+			{
+				if (!side_choice) { // found the first empty side
+					side_choice = &sd;
+					side_num = nb_sides;
+				}
+				if (sd["current_player"] == preferences::login()) {
+					side_choice = &sd;
+					side_num = nb_sides;
+					break;  // found the preferred one
 				}
 			}
+			++nb_sides;
 		}
-		if (static_cast<size_t>(side_choice) >= sides_list.size()) {
+		if (!side_choice) {
 			set_result(QUIT);
 			return;
 		}
 
-		const bool allow_changes = (*sides_list[side_choice])["allow_changes"] != "no";
+		bool allow_changes = (*side_choice)["allow_changes"].to_bool(true);
 
 		//if the client is allowed to choose their team, instead of having
 		//it set by the server, do that here.
 		std::string leader_choice, gender_choice;
-		size_t faction_choice = 0;
 
 		if(allow_changes) {
 			events::event_context context;
 
-			const config* era = level_.child("era");
+			const config &era = level_.child("era");
 			/** @todo Check whether we have the era. If we don't inform the user. */
-			if(era == NULL)
+			if (!era)
 				throw config::error(_("No era information found."));
-			const config::child_list& possible_sides =
-				era->get_children("multiplayer_side");
-			if(possible_sides.empty()) {
+			config::const_child_itors possible_sides = era.child_range("multiplayer_side");
+			if (possible_sides.first == possible_sides.second) {
 				set_result(QUIT);
 				throw config::error(_("No multiplayer sides found"));
 				return;
 			}
 
-			int color = side_choice;
-			const std::string color_str = (*sides_list[side_choice])["colour"];
+			int color = side_num;
+			const std::string color_str = (*side_choice)["color"];
 			if (!color_str.empty())
 				color = game_config::color_info(color_str).index() - 1;
 
+			std::vector<const config *> leader_sides;
+			foreach (const config &side, possible_sides) {
+				leader_sides.push_back(&side);
+			}
+
+			int forced_faction = find_suitable_faction(leader_sides, *side_choice);
+			if (forced_faction >= 0) {
+				const config *f = leader_sides[forced_faction];
+				leader_sides.clear();
+				leader_sides.push_back(f);
+			}
+
 			std::vector<std::string> choices;
-			for(config::child_list::const_iterator side =
-					possible_sides.begin(); side !=
-					possible_sides.end(); ++side)
+			foreach (const config *s, leader_sides)
 			{
-				const std::string& name = (**side)["name"];
-				const std::string& icon = (**side)["image"];
+				const config &side = *s;
+				const std::string &name = side["name"];
+				const std::string &icon = side["image"];
 
 				if (!icon.empty()) {
-					std::string rgb = (**side)["flag_rgb"];
+					std::string rgb = side["flag_rgb"];
 					if (rgb.empty())
 						rgb = "magenta";
 
@@ -296,29 +306,28 @@ void wait::join_game(bool observe)
 			}
 
 			std::vector<gui::preview_pane* > preview_panes;
-			leader_preview_pane leader_selector(disp(),
-					possible_sides, color);
+			leader_preview_pane leader_selector(disp(), leader_sides, color);
 			preview_panes.push_back(&leader_selector);
 
-			const int res = gui::show_dialog(disp(), NULL, _("Choose your faction:"), _("Starting position: ") + lexical_cast<std::string>(side_choice+1),
+			const int res = gui::show_dialog(disp(), NULL, _("Choose your faction:"), _("Starting position: ") + lexical_cast<std::string>(side_num + 1),
 						gui::OK_CANCEL, &choices, &preview_panes);
 			if(res < 0) {
 				set_result(QUIT);
 				return;
 			}
-			faction_choice = res;
+			const int faction_choice = res;
 			leader_choice = leader_selector.get_selected_leader();
 			gender_choice = leader_selector.get_selected_gender();
 
-			assert(faction_choice < possible_sides.size());
+			assert(static_cast<unsigned>(faction_choice) < leader_sides.size());
 
 			config faction;
 			config& change = faction.add_child("change_faction");
 			change["name"] = preferences::login();
-			change["faction"] = lexical_cast<std::string>(faction_choice);
+			change["faction"] = forced_faction >= 0 ? forced_faction : faction_choice;
 			change["leader"] = leader_choice;
 			change["gender"] = gender_choice;
-			network::send_data(faction, 0, true);
+			network::send_data(faction, 0);
 		}
 
 	}
@@ -333,10 +342,9 @@ const game_state& wait::get_state()
 
 void wait::start_game()
 {
-	const config* stats = level_.child("statistics");
-	if(stats != NULL) {
+	if (const config &stats = level_.child("statistics")) {
 		statistics::fresh_stats();
-		statistics::read_stats(*stats);
+		statistics::read_stats(stats);
 	}
 
 	/**
@@ -348,7 +356,7 @@ void wait::start_game()
 	 * and new way. (Of course it would be nice to unify the data
 	 * stored.)
 	 */
-	if(level_.child("player") == 0) {
+	if (!level_.child("player")) {
 		level_to_gamestate(level_, state_);
 	} else {
 
@@ -358,7 +366,7 @@ void wait::start_game()
 		// the old way, no clue why however. Code is a copy paste of
 		// playcampaign.cpp:576 which shows an 'Unknown scenario: '$scenario|'
 		// error. This seems to work and have no side effects....
-		if(!state_.scenario.empty() && state_.scenario != "null") {
+		if(!state_.classification().scenario.empty() && state_.classification().scenario != "null") {
 			DBG_NW << "Falling back to loading the old way.\n";
 			level_to_gamestate(level_, state_);
 		}
@@ -367,14 +375,6 @@ void wait::start_game()
 	LOG_NW << "starting game\n";
 }
 
-game_state& wait::request_snapshot(){
-	config cfg;
-
-	cfg.add_child("snapshot_request");
-
-	return state_;
-}
-
 void wait::layout_children(const SDL_Rect& rect)
 {
 	ui::layout_children(rect);
@@ -404,11 +404,11 @@ void wait::process_network_data(const config& data, const network::connection so
 	ui::process_network_data(data, sock);
 
 	if(data["message"] != "") {
-		/* GCC-3.3 needs a temp var otherwise compilation fails */
-		gui::dialog dlg(disp(),_("Response"),data["message"],gui::OK_ONLY);
-		dlg.show();
+		gui2::show_transient_message(disp().video()
+				, _("Response")
+				, data["message"]);
 	}
-	if(data["failed"] == "yes") {
+	if (data["failed"].to_bool()) {
 		set_result(QUIT);
 		return;
 	} else if(data.child("stop_updates")) {
@@ -420,16 +420,16 @@ void wait::process_network_data(const config& data, const network::connection so
 	} else if(data.child("leave_game")) {
 		set_result(QUIT);
 		return;
-	} else if(data.child("scenario_diff")) {
+	} else if (const config &c = data.child("scenario_diff")) {
 		LOG_NW << "received diff for scenario... applying...\n";
 		/** @todo We should catch config::error and then leave the game. */
-		level_.apply_diff(*data.child("scenario_diff"));
+		level_.apply_diff(c);
 		generate_menu();
 	} else if(data.child("side")) {
 		level_ = data;
 		LOG_NW << "got some sides. Current number of sides = "
-			<< level_.get_children("side").size() << ","
-			<< data.get_children("side").size() << "\n";
+			<< level_.child_count("side") << ','
+			<< data.child_count("side") << '\n';
 		generate_menu();
 	}
 }
@@ -442,68 +442,50 @@ void wait::generate_menu()
 	std::vector<std::string> details;
 	std::vector<std::string> playerlist;
 
-	const config::child_list& sides = level_.get_children("side");
-	for(config::child_list::const_iterator s = sides.begin(); s != sides.end(); ++s) {
-		const config& sd = **s;
-
-		if(sd["allow_player"] == "no") {
+	foreach (const config &sd, level_.child_range("side"))
+	{
+		if (!sd["allow_player"].to_bool(true)) {
 			continue;
 		}
 
-		// 1.6.0 incorrectly uses sd["description"] which is empty
-		// std::string description = sd["description"];
-		std::string description = sd["name"];
-		const std::string faction_id = sd["id"];
+		std::string description = sd["user_description"];
+		const std::string faction_id = sd["player_id"];
 
-		// 1.6.0 incorrectly uses sd["name"] which is the player's name
-		// t_string side_name = sd["name"];
-
-		// 1.6.0 host will not send this to us, so the column will be empty
 		t_string side_name = sd["faction_name"];
-		
 		std::string leader_type = sd["type"];
 		std::string gender_id = sd["gender"];
 
 		// Hack: if there is a unit which can recruit, use it as a
 		// leader. Necessary to display leader information when loading
 		// saves.
-		config::const_child_itors side_units = sd.child_range("unit");
-		for(;side_units.first != side_units.second; ++side_units.first) {
-			if(utils::string_bool((**side_units.first)["canrecruit"], false)) {
-				leader_type = (**side_units.first)["type"];
+		foreach (const config &side_unit, sd.child_range("unit"))
+		{
+			if (side_unit["canrecruit"].to_bool()) {
+				leader_type = side_unit["type"].str();
 				break;
 			}
 		}
 
-		if(!sd["id"].empty())
-			playerlist.push_back(sd["id"]);
+		if(!sd["player_id"].empty())
+			playerlist.push_back(sd["player_id"]);
 
 		std::string leader_name;
 		std::string leader_image;
-		unit_type_data::unit_type_map_wrapper& utypes = unit_type_data::types();
-		const unit_type* ut;
-		const unit_type* utg;
 
-		if (utypes.unit_type_exists(leader_type) && leader_type != "random") {
-			ut = &(utypes.find_unit_type(leader_type)->second);
-			if (!gender_id.empty()) {
-				if (gender_id == "female")
-					utg = &(ut->get_gender_unit_type(unit_race::FEMALE));
-				else
-					// FIXME: this will make it look male, even if it's random. But all this
-					// code will be wiped out when the MP UI gets unified, anyway.
-					utg = &(ut->get_gender_unit_type(unit_race::MALE));
-			} else
-				utg = ut;
+		const unit_type *ut = unit_types.find(leader_type);
 
-			leader_name = utg->type_name();
+		if (ut) {
+			const unit_type &utg = ut->get_gender_unit_type(gender_id);
+
+			leader_name = utg.type_name();
 #ifdef LOW_MEM
-			leader_image = utg->image();
+			leader_image = utg.image();
 #else
-			std::string RCcolor = sd["colour"];
+			std::string RCcolor = sd["color"];
+
 			if (RCcolor.empty())
-				RCcolor = sd["side"];
-			leader_image = utg->image() + std::string("~RC(") + std::string(utg->flag_rgb() + ">" + RCcolor + ")");
+				RCcolor = sd["side"].str();
+			leader_image = utg.image() + std::string("~RC(") + std::string(utg.flag_rgb() + ">" + RCcolor + ")");
 #endif
 		} else {
 			leader_image = leader_list_manager::random_enemy_picture;
@@ -532,10 +514,10 @@ void wait::generate_menu()
 			str << _("(") << leader_name << _(")");
 		str << COLUMN_SEPARATOR;
 		// Don't show gold for saved games
-		if(sd["allow_changes"] == "yes")
-			str << sd["gold"] << ' ' << _("Gold") << COLUMN_SEPARATOR;
+		if (sd["allow_changes"].to_bool())
+			str << sd["gold"] << ' ' << _n("multiplayer_starting_gold^Gold", "multiplayer_starting_gold^Gold", sd["gold"].to_int()) << COLUMN_SEPARATOR;
 
-		int income_amt = lexical_cast_default<int>(sd["income"], 0);
+		int income_amt = sd["income"];
 		if(income_amt != 0){
 			str << _("(") << _("Income") << ' ';
 			if(income_amt > 0)
@@ -543,22 +525,23 @@ void wait::generate_menu()
 			str << sd["income"] << _(")");
 		}
 
-		str	<< COLUMN_SEPARATOR << t_string().from_serialized(sd["user_team_name"].str());
-		int disp_color = lexical_cast_default<int>(sd["colour"], 0) - 1;
-		if(!sd["colour"].empty()) {
+		str << COLUMN_SEPARATOR << t_string::from_serialized(sd["user_team_name"].str());
+
+		int disp_color = sd["color"];
+		if(!sd["color"].empty()) {
 			try {
-				disp_color = game_config::color_info(sd["colour"]).index() - 1;
+				disp_color = game_config::color_info(sd["color"]).index();
 			} catch(config::error&) {
 				//ignore
 			}
 		} else {
 			/**
-			 * @todo we fall back to the side colour, but that's ugly rather
-			 * make the colour mandatory in 1.5.
+			 * @todo we fall back to the side color, but that's ugly rather
+			 * make the color mandatory in 1.5.
 			 */
-			disp_color = lexical_cast_default<int>(sd["side"], 0) - 1;
+			disp_color = sd["side"];
 		}
-		str << COLUMN_SEPARATOR << get_colour_string(disp_color);
+		str << COLUMN_SEPARATOR << get_color_string(disp_color - 1);
 		details.push_back(str.str());
 	}
 
@@ -566,7 +549,7 @@ void wait::generate_menu()
 
 	// Uses the actual connected player list if we do not have any
 	// "gamelist" user data
-	if (gamelist().child("user") == NULL) {
+	if (!gamelist().child("user")) {
 		set_user_list(playerlist, true);
 	}
 }
