@@ -1,11 +1,12 @@
-/* $Id: network.cpp 33199 2009-03-01 07:43:22Z mordante $ */
+/* $Id: network.cpp 51004 2011-09-03 13:13:58Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- * @file network.cpp
+ * @file
  * Networking
  */
 
@@ -22,7 +23,10 @@
 #include "gettext.hpp"
 #include "log.hpp"
 #include "network_worker.hpp"
+#include "serialization/string_utils.hpp"
 #include "thread.hpp"
+#include "util.hpp"
+#include "config.hpp"
 
 #include "filesystem.hpp"
 
@@ -32,6 +36,7 @@
 #include <iomanip>
 #include <set>
 #include <cstring>
+#include <stdexcept>
 
 #include <signal.h>
 #if defined(_WIN32) || defined(__WIN32__) || defined (WIN32)
@@ -52,10 +57,11 @@
 #define SOCKET int
 #endif
 
-#define DBG_NW LOG_STREAM(debug, network)
-#define LOG_NW LOG_STREAM(info, network)
-#define WRN_NW LOG_STREAM(warn, network)
-#define ERR_NW LOG_STREAM(err, network)
+static lg::log_domain log_network("network");
+#define DBG_NW LOG_STREAM(debug, log_network)
+#define LOG_NW LOG_STREAM(info, log_network)
+#define WRN_NW LOG_STREAM(warn, log_network)
+#define ERR_NW LOG_STREAM(err, log_network)
 // Only warnings and not errors to avoid DoS by log flooding
 
 namespace {
@@ -157,7 +163,7 @@ static void check_timeout()
 			<< "' Time since last ping: " << now - last_ping << "s\n";
 	// Reset last_ping if we didn't check for the last 10s.
 	if (last_ping_check + 10 <= now) last_ping = now;
-	if (static_cast<time_t>(last_ping + network::ping_timeout) <= now) {
+	if (static_cast<time_t>(last_ping + network::ping_interval + network::ping_timeout) <= now) {
 
 		time_t timeout = now - last_ping;
 		ERR_NW << "No server ping since " << timeout
@@ -218,7 +224,7 @@ connection_stats get_connection_stats(connection connection_num)
 	return connection_stats(get_send_stats(connection_num).total,get_receive_stats(connection_num).total,details.connected_at);
 }
 
-error::error(const std::string& msg, connection sock) : message(msg), socket(sock)
+error::error(const std::string& msg, connection sock) : game::error(msg), socket(sock)
 {
 	if(socket) {
 		bad_sockets.insert(socket);
@@ -244,12 +250,6 @@ manager::manager(size_t min_threads, size_t max_threads) : free_(true)
 		return;
 	}
 
-	// On Unix-based systems, set sigpipe to be ignored
-#if !(defined(_WIN32) || defined(__APPLE__) || defined(__AMIGAOS4__))
-	WRN_NW << "ignoring SIGPIPE\n";
-	signal(SIGPIPE,SIG_IGN);
-#endif
-
 	if(SDLNet_Init() == -1) {
 		ERR_NW << "could not initialize SDLNet; throwing error...\n";
 		throw error(SDL_GetError());
@@ -278,6 +278,32 @@ void set_raw_data_only()
 	network_worker_pool::set_raw_data_only();
 }
 
+// --- Proxy methods
+void enable_connection_through_proxy()
+{
+    throw std::runtime_error("Proxy not available while using SDL_net. Use ANA instead.");
+}
+void set_proxy_address ( const std::string& )
+{
+    throw std::runtime_error("Proxy not available while using SDL_net. Use ANA instead.");
+}
+
+void set_proxy_port    ( const std::string& )
+{
+    throw std::runtime_error("Proxy not available while using SDL_net. Use ANA instead.");
+}
+
+void set_proxy_user    ( const std::string& )
+{
+    throw std::runtime_error("Proxy not available while using SDL_net. Use ANA instead.");
+}
+
+void set_proxy_password( const std::string& )
+{
+    throw std::runtime_error("Proxy not available while using SDL_net. Use ANA instead.");
+}
+// --- End Proxy methods
+
 server_manager::server_manager(int port, CREATE_SERVER create_server) : free_(false), connection_(0)
 {
 	if(create_server != NO_SERVER && !server_socket) {
@@ -369,14 +395,15 @@ void connect_operation::run()
 	char* const hostname = host_.empty() ? NULL : const_cast<char*>(host_.c_str());
 	IPaddress ip;
 	if(SDLNet_ResolveHost(&ip,hostname,port_) == -1) {
-		error_ = "Could not connect to host";
+		error_ = N_("Could not connect to host.");
 		return;
 	}
 
 	TCPsocket sock = SDLNet_TCP_Open(&ip);
 	if(!sock) {
-		error_ = hostname == NULL ? "Could not bind to port" :
-		                            "Could not connect to host";
+		error_ = hostname == NULL
+				? "Could not bind to port"
+				: N_("Could not connect to host.");
 		return;
 	}
 
@@ -386,7 +413,8 @@ void connect_operation::run()
 	{
 	_TCPsocket* raw_sock = reinterpret_cast<_TCPsocket*>(sock);
 	int no = 0;
-	setsockopt(raw_sock->channel, IPPROTO_TCP, TCP_NODELAY, (char*)&no, sizeof(no));
+	setsockopt(raw_sock->channel,
+			IPPROTO_TCP, TCP_NODELAY, reinterpret_cast<char*>(&no), sizeof(no));
 	}
 #endif
 
@@ -428,9 +456,13 @@ void connect_operation::run()
 	}
 
 	// Send data telling the remote host that this is a new connection
-	char buf[4] ALIGN_4;
-	SDLNet_Write32(0,buf);
-	const int nbytes = SDLNet_TCP_Send(sock,buf,4);
+	union
+	{
+	char data[4] ALIGN_4;
+	Uint32 num;
+	} buf;
+	SDLNet_Write32(0, &buf);
+	const int nbytes = SDLNet_TCP_Send(sock,&buf,4);
 	if(nbytes != 4) {
 		SDLNet_TCP_Close(sock);
 		error_ = "Could not send initial handshake";
@@ -488,6 +520,70 @@ connection connect(const std::string& host, int port, threading::waiter& waiter)
 	return static_cast<connect_operation*>(op.get())->result();
 }
 
+namespace {
+
+connection accept_connection_pending(std::vector<TCPsocket>& pending_sockets,
+	SDLNet_SocketSet& pending_socket_set)
+{
+	DBG_NW << "pending socket activity...\n";
+
+	std::vector<TCPsocket>::iterator i = pending_sockets.begin();
+	while (i != pending_sockets.end() && !SDLNet_SocketReady(*i)) ++i;
+
+	if (i == pending_sockets.end()) return 0;
+
+	// Receive the 4 bytes telling us if they're a new connection
+	// or trying to recover a connection
+	union
+	{
+	char data[4] ALIGN_4;
+	Uint32 num;
+	} buf;
+
+	const TCPsocket psock = *i;
+	SDLNet_TCP_DelSocket(pending_socket_set,psock);
+	pending_sockets.erase(i);
+
+	DBG_NW << "receiving data from pending socket...\n";
+
+	const int len = SDLNet_TCP_Recv(psock,&buf,4);
+	if(len != 4) {
+		WRN_NW << "pending socket disconnected\n";
+		SDLNet_TCP_Close(psock);
+		return 0;
+	}
+
+	const int handle = SDLNet_Read32(&buf);
+
+	DBG_NW << "received handshake from client: '" << handle << "'\n";
+
+	const int res = SDLNet_TCP_AddSocket(socket_set,psock);
+	if(res == -1) {
+		ERR_NW << "SDLNet_GetError() is " << SDLNet_GetError() << "\n";
+		SDLNet_TCP_Close(psock);
+
+		throw network::error(_("Could not add socket to socket set"));
+	}
+
+	const connection connect = create_connection(psock,"",0);
+
+	// Send back their connection number
+	SDLNet_Write32(connect, &buf);
+	const int nbytes = SDLNet_TCP_Send(psock,&buf,4);
+	if(nbytes != 4) {
+		SDLNet_TCP_DelSocket(socket_set,psock);
+		SDLNet_TCP_Close(psock);
+		remove_connection(connect);
+		throw network::error(_("Could not send initial handshake"));
+	}
+
+	waiting_sockets.insert(connect);
+	sockets.push_back(connect);
+	return connect;
+}
+
+} //anon namespace
+
 connection accept_connection()
 {
 	if(!server_socket) {
@@ -508,7 +604,7 @@ connection accept_connection()
 	const TCPsocket sock = SDLNet_TCP_Accept(server_socket);
 	if(sock) {
 		DBG_NW << "received connection. Pending handshake...\n";
-		pending_sockets.push_back(sock);
+
 		if(pending_socket_set == 0) {
 			pending_socket_set = SDLNet_AllocSocketSet(32);
 		}
@@ -516,11 +612,16 @@ connection accept_connection()
 		if(pending_socket_set != 0) {
 			int res = SDLNet_TCP_AddSocket(pending_socket_set,sock);
 
-			if (res == -1)
-			{
+			if (res != -1) {
+				pending_sockets.push_back(sock);
+			} else {
 				ERR_NW << "Pending socket set is full! Disconnecting " << sock << " connection\n";
+				ERR_NW << "SDLNet_GetError() is " << SDLNet_GetError() << "\n";
+
 				SDLNet_TCP_Close(sock);
 			}
+		} else {
+			ERR_NW << "Error in SDLNet_AllocSocketSet\n";
 		}
 	}
 
@@ -533,60 +634,7 @@ connection accept_connection()
 		return 0;
 	}
 
-	DBG_NW << "pending socket activity...\n";
-
-	for(std::vector<TCPsocket>::iterator i = pending_sockets.begin(); i != pending_sockets.end(); ++i) {
-		if(!SDLNet_SocketReady(*i)) {
-			continue;
-		}
-
-		// Receive the 4 bytes telling us if they're a new connection
-		// or trying to recover a connection
-		char buf[4] ALIGN_4;
-
-		const TCPsocket sock = *i;
-		SDLNet_TCP_DelSocket(pending_socket_set,sock);
-		pending_sockets.erase(i);
-
-		DBG_NW << "receiving data from pending socket...\n";
-
-		const int len = SDLNet_TCP_Recv(sock,buf,4);
-		if(len != 4) {
-			WRN_NW << "pending socket disconnected\n";
-			SDLNet_TCP_Close(sock);
-			return 0;
-		}
-
-		const int handle = SDLNet_Read32(buf);
-
-		DBG_NW << "received handshake from client: '" << handle << "'\n";
-
-		const int res = SDLNet_TCP_AddSocket(socket_set,sock);
-		if(res == -1) {
-			SDLNet_TCP_Close(sock);
-
-			throw network::error(_("Could not add socket to socket set"));
-		}
-
-
-		const connection connect = create_connection(sock,"",0);
-
-		// Send back their connection number
-		SDLNet_Write32(connect,buf);
-		const int nbytes = SDLNet_TCP_Send(sock,buf,4);
-		if(nbytes != 4) {
-			SDLNet_TCP_DelSocket(socket_set,sock);
-			SDLNet_TCP_Close(sock);
-			remove_connection(connect);
-			throw network::error(_("Could not send initial handshake"));
-		}
-
-		waiting_sockets.insert(connect);
-		sockets.push_back(connect);
-		return connect;
-	}
-
-	return 0;
+	return accept_connection_pending(pending_sockets, pending_socket_set);
 }
 
 bool disconnect(connection s)
@@ -649,7 +697,8 @@ connection receive_data(config& cfg, connection connection_num, unsigned int tim
 {
 	unsigned int start_ticks = SDL_GetTicks();
 	while(true) {
-		const connection res = receive_data(cfg,connection_num,(bool*)0 , bandwidth_in);
+		const connection res = receive_data(
+				cfg,connection_num, bandwidth_in);
 		if(res != 0) {
 			return res;
 		}
@@ -667,7 +716,7 @@ connection receive_data(config& cfg, connection connection_num, unsigned int tim
 	return 0;
 }
 
-connection receive_data(config& cfg, connection connection_num, bool* gzipped, bandwidth_in_ptr* bandwidth_in)
+connection receive_data(config& cfg, connection connection_num, bandwidth_in_ptr* bandwidth_in)
 {
 	if(!socket_set) {
 		return 0;
@@ -699,13 +748,15 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped, b
 			// See if this socket is still waiting for it to be assigned its remote handle.
 			// If it is, then the first 4 bytes must be the remote handle.
 			if(is_pending_remote_handle(*i)) {
-				char buf[4] ALIGN_4;
-				int len = SDLNet_TCP_Recv(sock,buf,4);
+				union {
+				char data[4] ALIGN_4;
+				} buf;
+				int len = SDLNet_TCP_Recv(sock,&buf,4);
 				if(len != 4) {
 					throw error("Remote host disconnected",*i);
 				}
 
-				const int remote_handle = SDLNet_Read32(buf);
+				const int remote_handle = SDLNet_Read32(&buf);
 				set_remote_handle(*i,remote_handle);
 
 				continue;
@@ -727,7 +778,7 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped, b
 	{
 		bandwidth_in = &temp;
 	}
-	sock = network_worker_pool::get_received_data(sock,cfg, gzipped, *bandwidth_in);
+	sock = network_worker_pool::get_received_data(sock,cfg, *bandwidth_in);
 	if (sock == NULL) {
 		if (!is_server() && last_ping != 0 && ping_timeout != 0)
 		{
@@ -766,8 +817,7 @@ connection receive_data(config& cfg, connection connection_num, bool* gzipped, b
 	waiting_sockets.insert(result);
 	if(!is_server()) {
 		const time_t& now = time(NULL);
-		const string_map::const_iterator ping = cfg.values.find("ping");
-		if (ping != cfg.values.end()) {
+		if (cfg.has_attribute("ping")) {
 			LOG_NW << "Lag: " << (now - lexical_cast<time_t>(cfg["ping"])) << "\n";
 			last_ping = now;
 		} else if (last_ping != 0) {
@@ -809,13 +859,15 @@ connection receive_data(std::vector<char>& buf, bandwidth_in_ptr* bandwidth_in)
 			// See if this socket is still waiting for it to be assigned its remote handle.
 			// If it is, then the first 4 bytes must be the remote handle.
 			if(is_pending_remote_handle(*i)) {
-				char buf[4] ALIGN_4;
-				int len = SDLNet_TCP_Recv(sock,buf,4);
+				union {
+				char data[4] ALIGN_4;
+				} buf;
+				int len = SDLNet_TCP_Recv(sock,&buf,4);
 				if(len != 4) {
 					throw error("Remote host disconnected",*i);
 				}
 
-				const int remote_handle = SDLNet_Read32(buf);
+				const int remote_handle = SDLNet_Read32(&buf);
 				set_remote_handle(*i,remote_handle);
 
 				continue;
@@ -964,9 +1016,9 @@ std::string get_bandwidth_stats(int hour)
 
 	ss << "Hour stat starting from " << hour << "\n " << std::left << std::setw(bandwidth_stats::type_width) <<  "Type of packet" << "| "
 		<< std::setw(bandwidth_stats::packet_width)<< "out #"  << "| "
-		<< std::setw(bandwidth_stats::bytes_width) << "out kb" << "| " /* Are these bytes or bits? base10 or base2? */
+		<< std::setw(bandwidth_stats::bytes_width) << "out KiB" << "| "
 		<< std::setw(bandwidth_stats::packet_width)<< "in #"  << "| "
-		<< std::setw(bandwidth_stats::bytes_width) << "in kb" << "\n";
+		<< std::setw(bandwidth_stats::bytes_width) << "in KiB" << "\n";
 
 	bandwidth_stats_output outputer(ss);
 	std::for_each(hour_stats[hour].begin(), hour_stats[hour].end(), outputer);
@@ -1014,12 +1066,7 @@ void send_file(const std::string& filename, connection connection_num, const std
 
 }
 
-/**
- * @todo Note the gzipped parameter should be removed later, we want to send
- * all data gzipped. This can be done once the campaign server is also updated
- * to work with gzipped data.
- */
-size_t send_data(const config& cfg, connection connection_num, const bool gzipped, const std::string& packet_type)
+size_t send_data(const config& cfg, connection connection_num, const std::string& packet_type)
 {
 	DBG_NW << "in send_data()...\n";
 
@@ -1038,7 +1085,7 @@ size_t send_data(const config& cfg, connection connection_num, const bool gzippe
 		for(sockets_list::const_iterator i = sockets.begin();
 		    i != sockets.end(); ++i) {
 			DBG_NW << "server socket: " << server_socket << "\ncurrent socket: " << *i << "\n";
-			size = send_data(cfg,*i, gzipped, packet_type);
+			size = send_data(cfg,*i, packet_type);
 		}
 		return size;
 	}
@@ -1051,7 +1098,7 @@ size_t send_data(const config& cfg, connection connection_num, const bool gzippe
 	}
 
 	LOG_NW << "SENDING to: " << connection_num << ": " << cfg;
-	return network_worker_pool::queue_data(info->second.sock, cfg, gzipped, packet_type);
+	return network_worker_pool::queue_data(info->second.sock, cfg, packet_type);
 }
 
 void send_raw_data(const char* buf, int len, connection connection_num, const std::string& packet_type)
@@ -1089,15 +1136,14 @@ void process_send_queue(connection, size_t)
 	check_error();
 }
 
-/** @todo Note the gzipped parameter should be removed later. */
-void send_data_all_except(const config& cfg, connection connection_num, const bool gzipped, const std::string& packet_type)
+void send_data_all_except(const config& cfg, connection connection_num, const std::string& packet_type)
 {
 	for(sockets_list::const_iterator i = sockets.begin(); i != sockets.end(); ++i) {
 		if(*i == connection_num) {
 			continue;
 		}
 
-		send_data(cfg,*i, gzipped, packet_type);
+		send_data(cfg,*i, packet_type);
 	}
 }
 
@@ -1125,7 +1171,9 @@ statistics get_send_stats(connection handle)
 }
 statistics get_receive_stats(connection handle)
 {
-	return network_worker_pool::get_current_transfer_stats(handle == 0 ? get_socket(sockets.back()) : get_socket(handle)).second;
+    const statistics result = network_worker_pool::get_current_transfer_stats(handle == 0 ? get_socket(sockets.back()) : get_socket(handle)).second;
+
+	return result;
 }
 
 } // end namespace network
