@@ -1,13 +1,14 @@
-/* $Id: playsingle_controller.cpp 42333 2010-04-30 23:22:12Z loonycyborg $ */
+/* $Id: playsingle_controller.cpp 52082 2011-11-25 01:06:27Z anonymissimus $ */
 
 /*
-   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2011 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,7 +16,7 @@
 */
 
 /**
- *  @file playsingle_controller.cpp
+ *  @file
  *  Logic for single-player game.
  */
 
@@ -25,6 +26,7 @@
 #include "ai/manager.hpp"
 #include "ai/game_info.hpp"
 #include "ai/testing.hpp"
+#include "dialogs.hpp"
 #include "foreach.hpp"
 #include "game_end_exceptions.hpp"
 #include "game_events.hpp"
@@ -43,6 +45,8 @@
 #include "save_blocker.hpp"
 #include "soundsource.hpp"
 #include "storyscreen/interface.hpp"
+#include "whiteboard/manager.hpp"
+#include "util.hpp"
 
 static lg::log_domain log_engine("engine");
 #define ERR_NG LOG_STREAM(err, log_engine)
@@ -70,7 +74,7 @@ playsingle_controller::playsingle_controller(const config& level,
 		browse_ = linger_ = true;
 	}
 
-	ai::game_info ai_info(*gui_,map_,units_,teams_, tod_manager_, gamestate_);
+	ai::game_info ai_info;
 	ai::manager::set_ai_info(ai_info);
 	ai::manager::add_observer(this) ;
 }
@@ -100,17 +104,23 @@ void playsingle_controller::init_gui(){
 
 void playsingle_controller::recruit(){
 	if (!browse_)
-		menu_handler_.recruit(browse_, player_number_, mouse_handler_.get_last_hex());
+		menu_handler_.recruit(player_number_, mouse_handler_.get_last_hex());
+	else if (resources::whiteboard->is_active())
+		menu_handler_.recruit(resources::screen->viewing_side(), mouse_handler_.get_last_hex());
 }
 
 void playsingle_controller::repeat_recruit(){
 	if (!browse_)
 		menu_handler_.repeat_recruit(player_number_, mouse_handler_.get_last_hex());
+	else if (resources::whiteboard->is_active())
+		menu_handler_.repeat_recruit(resources::screen->viewing_side(), mouse_handler_.get_last_hex());
 }
 
 void playsingle_controller::recall(){
 	if (!browse_)
 		menu_handler_.recall(player_number_, mouse_handler_.get_last_hex());
+	else if (resources::whiteboard->is_active())
+		menu_handler_.recall(resources::screen->viewing_side(), mouse_handler_.get_last_hex());
 }
 
 void playsingle_controller::toggle_shroud_updates(){
@@ -138,13 +148,20 @@ void playsingle_controller::force_end_turn(){
 
 void playsingle_controller::check_end_level()
 {
-	if ((level_result_ == NONE) || linger_)
+	if (level_result_ == NONE || linger_)
+	{
+		team &t = teams_[gui_->viewing_team()];
+		if (!browse_ && t.objectives_changed()) {
+			dialogs::show_objectives(level_, t.objectives());
+			t.reset_objectives_changed();
+		}
 		return;
+	}
 	throw end_level_exception(level_result_);
 }
 
 void playsingle_controller::rename_unit(){
-	menu_handler_.rename_unit(mouse_handler_);
+	menu_handler_.rename_unit();
 }
 
 void playsingle_controller::create_unit(){
@@ -159,13 +176,12 @@ void playsingle_controller::label_terrain(bool team_only){
 	menu_handler_.label_terrain(mouse_handler_, team_only);
 }
 
-void playsingle_controller::continue_move(){
-	menu_handler_.continue_move(mouse_handler_, player_number_);
+void playsingle_controller::clear_labels(){
+	menu_handler_.clear_labels();
 }
 
-void playsingle_controller::add_waypoint(){
-	if (!browse_)
-		mouse_handler_.add_waypoint(mouse_handler_.get_last_hex());
+void playsingle_controller::continue_move(){
+	menu_handler_.continue_move(mouse_handler_, player_number_);
 }
 
 void playsingle_controller::unit_hold_position(){
@@ -183,7 +199,7 @@ void playsingle_controller::user_command(){
 }
 
 void playsingle_controller::custom_command(){
-	menu_handler_.custom_command(mouse_handler_, player_number_);
+	menu_handler_.custom_command();
 }
 
 void playsingle_controller::ai_formula(){
@@ -194,23 +210,52 @@ void playsingle_controller::clear_messages(){
 	menu_handler_.clear_messages();
 }
 
-#ifdef USRCMD2
-void playsingle_controller::user_command_2(){
-	menu_handler_.user_command_2();
+void playsingle_controller::whiteboard_toggle() {
+	resources::whiteboard->set_active(!resources::whiteboard->is_active());
+	//@todo Stop printing whiteboard help in the chat once we have better documentation/help
+	resources::whiteboard->print_help_once();
 }
 
-void playsingle_controller::user_command_3(){
-	menu_handler_.user_command_3();
+void playsingle_controller::whiteboard_execute_action(){
+	whiteboard_manager_->contextual_execute();
+}
+
+void playsingle_controller::whiteboard_execute_all_actions(){
+	whiteboard_manager_->execute_all_actions();
+}
+
+void playsingle_controller::whiteboard_delete_action(){
+	whiteboard_manager_->contextual_delete();
+}
+
+void playsingle_controller::whiteboard_bump_up_action()
+{
+	whiteboard_manager_->contextual_bump_up_action();
+}
+
+void playsingle_controller::whiteboard_bump_down_action()
+{
+	whiteboard_manager_->contextual_bump_down_action();
+}
+
+void playsingle_controller::whiteboard_suppose_dead()
+{
+	unit* curr_unit;
+	map_location loc;
+	{ wb::future_map future; //start planned unit map scope
+		curr_unit = &*menu_handler_.current_unit();
+		loc = curr_unit->get_location();
+	} // end planned unit map scope
+	whiteboard_manager_->save_suppose_dead(*curr_unit,loc);
 }
-#endif
 
 void playsingle_controller::report_victory(
 	std::ostringstream &report, int player_gold, int remaining_gold,
 	int finishing_bonus_per_turn, int turns_left, int finishing_bonus)
 {
-	end_level_data &end_level = get_end_level_data();
+	const end_level_data &end_level = get_end_level_data_const();
 	report << _("Remaining gold: ")
-		   << remaining_gold << "\n";
+		   << utils::half_signed_value(remaining_gold) << "\n";
 	if(end_level.gold_bonus) {
 		if (turns_left > -1) {
 			report << _("Early finish bonus: ")
@@ -222,15 +267,15 @@ void playsingle_controller::report_victory(
 				   << finishing_bonus << "\n";
 		}
 		report << _("Gold: ")
-		       << (remaining_gold + finishing_bonus);
+		       << utils::half_signed_value(remaining_gold + finishing_bonus);
 	}
 	if (remaining_gold > 0) {
 		report << '\n' << _("Carry over percentage: ") << end_level.carryover_percentage;
 	}
 	if(end_level.carryover_add) {
-		report << "\n<b>" << _("Bonus Gold: ") << player_gold <<"</b>";
+		report << "\n<b>" << _("Bonus Gold: ") << utils::half_signed_value(player_gold) <<"</b>";
 	} else {
-		report << "\n<b>" << _("Retained Gold: ") << player_gold << "</b>";
+		report << "\n<b>" << _("Retained Gold: ") << utils::half_signed_value(player_gold) << "</b>";
 	}
 
 	std::string goldmsg;
@@ -285,21 +330,10 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 	sound::commit_music_changes();
 
 	if(!skip_replay) {
-		foreach (const config &s, story) {
-			show_storyscreen(*gui_, vconfig(s, true), level_["name"]);
-		}
+		show_story(*gui_, level_["name"], story);
 	}
 	gui_->labels().read(level_);
 
-	// Find a list of 'items' (i.e. overlays) on the level, and add them
-	foreach (const config &overlay, level_.child_range("item"))
-	{
-		gui_->add_overlay(
-			map_location(overlay, resources::state_of_game),
-			overlay["image"], overlay["halo"], overlay["team_name"],
-			utils::string_bool(overlay["visible_in_fog"], true));
-	}
-
 	// Read sound sources
 	assert(soundsources_manager_ != NULL);
 	foreach (const config &s, level_.child_range("sound_source")) {
@@ -307,20 +341,24 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 		soundsources_manager_->add(spec);
 	}
 
-	set_victory_when_enemies_defeated(
-		utils::string_bool(level_["victory_when_enemies_defeated"], true));
+	set_victory_when_enemies_defeated(level_["victory_when_enemies_defeated"].to_bool(true));
 	end_level_data &end_level = get_end_level_data();
-	end_level.carryover_percentage = lexical_cast_default<int>(
-		level_["carryover_percentage"], game_config::gold_carryover_percentage);
-	end_level.carryover_add = utils::string_bool(level_["carryover_add"]);
+	end_level.carryover_percentage = level_["carryover_percentage"].to_int(game_config::gold_carryover_percentage);
+	end_level.carryover_add = level_["carryover_add"].to_bool();
+
+	bool past_prestart = false;
 
 	LOG_NG << "entering try... " << (SDL_GetTicks() - ticks_) << "\n";
 	try {
+
 		fire_prestart(!loading_game_);
 		init_gui();
 
+		past_prestart = true;
+
 		LOG_NG << "first_time..." << (recorder.is_skipping() ? "skipping" : "no skip") << "\n";
 
+		events::raise_draw_event();
 		fire_start(!loading_game_);
 		gui_->recalculate_minimap();
 
@@ -339,13 +377,9 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 		// if we loaded a save file in linger mode, skip to it.
 		if (linger_) {
 			//determine the bonus gold handling for this scenario
-			const config end_cfg = level_.child_or_empty("endlevel");
-			end_level.carryover_percentage = lexical_cast_default<int>(
-				level_["carryover_percentage"],
-				game_config::gold_carryover_percentage);
-			end_level.carryover_add = utils::string_bool(level_["carryover_add"]);
-			end_level.gold_bonus = utils::string_bool(end_cfg["bonus"], true);
+			end_level.read(level_.child_or_empty("endlevel"));
 			end_level.carryover_report = false;
+			end_level.disabled = true;
 			throw end_level_exception(SKIP_TO_LINGER);
 		}
 
@@ -358,15 +392,27 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 			save = true;
 		} //end for loop
 
-	} catch(game::load_game_exception& lge) {
+#ifdef _MSC_VER
+//MSVC claims that const game::load_game_exception& lge would be unreferenced...
+#pragma warning (push)
+#pragma warning (disable : 4101)
+#endif
+	} catch(const game::load_game_exception& lge) {
 		// Loading a new game is effectively a quit.
 		//
 		if (lge.game != "") {
 			gamestate_ = game_state();
 		}
-
-		throw lge;
+		throw;
 	} catch (end_level_exception &end_level_exn) {
+		if(!past_prestart) {
+			draw_solid_tinted_rectangle(
+				0, 0, gui_->video().getx(), gui_->video().gety(), 0, 0, 0, 1.0,
+				gui_->video().getSurface()
+			);
+			update_rect(0, 0, gui_->video().getx(), gui_->video().gety());
+		}
+
 		ai_testing::log_game_end();
 		LEVEL_RESULT end_level_result = end_level_exn.result;
 		if (!end_level.custom_endlevel_music.empty()) {
@@ -400,7 +446,7 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 				info["type"] = "termination";
 				info["condition"] = "game over";
 				info["result"] = gamestate_.classification().completion;
-				network::send_data(cfg, 0, true);
+				network::send_data(cfg, 0);
 			} else {
 				gui2::show_transient_message(gui_->video(),_("Game Over"),
 									_("The game is over."));
@@ -409,13 +455,11 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 		}
 
 		if (end_level_result == QUIT) {
-
 			return QUIT;
 		}
 		else if (end_level_result == DEFEAT)
 		{
 			gamestate_.classification().completion = "defeat";
-
 			game_events::fire("defeat");
 
 			if (!obs) {
@@ -453,11 +497,11 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 			LOG_NG << "Add units that survived the scenario to the recall list.\n";
 			for(unit_map::iterator un = units_.begin(); un != units_.end(); ++un) {
 
-				if(teams_[un->second.side()-1].persistent()) {
-					LOG_NG << "Added unit " << un->second.id() << ", " << un->second.name() << "\n";
-					un->second.new_turn();
-					un->second.new_scenario();
-					teams_[un->second.side()-1].recall_list().push_back(un->second);
+				if (teams_[un->side() - 1].persistent()) {
+					LOG_NG << "Added unit " << un->id() << ", " << un->name() << "\n";
+					un->new_turn();
+					un->new_scenario();
+					teams_[un->side() - 1].recall_list().push_back(*un);
 				}
 			}
 			//store all units that survived (recall list for the next scenario) in snapshot
@@ -485,8 +529,7 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 		}
 
 		savegame::game_savegame save(gamestate_, *gui_, to_config(), preferences::compress_saves());
-		/** @todo FIXME: remove the odd linebreak after the string freeze */
-		save.save_game_interactive(gui_->video(), _("A network disconnection has occurred, and the game\ncannot continue. Do you want to save the game?"), gui::YES_NO);
+		save.save_game_interactive(gui_->video(), _("A network disconnection has occurred, and the game cannot continue. Do you want to save the game?"), gui::YES_NO);
 		if(disconnect) {
 			throw network::error();
 		} else {
@@ -496,9 +539,13 @@ LEVEL_RESULT playsingle_controller::play_scenario(
 
 	return QUIT;
 }
+#ifdef _MSC_VER
+#pragma warning (pop)
+#endif
 
 void playsingle_controller::play_turn(bool save)
 {
+	resources::whiteboard->on_gamestate_change();
 	gui_->new_turn();
 	gui_->invalidate_game_status();
 	events::raise_draw_event();
@@ -518,7 +565,7 @@ void playsingle_controller::play_turn(bool save)
 			init_side(player_number_ - 1);
 		} catch (end_turn_exception) {
 			if (current_team().is_network() == false) {
-				turn_info turn_data(player_number_, replay_sender_, undo_stack_);
+				turn_info turn_data(player_number_, replay_sender_);
 				recorder.end_turn();
 				turn_data.sync_network();
 			}
@@ -530,10 +577,12 @@ void playsingle_controller::play_turn(bool save)
 			replaying_ = ::do_replay(player_number_);
 			LOG_NG << "result of replay: " << (replaying_?"true":"false") << "\n";
 		} else {
-			// If a side is dead end the turn.
-			if (current_team().is_human() && side_units(units_, player_number_) == 0)
+			// If a side is dead end the turn, but play at least side=1's
+			// turn in case all sides are dead
+			if (current_team().is_human() && side_units(player_number_) == 0
+				&& (resources::units->size() != 0 || player_number_ != 1))
 			{
-				turn_info turn_data(player_number_, replay_sender_, undo_stack_);
+				turn_info turn_data(player_number_, replay_sender_);
 				recorder.end_turn();
 				turn_data.sync_network();
 				continue;
@@ -557,13 +606,13 @@ void playsingle_controller::play_turn(bool save)
 		loading_game_ = false;
 	}
 
+	finish_turn();
+
 	// Time has run out
 	check_time_over();
-
-	finish_turn();
 }
 
-void playsingle_controller::play_side(const unsigned int team_index, bool save)
+void playsingle_controller::play_side(const unsigned int side_number, bool save)
 {
 	//check for team-specific items in the scenario
 	gui_->parse_team_overlays();
@@ -579,7 +628,7 @@ void playsingle_controller::play_side(const unsigned int team_index, bool save)
 			end_turn_ = false;
 
 
-		statistics::reset_turn_stats(teams_[team_index - 1].save_id());
+		statistics::reset_turn_stats(teams_[side_number - 1].save_id());
 
 		if(current_team().is_human() || temporary_human) {
 			LOG_NG << "is human...\n";
@@ -588,26 +637,20 @@ void playsingle_controller::play_side(const unsigned int team_index, bool save)
 				play_human_turn();
 				after_human_turn();
 			} catch(end_turn_exception& end_turn) {
-				if (end_turn.redo == team_index) {
+				if (end_turn.redo == side_number) {
 					player_type_changed_ = true;
 					// If new controller is not human,
 					// reset gui to prev human one
-					if (!teams_[team_index-1].is_human()) {
+					if (!teams_[side_number-1].is_human()) {
 						browse_ = true;
-						int t = find_human_team_before(team_index);
+						int t = find_human_team_before(side_number);
 						if (t > 0) {
-							gui_->set_team(t-1);
-							gui_->recalculate_minimap();
-							gui_->invalidate_all();
-							gui_->draw(true,true);
+							update_gui_to_player(t-1);
 						}
 					}
 				}
 			}
 
-			if(game_config::debug)
-				game_display::clear_debug_highlights();
-
 			LOG_NG << "human finished turn...\n";
 		} else if(current_team().is_ai()) {
 			try {
@@ -630,10 +673,6 @@ void playsingle_controller::before_human_turn(bool save)
 	browse_ = false;
 	linger_ = false;
 
-	gui_->set_team(player_number_ - 1);
-	gui_->recalculate_minimap();
-	gui_->invalidate_all();
-	gui_->draw(true,true);
 
 	ai::manager::raise_turn_started();
 
@@ -649,7 +688,7 @@ void playsingle_controller::before_human_turn(bool save)
 
 void playsingle_controller::show_turn_dialog(){
 	if(preferences::turn_dialog() && (level_result_ == NONE) ) {
-		std::string message = _("It is now $name|'s turn");
+		std::string message = _("It is now $name|’s turn");
 		utils::string_map symbols;
 		symbols["name"] = teams_[player_number_ - 1].current_player();
 		message = utils::interpolate_variables_into_string(message, &symbols);
@@ -709,25 +748,29 @@ void playsingle_controller::linger()
 
 	// End all unit moves
 	for (unit_map::iterator u = units_.begin(); u != units_.end(); ++u) {
-		u->second.set_user_end_turn(true);
+		u->set_user_end_turn(true);
 	}
 	try {
 		// Same logic as single-player human turn, but
 		// *not* the same as multiplayer human turn.
 		gui_->enable_menu("endturn", true);
+		end_turn_ = false;
 		while(!end_turn_) {
 			// Reset the team number to make sure we're the right team.
 			player_number_ = first_player_;
 			play_slice();
 			gui_->draw();
 		}
-	} catch(game::load_game_exception& lge) {
+#ifdef _MSC_VER
+#pragma warning (push)
+#pragma warning (disable : 4101)
+#endif
+	} catch(const game::load_game_exception& lge) {
 		// Loading a new game is effectively a quit.
 		if (lge.game != "") {
 			gamestate_ = game_state();
 		}
-
-		throw lge;
+		throw;
 	}
 
 	// revert the end-turn button text to its normal label
@@ -738,6 +781,9 @@ void playsingle_controller::linger()
 
 	LOG_NG << "ending end-of-scenario linger\n";
 }
+#ifdef _MSC_VER
+#pragma warning (pop)
+#endif
 
 void playsingle_controller::end_turn_record()
 {
@@ -752,6 +798,18 @@ void playsingle_controller::end_turn_record_unlock()
 	turn_over_ = false;
 }
 
+hotkey::ACTION_STATE playsingle_controller::get_action_state(hotkey::HOTKEY_COMMAND command, int index) const
+{
+	switch(command) {
+	case hotkey::HOTKEY_WB_TOGGLE:
+		return resources::whiteboard->is_active() ? hotkey::ACTION_ON : hotkey::ACTION_OFF;
+	default:
+		return play_controller::get_action_state(command, index);
+	}
+}
+
+
+
 void playsingle_controller::after_human_turn(){
 	browse_ = true;
 	end_turn_record();
@@ -775,7 +833,7 @@ void playsingle_controller::play_ai_turn(){
 
 	const cursor::setter cursor_setter(cursor::WAIT);
 
-	turn_info turn_data(player_number_, replay_sender_, undo_stack_);
+	turn_info turn_data(player_number_, replay_sender_);
 
 	try {
 		ai::manager::play_turn(player_number_);
@@ -803,7 +861,8 @@ void playsingle_controller::handle_generic_event(const std::string& name){
 }
 
 void playsingle_controller::check_time_over(){
-	bool b = next_turn();
+	bool b = tod_manager_.next_turn();
+	it_is_a_new_turn_ = true;
 	if(!b) {
 
 		LOG_NG << "firing time over event...\n";
@@ -850,10 +909,10 @@ void playsingle_controller::store_recalls() {
 		}
 	}
 	//add any players from starting_pos that do not have a team in the current scenario
-	foreach (const config* player_cfg, gamestate_.starting_pos.get_children("player")) {
-		if (side_ids.count((*player_cfg)["save_id"]) == 0) {
-			LOG_NG << "stored inactive side in snapshot:\n" << (*player_cfg)["save_id"] << std::endl;
-			gamestate_.snapshot.add_child("side", (*player_cfg));
+	foreach (const config &player_cfg, gamestate_.starting_pos.child_range("player")) {
+		if (side_ids.count(player_cfg["save_id"]) == 0) {
+			LOG_NG << "stored inactive side in snapshot:\n" << player_cfg["save_id"] << std::endl;
+			gamestate_.snapshot.add_child("side", player_cfg);
 		}
 	}
 }
@@ -869,6 +928,7 @@ void playsingle_controller::store_gold(bool obs)
 	if (obs) {
 		title = _("Scenario Report");
 	} else {
+		persist_.end_transaction();
 		title = _("Victory");
 		report << "<b>" << _("You have emerged victorious!") << "</b>\n\n";
 	}
@@ -878,7 +938,7 @@ void playsingle_controller::store_gold(bool obs)
 		if (t.persistent()) ++persistent_teams;
 	}
 
-	end_level_data &end_level = get_end_level_data();
+	const end_level_data &end_level = get_end_level_data_const();
 
 	if (persistent_teams > 0 && (has_next_scenario ||
 		gamestate_.classification().campaign_type == "test"))
@@ -886,13 +946,13 @@ void playsingle_controller::store_gold(bool obs)
 		int finishing_bonus_per_turn =
 			map_.villages().size() * game_config::village_income +
 			game_config::base_income;
-		int turns_left = std::max<int>(0, number_of_turns() - turn());
+		int turns_left = std::max<int>(0, tod_manager_.number_of_turns() - turn());
 		int finishing_bonus = (end_level.gold_bonus && turns_left > -1) ?
 			finishing_bonus_per_turn * turns_left : 0;
 		foreach (const team &t, teams_)
 		{
 			if (!t.persistent()) continue;
-			int carryover_gold = ((t.gold() + finishing_bonus) * end_level.carryover_percentage) / 100;
+			int carryover_gold = div100rounded((t.gold() + finishing_bonus) * end_level.carryover_percentage);
 			config::child_itors side_range = gamestate_.snapshot.child_range("side");
 			config::child_iterator side_it = side_range.first;
 
@@ -900,8 +960,8 @@ void playsingle_controller::store_gold(bool obs)
 			while (side_it != side_range.second) {
 				if ((*side_it)["save_id"] == t.save_id()) {
 					(*side_it)["gold"] = str_cast<int>(carryover_gold);
-					(*side_it)["gold_add"] = end_level.carryover_add ? "yes" : "no";
-					(*side_it)["colour"] = t.colour();
+					(*side_it)["gold_add"] = end_level.carryover_add;
+					(*side_it)["color"] = t.color();
 					(*side_it)["current_player"] = t.current_player();
 					(*side_it)["name"] = t.name();
 					break;
@@ -914,8 +974,8 @@ void playsingle_controller::store_gold(bool obs)
 				config &new_side = gamestate_.snapshot.add_child("side");
 				new_side["save_id"] = t.save_id();
 				new_side["gold"] = str_cast<int>(carryover_gold);
-				new_side["gold_add"] = end_level.carryover_add ? "yes" : "no";
-				new_side["colour"] = t.colour();
+				new_side["gold_add"] = end_level.carryover_add;
+				new_side["color"] = t.color();
 				new_side["current_player"] = t.current_player();
 				new_side["name"] = t.name();
 			}
@@ -932,7 +992,7 @@ void playsingle_controller::store_gold(bool obs)
 	}
 
 	if (end_level.carryover_report) {
-		gui2::show_transient_message(gui_->video(), title, report.str(), true);
+		gui2::show_transient_message(gui_->video(), title, report.str(), "", true);
 	}
 }
 
@@ -940,13 +1000,13 @@ bool playsingle_controller::can_execute_command(hotkey::HOTKEY_COMMAND command,
 {
 	bool res = true;
 	switch (command){
-		case hotkey::HOTKEY_ADD_WAYPOINT:
 		case hotkey::HOTKEY_UNIT_HOLD_POSITION:
 		case hotkey::HOTKEY_END_UNIT_TURN:
+			return !browse_ && !linger_ && !events::commands_disabled;
 		case hotkey::HOTKEY_RECRUIT:
 		case hotkey::HOTKEY_REPEAT_RECRUIT:
 		case hotkey::HOTKEY_RECALL:
-			return !browse_ && !linger_ && !events::commands_disabled;
+			return (!browse_ || resources::whiteboard->is_active()) && !linger_ && !events::commands_disabled;
 		case hotkey::HOTKEY_ENDTURN:
 			return (!browse_ || linger_) && !events::commands_disabled;
 
@@ -964,24 +1024,45 @@ bool playsingle_controller::can_execute_command(hotkey::HOTKEY_COMMAND command,
 		case hotkey::HOTKEY_CHANGE_SIDE:
 			return !events::commands_disabled && game_config::debug && map_.on_board(mouse_handler_.get_last_hex());
 
+		case hotkey::HOTKEY_CLEAR_LABELS:
+			res = !is_observer();
+			break;
 		case hotkey::HOTKEY_LABEL_TEAM_TERRAIN:
-		case hotkey::HOTKEY_LABEL_TERRAIN:
+		case hotkey::HOTKEY_LABEL_TERRAIN: {
+			const terrain_label *label = resources::screen->labels().get_label(mouse_handler_.get_last_hex());
 			res = !events::commands_disabled && map_.on_board(mouse_handler_.get_last_hex())
 				&& !gui_->shrouded(mouse_handler_.get_last_hex())
-				&& !is_observer();
+				&& !is_observer()
+				&& (!label || !label->immutable());
 			break;
-
+		}
 		case hotkey::HOTKEY_CONTINUE_MOVE: {
 			if(browse_ || events::commands_disabled)
 				return false;
 
-			if( (menu_handler_.current_unit(mouse_handler_) != units_.end())
-				&& (menu_handler_.current_unit(mouse_handler_)->second.move_interrupted()))
+			if( (menu_handler_.current_unit() != units_.end())
+				&& (menu_handler_.current_unit()->move_interrupted()))
 				return true;
 			const unit_map::const_iterator i = units_.find(mouse_handler_.get_selected_hex());
 			if (i == units_.end()) return false;
-			return i->second.move_interrupted();
+			return i->move_interrupted();
 		}
+		case hotkey::HOTKEY_WB_TOGGLE:
+			return true;
+		case hotkey::HOTKEY_WB_EXECUTE_ACTION:
+		case hotkey::HOTKEY_WB_EXECUTE_ALL_ACTIONS:
+			return resources::whiteboard->can_enable_execution_hotkeys();
+		case hotkey::HOTKEY_WB_DELETE_ACTION:
+			return resources::whiteboard->can_enable_modifier_hotkeys();
+		case hotkey::HOTKEY_WB_BUMP_UP_ACTION:
+		case hotkey::HOTKEY_WB_BUMP_DOWN_ACTION:
+			return resources::whiteboard->can_enable_reorder_hotkeys();
+		case hotkey::HOTKEY_WB_SUPPOSE_DEAD:
+		{
+			//@todo re-enable this once we figure out a decent UI for suppose_dead
+			return false;
+		}
+
 		default: return play_controller::can_execute_command(command, index);
 	}
 	return res;
