@@ -1,18 +1,19 @@
-/* $Id: preferences.hpp 33157 2009-02-28 16:12:49Z ivanovic $ */
+/* $Id: preferences.hpp 51367 2011-10-03 11:56:40Z espreon $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
-/** @file preferences.hpp */
+/** @file */
 
 #ifndef PREFERENCES_HPP_INCLUDED
 #define PREFERENCES_HPP_INCLUDED
@@ -22,11 +23,8 @@ class display;
 
 #include "game_config.hpp"
 #include "terrain_translation.hpp"
-#include "config.hpp"
 
-#include <string>
 #include <utility>
-#include <set>
 
 namespace preferences {
 
@@ -38,12 +36,15 @@ namespace preferences {
 
 	void write_preferences();
 
-	// Low-level, should be seen only by preferences_display ?
-	void set(const std::string& key, std::string value);
+	void set(const std::string& key, const std::string &value);
+	void set(const std::string& key, char const *value);
+	void set(const std::string &key, bool value);
+	void set(const std::string &key, int value);
 	void clear(const std::string& key);
 	void set_child(const std::string& key, const config& val);
-	config* get_child(const std::string& key);
+	const config &get_child(const std::string &key);
 	std::string get(const std::string& key);
+	bool get(const std::string &key, bool def);
 	void erase(const std::string& key);
 
 	void disable_preferences_save();
@@ -74,7 +75,7 @@ namespace preferences {
 	int idle_anim_rate();
 	void _set_idle_anim_rate(const int rate);
 
-	const std::string& language();
+	std::string language();
 	void set_language(const std::string& s);
 
 	// Don't rename it to sound() because of a gcc-3.3 branch bug,
@@ -116,10 +117,38 @@ namespace preferences {
 	void save_hotkeys();
 
 	void add_alias(const std::string& alias, const std::string& command);
-	config* get_alias();
+	const config &get_alias();
 
-	bool use_colour_cursors();
-	void _set_colour_cursors(bool value);
+	bool use_color_cursors();
+	void _set_color_cursors(bool value);
+
+	bool joystick_support_enabled();
+	int joystick_mouse_deadzone();
+	int joystick_num_mouse_xaxis();
+	int joystick_num_mouse_yaxis();
+	int joystick_mouse_xaxis_num();
+	int joystick_mouse_yaxis_num();
+
+	int joystick_scroll_deadzone();
+	int joystick_num_scroll_xaxis();
+	int joystick_num_scroll_yaxis();
+	int joystick_scroll_xaxis_num();
+	int joystick_scroll_yaxis_num();
+
+	int joystick_cursor_deadzone();
+	int joystick_num_cursor_xaxis();
+	int joystick_num_cursor_yaxis();
+	int joystick_cursor_xaxis_num();
+	int joystick_cursor_yaxis_num();
+	int joystick_cursor_threshold();
+
+	int joystick_thrusta_deadzone();
+	int joystick_num_thrusta_axis();
+	int joystick_thrusta_axis_num();
+
+	int joystick_thrustb_deadzone();
+	int joystick_num_thrustb_axis();
+	int joystick_thrustb_axis_num();
 
 	int scroll_speed();
 	void set_scroll_speed(const int scroll);
@@ -136,31 +165,29 @@ namespace preferences {
 	 */
 	int mouse_scroll_threshold();
 
-	/** Set editor red tint level. */
-	void set_editor_r(int value);
-	/** Set editor green tint level. */
-	void set_editor_g(int value);
-	/** Set editor blue tint level. */
-	void set_editor_b(int value);
-
-	/** Get editor red tint level. */
-	int editor_r();
-	/** Get editor green tint level. */
-	int editor_g();
-	/** Get editor blue tint level. */
-	int editor_b();
-
 	int draw_delay();
 	void set_draw_delay(int value);
 
 	bool animate_map();
+	void set_animate_map(bool value);
+
 	bool show_standing_animations();
+	void set_show_standing_animations(bool value);
 
 	bool show_fps();
 	void set_show_fps(bool value);
 
+	bool ellipses();
+	void _set_ellipses(bool ison);
+
 	bool grid();
 	void _set_grid(bool ison);
+
+	bool confirm_load_save_from_different_version();
+
+	bool use_twelve_hour_clock_format();
+	void set_use_twelve_hour_clock_format(bool value);
+
 } // end namespace preferences
 
 #endif
