@@ -1,12 +1,13 @@
-/* $Id: replay_controller.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: replay_controller.cpp 51826 2011-11-04 05:08:54Z shadowmaster $ */
 /*
-   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2011 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,10 +16,12 @@
 
 #include "global.hpp"
 
+#include "foreach.hpp"
 #include "game_end_exceptions.hpp"
 #include "game_events.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
+#include "map_label.hpp"
 #include "replay.hpp"
 #include "replay_controller.hpp"
 #include "resources.hpp"
@@ -36,7 +39,7 @@ LEVEL_RESULT play_replay_level(const config& game_config,
 {
 	try{
 		const int ticks = SDL_GetTicks();
-		const int num_turns = atoi((*level)["turns"].c_str());
+		int num_turns = (*level)["turns"].to_int(-1);
 		DBG_NG << "creating objects... " << (SDL_GetTicks() - ticks) << "\n";
 		replay_controller replaycontroller(*level, state_of_game, ticks, num_turns, game_config, video);
 		DBG_NG << "created objects... " << (SDL_GetTicks() - replaycontroller.get_ticks()) << "\n";
@@ -59,17 +62,21 @@ replay_controller::replay_controller(const config& level,
 		const config& game_config, CVideo& video) :
 	play_controller(level, state_of_game, ticks, num_turns, game_config, video, false),
 	teams_start_(),
-	gamestate_start_(state_of_game),
+	gamestate_start_(gamestate_),
 	units_start_(),
-	tod_manager_start_(level, num_turns, &state_of_game),
+	tod_manager_start_(level, num_turns),
 	current_turn_(1),
 	delay_(0),
 	is_playing_(false),
 	show_everything_(false),
 	show_team_(state_of_game.classification().campaign_type == "multiplayer" ? 0 : 1)
 {
+	units_start_ = units_;
+	teams_start_ = teams_;
+	tod_manager_start_ = tod_manager_;
+
 	init();
-	gamestate_start_ = gamestate_;
+	reset_replay();
 }
 
 replay_controller::~replay_controller()
@@ -77,7 +84,8 @@ replay_controller::~replay_controller()
 	//YogiHH
 	//not absolutely sure if this is needed, but it makes me feel a lot better ;-)
 	//feel free to delete this if it is not necessary
-	gui_->get_theme().theme_reset().detach_handler(this);
+	gui_->get_theme().theme_reset_event().detach_handler(this);
+	gui_->complete_redraw_event().detach_handler(this);
 }
 
 void replay_controller::init(){
@@ -86,20 +94,6 @@ void replay_controller::init(){
 	//guarantee the cursor goes back to 'normal' at the end of the level
 	const cursor::setter cursor_setter(cursor::NORMAL);
 	init_replay_display();
-
-	fire_prestart(true);
-	init_gui();
-	statistics::fresh_stats();
-	set_victory_when_enemies_defeated(
-		utils::string_bool(level_["victory_when_enemies_defeated"], true));
-
-	DBG_REPLAY << "first_time..." << (recorder.is_skipping() ? "skipping" : "no skip") << "\n";
-
-	fire_start(!loading_game_);
-	update_gui();
-
-	units_start_ = units_;
-	teams_start_ = teams_;
 }
 
 void replay_controller::init_gui(){
@@ -116,13 +110,16 @@ void replay_controller::init_gui(){
 	for(std::vector<team>::iterator t = teams_.begin(); t != teams_.end(); ++t) {
 		t->reset_objectives_changed();
 	}
+
+	update_replay_ui();
 }
 
 void replay_controller::init_replay_display(){
 	DBG_REPLAY << "initializing replay-display... " << (SDL_GetTicks() - ticks_) << "\n";
 
 	rebuild_replay_theme();
-	gui_->get_theme().theme_reset().attach_handler(this);
+	gui_->get_theme().theme_reset_event().attach_handler(this);
+	gui_->complete_redraw_event().attach_handler(this);
 	DBG_REPLAY << "done initializing replay-display... " << (SDL_GetTicks() - ticks_) << "\n";
 }
 
@@ -140,21 +137,125 @@ void replay_controller::rebuild_replay_theme()
 	}
 }
 
+gui::button* replay_controller::play_button()
+{
+	return gui_->find_button("button-playreplay");
+}
+
+gui::button* replay_controller::stop_button()
+{
+	return gui_->find_button("button-stopreplay");
+}
+
+gui::button* replay_controller::reset_button()
+{
+	return gui_->find_button("button-resetreplay");
+}
+
+gui::button* replay_controller::play_turn_button()
+{
+	return gui_->find_button("button-nextturn");
+}
+
+gui::button* replay_controller::play_side_button()
+{
+	return gui_->find_button("button-nextside");
+}
+
+void replay_controller::update_replay_ui()
+{
+	//check if we have all buttons - if someone messed with theme then some buttons may be missing
+	//if any of the buttons is missing, we just disable every one
+	if(!replay_ui_has_all_buttons()) {
+		gui::button *play_b = play_button(), *stop_b = stop_button(),
+		            *reset_b = reset_button(), *play_turn_b = play_turn_button(),
+		            *play_side_b = play_side_button();
+
+		if(play_b) {
+			play_b->enable(false);
+		}
+
+		if(stop_b) {
+			stop_b->enable(false);
+		}
+
+		if(reset_b) {
+			reset_b->enable(false);
+		}
+
+		if(play_turn_b) {
+			play_turn_b->enable(false);
+		}
+
+		if(play_side_b) {
+			play_side_b->enable(false);
+		}
+	}
+}
+
+void replay_controller::replay_ui_playback_should_start()
+{
+	if(!replay_ui_has_all_buttons())
+		return;
+
+	play_button()->enable(false);
+	reset_button()->enable(false);
+	play_turn_button()->enable(false);
+	play_side_button()->enable(false);
+}
+
+void replay_controller::replay_ui_playback_should_stop()
+{
+	if(!replay_ui_has_all_buttons())
+		return;
+
+	if(!recorder.at_end()) {
+		play_button()->enable(true);
+		reset_button()->enable(true);
+		play_turn_button()->enable(true);
+		play_side_button()->enable(true);
+
+		play_button()->release();
+		play_turn_button()->release();
+		play_side_button()->release();
+	} else {
+		reset_button()->enable(true);
+		stop_button()->enable(false);
+	}
+
+	if(!is_playing_) {
+		//user interrupted
+		stop_button()->release();
+	}
+}
+
+void replay_controller::reset_replay_ui()
+{
+	if(!replay_ui_has_all_buttons())
+		return;
+
+	play_button()->enable(true);
+	stop_button()->enable(true);
+	reset_button()->enable(true);
+	play_turn_button()->enable(true);
+	play_side_button()->enable(true);
+}
+
+
 void replay_controller::reset_replay(){
-	gui::button* b = gui_->find_button("button-playreplay");
-	if (b != NULL) { b->release(); }
-	b = gui_->find_button("button-stopreplay");
-	if (b != NULL) { b->release(); }
+
 	gui_->clear_chat_messages();
 	is_playing_ = false;
 	player_number_ = 1;
 	current_turn_ = 1;
+	it_is_a_new_turn_ = true;
+	skip_replay_ = false;
 	tod_manager_= tod_manager_start_;
 	recorder.start_replay();
+	recorder.set_skip(false);
 	units_ = units_start_;
 	gamestate_ = gamestate_start_;
 	teams_ = teams_start_;
-	statistics::fresh_stats();
 	if (events_manager_ ){
 		// NOTE: this double reset is required so that the new
 		// instance of game_events::manager isn't created before the
@@ -164,67 +265,61 @@ void replay_controller::reset_replay(){
 		events_manager_.reset(new game_events::manager(level_));
 	}
 
-	gui_->new_turn();
-	gui_->invalidate_game_status();
-	events::raise_draw_event();
-	(*gui_).invalidate_all();
-	(*gui_).draw();
+	gui_->labels().read(level_);
+
+	statistics::fresh_stats();
+	set_victory_when_enemies_defeated(level_["victory_when_enemies_defeated"].to_bool(true));
+
+	// Add era events for MP game.
+	if (const config &era_cfg = level_.child("era")) {
+		game_events::add_events(era_cfg.child_range("event"), "era_events");
+	}
 
 	fire_prestart(true);
-	fire_start(!loading_game_);
-	gui_->new_turn();
-	gui_->invalidate_game_status();
-	events::raise_draw_event();
-	(*gui_).invalidate_all();
-	(*gui_).draw();
-	b = gui_->find_button("button-resetreplay");
-	if (b != NULL) { b->release(); }
+	init_gui();
+	fire_start(true);
+	update_gui();
+
+	reset_replay_ui();
 }
 
 void replay_controller::stop_replay(){
 	is_playing_ = false;
-	gui::button* b = gui_->find_button("button-playreplay");
-	if (b != NULL) { b->release(); }
 }
 
 void replay_controller::replay_next_turn(){
 	is_playing_ = true;
+	replay_ui_playback_should_start();
+
 	play_turn();
 
- 	if (!skip_replay_){
+ 	if (!skip_replay_ || !is_playing_){
 		gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
 	}
-	is_playing_ = false;
-	gui::button* b = gui_->find_button("button-nextturn");
-	if (b != NULL) { b->release(); }
+
+	replay_ui_playback_should_stop();
 }
 
 void replay_controller::replay_next_side(){
 	is_playing_ = true;
+	replay_ui_playback_should_start();
+
 	play_side(player_number_ - 1, false);
 
-	if (static_cast<size_t>(player_number_) > teams_.size()) {
-		player_number_ = 1;
-		current_turn_++;
-	}
-
-	if (!skip_replay_) {
+	if (!skip_replay_ || !is_playing_) {
 		gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
 	}
 
-	is_playing_ = false;
-	gui::button* b = gui_->find_button("button-nextside");
-	if (b != NULL) { b->release(); }
+	replay_ui_playback_should_stop();
 }
 
 void replay_controller::process_oos(const std::string& msg) const
 {
 	if (game_config::ignore_replay_errors) return;
 
-	/** @todo FIXME: activate translation support after string freeze */
 	std::stringstream message;
-	message << "The replay is corrupt/out of sync. It might not make much sense to continue. Do you want to save the game?";
-	message << "\n\nError details:\n\n" << msg;
+	message << _("The replay is corrupt/out of sync. It might not make much sense to continue. Do you want to save the game?");
+	message << "\n\n" << _("Error details:") << "\n\n" << msg;
 
 	savegame::oos_savegame save(to_config());
 	save.save_game_interactive(resources::screen->video(), message.str(), gui::YES_NO); // can throw end_level_exception
@@ -252,35 +347,40 @@ void replay_controller::replay_show_team1(){
 }
 
 void replay_controller::replay_skip_animation(){
-	recorder.set_skip(!recorder.is_skipping());
 	skip_replay_ = !skip_replay_;
+	recorder.set_skip(skip_replay_);
 }
 
+//move all sides till stop/end
 void replay_controller::play_replay(){
-	gui::button* b = gui_->find_button("button-stopreplay");
-	if (b != NULL) { b->release(); }
+
 	if (recorder.at_end()){
+		//shouldn't actually happen
 		return;
 	}
 
 	try{
 		is_playing_ = true;
+		replay_ui_playback_should_start();
 
 		DBG_REPLAY << "starting main loop\n" << (SDL_GetTicks() - ticks_) << "\n";
 		for(; !recorder.at_end() && is_playing_; first_player_ = 1) {
 			play_turn();
-		} //end for loop
-		is_playing_ = false;
+		}
+
+		if (!is_playing_) {
+			gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
+		}
 	}
 	catch(end_level_exception& e){
-		if (e.result == QUIT) { throw e; }
+		if (e.result == QUIT) throw;
 	}
+
+	replay_ui_playback_should_stop();
 }
 
+//make all sides move, then stop
 void replay_controller::play_turn(){
-	if (recorder.at_end()){
-		return;
-	}
 
 	LOG_REPLAY << "turn: " << current_turn_ << "\n";
 
@@ -297,10 +397,8 @@ void replay_controller::play_turn(){
 	}
 }
 
+//make only one side move
 void replay_controller::play_side(const unsigned int /*team_index*/, bool){
-	if (recorder.at_end()){
-		return;
-	}
 
 	DBG_REPLAY << "Status turn number: " << turn() << "\n";
 	DBG_REPLAY << "Replay_Controller turn number: " << current_turn_ << "\n";
@@ -319,9 +417,9 @@ void replay_controller::play_side(const unsigned int /*team_index*/, bool){
 			finish_side_turn();
 
 			// This is necessary for replays in order to show possible movements.
-			for (unit_map::iterator uit = units_.begin(); uit != units_.end(); ++uit) {
-				if (uit->second.side() != player_number_) {
-					uit->second.new_turn();
+			foreach (unit &u, units_) {
+				if (u.side() != player_number_) {
+					u.new_turn();
 				}
 			}
 		}
@@ -329,10 +427,12 @@ void replay_controller::play_side(const unsigned int /*team_index*/, bool){
 		player_number_++;
 
 		if (static_cast<size_t>(player_number_) > teams_.size()) {
-			next_turn();
 			finish_turn();
+			tod_manager_.next_turn();
+			it_is_a_new_turn_ = true;
 			player_number_ = 1;
 			current_turn_++;
+			gui_->new_turn();
 		}
 
 		update_teams();
@@ -340,20 +440,25 @@ void replay_controller::play_side(const unsigned int /*team_index*/, bool){
 	}
 	catch(end_level_exception& e){
 		//VICTORY/DEFEAT end_level_exception shall not return to title screen
-		if ((e.result != VICTORY) && (e.result != DEFEAT)) { throw e; }
+		if (e.result != VICTORY && e.result != DEFEAT) throw;
 	}
 }
 
 void replay_controller::update_teams(){
+
 	int next_team = player_number_;
 	if(static_cast<size_t>(next_team) > teams_.size()) {
 		next_team = 1;
 	}
 
-	if (!show_team_)
+	if (!show_team_) {
 		gui_->set_team(next_team - 1, show_everything_);
+	} else {
+		gui_->set_team(show_team_ - 1, show_everything_);
+	}
 
 	::clear_shroud(next_team);
+
 	gui_->set_playing_team(next_team - 1);
 	gui_->invalidate_all();
 }
@@ -375,8 +480,18 @@ void replay_controller::show_statistics(){
 	menu_handler_.show_statistics(gui_->playing_team()+1);
 }
 
-void replay_controller::handle_generic_event(const std::string& /*name*/){
-	rebuild_replay_theme();
+void replay_controller::handle_generic_event(const std::string& name){
+
+	if( name == "completely_redrawn" ) {
+		update_replay_ui();
+
+		gui::button* skip_animation_button = gui_->find_button("skip-animation");
+		if(skip_animation_button) {
+			skip_animation_button->set_check(skip_replay_);
+		}
+	} else {
+		rebuild_replay_theme();
+	}
 }
 
 bool replay_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int index) const
