@@ -1,11 +1,12 @@
-/* $Id: sdl_utils.cpp 42157 2010-04-17 13:36:09Z silene $ */
+/* $Id: sdl_utils.cpp 49573 2011-05-19 18:16:10Z mordante $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,12 +14,13 @@
 */
 
 /**
- *  @file sdl_utils.cpp
+ *  @file
  *  Support-routines for the SDL-graphics-library.
  */
 
 #include "global.hpp"
 
+#include "sdl_utils.hpp"
 #include "video.hpp"
 
 #include <algorithm>
@@ -26,13 +28,63 @@
 #include <cstring>
 #include <iostream>
 
-SDL_Color int_to_color(const Uint32 rgb) {
-	SDL_Color to_return = {
-		(0x00FF0000 & rgb)>>16,
-		(0x0000FF00 & rgb)>>8,
-		(0x000000FF & rgb), 0
-	};
-	return to_return;
+
+/// Check that the surface is neutral bpp 32, possibly with an empty alpha channel.
+inline bool is_neutral(const surface& surf)
+{
+	return (surf->format->BytesPerPixel == 4 &&
+		surf->format->Rmask == 0xFF0000u &&
+		(surf->format->Amask | 0xFF000000u) == 0xFF000000u);
+}
+
+surface_lock::surface_lock(surface &surf) : surface_(surf), locked_(false)
+{
+	assert(is_neutral(surface_));
+	if (SDL_MUSTLOCK(surface_))
+		locked_ = SDL_LockSurface(surface_) == 0;
+}
+
+surface_lock::~surface_lock()
+{
+	if (locked_)
+		SDL_UnlockSurface(surface_);
+}
+
+const_surface_lock::const_surface_lock(const surface &surf) : surface_(surf), locked_(false)
+{
+	assert(is_neutral(surface_));
+	if (SDL_MUSTLOCK(surface_))
+		locked_ = SDL_LockSurface(surface_) == 0;
+}
+
+const_surface_lock::~const_surface_lock()
+{
+	if (locked_)
+		SDL_UnlockSurface(surface_);
+}
+
+SDL_Color int_to_color(const Uint32 rgb)
+{
+	SDL_Color result;
+	result.r = (0x00FF0000 & rgb )>> 16;
+	result.g = (0x0000FF00 & rgb) >> 8;
+	result.b = (0x000000FF & rgb);
+	result.unused = 0;
+	return result;
+}
+
+SDL_Color create_color(const unsigned char red
+		, unsigned char green
+		, unsigned char blue
+		, unsigned char unused)
+{
+	SDL_Color result;
+	result.r = red;
+	result.g = green;
+	result.b = blue;
+	result.unused = unused;
+
+	return result;
 }
 
 SDLKey sdl_keysym_from_name(std::string const &keyname)
@@ -73,35 +125,33 @@ SDL_Rect intersect_rects(SDL_Rect const &rect1, SDL_Rect const &rect2)
 	SDL_Rect res;
 	res.x = std::max<int>(rect1.x, rect2.x);
 	res.y = std::max<int>(rect1.y, rect2.y);
-	res.w = std::max<int>(std::min<int>(rect1.x + rect1.w, rect2.x + rect2.w) - res.x, 0);
-	res.h = std::max<int>(std::min<int>(rect1.y + rect1.h, rect2.y + rect2.h) - res.y, 0);
+	int w = std::min<int>(rect1.x + rect1.w, rect2.x + rect2.w) - res.x;
+	int h = std::min<int>(rect1.y + rect1.h, rect2.y + rect2.h) - res.y;
+	if (w <= 0 || h <= 0) return empty_rect;
+	res.w = w;
+	res.h = h;
 	return res;
 }
 
-SDL_Rect get_rect_union(SDL_Rect const &rect1, SDL_Rect const& rect2) {
-	const int left_side = std::max(rect1.x, rect2.x);
-	const int right_side = std::min(rect1.x + rect1.w, rect2.x + rect2.w);
-	if(left_side > right_side) {
-		return empty_rect;
-	}
-
-	const int top_side = std::max(rect1.y, rect2.y);
-	const int bottom_side = std::min(rect1.y + rect1.h, rect2.y + rect2.h);
-	if(top_side > bottom_side) {
-		return empty_rect;
-	}
-
-	SDL_Rect result = {
-			left_side,
-			top_side,
-			right_side - left_side,
-			bottom_side - top_side};
-
-	return result;
+SDL_Rect union_rects(SDL_Rect const &rect1, SDL_Rect const &rect2)
+{
+	if (rect1.w == 0 || rect1.h == 0) return rect2;
+	if (rect2.w == 0 || rect2.h == 0) return rect1;
+	SDL_Rect res;
+	res.x = std::min<int>(rect1.x, rect2.x);
+	res.y = std::min<int>(rect1.y, rect2.y);
+	res.w = std::max<int>(rect1.x + rect1.w, rect2.x + rect2.w) - res.x;
+	res.h = std::max<int>(rect1.y + rect1.h, rect2.y + rect2.h) - res.y;
+	return res;
 }
+
 SDL_Rect create_rect(const int x, const int y, const int w, const int h)
 {
-	SDL_Rect rect = { x, y, w, h };
+	SDL_Rect rect;
+	rect.x = x;
+	rect.y = y;
+	rect.w = w;
+	rect.h = h;
 	return rect;
 }
 
@@ -201,11 +251,11 @@ surface stretch_surface_horizontal(
 
 	{
 		// Extra scoping used for the surface_lock.
-		surface_lock src_lock(src);
+		const_surface_lock src_lock(src);
 		surface_lock dst_lock(dst);
 
-		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
-		Uint32* dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+		const Uint32* const src_pixels = src_lock.pixels();
+		Uint32* dst_pixels = dst_lock.pixels();
 
 		for(unsigned y = 0; y < static_cast<unsigned>(src->h); ++y) {
 			const Uint32 pixel = src_pixels [y * src->w];
@@ -246,11 +296,11 @@ surface stretch_surface_vertical(
 
 	{
 		// Extra scoping used for the surface_lock.
-		surface_lock src_lock(src);
+		const_surface_lock src_lock(src);
 		surface_lock dst_lock(dst);
 
-		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
-		Uint32* dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+		const Uint32* const src_pixels = src_lock.pixels();
+		Uint32* dst_pixels = dst_lock.pixels();
 
 		for(unsigned y = 0; y < static_cast<unsigned>(h); ++y) {
 		  for(unsigned x = 0; x < static_cast<unsigned>(src->w); ++x) {
@@ -293,15 +343,16 @@ surface scale_surface(const surface &surf, int w, int h, bool optimize)
 		return NULL;
 	}
 
-	const fixed_t xratio = fxpdiv(surf->w,w);
-	const fixed_t yratio = fxpdiv(surf->h,h);
-
+	if (w > surf->w || h > surf->h)
 	{
-		surface_lock src_lock(src);
+		const_surface_lock src_lock(src);
 		surface_lock dst_lock(dst);
 
-		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
-		Uint32* const dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+		const Uint32* const src_pixels = src_lock.pixels();
+		Uint32* const dst_pixels = dst_lock.pixels();
+
+		fixed_t xratio = fxpdiv(surf->w,w);
+		fixed_t yratio = fxpdiv(surf->h,h);
 
 		fixed_t ysrc = ftofxp(0.0);
 		for(int ydst = 0; ydst != h; ++ydst, ysrc += yratio) {
@@ -310,7 +361,7 @@ surface scale_surface(const surface &surf, int w, int h, bool optimize)
 				const int xsrcint = fxptoi(xsrc);
 				const int ysrcint = fxptoi(ysrc);
 
-				Uint32* const src_word = src_pixels + ysrcint*src->w + xsrcint;
+				const Uint32* const src_word = src_pixels + ysrcint*src->w + xsrcint;
 				Uint32* const dst_word = dst_pixels +    ydst*dst->w + xdst;
 				const int dx = (xsrcint + 1 < src->w) ? 1 : 0;
 				const int dy = (ysrcint + 1 < src->h) ? src->w : 0;
@@ -390,7 +441,7 @@ surface scale_surface(const surface &surf, int w, int h, bool optimize)
 				// freaks out.
 				//
 				// The alpha thresholding step attempts
-				// to accomodates this limitation.
+				// to accommodates this limitation.
 				// There is a small loss of quality.
 				// For example, skeleton bowstrings
 				// are not as good as they could be.
@@ -420,139 +471,16 @@ surface scale_surface(const surface &surf, int w, int h, bool optimize)
 			}
 		}
 	}
-
-	return optimize ? create_optimized_surface(dst) : dst;
-}
-
-
-// This is a copy-paste of the previous function scale_surface
-// We only removed the alpha channel and big comments
-// and change how we optimize the resulting surface
-surface scale_opaque_surface(const surface &surf, int w, int h, bool optimize_format)
-{
-	if(surf == NULL)
-		return NULL;
-
-	if(w == surf->w && h == surf->h) {
-		return surf;
-	}
-	assert(w >= 0);
-	assert(h >= 0);
-
-	surface dst(create_neutral_surface(w,h));
-
-	if (w == 0 || h ==0) {
-		std::cerr << "Create an empty image\n";
-		return create_optimized_surface(dst);
-	}
-
-	surface src(make_neutral_surface(surf));
-	// Now both surfaces are always in the "neutral" pixel format
-
-	if(src == NULL || dst == NULL) {
-		std::cerr << "Could not create surface to scale onto\n";
-		return NULL;
-	}
-
-	const fixed_t xratio = fxpdiv(surf->w,w);
-	const fixed_t yratio = fxpdiv(surf->h,h);
-
+	else
 	{
-		surface_lock src_lock(src);
+		const_surface_lock src_lock(src);
 		surface_lock dst_lock(dst);
 
-		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
-		Uint32* const dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+		const Uint32* const src_pixels = src_lock.pixels();
+		Uint32* const dst_pixels = dst_lock.pixels();
 
-		fixed_t ysrc = ftofxp(0.0);
-		for(int ydst = 0; ydst != h; ++ydst, ysrc += yratio) {
-			fixed_t xsrc = ftofxp(0.0);
-			for(int xdst = 0; xdst != w; ++xdst, xsrc += xratio) {
-				const int xsrcint = fxptoi(xsrc);
-				const int ysrcint = fxptoi(ysrc);
-
-				Uint32* const src_word = src_pixels + ysrcint*src->w + xsrcint;
-				Uint32* const dst_word = dst_pixels +    ydst*dst->w + xdst;
-				const int dx = (xsrcint + 1 < src->w) ? 1 : 0;
-				const int dy = (ysrcint + 1 < src->h) ? src->w : 0;
-
-				Uint8 r,g,b;
-				Uint32 rr,gg,bb;
-				Uint32 pix[4], bilin[4];
-
-				const fixed_t e = 0x000000FF & xsrc;
-				const fixed_t s = 0x000000FF & ysrc;
-				const fixed_t n = 0xFF - s;
-				const fixed_t w = 0xFF - e;
-
-				pix[0] = *src_word;              // northwest
-				pix[1] = *(src_word + dx);       // northeast
-				pix[2] = *(src_word + dy);       // southwest
-				pix[3] = *(src_word + dx + dy);  // southeast
-
-				bilin[0] = n*w;
-				bilin[1] = n*e;
-				bilin[2] = s*w;
-				bilin[3] = s*e;
-
-				int loc;
-				rr = gg = bb = 0;
-				for (loc=0; loc<4; loc++) {
-				  r = pix[loc] >> 16;
-				  g = pix[loc] >> 8;
-				  b = pix[loc] >> 0;
-				  rr += r * bilin[loc];
-				  gg += g * bilin[loc];
-				  bb += b * bilin[loc];
-				}
-				r = rr >> 16;
-				g = gg >> 16;
-				b = bb >> 16;
-				*dst_word = (255 << 24) + (r << 16) + (g << 8) + b;
-			}
-		}
-	}
-
-	return optimize_format ? display_format_alpha(dst) : dst;
-}
-
-
-surface scale_surface_blended(const surface &surf, int w, int h, bool optimize)
-{
-	if(surf== NULL)
-		return NULL;
-
-	if(w == surf->w && h == surf->h) {
-		return surf;
-	}
-	assert(w >= 0);
-	assert(h >= 0);
-
-	surface dst(create_neutral_surface(w,h));
-
-	if (w == 0 || h ==0) {
-		std::cerr << "Create an empty image\n";
-		return create_optimized_surface(dst);
-	}
-
-	surface src(make_neutral_surface(surf));
-
-	if(src == NULL || dst == NULL) {
-		std::cerr << "Could not create surface to scale onto\n";
-		return NULL;
-	}
-
-	const double xratio = static_cast<double>(surf->w)/
-			              static_cast<double>(w);
-	const double yratio = static_cast<double>(surf->h)/
-			              static_cast<double>(h);
-
-	{
-		surface_lock src_lock(src);
-		surface_lock dst_lock(dst);
-
-		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
-		Uint32* const dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+		double xratio = double(surf->w) / w;
+		double yratio = double(surf->h) / h;
 
 		double ysrc = 0.0;
 		for(int ydst = 0; ydst != h; ++ydst, ysrc += yratio) {
@@ -575,23 +503,24 @@ surface scale_surface_blended(const surface &surf, int w, int h, bool optimize)
 						Uint8 r,g,b,a;
 
 						SDL_GetRGBA(src_pixels[ysrcint*src->w + xsrcint],src->format,&r,&g,&b,&a);
-						const double value = xsize*ysize*double(a)/255.0;
+						double value = xsize * ysize;
 						summation += value;
-
-						red += r*value;
-						green += g*value;
-						blue += b*value;
-						alpha += a*value;
+						if (!a) continue;
+						value *= a;
+						alpha += value;
+						red += r * value;
+						green += g * value;
+						blue += b * value;
 					}
 				}
 
-				if(summation == 0.0)
-					summation = 1.0;
-
-				red /= summation;
-				green /= summation;
-				blue /= summation;
-				alpha /= summation;
+				if (alpha != 0.0) {
+					double factor = 1 / alpha;
+					alpha = alpha / summation + 0.5;
+					red = red * factor + 0.5;
+					green = green * factor + 0.5;
+					blue = blue * factor + 0.5;
+				}
 
 				dst_pixels[ydst*dst->w + xdst] = SDL_MapRGBA(dst->format,Uint8(red),Uint8(green),Uint8(blue),Uint8(alpha));
 			}
@@ -601,10 +530,13 @@ surface scale_surface_blended(const surface &surf, int w, int h, bool optimize)
 	return optimize ? create_optimized_surface(dst) : dst;
 }
 
-surface adjust_surface_colour(const surface &surf, int red, int green, int blue, bool optimize)
+surface adjust_surface_color(const surface &surf, int red, int green, int blue, bool optimize)
 {
-	if((red == 0 && green == 0 && blue == 0) || surf == NULL)
-		return create_optimized_surface(surf);
+	if(surf == NULL)
+		return NULL;
+
+	if((red == 0 && green == 0 && blue == 0))
+		return optimize ? create_optimized_surface(surf) : surf;
 
 	surface nsurf(make_neutral_surface(surf));
 
@@ -725,10 +657,10 @@ surface shadow_image(const surface &surf, bool optimize)
 
 
 surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb, bool optimize){
-	if(map_rgb.size()){
-		if(surf == NULL)
+	if(surf == NULL)
 		return NULL;
 
+	if(!map_rgb.empty()){
 	     surface nsurf(make_neutral_surface(surf));
 	     if(nsurf == NULL) {
 			std::cerr << "failed to make neutral surface\n";
@@ -739,8 +671,6 @@ surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb, boo
 		Uint32* beg = lock.pixels();
 		Uint32* end = beg + nsurf->w*surf->h;
 
-		std::map<Uint32, Uint32>::const_iterator map_rgb_end = map_rgb.end();
-
 		while(beg != end) {
 			Uint8 alpha = (*beg) >> 24;
 
@@ -884,6 +814,9 @@ surface mask_surface(const surface &surf, const surface &mask)
 	if(surf == NULL) {
 		return NULL;
 	}
+	if(mask == NULL) {
+		return surf;
+	}
 
 	surface nsurf = make_neutral_surface(surf);
 	surface nmask(make_neutral_surface(mask));
@@ -904,12 +837,12 @@ surface mask_surface(const surface &surf, const surface &mask)
 
 	{
 		surface_lock lock(nsurf);
-		surface_lock mlock(nmask);
+		const_surface_lock mlock(nmask);
 
 		Uint32* beg = lock.pixels();
 		Uint32* end = beg + nsurf->w*surf->h;
-		Uint32* mbeg = mlock.pixels();
-		Uint32* mend = mbeg + nmask->w*nmask->h;
+		const Uint32* mbeg = mlock.pixels();
+		const Uint32* mend = mbeg + nmask->w*nmask->h;
 
 		while(beg != end && mbeg != mend) {
 			Uint8 alpha = (*beg) >> 24;
@@ -937,11 +870,14 @@ surface mask_surface(const surface &surf, const surface &mask)
 
 bool in_mask_surface(const surface &surf, const surface &mask)
 {
-	if(surf == NULL || mask == NULL) {
+	if(surf == NULL) {
 		return false;
 	}
+	if(mask == NULL){
+		return true;
+	}
 
-	if (surf->w != surf->w || surf->h != mask->h ) {
+	if (surf->w != mask->w || surf->h != mask->h ) {
 		// not same size, consider it doesn't fit
 		return false;
 	}
@@ -956,10 +892,10 @@ bool in_mask_surface(const surface &surf, const surface &mask)
 
 	{
 		surface_lock lock(nsurf);
-		surface_lock mlock(nmask);
+		const_surface_lock mlock(nmask);
 
-		Uint32* mbeg = mlock.pixels();
-		Uint32* mend = mbeg + nmask->w*nmask->h;
+		const Uint32* mbeg = mlock.pixels();
+		const Uint32* mend = mbeg + nmask->w*nmask->h;
 		Uint32* beg = lock.pixels();
 		// no need for 'end', because both surfaces have same size
 
@@ -978,6 +914,132 @@ bool in_mask_surface(const surface &surf, const surface &mask)
 	return true;
 }
 
+surface submerge_alpha(const surface &surf, int depth, float alpha_base, float alpha_delta,  bool optimize)
+{
+	if(surf== NULL) {
+		return NULL;
+	}
+
+	surface nsurf(make_neutral_surface(surf));
+
+	{
+		surface_lock lock(nsurf);
+
+		Uint32* beg = lock.pixels();
+		Uint32* limit = beg + (nsurf->h-depth) * nsurf->w ;
+		Uint32* end = beg + nsurf->w * nsurf->h;
+		beg = limit; // directlt jump to the bottom part
+
+		while(beg != end){
+			Uint8 alpha = (*beg) >> 24;
+
+			if(alpha) {
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg);
+				int d = (beg-limit)/nsurf->w;  // current depth in pixels
+				float a = alpha_base - d * alpha_delta;
+				fixed_t amount = ftofxp(a<0?0:a);
+				alpha = std::min<unsigned>(unsigned(fxpmult(alpha,amount)),255);
+				*beg = (alpha << 24) + (r << 16) + (g << 8) + b;
+			}
+
+			++beg;
+		}
+
+/*
+		for(int y = submerge_height; y < nsurf->h; ++y) {
+			Uint32* cur = beg + y * nsurf->w;
+			Uint32* row_end = beg + (y+1) * nsurf->w;
+			float d = y * 1.0 / depth;
+			double a = 0.2;//std::max<double>(0, (1-d)*0.3);
+			fixed_t amount = ftofxp(a);
+			while(cur != row_end) {
+				Uint8 alpha = (*cur) >> 24;
+
+				if(alpha) {
+					Uint8 r, g, b;
+					r = (*cur) >> 16;
+					g = (*cur) >> 8;
+					b = (*cur);
+					alpha = std::min<unsigned>(unsigned(fxpmult(alpha,amount)),255);
+					*cur = (alpha << 24) + (r << 16) + (g << 8) + b;
+				}
+
+				++cur;
+			}
+		}*/
+
+	}
+
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
+
+}
+
+surface light_surface(const surface &surf, const surface &lightmap, bool optimize)
+{
+	if(surf == NULL) {
+		return NULL;
+	}
+	if(lightmap == NULL) {
+		return surf;
+	}
+
+	surface nsurf = make_neutral_surface(surf);
+	surface nlightmap = make_neutral_surface(lightmap);
+
+	if(nsurf == NULL || nlightmap == NULL) {
+		std::cerr << "could not make neutral surface...\n";
+		return NULL;
+	}
+	if (nsurf->w != nlightmap->w) {
+		// we don't support efficiently different width.
+		// (different height is not a real problem)
+		// This function is used on all hexes and usually only for that
+		// so better keep it simple and efficient for the normal case
+		std::cerr << "Detected an image with bad dimensions :" << nsurf->w << "x" << nsurf->h << "\n";
+		std::cerr << "It will not be lighted, please use :"<< nlightmap->w << "x" << nlightmap->h << "\n";
+		return nsurf;
+	}
+	{
+		surface_lock lock(nsurf);
+		const_surface_lock llock(nlightmap);
+
+		Uint32* beg = lock.pixels();
+		Uint32* end = beg + nsurf->w * nsurf->h;
+		const Uint32* lbeg = llock.pixels();
+		const Uint32* lend = lbeg + nlightmap->w * nlightmap->h;
+
+		while(beg != end && lbeg != lend) {
+			Uint8 alpha = (*beg) >> 24;
+ 			if(alpha) {
+				Uint8 lr, lg, lb;
+
+				lr = (*lbeg) >> 16;
+				lg = (*lbeg) >> 8;
+				lb = (*lbeg);
+
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg);
+
+				r = std::max<int>(0,std::min<int>(255,int(r) + lr - 128));
+				g = std::max<int>(0,std::min<int>(255,int(g) + lg - 128));
+				b = std::max<int>(0,std::min<int>(255,int(b) + lb - 128));
+
+				*beg = (alpha << 24) + (r << 16) + (g << 8) + b;
+			}
+			++beg;
+			++lbeg;
+		}
+	}
+
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
+}
+
+
 surface blur_surface(const surface &surf, int depth, bool optimize)
 {
 	if(surf == NULL) {
@@ -991,7 +1053,7 @@ surface blur_surface(const surface &surf, int depth, bool optimize)
 		return NULL;
 	}
 
-	SDL_Rect rect = { 0, 0, surf->w, surf->h };
+	SDL_Rect rect = create_rect(0, 0, surf->w, surf->h);
 	blur_surface(res, rect, depth);
 
 	return optimize ? create_optimized_surface(res) : res;
@@ -1003,9 +1065,6 @@ void blur_surface(surface& surf, SDL_Rect rect, unsigned depth)
 		return;
 	}
 
-	assert((surf->flags & SDL_RLEACCEL) == 0);
-	assert(surf->format->BitsPerPixel == 32);
-
 	const unsigned max_blur = 256;
 	if(depth > max_blur) {
 		depth = max_blur;
@@ -1240,6 +1299,9 @@ surface blur_alpha_surface(const surface &surf, int depth, bool optimize)
 
 surface cut_surface(const surface &surf, SDL_Rect const &r)
 {
+	if(surf == NULL)
+		return NULL;
+
 	surface res = create_compatible_surface(surf, r.w, r.h);
 
 	size_t sbpp = surf->format->BytesPerPixel;
@@ -1271,14 +1333,14 @@ surface cut_surface(const surface &surf, SDL_Rect const &r)
 	if(src_rect.x >= surf->w || src_rect.y >= surf->h)
 		return res;
 
-	surface_lock slock(surf);
+	const_surface_lock slock(surf);
 	surface_lock rlock(res);
 
-	Uint8* src = reinterpret_cast<Uint8 *>(slock.pixels());
+	const Uint8* src = reinterpret_cast<const Uint8 *>(slock.pixels());
 	Uint8* dest = reinterpret_cast<Uint8 *>(rlock.pixels());
 
 	for(int y = 0; y < src_rect.h && (src_rect.y + y) < surf->h; ++y) {
-		Uint8* line_src  = src  + (src_rect.y + y) * spitch + src_rect.x * sbpp;
+		const Uint8* line_src  = src  + (src_rect.y + y) * spitch + src_rect.x * sbpp;
 		Uint8* line_dest = dest + (dst_rect.y + y) * rpitch + dst_rect.x * rbpp;
 		size_t size = src_rect.w + src_rect.x <= surf->w ? src_rect.w : surf->w - src_rect.x;
 
@@ -1289,7 +1351,7 @@ surface cut_surface(const surface &surf, SDL_Rect const &r)
 	return res;
 }
 
-surface blend_surface(const surface &surf, double amount, Uint32 colour, bool optimize)
+surface blend_surface(const surface &surf, double amount, Uint32 color, bool optimize)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -1308,7 +1370,7 @@ surface blend_surface(const surface &surf, double amount, Uint32 colour, bool op
 		Uint32* end = beg + nsurf->w*surf->h;
 
 		Uint8 red, green, blue, alpha;
-		SDL_GetRGBA(colour,nsurf->format,&red,&green,&blue,&alpha);
+		SDL_GetRGBA(color,nsurf->format,&red,&green,&blue,&alpha);
 
 		red   = Uint8(red   * amount);
 		green = Uint8(green * amount);
@@ -1419,11 +1481,9 @@ void blit_surface(const surface& src,
 {
 	assert(src);
 	assert(dst);
-	assert((src->flags & SDL_RLEACCEL) == 0);
-	assert((dst->flags & SDL_RLEACCEL) == 0);
 
 	// Get the areas to blit
-	SDL_Rect dst_rect = { 0, 0, dst->w, dst->h };
+	SDL_Rect dst_rect = create_rect(0, 0, dst->w, dst->h);
 	if(dstrect) {
 		dst_rect.x = dstrect->x;
 		dst_rect.w -= dstrect->x;
@@ -1433,7 +1493,7 @@ void blit_surface(const surface& src,
 
 	}
 
-	SDL_Rect src_rect = { 0, 0, src->w, src->h };
+	SDL_Rect src_rect = create_rect(0, 0, src->w, src->h);
 	if(srcrect && srcrect->w && srcrect->h) {
 		src_rect.x = srcrect->x;
 		src_rect.y = srcrect->y;
@@ -1475,21 +1535,14 @@ void blit_surface(const surface& src,
 	// Get the blit size limits.
 	const unsigned width = std::min(src_rect.w, dst_rect.w);
 	const unsigned height = std::min(src_rect.h, dst_rect.h);
-// 	std::cout << width << " -- " << height << "\n";
-// 	std::cout << src->w << " -- " << src->h << "\n";
-// 	std::cout << srcrect->x << "," << srcrect->y << " - " << srcrect->w << "x" << srcrect->h << " - " "\n";
-// 	if (dstrect)
-// 		std::cout << dstrect->x << "," << dstrect->y << " - " << dstrect->w << "x" << dstrect->h << " - " "\n";
-// 	std::cout << src_rect.x << "," << src_rect.y << " - " << src_rect.w << "x" << src_rect.h << " - " "\n";
-// 	std::cout << dst_rect.x << "," << dst_rect.y << " - " << dst_rect.w << "x" << dst_rect.h << " - " "\n";
 
 	{
 		// Extra scoping used for the surface_lock.
-		surface_lock src_lock(src);
+		const_surface_lock src_lock(src);
 		surface_lock dst_lock(dst);
 
-		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
-		Uint32* dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+		const Uint32* const src_pixels = src_lock.pixels();
+		Uint32* dst_pixels = dst_lock.pixels();
 
 		for(unsigned y = 0; y < height; ++y) {
 			for(unsigned x = 0; x < width; ++x) {
@@ -1545,7 +1598,7 @@ void blit_surface(const surface& src,
 
 					// Destination and source party transparent.
 
-					// aquired the data now do the blitting
+					// acquired the data now do the blitting
 					const unsigned tmp_a = 255 - src_a;
 
 					const unsigned tmp_r = 1 + (src_r * src_a) + (dst_r * tmp_a);
@@ -1569,10 +1622,10 @@ void blit_surface(const surface& src,
 
 
 
-void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, const surface &target)
+void fill_rect_alpha(SDL_Rect &rect, Uint32 color, Uint8 alpha, surface &target)
 {
 	if(alpha == SDL_ALPHA_OPAQUE) {
-		SDL_FillRect(target,&rect,colour);
+		sdl_fill_rect(target,&rect,color);
 		return;
 	} else if(alpha == SDL_ALPHA_TRANSPARENT) {
 		return;
@@ -1584,9 +1637,9 @@ void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, const surface &
 	}
 
 	SDL_Rect r = {0,0,rect.w,rect.h};
-	SDL_FillRect(tmp,&r,colour);
+	sdl_fill_rect(tmp,&r,color);
 	SDL_SetAlpha(tmp,SDL_SRCALPHA,alpha);
-	SDL_BlitSurface(tmp,NULL,target,&rect);
+	sdl_blit(tmp,NULL,target,&rect);
 }
 
 surface get_surface_portion(const surface &src, SDL_Rect &area, bool optimize_format)
@@ -1615,7 +1668,7 @@ surface get_surface_portion(const surface &src, SDL_Rect &area, bool optimize_fo
 		return NULL;
 	}
 
-	SDL_BlitSurface(src, &area, dst, NULL);
+	sdl_blit(src, &area, dst, NULL);
 
 	return optimize_format ? display_format_alpha(dst) : dst;
 }
@@ -1638,7 +1691,7 @@ struct not_alpha
 SDL_Rect get_non_transparent_portion(const surface &surf)
 {
 	SDL_Rect res = {0,0,0,0};
-	const surface nsurf(make_neutral_surface(surf));
+	surface nsurf(make_neutral_surface(surf));
 	if(nsurf == NULL) {
 		std::cerr << "failed to make neutral surface\n";
 		return res;
@@ -1722,11 +1775,11 @@ bool operator!=(const SDL_Color& a, const SDL_Color& b) {
 	return !operator==(a,b);
 }
 
-SDL_Color inverse(const SDL_Color& colour) {
+SDL_Color inverse(const SDL_Color& color) {
 	SDL_Color inverse;
-	inverse.r = 255 - colour.r;
-	inverse.g = 255 - colour.g;
-	inverse.b = 255 - colour.b;
+	inverse.r = 255 - color.r;
+	inverse.g = 255 - color.g;
+	inverse.b = 255 - color.b;
 	inverse.unused = 0;
 	return inverse;
 }
@@ -1756,7 +1809,7 @@ void surface_restorer::restore(SDL_Rect const &dst) const
 	SDL_Rect src = dst2;
 	src.x -= rect_.x;
 	src.y -= rect_.y;
-	SDL_BlitSurface(surface_, &src, target_->getSurface(), &dst2);
+	sdl_blit(surface_, &src, target_->getSurface(), &dst2);
 	update_rect(dst2);
 }
 
@@ -1765,7 +1818,7 @@ void surface_restorer::restore() const
 	if (surface_.null())
 		return;
 	SDL_Rect dst = rect_;
-	SDL_BlitSurface(surface_, NULL, target_->getSurface(), &dst);
+	sdl_blit(surface_, NULL, target_->getSurface(), &dst);
 	update_rect(rect_);
 }
 
@@ -1782,18 +1835,18 @@ void surface_restorer::cancel()
 	surface_.assign(NULL);
 }
 
-void draw_rectangle(int x, int y, int w, int h, Uint32 colour,surface target)
+void draw_rectangle(int x, int y, int w, int h, Uint32 color,surface target)
 {
 
-	SDL_Rect top = {x,y,w,1};
-	SDL_Rect bot = {x,y+h-1,w,1};
-	SDL_Rect left = {x,y,1,h};
-	SDL_Rect right = {x+w-1,y,1,h};
+	SDL_Rect top = create_rect(x, y, w, 1);
+	SDL_Rect bot = create_rect(x, y + h - 1, w, 1);
+	SDL_Rect left = create_rect(x, y, 1, h);
+	SDL_Rect right = create_rect(x + w - 1, y, 1, h);
 
-	SDL_FillRect(target,&top,colour);
-	SDL_FillRect(target,&bot,colour);
-	SDL_FillRect(target,&left,colour);
-	SDL_FillRect(target,&right,colour);
+	sdl_fill_rect(target,&top,color);
+	sdl_fill_rect(target,&bot,color);
+	sdl_fill_rect(target,&left,color);
+	sdl_fill_rect(target,&right,color);
 }
 
 void draw_solid_tinted_rectangle(int x, int y, int w, int h,
@@ -1801,23 +1854,23 @@ void draw_solid_tinted_rectangle(int x, int y, int w, int h,
                                  double alpha, surface target)
 {
 
-	SDL_Rect rect = {x,y,w,h};
+	SDL_Rect rect = create_rect(x, y, w, h);
 	fill_rect_alpha(rect,SDL_MapRGB(target->format,r,g,b),Uint8(alpha*255),target);
 }
 
 void draw_centered_on_background(surface surf, const SDL_Rect& rect, const SDL_Color& color, surface target)
 {
-	clip_rect_setter clip_setter(target, rect);
+	clip_rect_setter clip_setter(target, &rect);
 
 	Uint32 col = SDL_MapRGBA(target->format, color.r, color.g, color.b, color.unused);
 	//TODO: only draw background outside the image
 	SDL_Rect r = rect;
-	SDL_FillRect(target, &r, col);
+	sdl_fill_rect(target, &r, col);
 
 	if (surf != NULL) {
 		r.x = rect.x + (rect.w-surf->w)/2;
 		r.y = rect.y + (rect.h-surf->h)/2;
-		SDL_BlitSurface(surf, NULL, target, &r);
+		sdl_blit(surf, NULL, target, &r);
 	}
 	update_rect(rect);
 }
