@@ -1,13 +1,14 @@
-/* $Id: parser.cpp 41171 2010-02-12 14:46:28Z ilor $ */
+/* $Id: parser.cpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 - 2010 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2011 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,8 +16,8 @@
 */
 
 /**
- * @file serialization/parser.cpp
- * Read/Write & analyse WML- and config-files.
+ * @file
+ * Read/Write & analyze WML- and config-files.
  */
 
 
@@ -30,6 +31,7 @@
 #include "serialization/preprocessor.hpp"
 #include "serialization/tokenizer.hpp"
 #include "serialization/string_utils.hpp"
+#include "serialization/validator.hpp"
 #include "foreach.hpp"
 
 #include <stack>
@@ -37,6 +39,7 @@
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
 #include <boost/algorithm/string/replace.hpp>
+#include <boost/variant.hpp>
 
 static lg::log_domain log_config("config");
 #define ERR_CF LOG_STREAM(err, log_config)
@@ -46,16 +49,16 @@ static lg::log_domain log_config("config");
 static const size_t max_recursion_levels = 1000;
 
 namespace {
-
 class parser
 {
 	parser();
 	parser(const parser&);
 	parser& operator=(const parser&);
 public:
-	parser(config& cfg, std::istream& in);
+	parser(config& cfg, std::istream& in,
+		   abstract_validator * validator = NULL);
 	~parser();
-	void operator() (std::string* error_log=NULL);
+	void operator()();
 
 private:
 	void parse_element();
@@ -66,32 +69,28 @@ private:
 
 	config& cfg_;
 	tokenizer *tok_;
+	abstract_validator *validator_;
 
 	struct element {
-		element(config *cfg, std::string
-			const &name, const size_t& start_line = 0, const std::string& file="") :
-				cfg(cfg),
-				name(name),
-				last_element_map(),
-				start_line(start_line),
-			    file(file)
-			{}
+		element(config *cfg, std::string const &name,
+			int start_line = 0, const std::string &file = "") :
+			cfg(cfg), name(name), start_line(start_line), file(file)
+		{}
 
 		config* cfg;
 		std::string name;
-
-		std::map<std::string, config*> last_element_map;
-		size_t start_line;
+		int start_line;
 		std::string file;
 	};
 
 	std::stack<element> elements;
 };
 
-parser::parser(config &cfg, std::istream &in) :
-		cfg_(cfg),
-		tok_(new tokenizer(in)),
-		elements()
+parser::parser(config &cfg, std::istream &in, abstract_validator * validator)
+			   :cfg_(cfg),
+			   tok_(new tokenizer(in)),
+			   validator_(validator),
+			   elements()
 {
 }
 
@@ -101,50 +100,37 @@ parser::~parser()
 	delete tok_;
 }
 
-void parser::operator()(std::string* error_log)
+void parser::operator()()
 {
 	cfg_.clear();
 	elements.push(element(&cfg_, ""));
 
 	do {
-		try {
-			tok_->next_token();
+		tok_->next_token();
 
-			switch(tok_->current_token().type) {
-			case token::LF:
-				continue;
-			case '[':
-				parse_element();
-				break;
-			case token::STRING:
-				parse_variable();
-				break;
-			default:
-				if (static_cast<unsigned char>(tok_->current_token().value[0]) == 0xEF &&
-				    static_cast<unsigned char>(tok_->next_token().value[0])    == 0xBB &&
-				    static_cast<unsigned char>(tok_->next_token().value[0])    == 0xBF)
-				{
-					ERR_CF << "Skipping over a utf8 BOM\n";
-				} else {
-					error(_("Unexpected characters at line start"));
-				}
-				break;
-			case token::END:
-				break;
+		switch(tok_->current_token().type) {
+		case token::LF:
+			continue;
+		case '[':
+			parse_element();
+			break;
+		case token::STRING:
+			parse_variable();
+			break;
+		default:
+			if (static_cast<unsigned char>(tok_->current_token().value[0]) == 0xEF &&
+			    static_cast<unsigned char>(tok_->next_token().value[0])    == 0xBB &&
+			    static_cast<unsigned char>(tok_->next_token().value[0])    == 0xBF)
+			{
+				ERR_CF << "Skipping over a utf8 BOM\n";
+			} else {
+				error(_("Unexpected characters at line start"));
 			}
-		} catch(config::error& e) {
-			if(error_log == NULL)
-				throw;
-
-			// On error, dump tokens to the next LF
-			while(tok_->current_token().type != token::LF &&
-					tok_->current_token().type != token::END) {
-				tok_->next_token();
-			}
-
-			*error_log += e.message + '\n';
+			break;
+		case token::END:
+			break;
 		}
-		increment_parser_progress();
+		loadscreen::increment_progress();
 	} while (tok_->current_token().type != token::END);
 
 	// The main element should be there. If it is not, this is a parser error.
@@ -165,18 +151,18 @@ void parser::parse_element()
 	tok_->next_token();
 	std::string elname;
 	config* current_element = NULL;
-	std::map<std::string, config*>::const_iterator last_element_itor;
-
 	switch(tok_->current_token().type) {
 	case token::STRING: // [element]
 		elname = tok_->current_token().value;
 		if (tok_->next_token().type != ']')
 			error(_("Unterminated [element] tag"));
-
 		// Add the element
 		current_element = &(elements.top().cfg->add_child(elname));
-		elements.top().last_element_map[elname] = current_element;
 		elements.push(element(current_element, elname, tok_->get_start_line(), tok_->get_file()));
+		if (validator_){
+			validator_->open_tag(elname,tok_->get_start_line(),
+								  tok_->get_file());
+		}
 		break;
 
 	case '+': // [+element]
@@ -188,13 +174,19 @@ void parser::parse_element()
 
 		// Find the last child of the current element whose name is
 		// element
-		last_element_itor = elements.top().last_element_map.find(elname);
-		if(last_element_itor == elements.top().last_element_map.end()) {
-			current_element = &elements.top().cfg->add_child(elname);
+		if (config &c = elements.top().cfg->child(elname, -1)) {
+			current_element = &c;
+			if (validator_){
+				validator_->open_tag(elname,tok_->get_start_line(),
+									 tok_->get_file(),true);
+			}
 		} else {
-			current_element = last_element_itor->second;
+			current_element = &elements.top().cfg->add_child(elname);
+			if (validator_){
+				validator_->open_tag(elname,tok_->get_start_line(),
+									 tok_->get_file());
+			}
 		}
-		elements.top().last_element_map[elname] = current_element;
 		elements.push(element(current_element, elname, tok_->get_start_line(), tok_->get_file()));
 		break;
 
@@ -215,7 +207,11 @@ void parser::parse_element()
 			error(lineno_string(i18n_symbols, ss.str(),
 					N_("Found invalid closing tag $tag2 for tag $tag1 (opened at $pos)")));
 		}
-
+		if(validator_){
+			element & el= elements.top();
+			validator_->validate(*el.cfg,el.name,el.start_line,el.file);
+			validator_->close_tag();
+		}
 		elements.pop();
 		break;
 	default:
@@ -252,15 +248,11 @@ void parser::parse_variable()
 	if(variables.back().empty())
 		error(_("Empty variable name"));
 
-	{
-		for(std::vector<std::string>::iterator curvar = variables.begin(); curvar != variables.end(); ++curvar) {
-			cfg[*curvar] = "";
-		}
-	}
+	t_string_base buffer;
 
 	std::vector<std::string>::const_iterator curvar = variables.begin();
 
-	bool ignore_next_newlines = false;
+	bool ignore_next_newlines = false, previous_string = false;
 	while(1) {
 		tok_->next_token();
 		assert(curvar != variables.end());
@@ -268,11 +260,19 @@ void parser::parse_variable()
 		switch (tok_->current_token().type) {
 		case ',':
 			if ((curvar+1) != variables.end()) {
-				curvar++;
-				cfg[*curvar] = "";
-				continue;
+				if (buffer.translatable())
+					cfg[*curvar] = t_string(buffer);
+				else
+					cfg[*curvar] = buffer.value();
+				if(validator_){
+					validator_->validate_key (cfg,*curvar,buffer.value(),
+											  tok_->get_start_line (),
+											  tok_->get_file ());
+				}
+				buffer = t_string_base();
+				++curvar;
 			} else {
-				cfg[*curvar] += ",";
+				buffer += ",";
 			}
 			break;
 		case '_':
@@ -282,42 +282,56 @@ void parser::parse_variable()
 				error(_("Unterminated quoted string"));
 				break;
 			case token::QSTRING:
-				cfg[*curvar] += t_string(tok_->current_token().value, tok_->textdomain());
+				buffer += t_string_base(tok_->current_token().value, tok_->textdomain());
 				break;
 			default:
-				cfg[*curvar] += "_";
-				cfg[*curvar] += tok_->current_token().value;
+				buffer += "_";
+				buffer += tok_->current_token().value;
 				break;
 			case token::END:
 			case token::LF:
-				return;
+				buffer += "_";
+				goto finish;
 			}
 			break;
 		case '+':
-			// Ignore this
-			break;
+			ignore_next_newlines = true;
+			continue;
+		case token::STRING:
+			if (previous_string) buffer += " ";
+			//nobreak
 		default:
-			cfg[*curvar] += tok_->current_token().leading_spaces + tok_->current_token().value;
+			buffer += tok_->current_token().value;
 			break;
 		case token::QSTRING:
-			cfg[*curvar] += tok_->current_token().value;
+			buffer += tok_->current_token().value;
 			break;
 		case token::UNTERMINATED_QSTRING:
 			error(_("Unterminated quoted string"));
 			break;
 		case token::LF:
-			if(!ignore_next_newlines)
-				return;
-			break;
+			if (ignore_next_newlines) continue;
+			//nobreak
 		case token::END:
-			return;
+			goto finish;
 		}
 
-		if (tok_->current_token().type == '+') {
-			ignore_next_newlines = true;
-		} else if (tok_->current_token().type != token::LF) {
-			ignore_next_newlines = false;
-		}
+		previous_string = tok_->current_token().type == token::STRING;
+		ignore_next_newlines = false;
+	}
+
+	finish:
+	if (buffer.translatable())
+		cfg[*curvar] = t_string(buffer);
+	else
+		cfg[*curvar] = buffer.value();
+	if(validator_){
+		validator_->validate_key (cfg,*curvar,buffer.value(),
+								  tok_->get_start_line (),
+								  tok_->get_file ());
+	}
+	while (++curvar != variables.end()) {
+		cfg[*curvar] = "";
 	}
 }
 
@@ -355,18 +369,18 @@ void parser::error(const std::string& error_type)
 
 } // end anon namespace
 
-void read(config &cfg, std::istream &in, std::string* error_log)
+void read(config &cfg, std::istream &in, abstract_validator * validator)
 {
-	parser(cfg, in)(error_log);
+	parser(cfg, in, validator)();
 }
 
-void read(config &cfg, std::string &in, std::string* error_log)
+void read(config &cfg, std::string &in, abstract_validator * validator)
 {
-	std::stringstream ss(in);
-	parser(cfg, ss)(error_log);
+	std::istringstream ss(in);
+	parser(cfg, ss, validator)();
 }
 
-void read_gz(config &cfg, std::istream &file, std::string* error_log)
+void read_gz(config &cfg, std::istream &file, abstract_validator * validator)
 {
 	//an empty gzip file seems to confuse boost on msvc
 	//so return early if this is the case
@@ -377,100 +391,102 @@ void read_gz(config &cfg, std::istream &file, std::string* error_log)
 	filter.push(boost::iostreams::gzip_decompressor());
 	filter.push(file);
 
-	parser(cfg, filter)(error_log);
+	parser(cfg, filter,validator)();
 }
 
-static char const *AttributeEquals = "=";
-
-static char const *TranslatableAttributePrefix = "_ \"";
-static char const *AttributePrefix = "\"";
-static char const *AttributePostfix = "\"";
-
-static char const* AttributeContPostfix = " + \n";
-static char const* AttributeEndPostfix = "\n";
-
-static char const* TextdomainPrefix = "#textdomain ";
-static char const* TextdomainPostfix = "\n";
-
-static char const *ElementPrefix = "[";
-static char const *ElementPostfix = "]\n";
-static char const *EndElementPrefix = "[/";
-static char const *EndElementPostfix = "]\n";
-
-static std::string escaped_string(const std::string& value) {
-	std::vector<char> res;
-	for(std::string::const_iterator i = value.begin(); i != value.end(); ++i) {
-		// double interior quotes
-		if(*i == '\"') res.push_back(*i);
-		res.push_back(*i);
+static std::string escaped_string(const std::string &value)
+{
+	std::string res;
+	std::string::const_iterator iter = value.begin();
+	while (iter != value.end()) {
+		const char c = *iter;
+		res.append(c == '"' ? 2 : 1, c);
+		++iter;
 	}
-	return std::string(res.begin(), res.end());
+	return res;
 }
 
-void write_key_val(std::ostream &out, const std::string &key, const t_string &value, unsigned int level, std::string& textdomain)
+struct write_key_val_visitor : boost::static_visitor<void>
+{
+	std::ostream &out_;
+	unsigned level_;
+	std::string &textdomain_;
+	const std::string &key_;
+
+	write_key_val_visitor(std::ostream &out, unsigned level,
+		std::string &textdomain, const std::string &key)
+		: out_(out), level_(level), textdomain_(textdomain), key_(key)
+	{}
+
+	void operator()(boost::blank const &) const
+	{ out_ << "\"\""; }
+	void operator()(bool b) const
+	{ out_ << (b ? "yes" : "no"); }
+	void operator()(double d) const
+	{ int i = d; if (d == i) out_ << i; else out_ << d; }
+	void operator()(std::string const &s) const
+	{ out_ << '"' << escaped_string(s) << '"'; }
+	void operator()(t_string const &s) const;
+};
+
+/**
+ * Writes all the parts of a translatable string.
+ * @note If the first part is translatable and in the wrong textdomain,
+ *       the textdomain change has to happen before the attribute name.
+ *       That is the reason for not outputting the key beforehand and
+ *       letting this function do it.
+ */
+void write_key_val_visitor::operator()(t_string const &value) const
 {
 	bool first = true;
-	if (value.empty()) {
-		out << std::string(level, '\t') << key << AttributeEquals
-			<< AttributePrefix << AttributePostfix
-			<< AttributeEndPostfix;;
-		return;
-	}
 
-	for(t_string::walker w(value); !w.eos(); w.next()) {
+	for (t_string::walker w(value); !w.eos(); w.next())
+	{
 		std::string part(w.begin(), w.end());
 
-		if(w.translatable()) {
-			if(w.textdomain() != textdomain) {
-				out << TextdomainPrefix
-					<< w.textdomain()
-					<< TextdomainPostfix;
-				textdomain = w.textdomain();
-			}
+		if (!first)
+			out_ << " +\n";
 
-			if(first) {
-				out << std::string(level, '\t')
-					<< key
-					<< AttributeEquals;
-			}
-
-			out << TranslatableAttributePrefix
-				<< escaped_string(part)
-				<< AttributePostfix;
-
-		} else {
-			if(first) {
-				out << std::string(level, '\t')
-					<< key
-					<< AttributeEquals;
-			}
-
-			out << AttributePrefix
-				<< escaped_string(part)
-				<< AttributePostfix;
+		if (w.translatable() && w.textdomain() != textdomain_) {
+			textdomain_ = w.textdomain();
+			out_ << "#textdomain " << textdomain_ << '\n';
 		}
 
-		if(w.last()) {
-			out << AttributeEndPostfix;
-		} else {
-			out << AttributeContPostfix;
-			out << std::string(level+1, '\t');
-		}
+		for (unsigned i = 0; i < level_; ++i) out_ << '\t';
 
+		if (first)
+			out_ << key_ << '=';
+		else
+			out_ << '\t';
+
+		if (w.translatable())
+			out_ << '_';
+
+		out_ << '"' << escaped_string(part) << '"';
 		first = false;
 	}
 }
 
+void write_key_val(std::ostream &out, const std::string &key,
+	const config::attribute_value &value, unsigned level,
+	std::string& textdomain)
+{
+	if (!boost::get<t_string const>(&value.value)) {
+		for (unsigned i = 0; i < level; ++i) out << '\t';
+		out << key << '=';
+	}
+	boost::apply_visitor(write_key_val_visitor(out, level, textdomain, key), value.value);
+	out << '\n';
+}
+
 void write_open_child(std::ostream &out, const std::string &child, unsigned int level)
 {
-	out << std::string(level, '\t')
-		<< ElementPrefix << child << ElementPostfix;
+	out << std::string(level, '\t') << '[' << child << "]\n";
 }
 
 void write_close_child(std::ostream &out, const std::string &child, unsigned int level)
 {
-	out << std::string(level, '\t')
-		<< EndElementPrefix << child << EndElementPostfix;
+	out << std::string(level, '\t') << "[/" << child << "]\n";
 }
 
 static void write_internal(config const &cfg, std::ostream &out, std::string& textdomain, size_t tab = 0)
@@ -496,3 +512,11 @@ void write(std::ostream &out, config const &cfg, unsigned int level)
 	write_internal(cfg, out, textdomain, level);
 }
 
+void write_gz(std::ostream &out, config const &cfg)
+{
+	boost::iostreams::filtering_stream<boost::iostreams::output> filter;
+	filter.push(boost::iostreams::gzip_compressor());
+	filter.push(out);
+
+	write(filter, cfg);
+}
