@@ -1,11 +1,12 @@
-/* $Id: ban.hpp 35119 2009-04-22 15:33:00Z soliton $ */
+/* $Id: ban.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by Pauli Nieminen <paniemin@cc.hut.fi>
+   Copyright (C) 2008 - 2011 by Pauli Nieminen <paniemin@cc.hut.fi>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,7 +15,7 @@
 
 #ifndef SERVER_GAME_HPP_INCLUDED
 #define SERVER_GAME_HPP_INCLUDED
-#include "game_errors.hpp"
+#include "exceptions.hpp"
 
 #include <set>
 #include <map>
@@ -39,21 +40,12 @@ namespace wesnothd {
 		bool operator()(const banned_ptr& a, const banned_ptr& b) const;
 	};
 
-	struct subnet_compare_setter;
 	struct banned_compare_subnet {
 		bool operator()(const banned_ptr& a, const banned_ptr& b) const;
 		private:
-		static void set_use_subnet_mask(bool);
 		bool less(const banned_ptr& a, const banned_ptr& b) const;
-		bool less_with_subnet(const banned_ptr& a, const banned_ptr& b) const;
 		typedef bool (banned_compare_subnet::*compare_fn)(const banned_ptr& a, const banned_ptr& b) const;
 		static compare_fn active_;
-		friend struct subnet_compare_setter;
-	};
-
-	struct subnet_compare_setter {
-		subnet_compare_setter();
-		~subnet_compare_setter();
 	};
 
 	typedef std::set<banned_ptr,banned_compare_subnet > ban_set;
@@ -110,6 +102,7 @@ namespace wesnothd {
 		{ return group_ == group; }
 
 		bool match_ip(const ip_mask& ip) const;
+		bool match_ipmask(const ip_mask& ip) const;
 
 		unsigned int get_mask_ip(unsigned int) const;
 		unsigned int get_int_ip() const
@@ -122,8 +115,8 @@ namespace wesnothd {
 
 		bool operator>(const banned& b) const;
 
-		struct error : public ::game::error {
-			error(const std::string& message) : ::game::error(message) {}
+		struct error : public game::error {
+			error(const std::string& message) : game::error(message) {}
 		};
 	};
 
@@ -166,8 +159,8 @@ namespace wesnothd {
 
 		void check_ban_times(time_t time_now);
 
-		void list_deleted_bans(std::ostringstream& out) const;
-		void list_bans(std::ostringstream& out) const;
+		void list_deleted_bans(std::ostringstream& out, const std::string& mask = "*") const;
+		void list_bans(std::ostringstream& out, const std::string& mask = "*") const;
 
 		std::string is_ip_banned(const std::string& ip) const;
 
