@@ -1,11 +1,12 @@
-/* $Id: forum_user_handler.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: forum_user_handler.hpp 49961 2011-06-21 00:53:33Z shadowmaster $ */
 /*
-   Copyright (C) 2008 - 2010 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
+   Copyright (C) 2008 - 2011 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -63,6 +64,8 @@ class fuh : public user_handler {
 
 		bool user_exists(const std::string& name);
 
+		bool user_is_active(const std::string& name);
+
 		bool user_is_moderator(const std::string& name);
 		void set_is_moderator(const std::string& name, const bool& is_moderator);
 
@@ -85,6 +88,7 @@ class fuh : public user_handler {
 		std::vector<std::string> get_ignores(const std::string& user);*/
 		time_t get_lastlogin(const std::string& user);
 		time_t get_registrationdate(const std::string& user);
+		bool is_inactive(const std::string& user);
 
 		void set_lastlogin(const std::string& user, const time_t& lastlogin);
 
