@@ -1,11 +1,12 @@
-/* $Id: player.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: player.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -16,12 +17,10 @@
 #define PLAYER_HPP_INCLUDED
 
 
-#include "../config.hpp"
 #include "simple_wml.hpp"
 
 #include <ctime>
 #include <set>
-#include <string>
 
 namespace wesnothd {
 
@@ -55,7 +54,6 @@ public:
 
 	const simple_wml::node* config_address() const { return &cfg_; }
 
-	bool silenced() const { return messages_since_flood_start_ > MaxMessages; }
 	bool is_message_flooding();
 
 	/**
