@@ -1,11 +1,12 @@
-/* $Id: room_manager.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: room_manager.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2009 - 2010 by Tomasz Sniatowski <kailoran@gmail.com>
+   Copyright (C) 2009 - 2011 by Tomasz Sniatowski <kailoran@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,10 +15,10 @@
 
 #include "room.hpp"
 
-#include "../config.hpp"
-
 #include <boost/utility.hpp>
 
+class config;
+
 #ifndef SERVER_ROOM_MANAGER_HPP_INCLUDED
 #define SERVER_ROOM_MANAGER_HPP_INCLUDED
 
@@ -50,18 +51,12 @@ public:
 	};
 
 	static PRIVILEGE_POLICY pp_from_string(const std::string& str);
-	static const char* string_from_pp(PRIVILEGE_POLICY pp);
 
 	/**
 	 * Load settings from the main config file
 	 */
 	void load_config(const config& cfg);
 
-	/**
-	 * Get the room storage file name. If empty, rooms are not stored on disk.
-	 */
-	const std::string& storage_filename() const;
-
 	/**
 	 * Reads stored rooms from a file on disk, or returns immediately
 	 * if load_config was not called before or the storage filename is empty
@@ -104,11 +99,6 @@ public:
 	 */
 	room* get_create_room(const std::string& name, network::connection player);
 
-	/**
-	 * Delete a room. Players still in the room are kicked from it.
-	 */
-	void delete_room(const std::string& name);
-
 	/**
 	 * @return true iif the player is in the lobby
 	 */
@@ -237,7 +227,7 @@ private:
 	t_player_stored_rooms_ player_stored_rooms_;
 
 	/**
-	 * Persistent room storage filename
+	 * Persistent room storage filename.  If empty, rooms are not stored on disk.
 	 */
 	std::string filename_;
 
