@@ -1,11 +1,12 @@
-/* $Id: sample_user_handler.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: sample_user_handler.cpp 51816 2011-11-03 17:24:38Z espreon $ */
 /*
-   Copyright (C) 2008 - 2010 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
+   Copyright (C) 2008 - 2011 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -38,7 +39,7 @@ suh::suh(config c)
 }
 
 void suh::add_user(const std::string& name, const std::string& mail, const std::string& password) {
-	if(user_exists(name)) throw error("This nick is already registered");
+	if(user_exists(name)) throw error("This nickname is already registered");
 
 	users_.insert(std::pair<std::string, user>(name, user()));
 
@@ -49,7 +50,7 @@ void suh::add_user(const std::string& name, const std::string& mail, const std::
 }
 
 void suh::remove_user(const std::string& name) {
-	if(!user_exists(name)) throw error("This nick is not registered");
+	if(!user_exists(name)) throw error("This nickname is not registered");
 
 	users_.erase(name);
 }
@@ -60,6 +61,11 @@ bool suh::user_exists(const std::string& name) {
 	return (u != users_.end());
 }
 
+bool suh::user_is_active(const std::string& /*name*/) {
+	// FIXME: add support for inactive users maybe?
+	return true;
+}
+
 std::vector<std::string> suh::users() {
 	std::vector<std::string> users;
 	for(std::map<std::string,user>::const_iterator u = users_.begin(); u != users_.end(); ++u) {
@@ -148,8 +154,8 @@ void suh::check_name(const std::string& name) {
 				"characters. Only alpha-numeric characters, underscores and hyphens"
 				"are allowed.");
 	}
-	if(name.size() > 18) {
-		throw error("This username is too long. Usernames must be 18 characers or less.");
+	if(name.size() > 20) {
+		throw error("This username is too long. Usernames must be 20 characters or less.");
 	}
 }
 
@@ -216,12 +222,21 @@ std::string suh::user_info(const std::string& name) {
 	time_t ll_date = get_lastlogin(name);
 
 	std::string reg_string = ctime(&reg_date);
-	std::string ll_string = ctime(&ll_date);
+	std::string ll_string;
+
+	if(ll_date) {
+		ll_string = ctime(&ll_date);
+	} else {
+		ll_string = "Never";
+	}
 
 	std::stringstream info;
 	info << "Name: " << name << "\n"
 		 << "Real name: " << get_realname(name) << "\n"
 		 << "Registered: " << reg_string
 		 << "Last login: " << ll_string;
+	if(!user_is_active(name)) {
+		info << "This account is currently inactive.\n";
+	}
 	return info.str();
 }
