@@ -1,11 +1,12 @@
-/* $Id: sample_user_handler.hpp 33638 2009-03-14 17:33:44Z baufo $ */
+/* $Id: sample_user_handler.hpp 49961 2011-06-21 00:53:33Z shadowmaster $ */
 /*
-   Copyright (C) 2008 - 2009 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
+   Copyright (C) 2008 - 2011 by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -17,11 +18,15 @@
 
 #include "user_handler.hpp"
 
-/**
- * @class An exmple how to implement user_handler to demonstrate the interface
- * If you use this on anything real you are insane
- */
+#include <map>
+#include <vector>
 
+#include <ctime>
+
+/**
+ * An example of how to implement user_handler.
+ * If you use this on anything real, you are insane.
+ */
 class suh : public user_handler {
 	public:
 		suh(config c);
@@ -37,6 +42,7 @@ class suh : public user_handler {
 		void password_reminder(const std::string& name);
 
 		bool user_exists(const std::string& name);
+		bool user_is_active(const std::string& name);
 
 		bool user_is_moderator(const std::string& name);
 		void set_is_moderator(const std::string& name, const bool& is_moderator);
@@ -62,7 +68,7 @@ class suh : public user_handler {
 		void set_user_detail(const std::string& user, const std::string& detail, const std::string& value);
 		std::string get_valid_details();
 
-		std::string create_pepper(const std::string&, int) { return ""; }
+		std::string create_pepper(const std::string&) { return ""; }
 		bool use_phpbb_encryption() const { return false; }
 
 	private:
@@ -82,7 +88,6 @@ class suh : public user_handler {
 		void set_realname(const std::string& user, const std::string& realname);
 
 		void set_lastlogin(const std::string& user, const time_t& lastlogin);
-		void set_registrationdate(const std::string& user, const time_t& registrationdate);
 
 		int user_expiration_;
 
