@@ -1,11 +1,12 @@
-/* $Id: settings.cpp 32513 2009-02-01 17:03:32Z soliton $ */
+/* $Id: settings.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2007 - 2009 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2007 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- *  @file src/settings.cpp
+ *  @file
  *  General settings and defaults for scenarios.
  */
 
@@ -22,6 +23,7 @@
 #include "settings.hpp"
 
 #include "serialization/string_utils.hpp"
+#include "util.hpp"
 
 namespace settings {
 
@@ -48,20 +50,5 @@ int get_xp_modifier(const std::string& value)
 	return lexical_cast_in_range<int>(value, 70, 30, 200);
 }
 
-bool use_fog(const std::string& value)
-{
-	return utils::string_bool(value, true);
-}
-
-bool use_random_start_time(const std::string& value)
-{
-	return utils::string_bool(value, true);
-}
-
-bool use_shroud(const std::string& value)
-{
-	return utils::string_bool(value, false);
-}
-
 } // end namespace settings
 
