@@ -1,11 +1,12 @@
-/* $Id: settings.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: settings.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2007 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2007 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- *  @file src/settings.hpp
+ *  @file
  *  General settings and defaults for scenarios.
  */
 
@@ -70,36 +71,6 @@ namespace settings {
 	 */
 	int get_xp_modifier(const std::string& value);
 
-	/**
-	 * Returns whether the fog of war should be used.
-	 * If no valid value is supplied, it will return a default.
-	 *
-	 * @param value         string containing a boolean value
-	 *
-	 * @returns             whether or not the fog of war should be used
-	 */
-	bool use_fog(const std::string& value);
-
-	/**
-	 * Returns whether the random start time feature should be used.
-	 * If no valid value is supplied, it will return a default.
-	 *
-	 * @param value         string containing a boolean value
-	 *
-	 * @returns             whether or not the random start time should be used
-	 */
-	bool use_random_start_time(const std::string& value);
-
-	/**
-	 * Returns whether the shroud should be used.
-	 * If no valid value is supplied, it will return a default.
-	 *
-	 * @param value         string containing a boolean value
-	 *
-	 * @returns             whether or not the shroud should be used
-	 */
-	bool use_shroud(const std::string& value);
-
 } // namespace settings
 
 #endif
