@@ -1,11 +1,12 @@
-/* $Id: soundsource.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: soundsource.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2006 - 2010 by Karol Nowak <grzywacz@sul.uni.lodz.pl>
+   Copyright (C) 2006 - 2011 by Karol Nowak <grzywacz@sul.uni.lodz.pl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,8 +16,6 @@
 #define SOUNDSOURCE_HPP_INCLUDED
 
 #include <map>
-#include <string>
-#include <vector>
 
 #include "generic_event.hpp"
 #include "map_location.hpp"
@@ -37,12 +36,12 @@ class manager;
  */
 class positional_source {
 	unsigned int last_played_;
-	unsigned int min_delay_;
-	unsigned int chance_;
-	unsigned int loops_;
+	int min_delay_;
+	int chance_;
+	int loops_;
 	const unsigned int id_;
-	unsigned int range_;
-	unsigned int faderange_;
+	int range_;
+	int faderange_;
 	bool check_fogged_;
 	bool check_shrouded_;
 	std::string files_;
@@ -101,7 +100,7 @@ public:
 	void update_positions();
 
 	/**
-	 * Serializes information into cfg as new childs of key
+	 * Serializes information into cfg as new children of key
 	 * "sound_source", appendend to existing content.
 	 */
 	void write_sourcespecs(config& cfg) const;
