@@ -1,11 +1,12 @@
-/* $Id: statistics.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: statistics.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,7 +14,7 @@
 */
 
 /**
- *  @file statistics.cpp
+ *  @file
  *  Manage statistics: recruitments, recalls, kills, losses, etc.
  */
 
@@ -121,7 +122,7 @@ static stats::str_int_map read_str_int_map(const config& cfg)
 {
 	stats::str_int_map m;
 	foreach (const config::attribute &i, cfg.attribute_range()) {
-		m[i.first] = atoi(i.second.c_str());
+		m[i.first] = i.second;
 	}
 
 	return m;
@@ -161,7 +162,7 @@ static stats::battle_result_map read_battle_result_map(const config& cfg)
 	foreach (const config &i, cfg.child_range("sequence"))
 	{
 		config item = i;
-		const int key = atoi(item["_num"].c_str());
+		int key = item["_num"];
 		item.remove_attribute("_num");
 		m[key] = read_str_int_map(item);
 	}
@@ -404,7 +405,7 @@ void stats::read(const config& cfg)
 	turn_expected_damage_inflicted = lexical_cast<long long>(cfg["turn_expected_damage_inflicted"]);
 	turn_expected_damage_taken = lexical_cast<long long>(cfg["turn_expected_damage_taken"]);
 
-	save_id = cfg["save_id"];
+	save_id = cfg["save_id"].str();
 }
 
 scenario_context::scenario_context(const std::string& name)
@@ -586,7 +587,7 @@ stats calculate_stats(int category, std::string save_id)
 config write_stats()
 {
 	config res;
-	res["mid_scenario"] = (mid_scenario ? "yes" : "no");
+	res["mid_scenario"] = mid_scenario;
 
 	for(std::vector<scenario_stats>::const_iterator i = master_stats.begin(); i != master_stats.end(); ++i) {
 		res.add_child("scenario",i->write());
@@ -609,7 +610,7 @@ void write_stats(config_writer &out)
 void read_stats(const config& cfg)
 {
 	fresh_stats();
-	mid_scenario = (utils::string_bool(cfg["mid_scenario"]));
+	mid_scenario = cfg["mid_scenario"].to_bool();
 
 	foreach (const config &s, cfg.child_range("scenario")) {
 		master_stats.push_back(scenario_stats(s));
