@@ -1,16 +1,20 @@
-/* $Id: test_serialization.cpp 32730 2009-02-09 22:19:08Z noyga $ */
+/* $Id: test_serialization.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2009 by Karol Nowak <grywacz@gmail.com>
+   Copyright (C) 2009 - 2011 by Karol Nowak <grywacz@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
+
+#define GETTEXT_DOMAIN "wesnoth-test"
+
 #include <vector>
 #include <string>
 #include "serialization/string_utils.hpp"
@@ -18,8 +22,6 @@
 
 BOOST_AUTO_TEST_CASE( utils_join_test )
 {
-	BOOST_MESSAGE( "Starting utils::join test!" );
-
 	std::vector<std::string> fruit;
 
 	BOOST_CHECK( utils::join(fruit) == "" );
