@@ -1,11 +1,12 @@
-/* $Id: test_team.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: test_team.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2007 - 2009 by Karol Nowak <grywacz@gmail.com>
+   Copyright (C) 2007 - 2011 by Karol Nowak <grywacz@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,7 +15,7 @@
 
 #define GETTEXT_DOMAIN "wesnoth-test"
 
-#include "utils/test_support.hpp"
+#include <boost/test/unit_test.hpp>
 
 
 BOOST_AUTO_TEST_SUITE( teams )
