@@ -1,11 +1,12 @@
-/* $Id: text.hpp 41494 2010-03-07 18:43:15Z mordante $ */
+/* $Id: text.hpp 51971 2011-11-14 20:37:16Z mordante $ */
 /*
-   Copyright (C) 2008 - 2010 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -32,7 +33,20 @@ namespace gui2 {
 
 namespace font {
 
-// add background colour and also font markup.
+/**
+ * Escapses the markup characters in a text.
+ *
+ * The markups escaped are the ones used in the pango markup. The special
+ * characters are @code <>'"&@endcode. They escaping is the same as for HTML.
+ *
+ * @param text                    The text to escape.
+ *
+ * @returns                       The escaped text.
+ */
+std::string escape_text(const std::string& text);
+
+
+// add background color and also font markup.
 
 /**
  * Text class.
@@ -104,7 +118,7 @@ public:
 
 	/**
 	 * The flags have the same values as the ones in SDL_TTF so it's easy to mix
-	 * them for now. To avoid including SDL_TTF in the header they'r only
+	 * them for now. To avoid including SDL_TTF in the header they're only
 	 * declared here. Once SDL_TTF is removed they can be moved in the header.
 	 */
 
@@ -169,7 +183,7 @@ public:
 
 	ttext& set_font_style(const unsigned font_style);
 
-	ttext& set_foreground_colour(const Uint32 colour);
+	ttext& set_foreground_color(const Uint32 color);
 
 	ttext& set_maximum_width(int width);
 
@@ -203,8 +217,8 @@ private:
 	/** The style of the font, this is an orred mask of the font flags. */
 	unsigned font_style_;
 
-	/** The foreground colour. */
-	Uint32 foreground_colour_;
+	/** The foreground color. */
+	Uint32 foreground_color_;
 
 	/**
 	 * The maximum width of the text.
@@ -280,13 +294,28 @@ private:
 	 * If needed frees the other surface and then creates a new buffer and
 	 * initializes the entire buffer with values 0.
 	 *
-	 * NOTE eventhough we're clearly modifing function we don't change the
+	 * NOTE eventhough we're clearly modifying function we don't change the
 	 * state of the object. The const is needed so other functions can also be
 	 * marked const (those also don't change the state of the object.
 	 *
 	 * @param size                The required size of the buffer.
 	 */
 	void create_surface_buffer(const size_t size) const;
+
+	/**
+	 * Sets the markup'ed text.
+	 *
+	 * It tries to set the text as markup. If the markup is invalid it will try
+	 * a bit harder to recover from the errors and still set the markup.
+	 *
+	 * @param text                The text to set as markup.
+	 *
+	 * @returns                   Whether the markup was set or an
+	 *                            unrecoverable error occurred and the text is
+	 *                            set as plain text with an error message.
+	 */
+	bool set_markup(const std::string& text);
+
 };
 
 } // namespace font
