@@ -1,11 +1,12 @@
-/* $Id: theme.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: theme.hpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,17 +14,13 @@
 */
 
 /**
- *  @file theme.hpp
+ *  @file
  *  Definitions related to theme-support.
  */
 
 #ifndef THEME_HPP_INCLUDED
 #define THEME_HPP_INCLUDED
 
-#include <map>
-#include <string>
-#include <vector>
-
 #include "SDL.h"
 #include "config.hpp"
 #include "generic_event.hpp"
@@ -226,7 +223,7 @@ public:
 
 	const tborder& border() const { return border_; }
 
-	events::generic_event& theme_reset() { return theme_reset_; }
+	events::generic_event& theme_reset_event() { return theme_reset_event_; }
 
 private:
 	theme::object& find_element(std::string id);
@@ -236,7 +233,7 @@ private:
 
 	//notify observers that the theme has been rebuilt completely
 	//atm this is used for replay_controller to add replay controls to the standard theme
-	events::generic_event theme_reset_;
+	events::generic_event theme_reset_event_;
 
 	static std::map<std::string, config> known_themes;
 	std::string cur_theme;
