@@ -1,27 +1,29 @@
-/* $Id: time_of_day.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: time_of_day.hpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
-/** @file time_of_day.hpp */
+/** @file */
 
 #ifndef TIME_OF_DAY_HPP_INCLUDED
 #define TIME_OF_DAY_HPP_INCLUDED
 
-#include "config.hpp"
 #include "tstring.hpp"
-#include <string>
+
 #include <vector>
 
+class config;
+
 /**
  * Object which defines a time of day
  * with associated bonuses, image, sounds etc.
@@ -57,7 +59,7 @@ struct time_of_day
 	std::string image_mask;
 
 	/**
-	 * The colour modifications that should be made
+	 * The color modifications that should be made
 	 * to the game board to reflect the time of day.
 	 */
 	int red, green, blue;
