@@ -1,12 +1,13 @@
-/* $Id: tstring.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: tstring.cpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
-   Copyright (C) 2005 - 2010 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2011 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,7 +15,7 @@
 */
 
 /**
- *  @file tstring.cpp
+ *  @file
  *  Routines for translatable strings.
  */
 
@@ -53,18 +54,6 @@ size_t t_string_base::hash_value() const {
 	return seed;
 }
 
-t_string_base::walker::walker(const t_string& string) :
-	string_(string.get().value_),
-	begin_(0),
-	end_(string_.size()),
-	textdomain_(),
-	translatable_(false)
-{
-	if(string.get().translatable_) {
-		update();
-	}
-}
-
 t_string_base::walker::walker(const t_string_base& string) :
 	string_(string.value_),
 	begin_(0),
@@ -77,6 +66,44 @@ t_string_base::walker::walker(const t_string_base& string) :
 	}
 }
 
+#ifdef _MSC_VER
+/*
+ * From the IRC log of 23.07.2010
+ * 07:52 <silene> Upth: what did it break?
+ * 07:53 <Upth> silene: since that revision, the windows executable crashes
+ * immediately before loading the main menu
+ * 07:54 <silene> what kind of crash?
+ * 07:54 <Upth> assertion failed in the std::string library
+ * 07:54 <Upth> then "fatal error"
+ * 07:54 <Upth> and abnormal termination
+ * 07:54 <silene> which assertion?
+ * 07:55 <Upth> Expression: ("_Myptr + _Off <= (((_Mystring
+ * *)this->_Mycont)->_Myptr() + ((_Mystring *)this->_Mycont)->_Mysize) &&
+ * _Myptr + _Off >= ((_Mystring *)this->_Mycont)->_Myptr()", 0)
+ * 07:56 <shadowmaster> ugly.
+ * 07:57 <Upth> in the iterator += overload, called from the iterator +
+ * overload, called from std::basic_string::end(), called from line 409 of
+ * parser.cpp in write_key_val
+ * 07:58 <Upth> err std::basic_string::end() is called from
+ * t_string::walker::end(), which is called on line 409 of parser.cpp
+ * 07:58 <silene> that doesn't make sense; as far as i can tell it's a compiler
+ * bug
+ * 07:58 <silene> which compiler is that so that the code is made conditional
+ * on it?
+ * 07:58 <Upth> MSVC9
+ */
+t_string_base::walker::walker(const t_string& string) :
+	string_(string.get().value_),
+	begin_(0),
+	end_(string_.size()),
+	textdomain_(),
+	translatable_(false)
+{
+	if(string.get().translatable_) {
+		update();
+	}
+}
+
 t_string_base::walker::walker(const std::string& string) :
 	string_(string),
 	begin_(0),
@@ -86,6 +113,7 @@ t_string_base::walker::walker(const std::string& string) :
 {
 	update();
 }
+#endif
 
 void t_string_base::walker::update()
 {
@@ -173,6 +201,10 @@ t_string_base::t_string_base() :
 {
 }
 
+t_string_base::~t_string_base()
+{
+}
+
 t_string_base::t_string_base(const t_string_base& string) :
 	value_(string.value_),
 	translated_value_(string.translated_value_),
@@ -423,6 +455,25 @@ t_string_base& t_string_base::operator+=(const char* string)
 	return *this;
 }
 
+bool t_string_base::operator==(const t_string_base &that) const
+{
+	return that.translatable_ == translatable_ && that.value_ == value_;
+}
+
+bool t_string_base::operator==(const std::string &that) const
+{
+	return !translatable_ && value_ == that;
+}
+
+bool t_string_base::operator==(const char *that) const
+{
+	return !translatable_ && value_ == that;
+}
+
+bool t_string_base::operator<(const t_string_base &that) const
+{
+	return value_ < that.value_;
+}
 
 const std::string& t_string_base::str() const
 {
@@ -448,6 +499,45 @@ const std::string& t_string_base::str() const
 	return translated_value_;
 }
 
+t_string::t_string() : super()
+{
+}
+
+t_string::~t_string()
+{
+}
+
+t_string::t_string(const t_string &o) : super(o)
+{
+}
+
+t_string::t_string(const base &o) : super(o)
+{
+}
+
+t_string::t_string(const char *o) : super(base(o))
+{
+}
+
+t_string::t_string(const std::string &o) : super(base(o))
+{
+}
+
+t_string::t_string(const std::string &o, const std::string &textdomain) : super(base(o, textdomain))
+{
+}
+
+t_string &t_string::operator=(const t_string &o)
+{
+	super::operator=(o);
+	return *this;
+}
+
+t_string &t_string::operator=(const char *o)
+{
+	super::operator=(base(o));
+	return *this;
+}
 
 void t_string::add_textdomain(const std::string &name, const std::string &path)
 {
