@@ -1,11 +1,12 @@
-/* $Id: unit_animation.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: unit_animation.hpp 51426 2011-10-10 02:43:29Z ai0867 $ */
 /*
-   Copyright (C) 2006 - 2010 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
+   Copyright (C) 2006 - 2011 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -16,14 +17,8 @@
 
 #include "animated.hpp"
 #include "config.hpp"
-#include "map_location.hpp"
 #include "unit_frame.hpp"
 
-#include <climits>
-#include <string>
-#include <vector>
-#include <set>
-
 class attack_type;
 class game_display;
 class unit;
@@ -59,7 +54,6 @@ class unit_animation
 		void start_animation(int start_time
 				, const map_location &src = map_location::null_location
 				, const map_location &dst = map_location::null_location
-				, bool cycles = false
 				, const std::string& text = ""
 				, const Uint32 text_color = 0
 				, const bool accelerate = true);
@@ -67,34 +61,34 @@ class unit_animation
                 void pause_animation();
                 void restart_animation();
 		int get_current_frame_begin_time() const{ return unit_anim_.get_current_frame_begin_time() ; };
-		void redraw(const frame_parameters& value);
-		bool invalidate(const frame_parameters& value );
+		void redraw(frame_parameters& value);
+		void clear_haloes();
+		bool invalidate(frame_parameters& value );
 
 	friend class unit;
+
+	explicit unit_animation(const config &cfg, const std::string &frame_string = "");
+
 	protected:
 	// reserved to class unit, for the special case of redrawing the unit base frame
-	const frame_parameters get_current_params(const frame_parameters & default_val = frame_parameters(),bool primary = true) const { return unit_anim_.parameters(default_val,primary); };
+	const frame_parameters get_current_params(const frame_parameters & default_val = frame_parameters()) const { return unit_anim_.parameters(default_val); };
 	private:
-		static config prepare_animation(const config &cfg
-				, const std::string& animation_tag);
-
-		explicit unit_animation(const config& cfg
-				,const std::string& frame_string = "");
-
 		explicit unit_animation(int start_time
 				, const unit_frame &frame
 				, const std::string& event = ""
-				, const int variation=DEFAULT_ANIM);
+				, const int variation=DEFAULT_ANIM
+				, const frame_builder & builder = frame_builder());
 
 		class particule:public animated<unit_frame>
 	{
 		public:
-			explicit particule(int start_time=0) :
+			explicit particule(int start_time=0,const frame_builder &builder = frame_builder()) :
 				animated<unit_frame>(start_time),
 				accelerate(true),
-				parameters_(),
+				parameters_(builder),
 				halo_id_(0),
-				last_frame_begin_time_(0)
+				last_frame_begin_time_(0),
+				cycles_(false)
 				{};
 			explicit particule(const config& cfg
 					, const std::string& frame_string ="frame");
@@ -102,17 +96,21 @@ class unit_animation
 			virtual ~particule();
 			bool need_update() const;
 			bool need_minimal_update() const;
+			typedef enum { UNSET,CYCLE,NO_CYCLE} cycle_state;
 			void override(int start_time
 					, int duration
+					, const cycle_state cycles
 					, const std::string& highlight = ""
 					, const std::string& blend_ratio =""
 					, Uint32 blend_color = 0
 					, const std::string& offset = ""
-					, const std::string& layer = "");
-			void redraw( const frame_parameters& value,const map_location &src, const map_location &dst, const bool primary=false);
-			std::set<map_location> get_overlaped_hex(const frame_parameters& value,const map_location &src, const map_location &dst, const bool primary = false);
-			void start_animation(int start_time, bool cycles=false);
-			const frame_parameters parameters(const frame_parameters & default_val,bool primary) const { return get_current_frame().merge_parameters(get_current_frame_time(),parameters_.parameters(get_animation_time()-get_begin_time()),default_val,primary); };
+					, const std::string& layer = ""
+					, const std::string& modifiers = "");
+			void redraw( const frame_parameters& value,const map_location &src, const map_location &dst);
+			std::set<map_location> get_overlaped_hex(const frame_parameters& value,const map_location &src, const map_location &dst);
+			void start_animation(int start_time);
+			const frame_parameters parameters(const frame_parameters & default_val) const { return get_current_frame().merge_parameters(get_current_frame_time(),parameters_.parameters(get_animation_time()-get_begin_time()),default_val); };
+			void clear_halo();
 			bool accelerate;
 		private:
 
@@ -120,6 +118,7 @@ class unit_animation
 			frame_parsed_parameters parameters_;
 			int halo_id_;
 			int last_frame_begin_time_;
+			bool cycles_;
 
 	};
 		t_translation::t_list terrain_types_;
@@ -139,7 +138,7 @@ class unit_animation
 		/* these are drawing parameters, but for efficiancy reason they are in the anim and not in the particle */
 		map_location src_;
 		map_location dst_;
-		// optimisation
+		// optimization
 		bool invalidated_;
 		bool play_offscreen_;
 		std::set<map_location> overlaped_hex_;
@@ -159,7 +158,6 @@ class unit_animator
 				, const unit_animation * animation
 				, const map_location &src = map_location::null_location
 				, bool with_bars = false
-				, bool cycles = false
 				, const std::string& text = ""
 				, const Uint32 text_color = 0);
 		void add_animation(unit* animated_unit
@@ -168,7 +166,6 @@ class unit_animator
 				, const map_location &dst = map_location::null_location
 				, const int value = 0
 				, bool with_bars = false
-				, bool cycles = false
 				, const std::string& text = ""
 				, const Uint32 text_color = 0
 				, const unit_animation::hit_type hit_type =
@@ -182,7 +179,6 @@ class unit_animator
 				, const map_location &dst = map_location::null_location
 				, const int value = 0
 				, bool with_bars = false
-				, bool cycles = false
 				, const std::string& text = ""
 				, const Uint32 text_color = 0
 				, const unit_animation::hit_type hit_type =
@@ -212,10 +208,8 @@ class unit_animator
 				text(),
 				text_color(0),
 				src(),
-				with_bars(false),
-				cycles(false)
-			{
-			}
+				with_bars(false)
+			{}
 
 			unit *my_unit;
 			const unit_animation * animation;
@@ -223,7 +217,6 @@ class unit_animator
 			Uint32 text_color;
 			map_location src;
 			bool with_bars;
-			bool cycles;
 		};
 
 		std::vector<anim_elem> animated_units_;
