@@ -1,13 +1,14 @@
-/* $Id: variable.cpp 34564 2009-04-05 19:21:44Z silene $ */
+/* $Id: variable.cpp 51791 2011-10-31 22:48:58Z alarantalara $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2011 by Philippe Plantier <ayin@anathas.org>
 
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,20 +16,29 @@
 */
 
 /**
- *  @file variable.cpp
+ *  @file
  *  Manage WML-variables.
  */
 
 #include "global.hpp"
 
+#include "variable.hpp"
+
+#include "foreach.hpp"
+#include "formula_string_utils.hpp"
 #include "gamestatus.hpp"
 #include "log.hpp"
-#include "formula_string_utils.hpp"
+#include "resources.hpp"
+#include "unit.hpp"
+#include "unit_map.hpp"
+#include "team.hpp"
 
+#include <boost/variant.hpp>
 
-#define LOG_NG LOG_STREAM(info, engine)
-#define WRN_NG LOG_STREAM(warn, engine)
-#define ERR_NG LOG_STREAM(err, engine)
+static lg::log_domain log_engine("engine");
+#define LOG_NG LOG_STREAM(info, log_engine)
+#define WRN_NG LOG_STREAM(warn, log_engine)
+#define ERR_NG LOG_STREAM(err, log_engine)
 
 namespace
 {
@@ -36,7 +46,7 @@ namespace
 	 * @todo FIXME: the variable repository should be
 	 * a class of variable.hpp, and not the game_state.
 	 */
-	game_state* repos = NULL;
+	#define repos (resources::state_of_game)
 
 	// keeps track of insert_tag variables used by get_parsed_config
 	std::set<std::string> vconfig_recursion;
@@ -174,12 +184,12 @@ vconfig::vconfig(const config* cfg, const config * cache_key) :
 	}
 }
 
-vconfig::vconfig(const config* cfg, bool is_volatile) :
-	cfg_(cfg), cache_key_(cfg)
+vconfig::vconfig(const config &cfg, bool is_volatile) :
+	cfg_(&cfg), cache_key_(&cfg)
 {
 	if(is_volatile) {
 		increment_config_usage(cache_key_);
-		if(cache_key_ != cfg) {
+		if (cache_key_ != &cfg) {
 			//location of volatile cfg has moved
 			cfg_ = cache_key_;
 		}
@@ -199,6 +209,16 @@ vconfig::~vconfig()
 	decrement_config_usage(cache_key_);
 }
 
+vconfig vconfig::empty_vconfig()
+{
+    return vconfig(config(), true);
+}
+
+vconfig vconfig::unconstructed_vconfig()
+{
+    return vconfig();
+}
+
 vconfig& vconfig::operator=(const vconfig& cfg)
 {
 	const config* prev_key = cache_key_;
@@ -209,32 +229,18 @@ vconfig& vconfig::operator=(const vconfig& cfg)
 	return *this;
 }
 
-vconfig& vconfig::operator=(const config* cfg)
-{
-	if(cfg_ != cfg) {
-		cfg_ = cfg;
-		decrement_config_usage(cache_key_);
-		cache_key_ = NULL;
-	}
-	return *this;
-}
-
 const config vconfig::get_parsed_config() const
 {
 	config res;
 
-	for(string_map::const_iterator itor = cfg_->values.begin();
-		itor != cfg_->values.end(); ++itor)
-	{
-		res[itor->first] = expand(itor->first);
+	foreach (const config::attribute &i, cfg_->attribute_range()) {
+		res[i.first] = expand(i.first);
 	}
 
-	for(config::all_children_iterator child = cfg_->ordered_begin();
-		child != cfg_->ordered_end(); ++child)
+	foreach (const config::any_child &child, cfg_->all_children_range())
 	{
-		const std::string &child_key = *(*child).first;
-		if(child_key == "insert_tag") {
-			vconfig insert_cfg(child->second);
+		if (child.key == "insert_tag") {
+			vconfig insert_cfg(child.cfg);
 			const t_string& name = insert_cfg["name"];
 			const t_string& vname = insert_cfg["variable"];
 			if(!vconfig_recursion.insert(vname).second) {
@@ -245,7 +251,7 @@ const config vconfig::get_parsed_config() const
 				if(!vinfo.is_valid) {
 					res.add_child(name); //add empty tag
 				} else if(vinfo.explicit_index) {
-					res.add_child(name, vconfig(&(vinfo.as_container())).get_parsed_config());
+					res.add_child(name, vconfig(vinfo.as_container()).get_parsed_config());
 				} else {
 					variable_info::array_range range = vinfo.as_array();
 					if(range.first == range.second) {
@@ -263,11 +269,11 @@ const config vconfig::get_parsed_config() const
 					res.add_child("insert_tag", insert_cfg.get_config());
 				} else {
 					// throw to the top [insert_tag] which started the recursion
-					throw err;
+					throw;
 				}
 			}
 		} else {
-			res.add_child(child_key, vconfig((*child).second).get_parsed_config());
+			res.add_child(child.key, vconfig(child.cfg).get_parsed_config());
 		}
 	}
 	return res;
@@ -277,19 +283,17 @@ vconfig::child_list vconfig::get_children(const std::string& key) const
 {
 	vconfig::child_list res;
 
-	for(config::all_children_iterator child = cfg_->ordered_begin();
-		child != cfg_->ordered_end(); ++child)
+	foreach (const config::any_child &child, cfg_->all_children_range())
 	{
-		const std::string &child_key = *(*child).first;
-		if(child_key == key) {
-			res.push_back(vconfig(child->second, cache_key_));
-		} else if(child_key == "insert_tag") {
-			vconfig insert_cfg(child->second);
+		if (child.key == key) {
+			res.push_back(vconfig(&child.cfg, cache_key_));
+		} else if (child.key == "insert_tag") {
+			vconfig insert_cfg(child.cfg);
 			if(insert_cfg["name"] == key) {
 				variable_info vinfo(insert_cfg["variable"], false, variable_info::TYPE_CONTAINER);
 				if(!vinfo.is_valid) {
 					//push back an empty tag
-					res.push_back(vconfig(&empty_config));
+					res.push_back(vconfig(empty_config));
 				} else if(vinfo.explicit_index) {
 					config * cp = &(vinfo.as_container());
 					res.push_back(vconfig(cp, cp));
@@ -297,10 +301,10 @@ vconfig::child_list vconfig::get_children(const std::string& key) const
 					variable_info::array_range range = vinfo.as_array();
 					if(range.first == range.second) {
 						//push back an empty tag
-						res.push_back(vconfig(&empty_config));
+						res.push_back(vconfig(empty_config));
 					}
 					while(range.first != range.second) {
-						config * cp = *range.first++;
+						config *cp = &*range.first++;
 						res.push_back(vconfig(cp, cp));
 					}
 				}
@@ -312,39 +316,32 @@ vconfig::child_list vconfig::get_children(const std::string& key) const
 
 vconfig vconfig::child(const std::string& key) const
 {
-	const config *natural = cfg_->child(key);
-	if(natural)
-	{
-		return vconfig(natural, cache_key_);
+	if (const config &natural = cfg_->child(key)) {
+		return vconfig(&natural, cache_key_);
 	}
-	for(config::const_child_itors chitors = cfg_->child_range("insert_tag");
-		chitors.first != chitors.second; ++chitors.first)
+	foreach (const config &ins, cfg_->child_range("insert_tag"))
 	{
-		vconfig insert_cfg(*chitors.first);
+		vconfig insert_cfg(ins);
 		if(insert_cfg["name"] == key) {
 			variable_info vinfo(insert_cfg["variable"], false, variable_info::TYPE_CONTAINER);
 			if(!vinfo.is_valid) {
-				return vconfig(&empty_config);
+				return vconfig(empty_config);
 			}
 			config * cp = &(vinfo.as_container());
 			return vconfig(cp, cp);
 		}
 	}
-	return vconfig();
+	return unconstructed_vconfig();
 }
 
 bool vconfig::has_child(const std::string& key) const
 {
-	if(!cfg_) {
-		return false;
-	}
-	if(cfg_->child(key) != NULL) {
+	if (cfg_->child(key)) {
 		return true;
 	}
-	for(config::const_child_itors chitors = cfg_->child_range("insert_tag");
-		chitors.first != chitors.second; ++chitors.first)
+	foreach (const config &ins, cfg_->child_range("insert_tag"))
 	{
-		vconfig insert_cfg(*chitors.first);
+		vconfig insert_cfg(ins);
 		if(insert_cfg["name"] == key) {
 			return true;
 		}
@@ -352,40 +349,48 @@ bool vconfig::has_child(const std::string& key) const
 	return false;
 }
 
-const t_string vconfig::expand(const std::string& key) const
+struct vconfig_expand_visitor : boost::static_visitor<void>
 {
-	if(!cfg_) {
-		return t_string();
+	config::attribute_value &result;
+	vconfig_expand_visitor(config::attribute_value &r): result(r) {}
+	template<typename T> void operator()(T const &) const {}
+	void operator()(const std::string &s) const
+	{
+		result = utils::interpolate_variables_into_string(s, *repos);
 	}
-	const t_string& val = (*cfg_)[key];
-	if(repos != NULL && !val.str().empty()) {
-		std::string interp = utils::interpolate_variables_into_string(val.str(), *repos);
-		if(val.str() != interp) {
-			return t_string(interp);
-		}
+	void operator()(const t_string &s) const
+	{
+		result = utils::interpolate_variables_into_tstring(s, *repos);
 	}
-	return t_string(val);
+};
+
+config::attribute_value vconfig::expand(const std::string &key) const
+{
+	config::attribute_value val = (*cfg_)[key];
+	if (repos)
+		boost::apply_visitor(vconfig_expand_visitor(val), val.value);
+	return val;
 }
 
-vconfig::all_children_iterator::all_children_iterator(config::all_children_iterator i, const config *cache_key)
-: i_(i), inner_index_(0), index_offset_(0), cache_key_(cache_key)
+vconfig::all_children_iterator::all_children_iterator(const Itor &i, const config *cache_key)
+: i_(i), inner_index_(0), cache_key_(cache_key)
 {
 }
 
 vconfig::all_children_iterator& vconfig::all_children_iterator::operator++()
 {
-	if(i_.get_key() == "insert_tag") {
-		variable_info vinfo(vconfig(&i_.get_child())["variable"], false, variable_info::TYPE_CONTAINER);
+	if (inner_index_ >= 0 && i_->key == "insert_tag")
+	{
+		variable_info vinfo(vconfig(i_->cfg)["variable"], false, variable_info::TYPE_CONTAINER);
 		if(vinfo.is_valid && !vinfo.explicit_index) {
 			variable_info::array_range range = vinfo.as_array();
-			if(range.first != range.second && range.first + (++inner_index_) != range.second) {
-				++index_offset_;
+			if (++inner_index_ < std::distance(range.first, range.second)) {
 				return *this;
 			}
+			inner_index_ = 0;
 		}
 	}
 	++i_;
-	inner_index_ = 0;
 	return *this;
 }
 
@@ -396,55 +401,50 @@ vconfig::all_children_iterator vconfig::all_children_iterator::operator++(int)
 	return i;
 }
 
-std::pair<const std::string,const vconfig> vconfig::all_children_iterator::operator*() const
+vconfig::all_children_iterator::reference vconfig::all_children_iterator::operator*() const
 {
-	return std::make_pair<const std::string, const vconfig>(get_key(), get_child());
+	return value_type(get_key(), get_child());
 }
 
 vconfig::all_children_iterator::pointer vconfig::all_children_iterator::operator->() const
 {
-	return pointer(new std::pair<const std::string, const vconfig>(get_key(), get_child()));
+	pointer_proxy p = { value_type(get_key(), get_child()) };
+	return p;
 }
 
+
 std::string vconfig::all_children_iterator::get_key() const
 {
-	const std::string& key = i_.get_key();
-	if(key == "insert_tag") {
-		return vconfig(&i_.get_child())["name"];
+	const std::string &key = i_->key;
+	if (inner_index_ >= 0 && key == "insert_tag") {
+		return vconfig(i_->cfg)["name"];
 	}
 	return key;
 }
 
-const vconfig vconfig::all_children_iterator::get_child() const
+vconfig vconfig::all_children_iterator::get_child() const
 {
-	if(i_.get_key() == "insert_tag") {
+	if (inner_index_ >= 0 && i_->key == "insert_tag")
+	{
 		config * cp;
-		variable_info vinfo(vconfig(&i_.get_child())["variable"], false, variable_info::TYPE_CONTAINER);
+		variable_info vinfo(vconfig(i_->cfg)["variable"], false, variable_info::TYPE_CONTAINER);
 		if(!vinfo.is_valid) {
-			return vconfig(&empty_config);
+			return vconfig(empty_config);
 		} else if(inner_index_ == 0) {
 			cp = &(vinfo.as_container());
 			return vconfig(cp, cp);
 		}
-		cp = *(vinfo.as_array().first + inner_index_);
+		variable_info::array_range r = vinfo.as_array();
+		std::advance(r.first, inner_index_);
+		cp = &*r.first;
 		return vconfig(cp, cp);
 	}
-	return vconfig(&i_.get_child(), cache_key_);
+	return vconfig(&i_->cfg, cache_key_);
 }
 
-size_t vconfig::all_children_iterator::get_index() const
+bool vconfig::all_children_iterator::operator==(const all_children_iterator &i) const
 {
-	return i_.get_index() + index_offset_;
-}
-
-bool vconfig::all_children_iterator::operator==(all_children_iterator i) const
-{
-	return (i_ == i.i_ && inner_index_ == i.inner_index_);
-}
-
-bool vconfig::all_children_iterator::operator!=(all_children_iterator i) const
-{
-	return (i_ != i.i_ || inner_index_ != i.inner_index_);
+	return i_ == i.i_ && inner_index_ == i.inner_index_;
 }
 
 vconfig::all_children_iterator vconfig::ordered_begin() const
@@ -459,14 +459,8 @@ vconfig::all_children_iterator vconfig::ordered_end() const
 
 namespace variable
 {
-	manager::manager(game_state* repository)
-	{
-		repos = repository;
-	}
-
 	manager::~manager()
 	{
-		repos = NULL;
 		hash_memory.clear();
 	}
 }
@@ -479,25 +473,24 @@ scoped_wml_variable::scoped_wml_variable(const std::string& var_name) :
 	repos->scoped_variables.push_back(this);
 }
 
-void scoped_wml_variable::store(const config& var_value)
+config &scoped_wml_variable::store(const config &var_value)
 {
-	const config::child_list& children = repos->get_variables().get_children(var_name_);
-	for(config::child_list::const_iterator i = children.begin(); i != children.end(); ++i) {
-		previous_val_.add_child(var_name_, **i);
+	foreach (const config &i, repos->get_variables().child_range(var_name_)) {
+		previous_val_.add_child(var_name_, i);
 	}
 	repos->clear_variable_cfg(var_name_);
-	repos->add_variable_cfg(var_name_, var_value);
+	config &res = repos->add_variable_cfg(var_name_, var_value);
 	LOG_NG << "scoped_wml_variable: var_name \"" << var_name_ << "\" has been auto-stored.\n";
 	activated_ = true;
+	return res;
 }
 
 scoped_wml_variable::~scoped_wml_variable()
 {
 	if(activated_) {
 		repos->clear_variable_cfg(var_name_);
-		config::child_list old_val =previous_val_.get_children(var_name_);
-		for(config::child_list::iterator j=old_val.begin(); j != old_val.end() ; j++){
-			repos->add_variable_cfg(var_name_,**j);
+		foreach (const config &i, previous_val_.child_range(var_name_)) {
+			repos->add_variable_cfg(var_name_, i);
 		}
 		LOG_NG << "scoped_wml_variable: var_name \"" << var_name_ << "\" has been reverted.\n";
 	}
@@ -507,38 +500,43 @@ scoped_wml_variable::~scoped_wml_variable()
 
 void scoped_xy_unit::activate()
 {
-	unit_map::const_iterator itor = umap_.find(map_location(x_,y_));
+	map_location loc = map_location(x_, y_);
+	unit_map::const_iterator itor = umap_.find(loc);
 	if(itor != umap_.end()) {
-		config tmp_cfg;
-		itor->second.write(tmp_cfg);
-		tmp_cfg["x"] = lexical_cast<std::string,int>(x_ + 1);
-		tmp_cfg["y"] = lexical_cast<std::string,int>(y_ + 1);
-		LOG_NG << "auto-storing $" << name() << " at (" << x_ << ',' << y_ << ")\n";
-		store(tmp_cfg);
+		config &tmp_cfg = store();
+		itor->write(tmp_cfg);
+		tmp_cfg["x"] = x_ + 1;
+		tmp_cfg["y"] = y_ + 1;
+		LOG_NG << "auto-storing $" << name() << " at (" << loc << ")\n";
 	} else {
-		ERR_NG << "failed to auto-store $" << name() << " at (" << x_ << ',' << y_ << ")\n";
+		ERR_NG << "failed to auto-store $" << name() << " at (" << loc << ")\n";
 	}
 }
 
 void scoped_weapon_info::activate()
 {
-	if(data_ != NULL) {
-		store(*data_);
+	if (data_) {
+		store(data_);
 	}
 }
 
 void scoped_recall_unit::activate()
 {
-	player_info* const player = repos->get_player(player_);
-	if(player != NULL) {
-		if(player->available_units.size() > recall_index_) {
-			config tmp_cfg;
-			player->available_units[recall_index_].write(tmp_cfg);
+	const std::vector<team>& teams = teams_manager::get_teams();
+	std::vector<team>::const_iterator team_it;
+	for (team_it = teams.begin(); team_it != teams.end(); ++team_it) {
+		if (team_it->save_id() == player_ )
+			break;
+	}
+
+	if(team_it != teams.end()) {
+		if(team_it->recall_list().size() > recall_index_) {
+			config &tmp_cfg = store();
+			team_it->recall_list()[recall_index_].write(tmp_cfg);
 			tmp_cfg["x"] = "recall";
 			tmp_cfg["y"] = "recall";
 			LOG_NG << "auto-storing $" << name() << " for player: " << player_
 				<< " at recall index: " << recall_index_ << '\n';
-			store(tmp_cfg);
 		} else {
 			ERR_NG << "failed to auto-store $" << name() << " for player: " << player_
 				<< " at recall index: " << recall_index_ << '\n';
@@ -586,10 +584,12 @@ variable_info::variable_info(const std::string& varname,
 	assert(repos != NULL);
 	activate_scope_variable(varname);
 
-	vars = &repos->variables;
+	vars = &repos->variables_;
 	key = varname;
 	std::string::const_iterator itor = std::find(key.begin(),key.end(),'.');
 	int dot_index = key.find('.');
+
+	bool force_length = false;
 	// example varname = "unit_store.modifications.trait[0]"
 	while(itor != key.end()) { // subvar access
 		std::string element=key.substr(0,dot_index);
@@ -610,7 +610,7 @@ variable_info::variable_info(const std::string& varname,
 			element = std::string(element.begin(),index_start);
 		}
 
-		size_t size = vars->get_children(element).size();
+		size_t size = vars->child_count(element);
 		if(size <= inner_index) {
 			if(force_valid) {
 				// Add elements to the array until the requested size is attained
@@ -623,34 +623,37 @@ variable_info::variable_info(const std::string& varname,
 				WRN_NG << "variable_info: invalid WML array index, "
 					<< varname << std::endl;
 				return;
-			} else if(key != "length") {
-				WRN_NG << "variable_info: retrieving member of non-existant WML container, "
-					<< varname << std::endl;
+			} else if(varname.length() >= 7 && varname.substr(varname.length()-7) == ".length") {
+				// require '.' to avoid matching suffixes -> requires varname over key to always find length
+				// return length 0 for non-existent WML array (handled below)
+				force_length = true;
+			} else {
+				WRN_NG << "variable_info: retrieving member of non-existent WML container, "
+				<< varname << std::endl;
 				return;
-			} //else return length 0 for non-existant WML array (handled below)
+			}
 		}
-		if(!inner_explicit_index && key == "length") {
+		if((!inner_explicit_index && key == "length") || force_length) {
 			switch(vartype) {
 			case variable_info::TYPE_ARRAY:
 			case variable_info::TYPE_CONTAINER:
 				WRN_NG << "variable_info: using reserved WML variable as wrong type, "
 					<< varname << std::endl;
-				is_valid = force_valid || repos->temporaries.child(varname) != NULL;
+				is_valid = force_valid || repos->temporaries_.child(varname);
 				break;
 			case variable_info::TYPE_SCALAR:
 			default:
 				// Store the length of the array as a temporary variable
-				repos->temporaries[varname] = lexical_cast<std::string>(size);
+				repos->temporaries_[varname] = int(size);
 				is_valid = true;
 				break;
 			}
 			key = varname;
-			vars = &repos->temporaries;
+			vars = &repos->temporaries_;
 			return;
 		}
 
-		//std::cerr << "Entering " << element << "[" << inner_index << "] of [" << vars->get_children(element).size() << "]\n";
-		vars = vars->get_children(element)[inner_index];
+		vars = &vars->child(element, inner_index);
 		itor = std::find(key.begin(),key.end(),'.');
 		dot_index = key.find('.');
 	} // end subvar access
@@ -667,7 +670,7 @@ variable_info::variable_info(const std::string& varname,
 			index = game_config::max_loop;
 		}
 		key = std::string(key.begin(),index_start);
-		size_t size = vars->get_children(key).size();
+		size_t size = vars->child_count(key);
 		if(size <= index) {
 			if(!force_valid) {
 				WRN_NG << "variable_info: invalid WML array index, " << varname << std::endl;
@@ -679,12 +682,12 @@ variable_info::variable_info(const std::string& varname,
 		}
 		switch(vartype) {
 		case variable_info::TYPE_ARRAY:
-			vars = vars->get_children(key)[index];
+			vars = &vars->child(key, index);
 			key = "__array";
-			is_valid = force_valid || vars->child(key) != NULL;
+			is_valid = force_valid || vars->child(key);
 			break;
 		case variable_info::TYPE_SCALAR:
-			vars = vars->get_children(key)[index];
+			vars = &vars->child(key, index);
 			key = "__value";
 			is_valid = force_valid || vars->has_attribute(key);
 			break;
@@ -694,8 +697,10 @@ variable_info::variable_info(const std::string& varname,
 			is_valid = true;
 			return;
 		}
-		WRN_NG << "variable_info: using explicitly indexed Container as wrong WML type, "
-			<< varname << std::endl;
+		if (force_valid) {
+			WRN_NG << "variable_info: using explicitly indexed "
+				"container as wrong WML type, " << varname << '\n';
+		}
 		explicit_index = false;
 		index = 0;
 	} else {
@@ -716,21 +721,21 @@ variable_info::variable_info(const std::string& varname,
 	}
 }
 
-t_string& variable_info::as_scalar() {
+config::attribute_value &variable_info::as_scalar()
+{
 	assert(is_valid);
-	return vars->values[key];
+	return (*vars)[key];
 }
 
 config& variable_info::as_container() {
 	assert(is_valid);
 	if(explicit_index) {
 		// Empty data for explicit index was already created if it was needed
-		return *vars->get_children(key)[index];
+		return vars->child(key, index);
 	}
-	config *temp = vars->child(key);
-	if(temp) {
+	if (config &temp = vars->child(key)) {
 		// The container exists, index not specified, return index 0
-		return *temp;
+		return temp;
 	}
 	// Add empty data for the new variable, since it does not exist yet
 	return vars->add_child(key);
