@@ -1,3 +1,18 @@
+/* $Id: variant.cpp 51328 2011-09-28 19:21:16Z mordante $ */
+/*
+   Copyright (C) 2008 - 2011 by David White <dave@whitevine.net>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
+
+   See the COPYING file for more details.
+*/
+
 #include "global.hpp"
 
 #include <cmath>
@@ -9,6 +24,7 @@
 #include "formatter.hpp"
 #include "formula_callable.hpp"
 #include "formula_function.hpp"
+#include "util.hpp"
 
 namespace {
 std::string variant_type_to_string(variant::TYPE type) {
@@ -17,6 +33,8 @@ std::string variant_type_to_string(variant::TYPE type) {
 		return "null";
 	case variant::TYPE_INT:
 		return "int";
+	case variant::TYPE_DECIMAL:
+		return "decimal";
 	case variant::TYPE_CALLABLE:
 		return "object";
 	case variant::TYPE_LIST:
@@ -59,21 +77,52 @@ std::string get_call_stack()
 	return res;
 }
 
-type_error::type_error(const std::string& str) : message(str) {
+type_error::type_error(const std::string& str) : game::error(str) {
 	std::cerr << "ERROR: " << message << "\n" << get_call_stack();
 }
 
-variant_iterator::variant_iterator() : type_(TYPE_NULL)
-{}
+variant_iterator::variant_iterator()
+	: type_(TYPE_NULL)
+	, list_iterator_()
+	, map_iterator_()
+{
+}
 
-variant_iterator::variant_iterator(const variant_iterator& iter) : type_(iter.type_), list_iterator_(iter.list_iterator_), map_iterator_(iter.map_iterator_)
-{}
+variant_iterator::variant_iterator(const variant_iterator& iter)
+	: type_(iter.type_)
+	, list_iterator_()
+	, map_iterator_()
+{
+	switch(type_) {
+		case TYPE_LIST :
+			list_iterator_ = iter.list_iterator_;
+			break;
 
-variant_iterator::variant_iterator(const std::vector<variant>::iterator& iter) : type_(TYPE_LIST), list_iterator_(iter)
-{}
+		case TYPE_MAP:
+			map_iterator_ = iter.map_iterator_;
+			break;
 
-variant_iterator::variant_iterator(const std::map<variant, variant>::iterator& iter) : type_(TYPE_MAP), map_iterator_(iter)
-{}
+		case TYPE_NULL:
+			/* DO NOTHING */
+			break;
+	}
+}
+
+variant_iterator::variant_iterator(
+		const std::vector<variant>::iterator& iter)
+	: type_(TYPE_LIST)
+	, list_iterator_(iter)
+	, map_iterator_()
+{
+}
+
+variant_iterator::variant_iterator(
+		const std::map<variant, variant>::iterator& iter)
+	: type_(TYPE_MAP)
+	, list_iterator_()
+	, map_iterator_(iter)
+{
+}
 
 variant variant_iterator::operator*() const
 {
@@ -89,7 +138,7 @@ variant variant_iterator::operator*() const
 		return variant();
 }
 
-variant_iterator variant_iterator::operator++()
+variant_iterator& variant_iterator::operator++()
 {
 	if (type_ == TYPE_LIST)
 	{
@@ -121,8 +170,20 @@ variant_iterator& variant_iterator::operator=(const variant_iterator& that)
 	if (this == &that)
 		return *this;
 	type_ = that.type_;
-	list_iterator_ = that.list_iterator_;
-	map_iterator_ = that.map_iterator_;
+	switch(type_) {
+		case TYPE_LIST :
+			list_iterator_ = that.list_iterator_;
+			break;
+
+		case TYPE_MAP:
+			map_iterator_ = that.map_iterator_;
+			break;
+
+		case TYPE_NULL:
+			/* DO NOTHING */
+			break;
+	}
+
 	return *this;
 }
 
@@ -163,22 +224,34 @@ bool variant_iterator::operator!=(const variant_iterator& that) const
 }
 
 struct variant_list {
-	variant_list() : refcount(0)
-	{}
+	variant_list()
+		: elements()
+		, refcount(0)
+	{
+	}
+
 	std::vector<variant> elements;
 	int refcount;
 };
 
 struct variant_string {
-	variant_string() : refcount(0)
-	{}
+	variant_string()
+		: str()
+		, refcount(0)
+	{
+	}
+
 	std::string str;
 	int refcount;
 };
 
 struct variant_map {
-	variant_map() : refcount(0)
-	{}
+	variant_map()
+		: elements()
+		, refcount(0)
+	{
+	}
+
 	std::map<variant,variant> elements;
 	int refcount;
 };
@@ -201,6 +274,7 @@ void variant::increment_refcount()
 
 	// These are not used here, add them to silence a compiler warning.
 	case TYPE_NULL:
+	case TYPE_DECIMAL:
 	case TYPE_INT :
 		break;
 	}
@@ -230,6 +304,7 @@ void variant::release()
 
 	// These are not used here, add them to silence a compiler warning.
 	case TYPE_NULL:
+	case TYPE_DECIMAL:
 	case TYPE_INT :
 		break;
 	}
@@ -241,6 +316,9 @@ variant::variant() : type_(TYPE_NULL), int_value_(0)
 variant::variant(int n) : type_(TYPE_INT), int_value_(n)
 {}
 
+variant::variant(int n, variant::DECIMAL_VARIANT_TYPE /*type*/) : type_(TYPE_DECIMAL), decimal_value_(n)
+{}
+
 variant::variant(const game_logic::formula_callable* callable)
 	: type_(TYPE_CALLABLE), callable_(callable)
 {
@@ -275,6 +353,7 @@ variant::variant(std::map<variant,variant>* map)
 }
 
 variant::variant(const variant& v)
+    : type_(v.type_)
 {
 	memcpy(this, &v, sizeof(v));
 	increment_refcount();
@@ -311,7 +390,7 @@ const variant& variant::operator[](size_t n) const
 	return list_->elements[n];
 }
 
-const variant& variant::operator[](const variant v) const
+const variant& variant::operator[](const variant& v) const
 {
 	if(type_ == TYPE_CALLABLE) {
 		assert(v.as_int() == 0);
@@ -359,22 +438,6 @@ variant variant::get_values() const
 	return variant(&tmp);
 }
 
-variant_iterator variant::get_iterator() const
-{
-	if(type_ == TYPE_LIST)
-	{
-		std::vector<variant>::iterator i;
-		return variant_iterator( i );
-	}
-	if(type_ == TYPE_MAP)
-	{
-		std::map<variant,variant>::iterator i;
-		return variant_iterator( i );
-	}
-
-	return variant_iterator();
-}
-
 variant_iterator variant::begin() const
 {
 	if(type_ == TYPE_LIST)
@@ -396,6 +459,21 @@ variant_iterator variant::end() const
 	return variant_iterator();
 }
 
+bool variant::is_empty() const
+{
+	if(type_ == TYPE_NULL) {
+		return true;
+	} else if (type_ == TYPE_LIST) {
+		assert(list_);
+		return list_->elements.empty();
+	} else if (type_ == TYPE_MAP) {
+		assert(map_);
+		return map_->elements.empty();
+	}
+
+	return false;
+}
+
 size_t variant::num_elements() const
 {
 	if(type_ == TYPE_CALLABLE) {
@@ -429,6 +507,22 @@ variant variant::get_member(const std::string& str) const
 	}
 }
 
+int variant::as_decimal() const
+{
+	if( type_ == TYPE_DECIMAL) {
+		return decimal_value_;
+	} else if( type_ == TYPE_INT ) {
+		return int_value_*1000;
+	} else if( type_ == TYPE_NULL) {
+		return 0;
+	} else {
+		throw type_error((formatter() << "type error: "
+			<< " expected integer or decimal but found "
+			<< variant_type_to_string(type_)
+			<< " (" << to_debug_string() << ")").str());
+	}
+}
+
 bool variant::as_bool() const
 {
 	switch(type_) {
@@ -436,6 +530,8 @@ bool variant::as_bool() const
 		return false;
 	case TYPE_INT:
 		return int_value_ != 0;
+	case TYPE_DECIMAL:
+		return decimal_value_ != 0;
 	case TYPE_CALLABLE:
 		return callable_ != NULL;
 	case TYPE_LIST:
@@ -487,22 +583,69 @@ variant variant::operator+(const variant& v) const
 			return variant(&res);
 		}
 	}
+	if(type_ == TYPE_DECIMAL || v.type_ == TYPE_DECIMAL) {
+		return variant( as_decimal() + v.as_decimal() , DECIMAL_VARIANT);
+	}
 
 	return variant(as_int() + v.as_int());
 }
 
 variant variant::operator-(const variant& v) const
 {
+	if(type_ == TYPE_DECIMAL || v.type_ == TYPE_DECIMAL) {
+		return variant( as_decimal() - v.as_decimal() , DECIMAL_VARIANT);
+	}
+
 	return variant(as_int() - v.as_int());
 }
 
 variant variant::operator*(const variant& v) const
 {
+	if(type_ == TYPE_DECIMAL || v.type_ == TYPE_DECIMAL) {
+
+		long long long_int = as_decimal();
+
+		long_int *= v.as_decimal();
+
+		long_int /= 100;
+
+		if( long_int%10 >= 5) {
+			long_int /= 10;
+			++long_int;
+		} else
+			long_int/=10;
+
+		return variant( static_cast<int>(long_int) , variant::DECIMAL_VARIANT );
+	}
+
 	return variant(as_int() * v.as_int());
 }
 
 variant variant::operator/(const variant& v) const
 {
+	if(type_ == TYPE_DECIMAL || v.type_ == TYPE_DECIMAL) {
+		int denominator = v.as_decimal();
+
+		if(denominator == 0) {
+			throw type_error((formatter() << "divide by zero error").str());
+		}
+
+		long long long_int = as_decimal();
+
+		long_int *= 10000;
+
+		long_int /= denominator;
+
+		if( long_int%10 >= 5) {
+			long_int /= 10;
+			++long_int;
+		} else
+			long_int/=10;
+
+		return variant(  static_cast<int>(long_int) , variant::DECIMAL_VARIANT);
+	}
+
+
 	const int numerator = as_int();
 	const int denominator = v.as_int();
 	if(denominator == 0) {
@@ -523,19 +666,43 @@ variant variant::operator%(const variant& v) const
 	return variant(numerator%denominator);
 }
 
+
 variant variant::operator^(const variant& v) const
 {
-	return variant(static_cast<int>(pow(static_cast<float>(as_int()), v.as_int())));
+	if( type_ == TYPE_DECIMAL || v.type_ == TYPE_DECIMAL ) {
+
+		double res = pow( as_decimal()/1000.0 , v.as_decimal()/1000.0 );
+
+		res *= 1000;
+		int i =  static_cast<int>( res );
+
+		res -= i;
+
+		if( res > 0.5 )
+			i++;
+
+		return variant( i , variant::DECIMAL_VARIANT);
+	}
+
+	return variant(static_cast<int>(
+			round_portable(pow(static_cast<double>(as_int()), v.as_int()))));
 }
 
 variant variant::operator-() const
 {
+	if( type_ == TYPE_DECIMAL)
+		return variant( -decimal_value_, variant::DECIMAL_VARIANT );
+
 	return variant(-as_int());
 }
 
 bool variant::operator==(const variant& v) const
 {
 	if(type_ != v.type_) {
+		if( type_ == TYPE_DECIMAL || v.type_ == TYPE_DECIMAL ) {
+			return as_decimal() == v.as_decimal();
+		}
+
 		return false;
 	}
 
@@ -552,6 +719,10 @@ bool variant::operator==(const variant& v) const
 		return int_value_ == v.int_value_;
 	}
 
+	case TYPE_DECIMAL: {
+		return decimal_value_ == v.decimal_value_;
+	}
+
 	case TYPE_LIST: {
 		if(num_elements() != v.num_elements()) {
 			return false;
@@ -587,6 +758,10 @@ bool variant::operator!=(const variant& v) const
 bool variant::operator<=(const variant& v) const
 {
 	if(type_ != v.type_) {
+		if( type_ == TYPE_DECIMAL || v.type_ == TYPE_DECIMAL ) {
+			return as_decimal() <= v.as_decimal();
+		}
+
 		return type_ < v.type_;
 	}
 
@@ -603,6 +778,10 @@ bool variant::operator<=(const variant& v) const
 		return int_value_ <= v.int_value_;
 	}
 
+	case TYPE_DECIMAL: {
+		return decimal_value_ <= v.decimal_value_;
+	}
+
 	case TYPE_LIST: {
 		for(size_t n = 0; n != num_elements() && n != v.num_elements(); ++n) {
 			if((*this)[n] < v[n]) {
@@ -643,6 +822,78 @@ bool variant::operator>(const variant& v) const
 	return !(*this <= v);
 }
 
+variant variant::list_elements_add(const variant& v) const
+{
+	must_be(TYPE_LIST);
+	v.must_be(TYPE_LIST);
+
+	if( num_elements() != v.num_elements() )
+		throw type_error("Operator '.+' requires two lists of the same length");
+
+	std::vector< variant > res;
+	res.reserve(num_elements());
+
+	for(size_t i = 0; i < num_elements(); ++i) {
+		res.push_back( (*this)[i] + v[i] );
+	}
+
+	return variant( &res );
+}
+
+variant variant::list_elements_sub(const variant& v) const
+{
+	must_be(TYPE_LIST);
+	v.must_be(TYPE_LIST);
+
+	if( num_elements() != v.num_elements() )
+		throw type_error("Operator '.-' requires two lists of the same length");
+
+	std::vector< variant > res;
+	res.reserve(num_elements());
+
+	for(size_t i = 0; i < num_elements(); ++i) {
+		res.push_back( (*this)[i] - v[i] );
+	}
+
+	return variant( &res );
+}
+
+variant variant::list_elements_mul(const variant& v) const
+{
+	must_be(TYPE_LIST);
+	v.must_be(TYPE_LIST);
+
+	if( num_elements() != v.num_elements() )
+		throw type_error("Operator '.*' requires two lists of the same length");
+
+	std::vector< variant > res;
+	res.reserve(num_elements());
+
+	for(size_t i = 0; i < num_elements(); ++i) {
+		res.push_back( (*this)[i] * v[i] );
+	}
+
+	return variant( &res );
+}
+
+variant variant::list_elements_div(const variant& v) const
+{
+	must_be(TYPE_LIST);
+	v.must_be(TYPE_LIST);
+
+	if( num_elements() != v.num_elements() )
+		throw type_error("Operator './' requires two lists of the same length");
+
+	std::vector< variant > res;
+	res.reserve(num_elements());
+
+	for(size_t i = 0; i < num_elements(); ++i) {
+		res.push_back( (*this)[i] / v[i] );
+	}
+
+	return variant( &res );
+}
+
 void variant::must_be(variant::TYPE t) const
 {
 	if(type_ != t) {
@@ -661,6 +912,28 @@ void variant::serialize_to_string(std::string& str) const
 	case TYPE_INT:
 		str += boost::lexical_cast<std::string>(int_value_);
 		break;
+	case TYPE_DECIMAL: {
+		std::ostringstream s;
+
+		int fractional = decimal_value_ % 1000;
+		int integer = (decimal_value_ - fractional) / 1000;
+
+		s << integer << ".";
+
+		fractional = abs(fractional);
+
+		if( fractional < 100) {
+			if( fractional < 10)
+				s << "00";
+			else
+				s << 0;
+		}
+
+		s << fractional;
+
+		str += s.str();
+		break;
+	}
 	case TYPE_CALLABLE:
 		callable_->serialize(str);
 		break;
@@ -706,7 +979,7 @@ void variant::serialize_to_string(std::string& str) const
 void variant::serialize_from_string(const std::string& str)
 {
 	try {
-		*this = game_logic::formula(str).execute();
+		*this = game_logic::formula(str).evaluate();
 	} catch(...) {
 		*this = variant(str);
 	}
@@ -739,6 +1012,27 @@ std::string variant::string_cast() const
 		return "0";
 	case TYPE_INT:
 		return boost::lexical_cast<std::string>(int_value_);
+	case TYPE_DECIMAL: {
+		std::ostringstream s;
+
+		int fractional = decimal_value_ % 1000;
+		int integer = (decimal_value_ - fractional) / 1000;
+
+		s << integer << ".";
+
+		fractional = abs(fractional);
+
+		if( fractional < 100) {
+			if( fractional < 10)
+				s << "00";
+			else
+				s << 0;
+		}
+
+		s << fractional;
+
+		return s.str();
+	}
 	case TYPE_CALLABLE:
 		return "(object)";
 	case TYPE_LIST: {
@@ -775,7 +1069,7 @@ std::string variant::string_cast() const
 	}
 }
 
-std::string variant::to_debug_string(std::vector<const game_logic::formula_callable*>* seen) const
+std::string variant::to_debug_string(std::vector<const game_logic::formula_callable*>* seen, bool verbose) const
 {
 	std::vector<const game_logic::formula_callable*> seen_stack;
 	if(!seen) {
@@ -789,6 +1083,25 @@ std::string variant::to_debug_string(std::vector<const game_logic::formula_calla
 	case TYPE_INT:
 		s << int_value_;
 		break;
+	case TYPE_DECIMAL: {
+		int fractional = decimal_value_ % 1000;
+		int integer = (decimal_value_ - fractional) / 1000;
+
+		s << integer << ".";
+
+		fractional = abs(fractional);
+
+		if( fractional < 100) {
+			if( fractional < 10)
+				s << "00";
+			else
+				s << 0;
+		}
+
+		s << fractional;
+
+		break;
+	}
 	case TYPE_LIST: {
 		s << "[";
 		for(size_t n = 0; n != num_elements(); ++n) {
@@ -796,7 +1109,7 @@ std::string variant::to_debug_string(std::vector<const game_logic::formula_calla
 				s << ", ";
 			}
 
-			s << operator[](n).to_debug_string(seen);
+			s << operator[](n).to_debug_string(seen, verbose);
 		}
 		s << "]";
 		break;
@@ -804,7 +1117,8 @@ std::string variant::to_debug_string(std::vector<const game_logic::formula_calla
 	case TYPE_CALLABLE: {
 		s << "{";
 		if(std::find(seen->begin(), seen->end(), callable_) == seen->end()) {
-			seen->push_back(callable_);
+			if(!verbose)
+				seen->push_back(callable_);
 			std::vector<game_logic::formula_input> v = callable_->inputs();
 			bool first = true;
 			for(size_t i=0; i<v.size(); ++i) {
@@ -820,7 +1134,7 @@ std::string variant::to_debug_string(std::vector<const game_logic::formula_calla
 					s << "(writeonly) ";
 				}
 
-				s << "-> " << callable_->query_value(input.name).to_debug_string(seen);
+				s << "-> " << callable_->query_value(input.name).to_debug_string(seen, verbose);
 			}
 		} else {
 			s << "...";
@@ -836,9 +1150,9 @@ std::string variant::to_debug_string(std::vector<const game_logic::formula_calla
 				s << ",";
 			}
 			first_time = false;
-			s << i->first.to_debug_string(seen);
+			s << i->first.to_debug_string(seen, verbose);
 			s << "->";
-			s << i->second.to_debug_string(seen);
+			s << i->second.to_debug_string(seen, verbose);
 		}
 		s << "]";
 		break;
