@@ -1,11 +1,12 @@
-/* $Id: button.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: button.cpp 48480 2011-02-12 16:20:24Z ivanovic $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -22,11 +23,13 @@
 #include "marked-up_text.hpp"
 #include "image.hpp"
 #include "log.hpp"
+#include "serialization/string_utils.hpp"
 #include "sound.hpp"
 #include "video.hpp"
 #include "wml_separators.hpp"
 
-#define ERR_DP LOG_STREAM(err, display)
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
 
 namespace gui {
 
@@ -95,6 +98,10 @@ button::button(CVideo& video, const std::string& label, button::TYPE type,
 	}
 }
 
+button::~button()
+{
+}
+
 void button::calculate_size()
 {
 	if (type_ == TYPE_IMAGE){
@@ -109,34 +116,35 @@ void button::calculate_size()
 
 	if (!change_size) {
 		unsigned w = loc.w - (type_ == TYPE_PRESS ? horizontal_padding : checkbox_horizontal_padding + base_width_);
-		if (type_ != TYPE_IMAGE){
-			label_ = font::make_text_ellipsis(label_, font_size, w, false);
+		if (type_ != TYPE_IMAGE)
+		{
+			int fs = font_size;
+			int style = TTF_STYLE_NORMAL;
+			std::string::const_iterator i_beg = label_.begin(), i_end = label_.end(),
+				i = font::parse_markup(i_beg, i_end, &fs, NULL, &style);
+			if (i != i_end) {
+				std::string tmp(i, i_end);
+				label_.erase(i - i_beg, i_end - i_beg);
+				label_ += font::make_text_ellipsis(tmp, fs, w, style);
+			}
 		}
 	}
 
 	if (type_ != TYPE_IMAGE){
 		textRect_ = font::draw_text(NULL, screen_area(), font_size,
-		                            font::BUTTON_COLOUR, label_, 0, 0);
+		                            font::BUTTON_COLOR, label_, 0, 0);
 	}
 
 	if (!change_size)
 		return;
 
-#ifdef USE_TINY_GUI
-	set_height(textRect_.h+vertical_padding);
-#else
 	set_height(std::max(textRect_.h+vertical_padding,base_height_));
-#endif
 	if(type_ == TYPE_PRESS) {
-#ifdef USE_TINY_GUI
-		set_width(textRect_.w + horizontal_padding);
-#else
 		if(spacing_ == MINIMUM_SPACE) {
 			set_width(textRect_.w + horizontal_padding);
 		} else {
 			set_width(std::max(textRect_.w+horizontal_padding,base_width_));
 		}
-#endif
 	} else {
 		if(label_.empty()) {
 			set_width(base_width_);
@@ -220,13 +228,13 @@ void button::draw_contents()
 		textx = loc.x + image_w + checkbox_horizontal_padding / 2;
 	}
 
-	SDL_Color button_colour = font::BUTTON_COLOUR;
+	SDL_Color button_color = font::BUTTON_COLOR;
 
 	if (!enabled()) {
 		static const Uint32 disabled_btn_color = 0xAAAAAA;
 		static const double disabled_btn_adjust = 0.18;
 		image = blend_surface(greyscale_image(image), disabled_btn_adjust, disabled_btn_color);
-		button_colour = font::GRAY_COLOUR;
+		button_color = font::GRAY_COLOR;
 	}
 
 	video().blit_surface(loc.x, loc.y, image);
@@ -235,7 +243,7 @@ void button::draw_contents()
 		clipArea.y += offset;
 		clipArea.w -= 2*offset;
 		clipArea.h -= 2*offset;
-		font::draw_text(&video(), clipArea, font_size, button_colour, label_, textx, texty);
+		font::draw_text(&video(), clipArea, font_size, button_color, label_, textx, texty);
 	}
 
 	update_rect(loc);
