@@ -1,11 +1,12 @@
-/* $Id: combo.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: combo.cpp 48480 2011-02-12 16:20:24Z ivanovic $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -23,12 +24,7 @@
 
 namespace gui {
 
-#ifdef USE_TINY_GUI
-	const std::string combo::empty_combo_label = "           ";
-#else
-	const std::string combo::empty_combo_label = "";
-#endif
-
+const std::string combo::empty_combo_label = "";
 
 const int combo::font_size = font::SIZE_SMALL;
 const int combo::horizontal_padding = 10;
@@ -60,11 +56,6 @@ void combo::set_items(const std::vector<std::string>& items)
 	selected_ = -1;
 }
 
-size_t combo::items_size() const
-{
-	return items_.size();
-}
-
 void combo::set_selected_internal(int val)
 {
 	const size_t index = size_t(val);
