@@ -1,11 +1,12 @@
-/* $Id: combo_drag.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: combo_drag.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -25,10 +26,17 @@ namespace gui {
 	const float combo_drag::MIN_DRAG_DISTANCE = 10.0;
 	const float combo_drag::RETURN_SPEED = 25.0;
 
-	combo_drag::combo_drag(display& disp, const std::vector<std::string>& items, const drop_group_manager_ptr group)
-		: combo(disp, items), drop_target(group, location()),
-		drag_target_(-1), old_drag_target_(-1),
-		old_location_(), drag_(NONE)
+	combo_drag::combo_drag(display& disp
+			, const std::vector<std::string>& items
+			, const drop_group_manager_ptr group)
+		: combo(disp, items)
+		, drop_target(group, location())
+		, drag_target_(-1)
+		, old_drag_target_(-1)
+		, old_location_()
+		, mouse_x_(-1)
+		, mouse_y_(-1)
+		, drag_(NONE)
 	{
 	}
 
