@@ -1,11 +1,12 @@
-/* $Id: combo_drag.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: combo_drag.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2008 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,16 +15,11 @@
 #ifndef COMBO_DRAG_H_INCLUDED
 #define COMBO_DRAG_H_INCLUDED
 
-#include "SDL.h"
-
-#include "events.hpp"
 #include "widgets/combo.hpp"
 #include "widgets/drop_target.hpp"
 
-#include <boost/shared_ptr.hpp>
 class display;
 
-
 namespace gui {
 
 	class combo_drag;
@@ -35,8 +31,8 @@ namespace gui {
 			combo_drag(display& disp, const std::vector<std::string>& items, const drop_group_manager_ptr group);
 
 			/**
-			 * used to query if this object was droped to a target
-			 * @return: -1 if not droped and other the id of target object
+			 * used to query if this object was dropped to a target
+			 * @return: -1 if not dropped and other the id of target object
 			 **/
 			int get_drop_target();
 			/**
