@@ -1,11 +1,12 @@
-/* $Id: drop_target.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: drop_target.cpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2008 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -65,7 +66,7 @@ namespace gui {
 					end,
 					boost::bind(&drop_target::hit_rect,
 						boost::bind(&drop_groups::value_type::second,_1)
-						,loc_, id_));
+						,boost::cref(loc_), id_));
 
 		if (itor == end)
 			return -1;
