@@ -1,37 +1,46 @@
-/* $Id: menu.cpp 15769 2007-02-14 05:09:02Z sapient $ */
+/* $Id: menu.cpp 49133 2011-04-09 16:00:13Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "global.hpp"
 
 #include "widgets/menu.hpp"
 
-#include "language.hpp"
+#include "game_config.hpp"
 #include "font.hpp"
+#include "language.hpp"
 #include "image.hpp"
 #include "marked-up_text.hpp"
-#include "sdl_utils.hpp"
-#include "util.hpp"
+#include "sound.hpp"
 #include "video.hpp"
 #include "wml_separators.hpp"
-#include "serialization/string_utils.hpp"
 
-#include <algorithm>
-#include <cassert>
 #include <numeric>
 
 namespace gui {
 
 menu::basic_sorter::basic_sorter()
+	: alpha_sort_()
+	, numeric_sort_()
+	, id_sort_()
+	, xp_sort_()
+	, level_sort_()
+	, redirect_sort_()
+	, pos_sort_()
+	, xp_col_(-1)
 {
 	set_id_sort(-1);
 }
@@ -48,6 +57,19 @@ menu::basic_sorter& menu::basic_sorter::set_numeric_sort(int column)
 	return *this;
 }
 
+menu::basic_sorter& menu::basic_sorter::set_xp_sort(int column)
+{
+	xp_sort_.insert(column);
+	return *this;
+}
+
+menu::basic_sorter& menu::basic_sorter::set_level_sort(int level_column, int xp_column)
+{
+	level_sort_.insert(level_column);
+	xp_col_ = xp_column;
+	return *this;
+}
+
 menu::basic_sorter& menu::basic_sorter::set_id_sort(int column)
 {
 	id_sort_.insert(column);
@@ -77,7 +99,27 @@ bool menu::basic_sorter::column_sortable(int column) const
 	}
 
 	return alpha_sort_.count(column) == 1 || numeric_sort_.count(column) == 1 ||
-		   pos_sort_.count(column) == 1 || id_sort_.count(column) == 1;
+		   pos_sort_.count(column) == 1 || id_sort_.count(column) == 1 ||
+		    xp_sort_.count(column) == 1 || level_sort_.count(column) == 1;
+}
+
+static std::pair<int, int> parse_fraction(const std::string& s)
+{
+	std::vector<std::string> parts = utils::split(s, '/', 0);
+	parts.resize(2);
+	int num = lexical_cast_default<int>(parts[0], 0);
+	int denom = lexical_cast_default<int>(parts[1], 0);
+	return std::make_pair(num, denom);
+}
+
+static int xp_to_advance(const std::string& s) {
+	std::pair<int,int> xp_frac = parse_fraction(s);
+
+	//consider units without AMLA or advancement as having xp_max=1000000
+	if(xp_frac.second == 0)
+		xp_frac.second = 1000000;
+
+	return xp_frac.second - xp_frac.first;
 }
 
 bool menu::basic_sorter::less(int column, const item& row1, const item& row2) const
@@ -99,33 +141,38 @@ bool menu::basic_sorter::less(int column, const item& row1, const item& row2) co
 		return true;
 	}
 
-	const std::string& item1 = row1.fields[column];
-	const std::string& item2 = row2.fields[column];
+	const std::string& item1 = font::del_tags(row1.fields[column]);
+	const std::string& item2 = font::del_tags(row2.fields[column]);
+
 	if(alpha_sort_.count(column) == 1) {
 		std::string::const_iterator begin1 = item1.begin(), end1 = item1.end(),
 		                            begin2 = item2.begin(), end2 = item2.end();
-		while(begin1 != end1 && (font::is_format_char(*begin1) || is_wml_separator(*begin1))) {
+		while(begin1 != end1 && is_wml_separator(*begin1)) {
 			++begin1;
 		}
 
-		while(begin2 != end2 && (font::is_format_char(*begin2) || is_wml_separator(*begin2))) {
+		while(begin2 != end2 && is_wml_separator(*begin2)) {
 			++begin2;
 		}
 
 		return std::lexicographical_compare(begin1,end1,begin2,end2,chars_less_insensitive);
 	} else if(numeric_sort_.count(column) == 1) {
-		const char* a = item1.c_str();
-		const char* b = item2.c_str();
+		int val_1 = lexical_cast_default<int>(item1, 0);
+		int val_2 = lexical_cast_default<int>(item2, 0);
 
-		while(*a != 0 && !isdigit(*a)) {
-			++a;
+		return val_1 > val_2;
+	} else if(xp_sort_.count(column) == 1) {
+		return xp_to_advance(item1) < xp_to_advance(item2);
+	} else if(level_sort_.count(column) == 1) {
+		int level_1 = lexical_cast_default<int>(item1, 0);
+		int level_2 = lexical_cast_default<int>(item2, 0);
+		if (level_1 == level_2) {
+			//break tie using xp
+			const std::string& xp_item1 = font::del_tags(row1.fields[xp_col_]);
+			const std::string& xp_item2 = font::del_tags(row2.fields[xp_col_]);
+			return xp_to_advance(xp_item1) < xp_to_advance(xp_item2);
 		}
-
-		while(*b != 0 && !isdigit(*b)) {
-			++b;
-		}
-
-		return atoi(a) > atoi(b);
+		return level_1 > level_2;
 	}
 
 	const std::map<int,std::vector<int> >::const_iterator itor = pos_sort_.find(column);
@@ -147,17 +194,17 @@ bool menu::basic_sorter::less(int column, const item& row1, const item& row2) co
 
 menu::menu(CVideo& video, const std::vector<std::string>& items,
            bool click_selects, int max_height, int max_width,
-		   const sorter* sorter_obj, style *menu_style)
-        : scrollarea(video),
+		   const sorter* sorter_obj, style *menu_style, const bool auto_join)
+        : scrollarea(video, auto_join), silent_(false),
           max_height_(max_height), max_width_(max_width), max_items_(-1), item_height_(-1),
 		  heading_height_(-1),
 	  cur_help_(-1,-1), help_string_(-1),
-	  selected_(0), click_selects_(click_selects),
+	  selected_(0), click_selects_(click_selects), out_(false),
 	  previous_button_(true), show_result_(false),
 	  double_clicked_(false),
 	  num_selects_(true),
 	  ignore_next_doubleclick_(false),
-	  last_was_doubleclick_(false),
+	  last_was_doubleclick_(false), use_ellipsis_(false),
 	  sorter_(sorter_obj), sortby_(-1), sortreversed_(false), highlight_heading_(-1)
 {
 	style_ = (menu_style) ? menu_style : &default_style;
@@ -165,6 +212,10 @@ menu::menu(CVideo& video, const std::vector<std::string>& items,
 	fill_items(items, true);
 }
 
+menu::~menu()
+{
+}
+
 void menu::fill_items(const std::vector<std::string>& items, bool strip_spaces)
 {
 	for(std::vector<std::string>::const_iterator itor = items.begin();
@@ -237,7 +288,7 @@ void menu::do_sort()
 	recalculate_pos();
 
 	if(selectid >= 0 && selectid < int(item_pos_.size())) {
-		move_selection(selectid,NO_MOVE_VIEWPORT);
+		move_selection_to(selectid, true, NO_MOVE_VIEWPORT);
 	}
 
 	set_dirty();
@@ -257,8 +308,7 @@ void menu::assert_pos()
 	size_t sz = items_.size();
 	assert(item_pos_.size() == sz);
 	for(size_t n = 0; n != sz; ++n) {
-		size_t i = item_pos_[n];
-		assert(i < sz && n == items_[i].id);
+		assert(item_pos_[n] < sz && n == items_[item_pos_[n]].id);
 	}
 }
 
@@ -292,20 +342,24 @@ void menu::update_size()
 {
 	unsigned int h = heading_height();
 	for(size_t i = get_position(),
-	    i_end = minimum(items_.size(), i + max_items_onscreen());
+	    i_end = std::min(items_.size(), i + max_items_onscreen());
 	    i < i_end; ++i)
 		h += get_item_rect(i).h;
-	h = maximum(h, height());
-	if (max_height_ > 0 && h > (unsigned)max_height_)
+	h = std::max(h, height());
+	if (max_height_ > 0 && h > static_cast<unsigned>(max_height_)) {
 		h = max_height_;
+	}
 
+	use_ellipsis_ = false;
 	std::vector<int> const &widths = column_widths();
 	unsigned w = std::accumulate(widths.begin(), widths.end(), 0);
 	if (items_.size() > max_items_onscreen())
 		w += scrollbar_width();
-	w = maximum(w, width());
-	if (max_width_ > 0 && w > (unsigned)max_width_)
+	w = std::max(w, width());
+	if (max_width_ > 0 && w > static_cast<unsigned>(max_width_)) {
+		use_ellipsis_ = true;
 		w = max_width_;
+	}
 
 	update_scrollbar_grip_height();
 	set_measurements(w, h);
@@ -380,6 +434,7 @@ void menu::set_heading(const std::vector<std::string>& heading)
 
 void menu::set_items(const std::vector<std::string>& items, bool strip_spaces, bool keep_viewport)
 {
+
 	const bool scrolled_to_max = (has_scrollbar() && get_position() == get_max_position());
 	items_.clear();
 	item_pos_.clear();
@@ -419,6 +474,9 @@ void menu::set_max_height(const int new_max_height)
 void menu::set_max_width(const int new_max_width)
 {
 	max_width_ = new_max_width;
+	itemRects_.clear();
+	column_widths_.clear();
+	update_size();
 }
 
 size_t menu::max_items_onscreen() const
@@ -428,6 +486,7 @@ size_t menu::max_items_onscreen() const
 	}
 
 	const size_t max_height = (max_height_ == -1 ? (video().gety()*66)/100 : max_height_) - heading_height();
+
 	std::vector<int> heights;
 	size_t n;
 	for(n = 0; n != items_.size(); ++n) {
@@ -453,43 +512,67 @@ void menu::adjust_viewport_to_selection()
 	adjust_position(selected_);
 }
 
+void menu::set_selection_pos(size_t new_selected, bool silent, SELECTION_MOVE_VIEWPORT move_viewport)
+{
+	if (new_selected >= items_.size())
+		return;
+
+	bool changed = false;
+	if (new_selected != selected_) {
+		invalidate_row_pos(selected_);
+		invalidate_row_pos(new_selected);
+		selected_ = new_selected;
+		changed = true;
+	}
+
+	if(move_viewport == MOVE_VIEWPORT) {
+		adjust_viewport_to_selection();
+		if(!silent_ && !silent && changed) {
+			sound::play_UI_sound(game_config::sounds::menu_select);
+		}
+	}
+}
+
 void menu::move_selection_up(size_t dep)
 {
 	set_selection_pos(selected_ > dep ? selected_ - dep : 0);
 }
 
-void menu::reset_selection()
-{
-	set_selection_pos(0);
-}
-
 void menu::move_selection_down(size_t dep)
 {
 	size_t nb_items = items_.size();
 	set_selection_pos(selected_ + dep >= nb_items ? nb_items - 1 : selected_ + dep);
 }
 
-void menu::set_selection_pos(size_t new_selected, SELECTION_MOVE_VIEWPORT move_viewport)
+// private function with control over sound and viewport
+void menu::move_selection_to(size_t id, bool silent, SELECTION_MOVE_VIEWPORT move_viewport)
 {
-	if (new_selected == selected_ || new_selected >= items_.size())
-		return;
-
-	invalidate_row_pos(selected_);
-	invalidate_row_pos(new_selected);
-	selected_ = new_selected;
-
-	if(move_viewport == MOVE_VIEWPORT) {
-		adjust_viewport_to_selection();
+	if(id < item_pos_.size()) {
+		set_selection_pos(item_pos_[id], silent, move_viewport);
 	}
 }
 
-void menu::move_selection(size_t id, SELECTION_MOVE_VIEWPORT move_viewport)
+// public function
+void menu::move_selection(size_t id)
 {
 	if(id < item_pos_.size()) {
-		set_selection_pos(item_pos_[id],move_viewport);
+		set_selection_pos(item_pos_[id], true, MOVE_VIEWPORT);
 	}
 }
 
+// public function
+void menu::move_selection_keeping_viewport(size_t id)
+{
+	if(id < item_pos_.size()) {
+		set_selection_pos(item_pos_[id], true, NO_MOVE_VIEWPORT);
+	}
+}
+
+void menu::reset_selection()
+{
+	set_selection_pos(0);
+}
+
 void menu::key_press(SDLKey key)
 {
 	if (!click_selects_) {
@@ -567,9 +650,9 @@ void menu::handle_event(const SDL_Event& event)
 		// Only pass key events if we have the focus
 		if (focus(&event))
 			key_press(event.key.keysym.sym);
-	} else if(event.type == SDL_MOUSEBUTTONDOWN &&
-	          event.button.button == SDL_BUTTON_LEFT ||
-		  event.type == DOUBLE_CLICK_EVENT) {
+	} else if(!mouse_locked() && ((event.type == SDL_MOUSEBUTTONDOWN &&
+	         (event.button.button == SDL_BUTTON_LEFT || event.button.button == SDL_BUTTON_RIGHT)) ||
+	         event.type == DOUBLE_CLICK_EVENT)) {
 
 		int x = 0;
 		int y = 0;
@@ -577,14 +660,14 @@ void menu::handle_event(const SDL_Event& event)
 			x = event.button.x;
 			y = event.button.y;
 		} else {
-			x = (long)event.user.data1;
-			y = (long)event.user.data2;
+			x = reinterpret_cast<long>(event.user.data1);
+			y = reinterpret_cast<long>(event.user.data2);
 		}
 
 		const int item = hit(x,y);
 		if(item != -1) {
 			set_focus(true);
-			move_selection(item);
+			move_selection_to(item);
 
 			if(click_selects_) {
 				show_result_ = true;
@@ -596,6 +679,9 @@ void menu::handle_event(const SDL_Event& event)
 				} else {
 					double_clicked_ = true;
 					last_was_doubleclick_ = true;
+					if(!silent_) {
+						sound::play_UI_sound(game_config::sounds::button_press);
+					}
 				}
 			} else if (last_was_doubleclick_) {
 				// If we have a double click as the next event, it means
@@ -618,11 +704,17 @@ void menu::handle_event(const SDL_Event& event)
 				sort_by(heading);
 			}
 		}
-	} else if(event.type == SDL_MOUSEMOTION) {
+	} else if(!mouse_locked() && event.type == SDL_MOUSEMOTION) {
 		if(click_selects_) {
 			const int item = hit(event.motion.x,event.motion.y);
-			if (item != -1)
-				move_selection(item);
+			const bool out = (item == -1);
+			if (out_ != out) {
+					out_ = out;
+					invalidate_row_pos(selected_);
+			}
+			if (item != -1) {
+				move_selection_to(item);
+			}
 		}
 
 		const int heading_item = hit_heading(event.motion.x,event.motion.y);
@@ -666,6 +758,17 @@ void menu::scroll(unsigned int)
 	set_dirty();
 }
 
+void menu::set_sorter(sorter *s)
+{
+	if(sortby_ >= 0) {
+		//clear an existing sort
+		sort_by(-1);
+	}
+	sorter_ = s;
+	sortreversed_ = false;
+	sortby_ = -1;
+}
+
 void menu::sort_by(int column)
 {
 	const bool already_sorted = (column == sortby_);
@@ -689,9 +792,9 @@ void menu::sort_by(int column)
 
 SDL_Rect menu::style::item_size(const std::string& item) const {
 	SDL_Rect res = {0,0,0,0};
-	std::vector<std::string> img_text_items = utils::split(item, IMG_TEXT_SEPARATOR);
+	std::vector<std::string> img_text_items = utils::split(item, IMG_TEXT_SEPARATOR, utils::REMOVE_EMPTY);
 	for (std::vector<std::string>::const_iterator it = img_text_items.begin();
-		 it != img_text_items.end(); it++) {
+		 it != img_text_items.end(); ++it) {
 		if (res.w > 0 || res.h > 0) {
 			// Not the first item, add the spacing.
 			res.w += 5;
@@ -699,17 +802,17 @@ SDL_Rect menu::style::item_size(const std::string& item) const {
 		const std::string str = *it;
 		if (!str.empty() && str[0] == IMAGE_PREFIX) {
 			const std::string image_name(str.begin()+1,str.end());
-			surface const img = image::get_image(image_name,image::UNSCALED);
+			surface const img = get_item_image(image_name);
 			if(img != NULL) {
 				res.w += img->w;
-				res.h = maximum<int>(img->h, res.h);
+				res.h = std::max<int>(img->h, res.h);
 			}
 		} else {
 			const SDL_Rect area = {0,0,10000,10000};
 			const SDL_Rect font_size =
-				font::draw_text(NULL,area,get_font_size(),font::NORMAL_COLOUR,str,0,0);
+				font::draw_text(NULL,area,get_font_size(),font::NORMAL_COLOR,str,0,0);
 			res.w += font_size.w;
-			res.h = maximum<int>(font_size.h, res.h);
+			res.h = std::max<int>(font_size.h, res.h);
 		}
 	}
 	return res;
@@ -827,17 +930,17 @@ void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 		const int last_x = xpos;
 		column.w = widths[i];
 		std::string str = row[i];
-		std::vector<std::string> img_text_items = utils::split(str, IMG_TEXT_SEPARATOR);
+		std::vector<std::string> img_text_items = utils::split(str, IMG_TEXT_SEPARATOR, utils::REMOVE_EMPTY);
 		for (std::vector<std::string>::const_iterator it = img_text_items.begin();
-			 it != img_text_items.end(); it++) {
+			 it != img_text_items.end(); ++it) {
 			str = *it;
 			if (!str.empty() && str[0] == IMAGE_PREFIX) {
 				const std::string image_name(str.begin()+1,str.end());
-				const surface img = image::get_image(image_name,image::UNSCALED);
-				const int max_width = max_width_ < 0 ? area.w :
-					minimum<int>(max_width_, area.w - xpos);
-				if(img != NULL && (xpos - rect.x) + img->w < max_width
-				   && rect.y + img->h < area.h) {
+				const surface img = style_->get_item_image(image_name);
+				const int remaining_width = max_width_ < 0 ? area.w :
+				std::min<int>(max_width_, ((lang_rtl)? xpos - rect.x : rect.x + rect.w - xpos));
+				if(img != NULL && img->w <= remaining_width
+				&& rect.y + img->h < area.h) {
 					const size_t y = rect.y + (rect.h - img->h)/2;
 					const size_t w = img->w + 5;
 					const size_t x = xpos + ((lang_rtl) ? widths[i] - w : 0);
@@ -848,15 +951,29 @@ void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 				}
 			} else {
 				column.x = xpos;
-				const std::string to_show = max_width_ > -1 ?
-					font::make_text_ellipsis(str, style_->get_font_size(), loc.w - (xpos - rect.x)) : str;
+				const bool has_wrap = (str.find_first_of("\r\n") != std::string::npos);
+				//prevent ellipsis calculation if there is any line wrapping
+				std::string to_show = str;
+				if (use_ellipsis_ && !has_wrap)
+				{
+					int fs = style_->get_font_size();
+					int style = TTF_STYLE_NORMAL;
+					int w = loc.w - (xpos - rect.x) - 2 * style_->get_thickness();
+					std::string::const_iterator i_beg = to_show.begin(), i_end = to_show.end(),
+						i = font::parse_markup(i_beg, i_end, &fs, NULL, &style);
+					if (i != i_end) {
+						std::string tmp(i, i_end);
+						to_show.erase(i - i_beg, i_end - i_beg);
+						to_show += font::make_text_ellipsis(tmp, fs, w, style);
+					}
+				}
 				const SDL_Rect& text_size = font::text_area(str,style_->get_font_size());
 				const size_t y = rect.y + (rect.h - text_size.h)/2;
-				font::draw_text(&video(),column,style_->get_font_size(),font::NORMAL_COLOUR,to_show,xpos,y);
+				font::draw_text(&video(),column,style_->get_font_size(),font::NORMAL_COLOR,to_show,xpos,y);
 
 				if(type == HEADING_ROW && sortby_ == int(i)) {
 					const surface sort_img = image::get_image(sortreversed_ ? "misc/sort-arrow.png" :
-					                                   "misc/sort-arrow-reverse.png", image::UNSCALED);
+					                                   "misc/sort-arrow-reverse.png");
 					if(sort_img != NULL && sort_img->w <= widths[i] && sort_img->h <= rect.h) {
 						const size_t sort_x = xpos + widths[i] - sort_img->w;
 						const size_t sort_y = rect.y + rect.h/2 - sort_img->h/2;
@@ -881,7 +998,8 @@ void menu::draw_contents()
 	style_->draw_row(*this,0,heading_rect,HEADING_ROW);
 
 	for(size_t i = 0; i != item_pos_.size(); ++i) {
-		style_->draw_row(*this,item_pos_[i],get_item_rect(i),item_pos_[i] == selected_ ? SELECTED_ROW : NORMAL_ROW);
+		style_->draw_row(*this,item_pos_[i],get_item_rect(i),
+			 (!out_ && item_pos_[i] == selected_) ? SELECTED_ROW : NORMAL_ROW);
 	}
 }
 
@@ -904,7 +1022,8 @@ void menu::draw()
 				const unsigned int pos = item_pos_[*i];
 				const SDL_Rect& rect = get_item_rect(*i);
 				bg_restore(rect);
-				style_->draw_row(*this,pos,rect,pos == selected_ ? SELECTED_ROW : NORMAL_ROW);
+				style_->draw_row(*this,pos,rect,
+					(!out_ && pos == selected_) ? SELECTED_ROW : NORMAL_ROW);
 				update_rect(rect);
 			}
 		}
@@ -917,9 +1036,8 @@ void menu::draw()
 
 	bg_restore();
 
-	util::scoped_ptr<clip_rect_setter> clipper(NULL);
-	if(clip_rect())
-		clipper.assign(new clip_rect_setter(video().getSurface(), *clip_rect()));
+	clip_rect_setter clipping_rect =
+			clip_rect_setter(video().getSurface(), clip_rect(), clip_rect() != NULL);
 
 	draw_contents();
 
@@ -944,15 +1062,13 @@ int menu::hit(int x, int y) const
 int menu::hit_column(int x) const
 {
 	std::vector<int> const &widths = column_widths();
-	x -= location().x;
-	for(int j = 0, j_end = widths.size(); j != j_end; ++j) {
-		x -= widths[j];
-		if (x < 0) {
-			return j;
+	int j = -1, j_end = widths.size();
+	for(x -= location().x; x >= 0; x -= widths[j]) {
+		if(++j == j_end) {
+			return -1;
 		}
 	}
-
-	return -1;
+	return j;
 }
 
 std::pair<int,int> menu::hit_cell(int x, int y) const
@@ -974,7 +1090,7 @@ int menu::hit_heading(int x, int y) const
 {
 	const size_t height = heading_height();
 	const SDL_Rect& loc = inner_location();
-	if(y >= loc.y && (size_t)y < loc.y + height) {
+	if(y >= loc.y && static_cast<size_t>(y) < loc.y + height) {
 		return hit_column(x);
 	} else {
 		return -1;
@@ -988,7 +1104,6 @@ SDL_Rect menu::get_item_rect(int item) const
 
 SDL_Rect menu::get_item_rect_internal(size_t item) const
 {
-	const SDL_Rect empty_rect = {0,0,0,0};
 	unsigned int first_item_on_screen = get_position();
 	if (item < first_item_on_screen ||
 	    size_t(item) >= first_item_on_screen + max_items_onscreen()) {
@@ -1007,7 +1122,7 @@ SDL_Rect menu::get_item_rect_internal(size_t item) const
 		y = prev.y + prev.h;
 	}
 
-	SDL_Rect res = { loc.x, y, loc.w, get_item_height(item) };
+	SDL_Rect res = create_rect(loc.x, y, loc.w, get_item_height(item));
 
 	SDL_Rect const &screen_area = ::screen_area();
 
@@ -1036,7 +1151,7 @@ size_t menu::get_item_height_internal(const std::vector<std::string>& item) cons
 	size_t res = 0;
 	for(std::vector<std::string>::const_iterator i = item.begin(); i != item.end(); ++i) {
 		SDL_Rect rect = style_->item_size(*i);
-		res = maximum<int>(rect.h,res);
+		res = std::max<int>(rect.h,res);
 	}
 
 	return res;
@@ -1048,7 +1163,7 @@ size_t menu::heading_height() const
 		heading_height_ = int(get_item_height_internal(heading_));
 	}
 
-	return minimum<unsigned int>(heading_height_,max_height_);
+	return std::min<unsigned int>(heading_height_,max_height_);
 }
 
 size_t menu::get_item_height(int) const
@@ -1058,7 +1173,7 @@ size_t menu::get_item_height(int) const
 
 	size_t max_height = 0;
 	for(size_t n = 0; n != items_.size(); ++n) {
-		max_height = maximum<int>(max_height,get_item_height_internal(items_[n].fields));
+		max_height = std::max<int>(max_height,get_item_height_internal(items_[n].fields));
 	}
 
 	return item_height_ = max_height;
