@@ -1,26 +1,24 @@
-/* $Id: scrollarea.hpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: scrollarea.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2004 - 2010 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2004 - 2011 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
-/** @file widgets/scrollarea.hpp */
+/** @file */
 
 #ifndef SCROLLAREA_HPP_INCLUDED
 #define SCROLLAREA_HPP_INCLUDED
 
-#include "SDL.h"
-#include "../sdl_utils.hpp"
 #include "scrollbar.hpp"
-#include "widget.hpp"
 
 namespace gui {
 
