@@ -1,11 +1,12 @@
-/* $Id: textbox.hpp 42225 2010-04-23 13:57:22Z ilor $ */
+/* $Id: textbox.hpp 48153 2011-01-01 15:57:50Z mordante $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2
-   or at your option any later version.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -16,13 +17,10 @@
 #define TEXTBOX_HPP_INCLUDED
 
 #include "../serialization/string_utils.hpp"
-#include "../sdl_utils.hpp"
 #include "font.hpp"
 
 #include "scrollarea.hpp"
 
-#include "SDL.h"
-
 namespace gui {
 
 class textbox : public scrollarea
@@ -32,8 +30,8 @@ public:
 	virtual ~textbox();
 
 	const std::string text() const;
-	void set_text(const std::string& text, const SDL_Color& color =font::NORMAL_COLOUR);
-	void append_text(const std::string& text,bool auto_scroll = false, const SDL_Color& color =font::NORMAL_COLOUR);
+	void set_text(const std::string& text, const SDL_Color& color =font::NORMAL_COLOR);
+	void append_text(const std::string& text,bool auto_scroll = false, const SDL_Color& color =font::NORMAL_COLOR);
 	void clear();
 
 	void set_editable(bool value);
@@ -85,8 +83,8 @@ private:
 	void handle_event(const SDL_Event& event);
 
 	void draw_cursor(int pos, CVideo &video) const;
-	void update_text_cache(bool reset = false, const SDL_Color& color =font::NORMAL_COLOUR);
-	surface add_text_line(const wide_string& text, const SDL_Color& color =font::NORMAL_COLOUR);
+	void update_text_cache(bool reset = false, const SDL_Color& color =font::NORMAL_COLOR);
+	surface add_text_line(const wide_string& text, const SDL_Color& color =font::NORMAL_COLOR);
 	bool is_selection();
 	void erase_selection();
 
